@@ -77,13 +77,13 @@ typedef uint32_t vmmDevHypPhys;
  */
 
 /** guest can (== wants to) handle absolute coordinates */
-#define VBOXGUEST_MOUSE_GUEST_CAN_ABSOLUTE      BIT(0)
+#define VBOXGUEST_MOUSE_GUEST_CAN_ABSOLUTE      RT_BIT(0)
 /** host can (== wants to) send absolute coordinates */
-#define VBOXGUEST_MOUSE_HOST_CAN_ABSOLUTE       BIT(1)
+#define VBOXGUEST_MOUSE_HOST_CAN_ABSOLUTE       RT_BIT(1)
 /** guest can *NOT* switch to software cursor and therefore depends on the host cursor */
-#define VBOXGUEST_MOUSE_GUEST_NEEDS_HOST_CURSOR BIT(2)
+#define VBOXGUEST_MOUSE_GUEST_NEEDS_HOST_CURSOR RT_BIT(2)
 /** host does NOT provide support for drawing the cursor itself (e.g. L4 console) */
-#define VBOXGUEST_MOUSE_HOST_CANNOT_HWPOINTER   BIT(3)
+#define VBOXGUEST_MOUSE_HOST_CANNOT_HWPOINTER   RT_BIT(3)
 
 /** fictive start address of the hypervisor physical memory for MmMapIoSpace */
 #define HYPERVISOR_PHYSICAL_START  0xf8000000
@@ -103,9 +103,9 @@ typedef uint32_t vmmDevHypPhys;
  * 
  * @remark  These defines also live in the 16-bit and assembly versions of this header.
  */
-#define VMMDEV_VERSION_MAJOR (0x1)
-#define VMMDEV_VERSION_MINOR (0x4)
-#define VMMDEV_VERSION ((VMMDEV_VERSION_MAJOR << 16) | VMMDEV_VERSION_MINOR)
+#define VMMDEV_VERSION       0x00010004
+#define VMMDEV_VERSION_MAJOR (VMMDEV_VERSION >> 16)
+#define VMMDEV_VERSION_MINOR (VMMDEV_VERSION & 0xffff)
 
 /**
  * VMMDev request types.
@@ -915,17 +915,17 @@ typedef struct
  */
 
 /** Host mouse capabilities has been changed. */
-#define VMMDEV_EVENT_MOUSE_CAPABILITIES_CHANGED     BIT(0)
+#define VMMDEV_EVENT_MOUSE_CAPABILITIES_CHANGED     RT_BIT(0)
 /** HGCM event. */
-#define VMMDEV_EVENT_HGCM                           BIT(1)
+#define VMMDEV_EVENT_HGCM                           RT_BIT(1)
 /** A display change request has been issued. */
-#define VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST         BIT(2)
+#define VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST         RT_BIT(2)
 /** Credentials are available for judgement. */
-#define VMMDEV_EVENT_JUDGE_CREDENTIALS              BIT(3)
+#define VMMDEV_EVENT_JUDGE_CREDENTIALS              RT_BIT(3)
 /** The guest has been restored. */
-#define VMMDEV_EVENT_RESTORED                       BIT(4)
+#define VMMDEV_EVENT_RESTORED                       RT_BIT(4)
 /** Seamless mode state changed */
-#define VMMDEV_EVENT_SEAMLESS_MODE_CHANGE_REQUEST   BIT(5)
+#define VMMDEV_EVENT_SEAMLESS_MODE_CHANGE_REQUEST   RT_BIT(5)
 
 
 /** @} */
@@ -1144,26 +1144,26 @@ typedef struct _VBoxGuestHGCMCallInfo
 #define VMMDEV_CREDENTIALS_STRLEN           128
 
 /** query from host whether credentials are present */
-#define VMMDEV_CREDENTIALS_QUERYPRESENCE     BIT(1)
+#define VMMDEV_CREDENTIALS_QUERYPRESENCE     RT_BIT(1)
 /** read credentials from host (can be combined with clear) */
-#define VMMDEV_CREDENTIALS_READ              BIT(2)
+#define VMMDEV_CREDENTIALS_READ              RT_BIT(2)
 /** clear credentials on host (can be combined with read) */
-#define VMMDEV_CREDENTIALS_CLEAR             BIT(3)
+#define VMMDEV_CREDENTIALS_CLEAR             RT_BIT(3)
 /** read credentials for judgement in the guest */
-#define VMMDEV_CREDENTIALS_READJUDGE         BIT(8)
+#define VMMDEV_CREDENTIALS_READJUDGE         RT_BIT(8)
 /** clear credentials for judegement on the host */
-#define VMMDEV_CREDENTIALS_CLEARJUDGE        BIT(9)
+#define VMMDEV_CREDENTIALS_CLEARJUDGE        RT_BIT(9)
 /** report credentials acceptance by guest */
-#define VMMDEV_CREDENTIALS_JUDGE_OK          BIT(10)
+#define VMMDEV_CREDENTIALS_JUDGE_OK          RT_BIT(10)
 /** report credentials denial by guest */
-#define VMMDEV_CREDENTIALS_JUDGE_DENY        BIT(11)
+#define VMMDEV_CREDENTIALS_JUDGE_DENY        RT_BIT(11)
 /** report that no judgement could be made by guest */
-#define VMMDEV_CREDENTIALS_JUDGE_NOJUDGEMENT BIT(12)
+#define VMMDEV_CREDENTIALS_JUDGE_NOJUDGEMENT RT_BIT(12)
 
 /** flag telling the guest that credentials are present */
-#define VMMDEV_CREDENTIALS_PRESENT           BIT(16)
+#define VMMDEV_CREDENTIALS_PRESENT           RT_BIT(16)
 /** flag telling guest that local logons should be prohibited */
-#define VMMDEV_CREDENTIALS_NOLOCALLOGON      BIT(17)
+#define VMMDEV_CREDENTIALS_NOLOCALLOGON      RT_BIT(17)
 
 /** credentials request structure */
 #pragma pack(4)
