@@ -32,8 +32,12 @@ __BEGIN_DECLS
 /** CPU mode flags (DISCPUSTATE::mode).
  * @{
  */
-#define CPUMODE_16BIT                   1
-#define CPUMODE_32BIT                   2
+typedef enum
+{
+    CPUMODE_16BIT = 1,
+    CPUMODE_32BIT = 2,
+    CPUMODE_64BIT = 3
+} DISCPUMODE;
 /** @} */
 
 /** Prefix byte flags
@@ -57,63 +61,63 @@ __BEGIN_DECLS
 /**
  * Operand type.
  */
-#define OPTYPE_INVALID               BIT(0)
-#define OPTYPE_HARMLESS              BIT(1)
-#define OPTYPE_CONTROLFLOW           BIT(2)
-#define OPTYPE_POTENTIALLY_DANGEROUS BIT(3)
-#define OPTYPE_DANGEROUS             BIT(4)
-#define OPTYPE_PORTIO                BIT(5)
-#define OPTYPE_PRIVILEGED            BIT(6)
-#define OPTYPE_PRIVILEGED_NOTRAP     BIT(7)
-#define OPTYPE_UNCOND_CONTROLFLOW    BIT(8)
-#define OPTYPE_RELATIVE_CONTROLFLOW  BIT(9)
-#define OPTYPE_COND_CONTROLFLOW      BIT(10)
-#define OPTYPE_INTERRUPT             BIT(11)
-#define OPTYPE_ILLEGAL               BIT(12)
-#define OPTYPE_RRM_DANGEROUS         BIT(14) /**< Some additional dangerouse ones when recompiling raw r0. */
-#define OPTYPE_RRM_DANGEROUS_16      BIT(15) /**< Some additional dangerouse ones when recompiling 16-bit raw r0. */
+#define OPTYPE_INVALID               RT_BIT(0)
+#define OPTYPE_HARMLESS              RT_BIT(1)
+#define OPTYPE_CONTROLFLOW           RT_BIT(2)
+#define OPTYPE_POTENTIALLY_DANGEROUS RT_BIT(3)
+#define OPTYPE_DANGEROUS             RT_BIT(4)
+#define OPTYPE_PORTIO                RT_BIT(5)
+#define OPTYPE_PRIVILEGED            RT_BIT(6)
+#define OPTYPE_PRIVILEGED_NOTRAP     RT_BIT(7)
+#define OPTYPE_UNCOND_CONTROLFLOW    RT_BIT(8)
+#define OPTYPE_RELATIVE_CONTROLFLOW  RT_BIT(9)
+#define OPTYPE_COND_CONTROLFLOW      RT_BIT(10)
+#define OPTYPE_INTERRUPT             RT_BIT(11)
+#define OPTYPE_ILLEGAL               RT_BIT(12)
+#define OPTYPE_RRM_DANGEROUS         RT_BIT(14) /**< Some additional dangerouse ones when recompiling raw r0. */
+#define OPTYPE_RRM_DANGEROUS_16      RT_BIT(15) /**< Some additional dangerouse ones when recompiling 16-bit raw r0. */
 #define OPTYPE_RRM_MASK              (OPTYPE_RRM_DANGEROUS | OPTYPE_RRM_DANGEROUS_16)
-#define OPTYPE_INHIBIT_IRQS          BIT(16) /**< Will or can inhibit irqs (sti, pop ss, mov ss) */
-#define OPTYPE_PORTIO_READ           BIT(17)
-#define OPTYPE_PORTIO_WRITE          BIT(18)
+#define OPTYPE_INHIBIT_IRQS          RT_BIT(16) /**< Will or can inhibit irqs (sti, pop ss, mov ss) */
+#define OPTYPE_PORTIO_READ           RT_BIT(17)
+#define OPTYPE_PORTIO_WRITE          RT_BIT(18)
 #define OPTYPE_ALL                   (0xffffffff)
 
 /** Parameter usage flags.
  * @{
  */
-#define USE_BASE                        BIT(0)
-#define USE_INDEX                       BIT(1)
-#define USE_SCALE                       BIT(2)
-#define USE_REG_GEN8                    BIT(3)
-#define USE_REG_GEN16                   BIT(4)
-#define USE_REG_GEN32                   BIT(5)
-#define USE_REG_FP                      BIT(6)
-#define USE_REG_MMX                     BIT(7)
-#define USE_REG_XMM                     BIT(8)
-#define USE_REG_CR                      BIT(9)
-#define USE_REG_DBG                     BIT(10)
-#define USE_REG_SEG                     BIT(11)
-#define USE_REG_TEST                    BIT(12)
-#define USE_DISPLACEMENT8               BIT(13)
-#define USE_DISPLACEMENT16              BIT(14)
-#define USE_DISPLACEMENT32              BIT(15)
-#define USE_IMMEDIATE8                  BIT(16)
-#define USE_IMMEDIATE8_REL              BIT(17)
-#define USE_IMMEDIATE16                 BIT(18)
-#define USE_IMMEDIATE16_REL             BIT(19)
-#define USE_IMMEDIATE32                 BIT(20)
-#define USE_IMMEDIATE32_REL             BIT(21)
-#define USE_IMMEDIATE64                 BIT(22)
-#define USE_IMMEDIATE_ADDR_0_32         BIT(23)
-#define USE_IMMEDIATE_ADDR_16_32        BIT(24)
-#define USE_IMMEDIATE_ADDR_0_16         BIT(25)
-#define USE_IMMEDIATE_ADDR_16_16        BIT(26)
+#define USE_BASE                        RT_BIT(0)
+#define USE_INDEX                       RT_BIT(1)
+#define USE_SCALE                       RT_BIT(2)
+#define USE_REG_GEN8                    RT_BIT(3)
+#define USE_REG_GEN16                   RT_BIT(4)
+#define USE_REG_GEN32                   RT_BIT(5)
+#define USE_REG_FP                      RT_BIT(6)
+#define USE_REG_MMX                     RT_BIT(7)
+#define USE_REG_XMM                     RT_BIT(8)
+#define USE_REG_CR                      RT_BIT(9)
+#define USE_REG_DBG                     RT_BIT(10)
+#define USE_REG_SEG                     RT_BIT(11)
+#define USE_REG_TEST                    RT_BIT(12)
+#define USE_DISPLACEMENT8               RT_BIT(13)
+#define USE_DISPLACEMENT16              RT_BIT(14)
+#define USE_DISPLACEMENT32              RT_BIT(15)
+#define USE_IMMEDIATE8                  RT_BIT(16)
+#define USE_IMMEDIATE8_REL              RT_BIT(17)
+#define USE_IMMEDIATE16                 RT_BIT(18)
+#define USE_IMMEDIATE16_REL             RT_BIT(19)
+#define USE_IMMEDIATE32                 RT_BIT(20)
+#define USE_IMMEDIATE32_REL             RT_BIT(21)
+#define USE_IMMEDIATE64                 RT_BIT(22)
+#define USE_IMMEDIATE_ADDR_0_32         RT_BIT(23)
+#define USE_IMMEDIATE_ADDR_16_32        RT_BIT(24)
+#define USE_IMMEDIATE_ADDR_0_16         RT_BIT(25)
+#define USE_IMMEDIATE_ADDR_16_16        RT_BIT(26)
 /** DS:ESI */
-#define USE_POINTER_DS_BASED            BIT(27)
+#define USE_POINTER_DS_BASED            RT_BIT(27)
 /** ES:EDI */
-#define USE_POINTER_ES_BASED            BIT(28)
-#define USE_IMMEDIATE16_SX8             BIT(29)
-#define USE_IMMEDIATE32_SX8             BIT(30)
+#define USE_POINTER_ES_BASED            RT_BIT(28)
+#define USE_IMMEDIATE16_SX8             RT_BIT(29)
+#define USE_IMMEDIATE32_SX8             RT_BIT(30)
 
 #define USE_IMMEDIATE                   (USE_IMMEDIATE8|USE_IMMEDIATE16|USE_IMMEDIATE32|USE_IMMEDIATE64|USE_IMMEDIATE8_REL|USE_IMMEDIATE16_REL|USE_IMMEDIATE32_REL|USE_IMMEDIATE_ADDR_0_32|USE_IMMEDIATE_ADDR_16_32|USE_IMMEDIATE_ADDR_0_16|USE_IMMEDIATE_ADDR_16_16|USE_IMMEDIATE16_SX8|USE_IMMEDIATE32_SX8)
 
@@ -213,12 +217,12 @@ __BEGIN_DECLS
 /** Used by DISQueryParamVal & EMIQueryParamVal
  * @{
  */
-#define PARAM_VAL8             BIT(0)
-#define PARAM_VAL16            BIT(1)
-#define PARAM_VAL32            BIT(2)
-#define PARAM_VAL64            BIT(3)
-#define PARAM_VALFARPTR16      BIT(4)
-#define PARAM_VALFARPTR32      BIT(5)
+#define PARAM_VAL8             RT_BIT(0)
+#define PARAM_VAL16            RT_BIT(1)
+#define PARAM_VAL32            RT_BIT(2)
+#define PARAM_VAL64            RT_BIT(3)
+#define PARAM_VALFARPTR16      RT_BIT(4)
+#define PARAM_VALFARPTR32      RT_BIT(5)
 
 #define PARMTYPE_REGISTER      1
 #define PARMTYPE_ADDRESS       2
@@ -312,7 +316,7 @@ typedef struct _OPCODE *POPCODE;
 /** Pointer to const opcode. */
 typedef const struct _OPCODE *PCOPCODE;
 
-typedef DECLCALLBACK(int32_t) FN_DIS_READBYTES(RTUINTPTR pSrc, uint8_t *pDest, uint32_t size, RTUINTPTR dwUserdata);
+typedef DECLCALLBACK(int) FN_DIS_READBYTES(RTUINTPTR pSrc, uint8_t *pDest, uint32_t size, void *pvUserdata);
 typedef FN_DIS_READBYTES *PFN_DIS_READBYTES;
 
 /* forward decl */
@@ -322,7 +326,7 @@ typedef struct _DISCPUSTATE *PDISCPUSTATE;
 
 /** Parser callback.
  * @remark no DECLCALLBACK() here because it's considered to be internal (really, I'm too lazy to update all the functions). */
-typedef int FNDISPARSE(RTUINTPTR pu8CodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDISCPUSTATE pCpu);
+typedef unsigned FNDISPARSE(RTUINTPTR pu8CodeBlock, PCOPCODE pOp, POP_PARAMETER pParam, PDISCPUSTATE pCpu);
 typedef FNDISPARSE *PFNDISPARSE;
 
 typedef struct _DISCPUSTATE
@@ -369,7 +373,7 @@ typedef struct _DISCPUSTATE
     /** Pointer to the current instruction. */
     PCOPCODE        pCurInstr;
 
-    RTUINTPTR       dwUserData[3];
+    void           *apvUserData[3];
 
     /** Optional read function */
     PFN_DIS_READBYTES pfnReadBytes;
@@ -401,7 +405,7 @@ typedef struct _OPCODE
 /**
  * Disassembles a code block.
  *
- * @returns Success indicator.
+ * @returns VBox error code
  * @param   pCpu            Pointer to cpu structure which have DISCPUSTATE::mode
  *                          set correctly.
  * @param   pvCodeBlock     Pointer to the strunction to disassemble.
@@ -418,12 +422,12 @@ typedef struct _OPCODE
  * @remark  cbMax isn't respected as a boundry. DISInstr() will read beyond cbMax.
  *          This means *pcbSize >= cbMax sometimes.
  */
-DISDECL(bool) DISBlock(PDISCPUSTATE pCpu, RTUINTPTR pvCodeBlock, int32_t cbMax, uint32_t *pSize);
+DISDECL(int) DISBlock(PDISCPUSTATE pCpu, RTUINTPTR pvCodeBlock, unsigned cbMax, unsigned *pSize);
 
 /**
  * Disassembles one instruction
  *
- * @returns Success indicator.
+ * @returns VBox error code
  * @param   pCpu            Pointer to cpu structure which have DISCPUSTATE::mode
  *                          set correctly.
  * @param   pu8Instruction  Pointer to the instrunction to disassemble.
@@ -434,12 +438,12 @@ DISDECL(bool) DISBlock(PDISCPUSTATE pCpu, RTUINTPTR pvCodeBlock, int32_t cbMax,
  *
  * @todo    Define output callback.
  */
-DISDECL(bool) DISInstr(PDISCPUSTATE pCpu, RTUINTPTR pu8Instruction, uint32_t u32EipOffset, uint32_t *pcbSize, char *pszOutput);
+DISDECL(int) DISInstr(PDISCPUSTATE pCpu, RTUINTPTR pu8Instruction, unsigned u32EipOffset, unsigned *pcbSize, char *pszOutput);
 
 /**
  * Disassembles one instruction
  *
- * @returns Success indicator.
+ * @returns VBox error code
  * @param   pCpu            Pointer to cpu structure which have DISCPUSTATE::mode
  *                          set correctly.
  * @param   pu8Instruction  Pointer to the strunction to disassemble.
@@ -451,20 +455,20 @@ DISDECL(bool) DISInstr(PDISCPUSTATE pCpu, RTUINTPTR pu8Instruction, uint32_t u32
  *
  * @todo    Define output callback.
  */
-DISDECL(bool) DISInstrEx(PDISCPUSTATE pCpu, RTUINTPTR pu8Instruction, uint32_t u32EipOffset, uint32_t *pcbSize,
-                         char *pszOutput, uint32_t uFilter);
+DISDECL(int) DISInstrEx(PDISCPUSTATE pCpu, RTUINTPTR pu8Instruction, uint32_t u32EipOffset, uint32_t *pcbSize,
+                         char *pszOutput, unsigned uFilter);
 
 /**
  * Parses one instruction.
  * The result is found in pCpu.
  *
- * @returns Success indicator.
+ * @returns VBox error code
  * @param   pCpu            Pointer to cpu structure which has DISCPUSTATE::mode set correctly.
  * @param   InstructionAddr Pointer to the instruction to parse.
  * @param   pcbInstruction  Where to store the size of the instruction.
  *                          NULL is allowed.
  */
-DISDECL(bool) DISCoreOne(PDISCPUSTATE pCpu, RTUINTPTR InstructionAddr, unsigned *pcbInstruction);
+DISDECL(int) DISCoreOne(PDISCPUSTATE pCpu, RTUINTPTR InstructionAddr, unsigned *pcbInstruction);
 
 /**
  * Parses one guest instruction.
