@@ -744,7 +744,7 @@
 #define VERR_PDM_MISSING_INTERFACE                  (-2804)
 /** A driver encountered an unknown configuration value.
  * This means that the driver is potentially misconfigured and the driver
- * construction. */
+ * construction failed because of this. */
 #define VERR_PDM_DRVINS_UNKNOWN_CFG_VALUES          (-2805)
 /** The PCI bus assigned to a device didn't have room for it.
  * Either too many devices are configured on the same PCI bus, or there are
@@ -905,6 +905,9 @@
 /** Couldn't find the USB Proxy device. Using OSE? */
 #define VERR_PDM_NO_USBPROXY                        (-2862)
 
+/** The driver properties were invalid, and as a consequence construction
+ * failed. Caused my unusable media or similar problems. */
+#define VERR_PDM_DRIVER_INVALID_PROPERTIES          (-2866)
 /** @} */
 
 
@@ -1102,6 +1105,15 @@
 #define VERR_HWACCM_NO_CPUID                        (-4101)
 /** @} */
 
+/** @name VBox Disassembler Status Codes
+ * @{
+ */
+/** Invalid opcode byte(s) */
+#define VERR_DIS_INVALID_OPCODE                     (-5000)
+/** Generic failure during disassembly. */
+#define VERR_DIS_GEN_FAILURE                        (-5001)
+/** @} */
+
 /* SED-END */
 
 
