@@ -221,38 +221,106 @@ typedef PDMDRVREG const *PCPDMDRVREG;
 /** PDM Driver Classes.
  * @{ */
 /** Mouse input driver. */
-#define PDM_DRVREG_CLASS_MOUSE          BIT(0)
+#define PDM_DRVREG_CLASS_MOUSE          RT_BIT(0)
 /** Keyboard input driver. */
-#define PDM_DRVREG_CLASS_KEYBOARD       BIT(1)
+#define PDM_DRVREG_CLASS_KEYBOARD       RT_BIT(1)
 /** Display driver. */
-#define PDM_DRVREG_CLASS_DISPLAY        BIT(2)
+#define PDM_DRVREG_CLASS_DISPLAY        RT_BIT(2)
 /** Network transport driver. */
-#define PDM_DRVREG_CLASS_NETWORK        BIT(3)
+#define PDM_DRVREG_CLASS_NETWORK        RT_BIT(3)
 /** Block driver. */
-#define PDM_DRVREG_CLASS_BLOCK          BIT(4)
+#define PDM_DRVREG_CLASS_BLOCK          RT_BIT(4)
 /** Media driver. */
-#define PDM_DRVREG_CLASS_MEDIA          BIT(5)
+#define PDM_DRVREG_CLASS_MEDIA          RT_BIT(5)
 /** Mountable driver. */
-#define PDM_DRVREG_CLASS_MOUNTABLE      BIT(6)
+#define PDM_DRVREG_CLASS_MOUNTABLE      RT_BIT(6)
 /** Audio driver. */
-#define PDM_DRVREG_CLASS_AUDIO          BIT(7)
+#define PDM_DRVREG_CLASS_AUDIO          RT_BIT(7)
 /** VMMDev driver. */
-#define PDM_DRVREG_CLASS_VMMDEV         BIT(8)
+#define PDM_DRVREG_CLASS_VMMDEV         RT_BIT(8)
 /** Status driver. */
-#define PDM_DRVREG_CLASS_STATUS         BIT(9)
+#define PDM_DRVREG_CLASS_STATUS         RT_BIT(9)
 /** ACPI driver. */
-#define PDM_DRVREG_CLASS_ACPI           BIT(10)
+#define PDM_DRVREG_CLASS_ACPI           RT_BIT(10)
 /** USB related driver. */
-#define PDM_DRVREG_CLASS_USB            BIT(11)
+#define PDM_DRVREG_CLASS_USB            RT_BIT(11)
 /** ISCSI Transport related driver. */
-#define PDM_DRVREG_CLASS_ISCSITRANSPORT BIT(12)
+#define PDM_DRVREG_CLASS_ISCSITRANSPORT RT_BIT(12)
 /** Char driver. */
-#define PDM_DRVREG_CLASS_CHAR           BIT(13)
+#define PDM_DRVREG_CLASS_CHAR           RT_BIT(13)
 /** Stream driver. */
-#define PDM_DRVREG_CLASS_STREAM         BIT(14)
+#define PDM_DRVREG_CLASS_STREAM         RT_BIT(14)
 /** @} */
 
 
+/**
+ * USB hub registration structure.
+ */
+typedef struct PDMUSBHUBREG
+{
+    /** Structure version number. PDM_USBHUBREG_VERSION defines the current version. */
+    uint32_t            u32Version;
+
+    /**
+     * Request the hub to attach of the specified device.
+     *
+     * @returns VBox status code.
+     * @param   pDrvIns     The hub instance.
+     * @param   pUsbIns     The device to attach.
+     * @param   piPort      Where to store the port number the device was attached to.
+     * @thread EMT.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnAttachDevice,(PPDMDRVINS pDrvIns, PPDMUSBINS pUsbIns, uint32_t *piPort));
+
+    /**
+     * Request the hub to detach of the specified device.
+     *
+     * The device has previously been attached to the hub with the
+     * pfnAttachDevice call. This call is not currently expected to
+     * fail.
+     *
+     * @returns VBox status code.
+     * @param   pDrvIns     The hub instance.
+     * @param   pUsbIns     The device to detach.
+     * @param   iPort       The port number returned by the attach call.
+     * @thread EMT.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnDetachDevice,(PPDMDRVINS pDrvIns, PPDMUSBINS pUsbIns, uint32_t iPort));
+
+    /** Counterpart to u32Version, same value. */
+    uint32_t            u32TheEnd;
+} PDMUSBHUBREG;
+/** Pointer to a const USB hub registration structure. */
+typedef const PDMUSBHUBREG *PCPDMUSBHUBREG;
+
+/** Current PDMUSBHUBREG version number. */
+#define PDM_USBHUBREG_VERSION       0xeb010000
+
+
+/**
+ * USB hub helpers.
+ * This is currently just a place holder.
+ */
+typedef struct PDMUSBHUBHLP
+{
+    /** Structure version. PDM_USBHUBHLP_VERSION defines the current version. */
+    uint32_t                    u32Version;
+
+    /** Just a safety precaution. */
+    uint32_t                    u32TheEnd;
+} PDMUSBHUBHLP;
+/** Pointer to PCI helpers. */
+typedef PDMUSBHUBHLP *PPDMUSBHUBHLP;
+/** Pointer to const PCI helpers. */
+typedef const PDMUSBHUBHLP *PCPDMUSBHUBHLP;
+/** Pointer to const PCI helpers pointer. */
+typedef PCPDMUSBHUBHLP *PPCPDMUSBHUBHLP;
+
+/** Current PDMUSBHUBHLP version number. */
+#define PDM_USBHUBHLP_VERSION       0xea010000
+
+
+
 /**
  * Poller callback.
  *
@@ -438,7 +506,7 @@ typedef struct PDMDRVHLP
      *                          until the timer is fully destroyed (i.e. a bit after TMTimerDestroy()).
      * @param   ppTimer         Where to store the timer on success.
      */
-    DECLR3CALLBACKMEMBER(int, pfnTMTimerCreate,(PPDMDRVINS pDrvIns, TMCLOCK enmClock, PFNTMTIMERDRV pfnCallback, const char *pszDesc, PPTMTIMERHC ppTimer));
+    DECLR3CALLBACKMEMBER(int, pfnTMTimerCreate,(PPDMDRVINS pDrvIns, TMCLOCK enmClock, PFNTMTIMERDRV pfnCallback, const char *pszDesc, PPTMTIMERR3 ppTimer));
 
     /**
      * Register a save state data unit.
@@ -545,19 +613,21 @@ typedef struct PDMDRVHLP
      *
      * @returns VBox status code.
      * @param   pDrvIns         The driver instance.
-     * @param   pvReserved      Reserved for future inteface callback structure.
-     * @param   ppvReservedHlp  Reserved for future helper callback structure.
+     * @param   fVersions       Indicates the kinds of USB devices that can be attached to this HUB.
+     * @param   cPorts          The number of ports.
+     * @param   pUsbHubReg      The hub callback structure that PDMUsb uses to interact with it.
+     * @param   ppUsbHubHlp     The helper callback structure that the hub uses to talk to PDMUsb.
      *
      * @thread  EMT.
      */
-    DECLR3CALLBACKMEMBER(int, pfnUSBRegisterHub,(PPDMDRVINS pDrvIns, void *pvReservedIn, void **ppvReservedHlp));
+    DECLR3CALLBACKMEMBER(int, pfnUSBRegisterHub,(PPDMDRVINS pDrvIns, uint32_t fVersions, uint32_t cPorts, PCPDMUSBHUBREG pUsbHubReg, PPCPDMUSBHUBHLP ppUsbHubHlp));
 
     /**
      * Creates a PDM thread.
-     * 
-     * This differs from the RTThreadCreate() API in that PDM takes care of suspending, 
+     *
+     * This differs from the RTThreadCreate() API in that PDM takes care of suspending,
      * resuming, and destroying the thread as the VM state changes.
-     * 
+     *
      * @returns VBox status code.
      * @param   pDrvIns     The driver instance.
      * @param   ppThread    Where to store the thread 'handle'.
@@ -603,24 +673,24 @@ typedef struct PDMDRVINS
     } Internal;
 
     /** Pointer the PDM Driver API. */
-    HCPTRTYPE(PCPDMDRVHLP)      pDrvHlp;
+    R3PTRTYPE(PCPDMDRVHLP)      pDrvHlp;
     /** Pointer to driver registration structure.  */
-    HCPTRTYPE(PCPDMDRVREG)      pDrvReg;
+    R3PTRTYPE(PCPDMDRVREG)      pDrvReg;
     /** Configuration handle. */
-    HCPTRTYPE(PCFGMNODE)        pCfgHandle;
+    R3PTRTYPE(PCFGMNODE)        pCfgHandle;
     /** Driver instance number. */
     RTUINT                      iInstance;
     /** Pointer to the base interface of the device/driver instance above. */
-    HCPTRTYPE(PPDMIBASE)        pUpBase;
+    R3PTRTYPE(PPDMIBASE)        pUpBase;
     /** Pointer to the base interface of the driver instance below. */
-    HCPTRTYPE(PPDMIBASE)        pDownBase;
+    R3PTRTYPE(PPDMIBASE)        pDownBase;
     /** The base interface of the driver.
      * The driver constructor initializes this. */
     PDMIBASE                    IBase;
     /* padding to make achInstanceData aligned at 16 byte boundrary. */
     uint32_t                    au32Padding[HC_ARCH_BITS == 32 ? 3 : 1];
     /** Pointer to driver instance data. */
-    HCPTRTYPE(void *)           pvInstanceData;
+    R3PTRTYPE(void *)           pvInstanceData;
     /** Driver instance data. The size of this area is defined
      * in the PDMDRVREG::cbInstanceData field. */
     char                        achInstanceData[4];
@@ -667,7 +737,7 @@ DECLINLINE(int) PDMDrvHlpVMSetRuntimeError(PPDMDRVINS pDrvIns, bool fFatal, cons
  * Set the VM runtime error. See PDMDrvHlpVMSetRuntimeError() for printf like message formatting.
  */
 #define PDMDRV_SET_RUNTIME_ERROR(pDrvIns, fFatal, pszErrorID, pszError)  \
-    PDMDrvHlpVMSetError(pDrvIns, fFatal, pszErrorID, "%s", pszError)
+    PDMDrvHlpVMSetRuntimeError(pDrvIns, fFatal, pszErrorID, "%s", pszError)
 
 #endif /* IN_RING3 */
 
@@ -715,9 +785,9 @@ DECLINLINE(void) PDMDrvHlpSTAMRegisterF(PPDMDRVINS pDrvIns, void *pvSample, STAM
 /**
  * @copydoc PDMDRVHLP::pfnUSBRegisterHub
  */
-DECLINLINE(int) PDMDrvHlpUSBRegisterHub(PPDMDRVINS pDrvIns, void *pvReservedIn, void **ppvReservedHlp)
+DECLINLINE(int) PDMDrvHlpUSBRegisterHub(PPDMDRVINS pDrvIns, uint32_t fVersions, uint32_t cPorts, PCPDMUSBHUBREG pUsbHubReg, PPCPDMUSBHUBHLP ppUsbHubHlp)
 {
-    return pDrvIns->pDrvHlp->pfnUSBRegisterHub(pDrvIns, pvReservedIn, ppvReservedHlp);
+    return pDrvIns->pDrvHlp->pfnUSBRegisterHub(pDrvIns, fVersions, cPorts, pUsbHubReg, ppUsbHubHlp);
 }
 
 /**
