@@ -189,9 +189,9 @@ typedef struct PDMIMOUSEPORT
 
 /** Mouse button defines for PDMIMOUSEPORT::pfnPutEvent.
  * @{ */
-#define PDMIMOUSEPORT_BUTTON_LEFT   BIT(0)
-#define PDMIMOUSEPORT_BUTTON_RIGHT  BIT(1)
-#define PDMIMOUSEPORT_BUTTON_MIDDLE BIT(2)
+#define PDMIMOUSEPORT_BUTTON_LEFT   RT_BIT(0)
+#define PDMIMOUSEPORT_BUTTON_RIGHT  RT_BIT(1)
+#define PDMIMOUSEPORT_BUTTON_MIDDLE RT_BIT(2)
 /** @} */
 
 
@@ -1080,6 +1080,13 @@ typedef struct PDMIISCSITRANSPORT
     DECLR3CALLBACKMEMBER(int, pfnClose,(PPDMIISCSITRANSPORT pTransport));
 } PDMIISCSITRANSPORT;
 
+/** @name Bit mask definitions for status line type
+ * @{ */
+#define PDM_ICHAR_STATUS_LINES_DCD  RT_BIT(0)
+#define PDM_ICHAR_STATUS_LINES_RI   RT_BIT(1)
+#define PDM_ICHAR_STATUS_LINES_DSR  RT_BIT(2)
+#define PDM_ICHAR_STATUS_LINES_CTS  RT_BIT(3)
+/** @} */
 
 /** Pointer to a char port interface. */
 typedef struct PDMICHARPORT *PPDMICHARPORT;
@@ -1099,6 +1106,16 @@ typedef struct PDMICHARPORT
      * @thread  Any thread.
      */
     DECLR3CALLBACKMEMBER(int, pfnNotifyRead,(PPDMICHARPORT pInterface, const void *pvBuf, size_t *pcbRead));
+
+    /**
+     * Notify the device/driver when the status lines changed.
+     *
+     * @returns VBox status code.
+     * @param   pInterface      Pointer to the interface structure containing the called function pointer.
+     * @param   fNewStatusLine  New state of the status line pins.
+     * @thread  Any thread.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnNotifyStatusLinesChanged,(PPDMICHARPORT pInterface, uint32_t fNewStatusLines));
 } PDMICHARPORT;
 
 /** Pointer to a char interface. */
@@ -1133,6 +1150,17 @@ typedef struct PDMICHAR
      */
     DECLR3CALLBACKMEMBER(int, pfnSetParameters,(PPDMICHAR pInterface, unsigned Bps, char chParity, unsigned cDataBits, unsigned cStopBits));
 
+    /**
+     * Set the state of the modem lines.
+     *
+     * @returns VBox status code.
+     * @param   pInterface          Pointer to the interface structure containing the called function pointer.
+     * @param   fRequestToSend      Set to true to make the Request to Send line active otherwise to 0.
+     * @param   fDataTerminalReady  Set to true to make the Data Terminal Ready line active otherwise 0.
+     * @thread  Any thread.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnSetModemLines,(PPDMICHAR pInterface, bool fRequestToSend, bool fDataTerminalReady));
+
 } PDMICHAR;
 
 
@@ -1278,6 +1306,23 @@ typedef struct PDMIACPIPORT
      * @param   pInterface      Pointer to the interface structure containing the called function pointer.
      */
     DECLR3CALLBACKMEMBER(int, pfnPowerButtonPress,(PPDMIACPIPORT pInterface));
+
+    /**
+     * Send an ACPI sleep button event.
+     *
+     * @returns VBox status code
+     * @param   pInterface      Pointer to the interface structure containing the called function pointer.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnSleepButtonPress,(PPDMIACPIPORT pInterface));
+
+    /**
+     * Check if the last power button event was handled by the guest.
+     *
+     * @returns VBox status code
+     * @param   pInterface      Pointer to the interface structure containing the called function pointer.
+     * @param   pfHandled       Is set to true if the last power button event was handled, false otherwise.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnGetPowerButtonHandled,(PPDMIACPIPORT pInterface, bool *pfHandled));
 } PDMIACPIPORT;
 
 /** Pointer to an ACPI connector interface. */
@@ -1809,13 +1854,13 @@ typedef union PDMLEDCORE
 /** LED bit masks for the u32 view.
  * @{ */
 /** Reading/Receiving indicator. */
-#define PDMLED_READING  BIT(0)
+#define PDMLED_READING  RT_BIT(0)
 /** Writing/Sending indicator. */
-#define PDMLED_WRITING  BIT(1)
+#define PDMLED_WRITING  RT_BIT(1)
 /** Busy indicator. */
-#define PDMLED_BUSY     BIT(2)
+#define PDMLED_BUSY     RT_BIT(2)
 /** Error indicator. */
-#define PDMLED_ERROR    BIT(3)
+#define PDMLED_ERROR    RT_BIT(3)
 /** @} */
 
 
