@@ -27,6 +27,7 @@
 #include <VBox/dbgf.h>
 #include <VBox/mm.h>
 #include <VBox/err.h>
+#include <VBox/vusb.h>
 #include <iprt/stdarg.h>
 
 __BEGIN_DECLS
@@ -36,6 +37,31 @@ __BEGIN_DECLS
  * @{
  */
 
+/**
+ * USB descriptor cache.
+ *
+ * This structure is owned by the USB device but provided to the PDM/VUSB layer
+ * thru the PDMUSBREG::pfnGetDescriptorCache method. PDM/VUSB will use the
+ * information here to map addresses to endpoints, perform SET_CONFIGURATION
+ * requests, and optionally perform GET_DESCRIPTOR requests (see flag).
+ *
+ * Currently, only device and configuration descriptors are cached.
+ */
+typedef struct PDMUSBDESCCACHE
+{
+    /** USB device descriptor */
+    PCVUSBDESCDEVICE    pDevice;
+    /** USB Descriptor arrays (pDev->bNumConfigurations) */
+    PCVUSBDESCCONFIGEX  paConfigs;
+    /** Use the cached descriptors for GET_DESCRIPTOR requests. */
+    bool                fUseCachedDescriptors;
+} PDMUSBDESCCACHE;
+/** Pointer to an USB descriptor cache. */
+typedef PDMUSBDESCCACHE *PPDMUSBDESCCACHE;
+/** Pointer to a const USB descriptor cache. */
+typedef const PDMUSBDESCCACHE *PCPDMUSBDESCCACHE;
+
+
 
 /** PDM USB Device Registration Structure,
  *
@@ -77,18 +103,6 @@ typedef struct PDMUSBREG
      */
     DECLR3CALLBACKMEMBER(int, pfnConstruct,(PPDMUSBINS pUsbIns, int iInstance, PCFGMNODE pCfg, PCFGMNODE pCfgGlobal));
 
-    /**
-     * Init complete notification.
-     *
-     * This can be done to do communication with other devices and other
-     * initialization which requires everything to be in place.
-     *
-     * @returns VBOX status code.
-     * @param   pUsbIns     The USB device instance data.
-     * @remarks Optional.
-     */
-    DECLR3CALLBACKMEMBER(int, pfnInitComplete,(PPDMUSBINS pUsbIns));
-
     /**
      * Destruct an USB device instance.
      *
@@ -98,57 +112,91 @@ typedef struct PDMUSBREG
      * This method will be called regardless of the pfnConstruc result to avoid
      * complicated failure paths.
      *
-     * @returns VBox status.
      * @param   pUsbIns     The USB device instance data.
      * @remarks Optional.
      */
-    DECLR3CALLBACKMEMBER(int, pfnDestruct,(PPDMUSBINS pUsbIns));
+    DECLR3CALLBACKMEMBER(void, pfnDestruct,(PPDMUSBINS pUsbIns));
+
 
     /**
-     * Power On notification.
+     * Init complete notification.
+     *
+     * This can be done to do communication with other devices and other
+     * initialization which requires everything to be in place.
+     *
+     * @returns VBOX status code.
+     * @param   pUsbIns     The USB device instance data.
+     * @remarks Optional.
+     * @remarks Not called when hotplugged.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnVMInitComplete,(PPDMUSBINS pUsbIns));
+
+    /**
+     * VM Power On notification.
      *
      * @returns VBox status.
      * @param   pUsbIns     The USB device instance data.
      * @remarks Optional.
      */
-    DECLR3CALLBACKMEMBER(void, pfnPowerOn,(PPDMUSBINS pUsbIns));
+    DECLR3CALLBACKMEMBER(void, pfnVMPowerOn,(PPDMUSBINS pUsbIns));
 
     /**
-     * Reset notification.
+     * VM Reset notification.
      *
      * @returns VBox status.
      * @param   pUsbIns     The USB device instance data.
      * @remarks Optional.
      */
-    DECLR3CALLBACKMEMBER(void, pfnReset,(PPDMUSBINS pUsbIns));
+    DECLR3CALLBACKMEMBER(void, pfnVMReset,(PPDMUSBINS pUsbIns));
 
     /**
-     * Suspend notification.
+     * VM Suspend notification.
      *
      * @returns VBox status.
      * @param   pUsbIns     The USB device instance data.
      * @remarks Optional.
      */
-    DECLR3CALLBACKMEMBER(void, pfnSuspend,(PPDMUSBINS pUsbIns));
+    DECLR3CALLBACKMEMBER(void, pfnVMSuspend,(PPDMUSBINS pUsbIns));
 
     /**
-     * Resume notification.
+     * VM Resume notification.
      *
      * @returns VBox status.
      * @param   pUsbIns     The USB device instance data.
      * @remarks Optional.
      */
-    DECLR3CALLBACKMEMBER(void, pfnResume,(PPDMUSBINS pUsbIns));
+    DECLR3CALLBACKMEMBER(void, pfnVMResume,(PPDMUSBINS pUsbIns));
 
     /**
-     * Power Off notification.
+     * VM Power Off notification.
      *
      * @param   pUsbIns     The USB device instance data.
      */
-    DECLR3CALLBACKMEMBER(void, pfnPowerOff,(PPDMUSBINS pUsbIns));
+    DECLR3CALLBACKMEMBER(void, pfnVMPowerOff,(PPDMUSBINS pUsbIns));
 
     /**
-     * Attach command.
+     * Called after the constructor when attaching a device at run time.
+     *
+     * This can be used to do tasks normally assigned to pfnInitComplete and/or pfnVMPowerOn.
+     *
+     * @returns VBox status.
+     * @param   pUsbIns     The USB device instance data.
+     * @remarks Optional.
+     */
+    DECLR3CALLBACKMEMBER(void, pfnHotPlugged,(PPDMUSBINS pUsbIns));
+
+    /**
+     * Called before the destructor when a device is unplugged at run time.
+     *
+     * This can be used to do tasks normally assigned to pfnVMSuspend and/or pfnVMPowerOff.
+     *
+     * @returns VBox status.
+     * @param   pUsbIns     The USB device instance data.
+     * @remarks Optional.
+     */
+    DECLR3CALLBACKMEMBER(void, pfnHotUnplugged,(PPDMUSBINS pUsbIns));
+    /**
+     * Driver Attach command.
      *
      * This is called to let the USB device attach to a driver for a specified LUN
      * at runtime. This is not called during VM construction, the device constructor
@@ -159,10 +207,10 @@ typedef struct PDMUSBREG
      * @param   iLUN        The logical unit which is being detached.
      * @remarks Optional.
      */
-    DECLR3CALLBACKMEMBER(int, pfnAttach,(PPDMUSBINS pUsbIns, unsigned iLUN));
+    DECLR3CALLBACKMEMBER(int, pfnDriverAttach,(PPDMUSBINS pUsbIns, unsigned iLUN));
 
     /**
-     * Detach notification.
+     * Driver Detach notification.
      *
      * This is called when a driver is detaching itself from a LUN of the device.
      * The device should adjust it's state to reflect this.
@@ -171,7 +219,7 @@ typedef struct PDMUSBREG
      * @param   iLUN        The logical unit which is being detached.
      * @remarks Optional.
      */
-    DECLR3CALLBACKMEMBER(void, pfnDetach,(PPDMUSBINS pUsbIns, unsigned iLUN));
+    DECLR3CALLBACKMEMBER(void, pfnDriverDetach,(PPDMUSBINS pUsbIns, unsigned iLUN));
 
     /**
      * Query the base interface of a logical unit.
@@ -184,6 +232,118 @@ typedef struct PDMUSBREG
      */
     DECLR3CALLBACKMEMBER(int, pfnQueryInterface,(PPDMUSBINS pUsbIns, unsigned iLUN, PPDMIBASE *ppBase));
 
+    /**
+     * Requests the USB device to reset.
+     *
+     * @returns VBox status code.
+     * @param   pUsbIns         The USB device instance.
+     * @param   fResetOnLinux   A hint to the usb proxy.
+     *                          Don't use this unless you're the linux proxy device.
+     * @thread  Any thread.
+     * @remarks Optional.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnUsbReset,(PPDMUSBINS pUsbIns, bool fResetOnLinux));
+
+    /**
+     * Query device and configuration descriptors for the caching and servicing
+     * relevant GET_DESCRIPTOR requests.
+     *
+     * @returns Pointer to the descriptor cache (read-only).
+     * @param   pUsbIns             The USB device instance.
+     * @remarks Mandatory.
+     */
+    DECLR3CALLBACKMEMBER(PCPDMUSBDESCCACHE, pfnUsbGetDescriptorCache,(PPDMUSBINS pUsbIns));
+
+    /**
+     * SET_CONFIGURATION request.
+     *
+     * @returns VBox status code.
+     * @param   pUsbIns             The USB device instance.
+     * @param   bConfigurationValue The bConfigurationValue of the new configuration.
+     * @param   pvOldCfgDesc        Internal - for the device proxy.
+     * @param   pvOldIfState        Internal - for the device proxy.
+     * @param   pvNewCfgDesc        Internal - for the device proxy.
+     * @remarks Optional.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnUsbSetConfiguration,(PPDMUSBINS pUsbIns, uint8_t bConfigurationValue,
+                                                      const void *pvOldCfgDesc, const void *pvOldIfState, const void *pvNewCfgDesc));
+
+    /**
+     * SET_INTERFACE request.
+     *
+     * @returns VBox status code.
+     * @param   pUsbIns             The USB device instance.
+     * @param   bInterfaceNumber    The interface number.
+     * @param   bAlternateSetting   The alternate setting.
+     * @remarks Optional.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnUsbSetInterface,(PPDMUSBINS pUsbIns, uint8_t bInterfaceNumber, uint8_t bAlternateSetting));
+
+    /**
+     * Clears the halted state of an endpoint. (Optional)
+     *
+     * This called when VUSB sees a CLEAR_FEATURE(ENDPOINT_HALT) on request
+     * on the zero pipe.
+     *
+     * @returns VBox status code.
+     * @param   pUsbIns             The USB device instance.
+     * @param   uEndpoint           The endpoint to clear.
+     * @remarks Optional.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnUsbClearHaltedEndpoint,(PPDMUSBINS pUsbIns, unsigned uEndpoint));
+
+    /**
+     * Allocates an URB.
+     *
+     * This can be used to make use of shared user/kernel mode buffers.
+     *
+     * @returns VBox status code.
+     * @param   pUsbIns             The USB device instance.
+     * @param   cbData              The size of the data buffer.
+     * @param   cTds                The number of TDs.
+     * @param   enmType             The type of URB.
+     * @param   ppUrb               Where to store the allocated URB.
+     * @remarks Optional.
+     * @remarks Not implemented yet.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnUrbNew,(PPDMUSBINS pUsbIns, size_t cbData, size_t cTds, VUSBXFERTYPE enmType, PVUSBURB *ppUrb));
+
+    /**
+     * Queues an URB for processing.
+     *
+     * @returns VBox status code.
+     * @retval  VINF_SUCCESS on success.
+     * @retval  VERR_VUSB_DEVICE_NOT_ATTACHED if the device has been disconnected.
+     * @retval  VERR_VUSB_FAILED_TO_QUEUE_URB as a general failure kind of thing.
+     * @retval  TBD - document new stuff!
+     *
+     * @param   pUsbIns             The USB device instance.
+     * @param   pUrb                The URB to process.
+     * @remarks Mandatory.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnUrbQueue,(PPDMUSBINS pUsbIns, PVUSBURB pUrb));
+
+    /**
+     * Cancels an URB.
+     *
+     * @returns VBox status code.
+     * @param   pUsbIns             The USB device instance.
+     * @param   pUrb                The URB to cancel.
+     * @remarks Mandatory.
+     */
+    DECLR3CALLBACKMEMBER(int, pfnUrbCancel,(PPDMUSBINS pUsbIns, PVUSBURB pUrb));
+
+    /**
+     * Reaps an URB.
+     *
+     * @returns A ripe URB, NULL if none.
+     * @param   pUsbIns             The USB device instance.
+     * @param   cMillies            How log to wait for an URB to become ripe.
+     * @remarks Mandatory.
+     */
+    DECLR3CALLBACKMEMBER(PVUSBURB, pfnUrbReap,(PPDMUSBINS pUsbIns, unsigned cMillies));
+
+
     /** Just some init precaution. Must be set to PDM_USBREG_VERSION. */
     uint32_t            u32TheEnd;
 } PDMUSBREG;
@@ -414,6 +574,9 @@ typedef struct PDMUSBINS
      * The device constructor initializes this if it has any device level
      * interfaces to export. To obtain this interface call PDMR3QueryUSBDevice(). */
     PDMIBASE                    IBase;
+#if HC_ARCH_BITS == 32
+    uint32_t                    u32Alignment; /**< Alignment padding. */
+#endif
 
     /** Internal data. */
     union
@@ -421,7 +584,7 @@ typedef struct PDMUSBINS
 #ifdef PDMUSBINSINT_DECLARED
         PDMUSBINSINT            s;
 #endif
-        uint8_t                 padding[HC_ARCH_BITS == 32 ? 48 : 96];
+        uint8_t                 padding[HC_ARCH_BITS == 32 ? 64 : 96];
     } Internal;
 
     /** Pointer the PDM USB Device API. */
@@ -434,8 +597,15 @@ typedef struct PDMUSBINS
     R3PTRTYPE(PCFGMNODE)        pCfgGlobal;
     /** Pointer to device instance data. */
     R3PTRTYPE(void *)           pvInstanceDataR3;
-    /* padding to make achInstanceData aligned at 32 byte boundrary. */
-    uint32_t                    au32Padding[HC_ARCH_BITS == 32 ? 4 : 6];
+    /** Pointer to the VUSB Device structure.
+     * Internal to VUSB, don't touch.
+     * @todo Moved this to PDMUSBINSINT. */
+    R3PTRTYPE(void *)           pvVUsbDev2;
+    /** Device name for using when logging.
+     * The constructor sets this and the destructor frees it. */
+    R3PTRTYPE(char *)           pszName;
+    /** Padding to make achInstanceData aligned at 32 byte boundrary. */
+    uint32_t                    au32Padding[HC_ARCH_BITS == 32 ? 5 : 2];
     /** Device instance data. The size of this area is defined
      * in the PDMUSBREG::cbInstanceData field. */
     char                        achInstanceData[8];
@@ -560,6 +730,44 @@ typedef struct PDMUSBREGCB
  */
 typedef DECLCALLBACK(int) FNPDMVBOXUSBREGISTER(PCPDMUSBREGCB pCallbacks, uint32_t u32Version);
 
+
+/**
+ * Creates a USB proxy device instance.
+ *
+ * This will find an appropriate HUB for the USB device, create the necessary CFGM stuff
+ * and try instantiate the proxy device.
+ *
+ * @returns VBox status code.
+ * @param   pVM             The VM handle.
+ * @param   pUuid           The UUID thats to be associated with the device.
+ * @param   fRemote         Whether it's a remove or local device.
+ * @param   pszAddress      The address string.
+ * @param   pvBackend       Pointer to the backend.
+ * @param   iUsbVersion     The preferred USB version.
+ * @param   fMaskedIfs      The interfaces to hide from the guest.
+ */
+PDMR3DECL(int) PDMR3USBCreateProxyDevice(PVM pVM, PCRTUUID pUuid, bool fRemote, const char *pszAddress, void *pvBackend,
+                                         uint32_t iUsbVersion, uint32_t fMaskedIfs);
+
+/**
+ * Detaches and destroys a USB device.
+ *
+ * @returns VBox status code.
+ * @param   pVM             The VM handle.
+ * @param   pUuid           The UUID associated with the device to detach.
+ * @thread  EMT
+ */
+PDMR3DECL(int) PDMR3USBDetachDevice(PVM pVM, PCRTUUID pUuid);
+
+/**
+ * Checks if there are any USB hubs attached.
+ *
+ * @returns true / false accordingly.
+ * @param   pVM     Pointer to the shared VM structure.
+ */
+PDMR3DECL(bool) PDMR3USBHasHub(PVM pVM);
+
+
 /** @} */
 
 __END_DECLS
