@@ -322,6 +322,7 @@ SUPR3DECL(int) SUPSetVMForFastIOCtl(PVMR0 pVMR0);
 SUPR3DECL(int) SUPCallVMMR0(PVMR0 pVMR0, unsigned uOperation, void *pvArg);
 
 /**
+n
  * Calls the HC R0 VMM entry point, in a safer but slower manner than SUPCallVMMR0.
  * When entering using this call the R0 components can call into the host kernel
  * (i.e. use the SUPR0 and RT APIs).
@@ -367,6 +368,44 @@ SUPR3DECL(int) SUPPageAlloc(size_t cPages, void **ppvPages);
  */
 SUPR3DECL(int) SUPPageFree(void *pvPages, size_t cPages);
 
+/**
+ * Allocate zero-filled locked pages.
+ *
+ * Use this to allocate a number of pages rather than using RTMem*() and mess with
+ * alignment. The returned address is of course page aligned. Call SUPPageFreeLocked()
+ * to free the pages once done with them.
+ *
+ * @returns VBox status.
+ * @param   cPages          Number of pages to allocate.
+ * @param   ppvPages        Where to store the base pointer to the allocated pages.
+ */
+SUPR3DECL(int) SUPPageAllocLocked(size_t cPages, void **ppvPages);
+
+/**
+ * Allocate zero-filled locked pages.
+ *
+ * Use this to allocate a number of pages rather than using RTMem*() and mess with
+ * alignment. The returned address is of course page aligned. Call SUPPageFreeLocked()
+ * to free the pages once done with them.
+ *
+ * @returns VBox status code.
+ * @param   cPages          Number of pages to allocate.
+ * @param   ppvPages        Where to store the base pointer to the allocated pages.
+ * @param   paPages         Where to store the physical page addresses returned.
+ *                          On entry this will point to an array of with cbMemory >> PAGE_SHIFT entries.
+ *                          NULL is allowed.
+ */
+SUPR3DECL(int) SUPPageAllocLockedEx(size_t cPages, void **ppvPages, PSUPPAGE paPages);
+
+/**
+ * Frees locked pages allocated with SUPPageAllocLocked().
+ *
+ * @returns VBox status.
+ * @param   pvPages         Pointer returned by SUPPageAlloc().
+ * @param   cPages          Number of pages that was allocated.
+ */
+SUPR3DECL(int) SUPPageFreeLocked(void *pvPages, size_t cPages);
+
 /**
  * Locks down the physical memory backing a virtual memory
  * range in the current process.
@@ -559,7 +598,9 @@ SUPR0DECL(int) SUPR0LowFree(PSUPDRVSESSION pSession, RTHCUINTPTR uPtr);
 SUPR0DECL(int) SUPR0MemAlloc(PSUPDRVSESSION pSession, uint32_t cb, PRTR0PTR ppvR0, PRTR3PTR ppvR3);
 SUPR0DECL(int) SUPR0MemGetPhys(PSUPDRVSESSION pSession, RTHCUINTPTR uPtr, PSUPPAGE paPages);
 SUPR0DECL(int) SUPR0MemFree(PSUPDRVSESSION pSession, RTHCUINTPTR uPtr);
-SUPR0DECL(int) SUPR0GipMap(PSUPDRVSESSION pSession, PRTR3PTR ppGipR3, PRTHCPHYS pHCPhysGid);
+SUPR0DECL(int) SUPR0PageAlloc(PSUPDRVSESSION pSession, uint32_t cPages, PRTR3PTR ppvR3, PSUPPAGE paPages);
+SUPR0DECL(int) SUPR0PageFree(PSUPDRVSESSION pSession, RTR3PTR pvR3);
+SUPR0DECL(int) SUPR0GipMap(PSUPDRVSESSION pSession, PRTR3PTR ppGipR3, PRTHCPHYS pHCPhysGip);
 SUPR0DECL(int) SUPR0GipUnmap(PSUPDRVSESSION pSession);
 SUPR0DECL(int) SUPR0Printf(const char *pszFormat, ...);
 
