@@ -400,6 +400,15 @@ RTDECL(PCRTWINERRMSG) RTErrWinGet(long rc);
 #define VERR_ADDRESS_TOO_BIG                (-74)
 /** Failed to map a memory object. */
 #define VERR_MAP_FAILED                     (-75)
+/** Trailing characters. */
+#define VERR_TRAILING_CHARS                 (-76)
+/** Trailing characters. */
+#define VWRN_TRAILING_CHARS                 76
+/** Trailing spaces. */
+#define VERR_TRAILING_SPACES                (-77)
+/** Trailing spaces. */
+#define VWRN_TRAILING_SPACES                77
+
 /** @} */
 
 
