@@ -67,6 +67,9 @@ static struct audio_driver *drvtab[] = {
 #ifdef VBOX_WITH_ALSA
     &alsa_audio_driver,
 #endif
+#ifdef VBOX_WITH_PULSE
+    &pulse_audio_driver,
+#endif
 #endif
 #ifdef RT_OS_DARWIN
     &coreaudio_audio_driver,
@@ -80,6 +83,13 @@ static struct audio_driver *drvtab[] = {
     &no_audio_driver
 };
 
+static char *audio_streamname;
+
+const char *audio_get_stream_name(void)
+{
+    return audio_streamname;
+}
+
 struct fixed_settings {
     int enabled;
     int nb_voices;
@@ -254,11 +264,17 @@ static const char *audio_audfmt_to_string (audfmt_e fmt)
     case AUD_FMT_U16:
         return "U16";
 
+    case AUD_FMT_U32:
+        return "U32";
+
     case AUD_FMT_S8:
         return "S8";
 
     case AUD_FMT_S16:
         return "S16";
+
+    case AUD_FMT_S32:
+        return "S32";
     }
 
     dolog ("Bogus audfmt %d returning S16\n", fmt);
@@ -276,6 +292,10 @@ static audfmt_e audio_string_to_audfmt (const char *s, audfmt_e defval,
         *defaultp = 0;
         return AUD_FMT_U16;
     }
+    else if (!strcasecmp (s, "u32")) {
+        *defaultp = 0;
+        return AUD_FMT_U32;
+    }
     else if (!strcasecmp (s, "s8")) {
         *defaultp = 0;
         return AUD_FMT_S8;
@@ -284,6 +304,10 @@ static audfmt_e audio_string_to_audfmt (const char *s, audfmt_e defval,
         *defaultp = 0;
         return AUD_FMT_S16;
     }
+    else if (!strcasecmp (s, "s32")) {
+        *defaultp = 0;
+        return AUD_FMT_S32;
+    }
     else {
         dolog ("Bogus audio format `%s' using %s\n",
                s, audio_audfmt_to_string (defval));
@@ -336,23 +360,26 @@ static const char *audio_get_conf_str (const char *key,
     }
 }
 
-void AUD_vlog (const char *cap, const char *fmt, va_list ap)
+void AUD_vlog (const char *cap, const char *fmt, va_list va)
 {
+    va_list va2;
+    va_copy (va2, va); /* Have to make a copy here or GCC will break. */
     if (cap) {
-        Log (("%s: %N", cap, fmt, &ap));
+        Log (("%s: %N", cap, fmt, &va2));
     }
     else {
-        Log (("%N", fmt, &ap));
+        Log (("%N", fmt, &va2));
     }
+    va_end (va2);
 }
 
 void AUD_log (const char *cap, const char *fmt, ...)
 {
-    va_list ap;
+    va_list va;
 
-    va_start (ap, fmt);
-    AUD_vlog (cap, fmt, ap);
-    va_end (ap);
+    va_start (va, fmt);
+    AUD_vlog (cap, fmt, va);
+    va_end (va);
 }
 
 static void audio_process_options (const char *prefix,
@@ -459,6 +486,12 @@ static void audio_print_settings (audsettings_t *as)
     case AUD_FMT_U16:
         AUD_log (NULL, "U16");
         break;
+    case AUD_FMT_S32:
+        AUD_log (NULL, "S32");
+        break;
+    case AUD_FMT_U32:
+        AUD_log (NULL, "U32");
+        break;
     default:
         AUD_log (NULL, "invalid(%d)", as->fmt);
         break;
@@ -491,6 +524,8 @@ static int audio_validate_settings (audsettings_t *as)
     case AUD_FMT_U8:
     case AUD_FMT_S16:
     case AUD_FMT_U16:
+    case AUD_FMT_S32:
+    case AUD_FMT_U32:
         break;
     default:
         invalid = 1;
@@ -516,6 +551,12 @@ static int audio_pcm_info_eq (struct audio_pcm_info *info, audsettings_t *as)
     case AUD_FMT_U16:
         bits = 16;
         break;
+
+    case AUD_FMT_S32:
+        sign = 1;
+    case AUD_FMT_U32:
+        bits = 32;
+        break;
     }
     return info->freq == as->freq
         && info->nchannels == as->nchannels
@@ -539,6 +580,12 @@ void audio_pcm_init_info (struct audio_pcm_info *info, audsettings_t *as)
     case AUD_FMT_U16:
         bits = 16;
         break;
+
+    case AUD_FMT_S32:
+        sign = 1;
+    case AUD_FMT_U32:
+        bits = 32;
+        break;
     }
 
     info->freq = as->freq;
@@ -1454,7 +1501,8 @@ static void audio_atexit (void)
     HWVoiceOut *hwo = NULL;
     HWVoiceIn *hwi = NULL;
 
-    while ((hwo = audio_pcm_hw_find_any_enabled_out (s, hwo))) {
+    /* VBox change: audio_pcm_hw_find_any_enabled_out => audio_pcm_hw_find_any_out */
+    while ((hwo = audio_pcm_hw_find_any_out (s, hwo))) {
         SWVoiceCap *sc;
 
         hwo->pcm_ops->ctl_out (hwo, VOICE_DISABLE);
@@ -1470,7 +1518,8 @@ static void audio_atexit (void)
         }
     }
 
-    while ((hwi = audio_pcm_hw_find_any_enabled_in (s, hwi))) {
+    /* VBox change: audio_pcm_hw_find_any_enabled_in => audio_pcm_hw_find_any_in */
+    while ((hwi = audio_pcm_hw_find_any_in (s, hwi))) {
         hwi->pcm_ops->ctl_in (hwi, VOICE_DISABLE);
         hwi->pcm_ops->fini_in (hwi);
     }
@@ -1604,6 +1653,12 @@ int AUD_init_null(void)
 {
     AudioState *s = &glob_audio_state;
 
+#ifdef VBOX
+    if (s->drv)
+        s->drv->fini (s->drv_opaque);
+#endif
+
+    LogRel(("Audio: Using NULL audio driver\n"));
     return audio_driver_init (s, &no_audio_driver);
 }
 
@@ -1860,7 +1915,7 @@ static DECLCALLBACK(int) drvAudioConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHan
     /*
      * Validate the config.
      */
-    if (!CFGMR3AreValuesValid(pCfgHandle, "AudioDriver\0"))
+    if (!CFGMR3AreValuesValid(pCfgHandle, "AudioDriver\0StreamName\0"))
         return VERR_PDM_DRVINS_UNKNOWN_CFG_VALUES;
 
     /*
@@ -1878,6 +1933,10 @@ static DECLCALLBACK(int) drvAudioConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHan
     if (VBOX_FAILURE (rc))
         return rc;
 
+    rc = CFGMR3QueryStringAlloc (pCfgHandle, "StreamName", &audio_streamname);
+    if (VBOX_FAILURE (rc))
+        audio_streamname = NULL;
+
     rc = AUD_init (pDrvIns, drvname);
     if (VBOX_FAILURE (rc))
         return rc;
