@@ -1,4 +1,4 @@
-/* $Id: Builtins.h 4144 2007-08-14 08:59:40Z vboxsync $ */
+/* $Id: Builtins.h 28195 2008-02-18 15:57:41Z frank $ */
 /** @file
  * Built-in drivers & devices (part 1) header.
  */
@@ -39,15 +39,25 @@ extern const PDMDEVREG g_DeviceFloppyController;
 extern const PDMDEVREG g_DeviceVga;
 extern const PDMDEVREG g_DeviceVMMDev;
 extern const PDMDEVREG g_DevicePCNet;
+#ifdef VBOX_WITH_E1000
+extern const PDMDEVREG g_DeviceE1000;
+#endif
 extern const PDMDEVREG g_DeviceNE2000;
+#ifdef VBOX_WITH_INIP 
+extern const PDMDEVREG g_DeviceINIP; 
+#endif 
 extern const PDMDEVREG g_DeviceICHAC97;
 extern const PDMDEVREG g_DeviceAudioSniffer;
 extern const PDMDEVREG g_DeviceOHCI;
+extern const PDMDEVREG g_DeviceEHCI;
 extern const PDMDEVREG g_DeviceACPI;
 extern const PDMDEVREG g_DeviceDMA;
 extern const PDMDEVREG g_DeviceFloppyController;
 extern const PDMDEVREG g_DeviceSerialPort;
 extern const PDMDEVREG g_DeviceParallelPort;
+#ifdef VBOX_WITH_AHCI
+extern const PDMDEVREG g_DeviceAHCI;
+#endif
 
 extern const PDMDRVREG g_DrvMouseQueue;
 extern const PDMDRVREG g_DrvKeyboardQueue;
