@@ -771,8 +771,9 @@ static void pci_bios_init_device(PCIDevice *d)
     switch(devclass)
     {
     case 0x0101:
-        if (vendor_id == 0x8086 && device_id == 0x7010) {
-            /* PIIX3 IDE */
+        if (vendor_id == 0x8086 && 
+            (device_id == 0x7010 || device_id == 0x7111)) {
+            /* PIIX3 or PIIX4 IDE */
             pci_config_writew(d, 0x40, 0x8000); /* enable IDE0 */
             pci_config_writew(d, 0x42, 0x8000); /* enable IDE1 */
             goto default_map;
@@ -1447,7 +1448,7 @@ static DECLCALLBACK(int)   pciConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGM
         fUseIoApic = false;
     else if (VBOX_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("Configuration error: Failed to query boolean value \"IOAPIC\"."));
+                                N_("Configuration error: Failed to query boolean value \"IOAPIC\""));
 
     /* check if GC code is enabled. */
     rc = CFGMR3QueryBool(pCfgHandle, "GCEnabled", &fGCEnabled);
@@ -1455,7 +1456,7 @@ static DECLCALLBACK(int)   pciConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGM
         fGCEnabled = true;
     else if (VBOX_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("Configuration error: Failed to query boolean value \"GCEnabled\"."));
+                                N_("Configuration error: Failed to query boolean value \"GCEnabled\""));
     Log(("PCI: fGCEnabled=%d\n", fGCEnabled));
 
     /* check if R0 code is enabled. */
@@ -1464,7 +1465,7 @@ static DECLCALLBACK(int)   pciConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGM
         fR0Enabled = true;
     else if (VBOX_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("Configuration error: Failed to query boolean value \"R0Enabled\"."));
+                                N_("Configuration error: Failed to query boolean value \"R0Enabled\""));
     Log(("PCI: fR0Enabled=%d\n", fR0Enabled));
 
     /*
