@@ -52,13 +52,6 @@
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
 *******************************************************************************/
-typedef enum ASYNCSTATE
-{
-    //ASYNCSTATE_SUSPENDED = 1,
-    ASYNCSTATE_RUNNING,
-    ASYNCSTATE_TERMINATE
-} ASYNCSTATE;
-
 /**
  * Block driver instance data.
  */
@@ -77,10 +70,8 @@ typedef struct DRVTAP
     RTFILE                  PipeWrite;
     /** The read end of the control pipe. */
     RTFILE                  PipeRead;
-    /** The thread state. */
-    ASYNCSTATE volatile     enmState;
     /** Reader thread. */
-    RTTHREAD                Thread;
+    PPDMTHREAD              pThread;
     /** We are waiting for more receive buffers. */
     uint32_t volatile       fOutOfSpace;
     /** Event semaphore for blocking on receive. */
@@ -210,10 +201,14 @@ static DECLCALLBACK(void) drvTAPNotifyCanReceive(PPDMINETWORKCONNECTOR pInterfac
  * @param   Thread          Thread handle.
  * @param   pvUser          Pointer to a DRVTAP structure.
  */
-static DECLCALLBACK(int) drvTAPAsyncIoThread(RTTHREAD ThreadSelf, void *pvUser)
+static DECLCALLBACK(int) drvTAPAsyncIoThread(PPDMDRVINS pDrvIns, PPDMTHREAD pThread)
 {
-    PDRVTAP pData = (PDRVTAP)pvUser;
+    PDRVTAP pData = PDMINS2DATA(pDrvIns, PDRVTAP);
     LogFlow(("drvTAPAsyncIoThread: pData=%p\n", pData));
+
+    if (pThread->enmState == PDMTHREADSTATE_INITIALIZING)
+        return VINF_SUCCESS;
+
     STAM_PROFILE_ADV_START(&pData->StatReceive, a);
 
     int rc = RTSemEventCreate(&pData->EventOutOfSpace);
@@ -222,7 +217,7 @@ static DECLCALLBACK(int) drvTAPAsyncIoThread(RTTHREAD ThreadSelf, void *pvUser)
     /*
      * Polling loop.
      */
-    for (;;)
+    while (pThread->enmState == PDMTHREADSTATE_RUNNING)
     {
         /*
          * Wait for something to become available.
@@ -237,6 +232,11 @@ static DECLCALLBACK(int) drvTAPAsyncIoThread(RTTHREAD ThreadSelf, void *pvUser)
         STAM_PROFILE_ADV_STOP(&pData->StatReceive, a);
         errno=0;
         rc = poll(&aFDs[0], ELEMENTS(aFDs), -1 /* infinite */);
+
+        /* this might have changed in the meantime */
+        if (pThread->enmState != PDMTHREADSTATE_RUNNING)
+            break;
+
         STAM_PROFILE_ADV_START(&pData->StatReceive, a);
         if (    rc > 0
             &&  (aFDs[0].revents & (POLLIN | POLLPRI))
@@ -254,15 +254,25 @@ static DECLCALLBACK(int) drvTAPAsyncIoThread(RTTHREAD ThreadSelf, void *pvUser)
 
                 /*
                  * Wait for the device to have space for this frame.
+                 * Most guests use frame-sized receive buffers, hence non-zero cbMax
+                 * automatically means there is enough room for entire frame. Some
+                 * guests (eg. Solaris) use large chains of small receive buffers
+                 * (each 128 or so bytes large). We will still start receiving as soon
+                 * as cbMax is non-zero because:
+                 *  - it would be quite expensive for pfnCanReceive to accurately
+                 *    determine free receive buffer space
+                 *  - if we were waiting for enough free buffers, there is a risk
+                 *    of deadlocking because the guest could be waiting for a receive
+                 *    overflow error to allocate more receive buffers
                  */
                 size_t cbMax = pData->pPort->pfnCanReceive(pData->pPort);
-                if (cbMax < cbRead)
+                if (cbMax == 0)
                 {
                     /** @todo receive overflow handling needs serious improving! */
                     STAM_PROFILE_ADV_STOP(&pData->StatReceive, a);
                     STAM_PROFILE_START(&pData->StatRecvOverflows, b);
-                    while (   cbMax < cbRead
-                           && pData->enmState != ASYNCSTATE_TERMINATE)
+                    while (   cbMax == 0
+                           && pThread->enmState == PDMTHREADSTATE_RUNNING)
                     {
                         LogFlow(("drvTAPAsyncIoThread: cbMax=%d cbRead=%d waiting...\n", cbMax, cbRead));
 #if 1
@@ -277,7 +287,7 @@ static DECLCALLBACK(int) drvTAPAsyncIoThread(RTTHREAD ThreadSelf, void *pvUser)
                     ASMAtomicXchgU32(&pData->fOutOfSpace, false);
                     STAM_PROFILE_STOP(&pData->StatRecvOverflows, b);
                     STAM_PROFILE_ADV_START(&pData->StatReceive, a);
-                    if (pData->enmState == ASYNCSTATE_TERMINATE)
+                    if (pThread->enmState != PDMTHREADSTATE_RUNNING)
                         break;
                 }
 
@@ -309,9 +319,7 @@ static DECLCALLBACK(int) drvTAPAsyncIoThread(RTTHREAD ThreadSelf, void *pvUser)
         else if (   rc > 0
                  && aFDs[1].revents)
         {
-            LogFlow(("drvTAPAsyncIoThread: Control message: enmState=%d revents=%#x\n", pData->enmState, aFDs[1].revents));
-            if (pData->enmState == ASYNCSTATE_TERMINATE)
-                break;
+            LogFlow(("drvTAPAsyncIoThread: Control message: enmState=%d revents=%#x\n", pThread->enmState, aFDs[1].revents));
             if (aFDs[1].revents & (POLLHUP | POLLERR | POLLNVAL))
                 break;
 
@@ -338,13 +346,38 @@ static DECLCALLBACK(int) drvTAPAsyncIoThread(RTTHREAD ThreadSelf, void *pvUser)
 
     rc = RTSemEventDestroy(pData->EventOutOfSpace);
     AssertRC(rc);
+    pData->EventOutOfSpace = NIL_RTSEMEVENT;
 
     LogFlow(("drvTAPAsyncIoThread: returns %Vrc\n", VINF_SUCCESS));
     STAM_PROFILE_ADV_STOP(&pData->StatReceive, a);
     return VINF_SUCCESS;
 }
 
-#else
+
+/**
+ * Unblock the send thread so it can respond to a state change.
+ *
+ * @returns VBox status code.
+ * @param   pDevIns     The pcnet device instance.
+ * @param   pThread     The send thread.
+ */
+static DECLCALLBACK(int) drvTapAsyncIoWakeup(PPDMDRVINS pDrvIns, PPDMTHREAD pThread)
+{
+    PDRVTAP pData = PDMINS2DATA(pDrvIns, PDRVTAP);
+
+    /* Ensure that it does not spin in the CanReceive loop.
+     * (May assert in IPRT if we're really unlucky.) */
+    if (ASMAtomicXchgU32(&pData->fOutOfSpace, false))
+        RTSemEventSignal(pData->EventOutOfSpace);
+
+    int rc = RTFileWrite(pData->PipeWrite, "", 1, NULL);
+    AssertRC(rc);
+
+    return VINF_SUCCESS;
+}
+
+#else /* !ASYNC_NET */
+
 /**
  * Poller callback.
  */
@@ -398,7 +431,7 @@ static DECLCALLBACK(void) drvTAPPoller(PPDMDRVINS pDrvIns)
 
     STAM_PROFILE_ADV_STOP(&pData->StatReceive, a);
 }
-#endif
+#endif /* ASYNC_NET */
 
 
 /**
@@ -440,23 +473,6 @@ static DECLCALLBACK(void) drvTAPDestruct(PPDMDRVINS pDrvIns)
 #ifdef ASYNC_NET
     PDRVTAP pData = PDMINS2DATA(pDrvIns, PDRVTAP);
 
-    /*
-     * Terminate the Async I/O Thread.
-     */
-    ASMAtomicXchgSize(&pData->enmState, ASYNCSTATE_TERMINATE);
-    if (pData->Thread != NIL_RTTHREAD)
-    {
-        /* Ensure that it does not spin in the CanReceive loop */
-        if (ASMAtomicXchgU32(&pData->fOutOfSpace, false))
-            RTSemEventSignal(pData->EventOutOfSpace);
-
-        int rc = RTFileWrite(pData->PipeWrite, "", 1, NULL);
-        AssertRC(rc);
-        rc = RTThreadWait(pData->Thread, 5000, NULL);
-        AssertRC(rc);
-        pData->Thread = NIL_RTTHREAD;
-    }
-
     /*
      * Terminate the control pipe.
      */
@@ -495,10 +511,7 @@ static DECLCALLBACK(int) drvTAPConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandl
      */
     pData->pDrvIns                      = pDrvIns;
     pData->FileDevice                   = NIL_RTFILE;
-#ifdef ASYNC_NET
-    pData->Thread                       = NIL_RTTHREAD;
-    pData->enmState                     = ASYNCSTATE_RUNNING;
-#endif
+
     /* IBase */
     pDrvIns->IBase.pfnQueryInterface    = drvTAPQueryInterface;
     /* INetwork */
@@ -519,7 +532,7 @@ static DECLCALLBACK(int) drvTAPConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandl
     int rc = pDrvIns->pDrvHlp->pfnAttach(pDrvIns, NULL);
     if (rc != VERR_PDM_NO_ATTACHED_DRIVER)
         return PDMDRV_SET_ERROR(pDrvIns, VERR_PDM_DRVINS_NO_ATTACH,
-                                N_("Configuration error: Cannot attach drivers to the TAP driver!"));
+                                N_("Configuration error: Cannot attach drivers to the TAP driver"));
 
     /*
      * Query the network port interface.
@@ -527,7 +540,7 @@ static DECLCALLBACK(int) drvTAPConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandl
     pData->pPort = (PPDMINETWORKPORT)pDrvIns->pUpBase->pfnQueryInterface(pDrvIns->pUpBase, PDMINTERFACE_NETWORK_PORT);
     if (!pData->pPort)
         return PDMDRV_SET_ERROR(pDrvIns, VERR_PDM_MISSING_INTERFACE_ABOVE,
-                                N_("Configuration error: The above device/driver didn't export the network port interface!"));
+                                N_("Configuration error: The above device/driver didn't export the network port interface"));
 
     /*
      * Read the configuration.
@@ -536,11 +549,11 @@ static DECLCALLBACK(int) drvTAPConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandl
     rc = CFGMR3QueryS32(pCfgHandle, "FileHandle", &iFile);
     if (VBOX_FAILURE(rc))
         return PDMDRV_SET_ERROR(pDrvIns, rc,
-                                N_("Configuration error: Query for \"FileHandle\" 32-bit signed integer failed!"));
+                                N_("Configuration error: Query for \"FileHandle\" 32-bit signed integer failed"));
     pData->FileDevice = (RTFILE)iFile;
     if (!RTFileIsValid(pData->FileDevice))
         return PDMDrvHlpVMSetError(pDrvIns, VERR_INVALID_HANDLE, RT_SRC_POS,
-                                   N_("The TAP file handle %RTfile is not valid!"), pData->FileDevice);
+                                   N_("The TAP file handle %RTfile is not valid"), pData->FileDevice);
 
     /*
      * Make sure the descriptor is non-blocking and valid.
@@ -576,7 +589,7 @@ static DECLCALLBACK(int) drvTAPConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandl
     /*
      * Create the async I/O thread.
      */
-    rc = RTThreadCreate(&pData->Thread, drvTAPAsyncIoThread, pData, 128*_1K, RTTHREADTYPE_IO, RTTHREADFLAGS_WAITABLE, "TAP");
+    rc = PDMDrvHlpPDMThreadCreate(pDrvIns, &pData->pThread, pData, drvTAPAsyncIoThread, drvTapAsyncIoWakeup, 128 * _1K, RTTHREADTYPE_IO, "TAP");
     AssertRCReturn(rc, rc);
 #else
     /*
