@@ -1,4 +1,4 @@
-/** $Id: DrvTAPOs2.cpp 4372 2007-08-24 21:21:51Z vboxsync $ */
+/** $Id: DrvTAPOs2.cpp 27168 2008-01-09 16:58:25Z frank $ */
 /** @file
  * VBox network devices: OS/2 TAP network transport driver.
  */
@@ -233,15 +233,15 @@ static DECLCALLBACK(int) drvTAPOs2AsyncIoThread(PPDMDRVINS pDrvIns, PPDMTHREAD p
             AssertMsg(cbRead <= 1536, ("cbRead=%d\n", cbRead));
 
             /*
-             * Wait for the device to have space for this frame.
+             * Wait for the device to have some room.
              */
             size_t cbMax = pData->pPort->pfnCanReceive(pData->pPort);
-            if (cbMax < cbRead)
+            if (cbMax == 0)
             {
                 /** @todo receive overflow handling needs serious improving! */
                 STAM_PROFILE_ADV_STOP(&pData->StatReceive, a);
                 STAM_PROFILE_START(&pData->StatRecvOverflows, b);
-                while (   cbMax < cbRead
+                while (   cbMax == 0
                        && pData->enmState != ASYNCSTATE_TERMINATE)
                 {
                     LogFlow(("drvTAPOs2AsyncIoThread: cbMax=%d cbRead=%d waiting...\n", cbMax, cbRead));
@@ -384,7 +384,7 @@ static DECLCALLBACK(int) drvTAPOs2Construct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHa
     int rc = pDrvIns->pDrvHlp->pfnAttach(pDrvIns, NULL);
     if (rc != VERR_PDM_NO_ATTACHED_DRIVER)
         return PDMDRV_SET_ERROR(pDrvIns, VERR_PDM_DRVINS_NO_ATTACH,
-                                N_("Configuration error: Cannot attach drivers to the TAP driver!"));
+                                N_("Configuration error: Cannot attach drivers to the TAP driver"));
 
     /*
      * Query the network port interface.
@@ -392,7 +392,7 @@ static DECLCALLBACK(int) drvTAPOs2Construct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHa
     pData->pPort = (PPDMINETWORKPORT)pDrvIns->pUpBase->pfnQueryInterface(pDrvIns->pUpBase, PDMINTERFACE_NETWORK_PORT);
     if (!pData->pPort)
         return PDMDRV_SET_ERROR(pDrvIns, VERR_PDM_MISSING_INTERFACE_ABOVE,
-                                N_("Configuration error: The above device/driver didn't export the network port interface!"));
+                                N_("Configuration error: The above device/driver didn't export the network port interface"));
 
     /*
      * Read the configuration.
@@ -401,11 +401,11 @@ static DECLCALLBACK(int) drvTAPOs2Construct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHa
     rc = CFGMR3QueryS32(pCfgHandle, "FileHandle", &iFile);
     if (VBOX_FAILURE(rc))
         return PDMDRV_SET_ERROR(pDrvIns, rc,
-                                N_("Configuration error: Query for \"FileHandle\" 32-bit signed integer failed!"));
+                                N_("Configuration error: Query for \"FileHandle\" 32-bit signed integer failed"));
     pData->FileDevice = (RTFILE)iFile;
     if (!RTFileIsValid(pData->FileDevice))
         return PDMDrvHlpVMSetError(pDrvIns, VERR_INVALID_HANDLE, RT_SRC_POS,
-                                   N_("The TAP file handle %RTfile is not valid!"), pData->FileDevice);
+                                   N_("The TAP file handle %RTfile is not valid"), pData->FileDevice);
 
     /*
      * Make sure the descriptor is non-blocking and valid.
