@@ -515,7 +515,10 @@ sosendto(PNATState pData, struct socket *so, struct mbuf *m)
 #endif
             break;
 	  case CTL_DNS:
-	    addr.sin_addr = dns_addr;
+            if (!get_dns_addr(pData, &dns_addr))
+                addr.sin_addr = dns_addr;
+            else
+                addr.sin_addr = loopback_addr;
 	    break;
 	  case CTL_ALIAS:
 	  default:
