@@ -1,4 +1,4 @@
-/* $Id: DevACPI.cpp 4382 2007-08-27 11:28:55Z vboxsync $ */
+/* $Id: DevACPI.cpp 27987 2008-02-07 20:38:33Z frank $ */
 /** @file
  * Advanced Configuration and Power Interface (ACPI) Device.
  */
@@ -64,44 +64,44 @@
 #define ACPI_RESET_BLK  0x00004050
 
 /* PM1x status register bits */
-#define TMR_STS         BIT(0)
-#define RSR1_STS        (BIT(1) | BIT(2) | BIT(3))
-#define BM_STS          BIT(4)
-#define GBL_STS         BIT(5)
-#define RSR2_STS        (BIT(6) | BIT(7))
-#define PWRBTN_STS      BIT(8)
-#define SLPBTN_STS      BIT(9)
-#define RTC_STS         BIT(10)
-#define IGN_STS         BIT(11)
-#define RSR3_STS        (BIT(12) | BIT(13) | BIT(14))
-#define WAK_STS         BIT(15)
+#define TMR_STS         RT_BIT(0)
+#define RSR1_STS        (RT_BIT(1) | RT_BIT(2) | RT_BIT(3))
+#define BM_STS          RT_BIT(4)
+#define GBL_STS         RT_BIT(5)
+#define RSR2_STS        (RT_BIT(6) | RT_BIT(7))
+#define PWRBTN_STS      RT_BIT(8)
+#define SLPBTN_STS      RT_BIT(9)
+#define RTC_STS         RT_BIT(10)
+#define IGN_STS         RT_BIT(11)
+#define RSR3_STS        (RT_BIT(12) | RT_BIT(13) | RT_BIT(14))
+#define WAK_STS         RT_BIT(15)
 #define RSR_STS         (RSR1_STS | RSR2_STS | RSR3_STS)
 
 /* PM1x enable register bits */
-#define TMR_EN          BIT(0)
-#define RSR1_EN         (BIT(1) | BIT(2) | BIT(3) | BIT(4))
-#define GBL_EN          BIT(5)
-#define RSR2_EN         (BIT(6) | BIT(7))
-#define PWRBTN_EN       BIT(8)
-#define SLPBTN_EN       BIT(9)
-#define RTC_EN          BIT(10)
-#define RSR3_EN         (BIT(11) | BIT(12) | BIT(13) | BIT(14) | BIT(15))
+#define TMR_EN          RT_BIT(0)
+#define RSR1_EN         (RT_BIT(1) | RT_BIT(2) | RT_BIT(3) | RT_BIT(4))
+#define GBL_EN          RT_BIT(5)
+#define RSR2_EN         (RT_BIT(6) | RT_BIT(7))
+#define PWRBTN_EN       RT_BIT(8)
+#define SLPBTN_EN       RT_BIT(9)
+#define RTC_EN          RT_BIT(10)
+#define RSR3_EN         (RT_BIT(11) | RT_BIT(12) | RT_BIT(13) | RT_BIT(14) | RT_BIT(15))
 #define RSR_EN          (RSR1_EN | RSR2_EN | RSR3_EN)
 #define IGN_EN          0
 
 /* PM1x control register bits */
-#define SCI_EN          BIT(0)
-#define BM_RLD          BIT(1)
-#define GBL_RLS         BIT(2)
-#define RSR1_CNT        (BIT(3) | BIT(4) | BIT(5) | BIT(6) | BIT(7) | BIT(8))
-#define IGN_CNT         BIT(9)
+#define SCI_EN          RT_BIT(0)
+#define BM_RLD          RT_BIT(1)
+#define GBL_RLS         RT_BIT(2)
+#define RSR1_CNT        (RT_BIT(3) | RT_BIT(4) | RT_BIT(5) | RT_BIT(6) | RT_BIT(7) | RT_BIT(8))
+#define IGN_CNT         RT_BIT(9)
 #define SLP_TYPx_SHIFT  10
 #define SLP_TYPx_MASK    7
-#define SLP_EN          BIT(13)
-#define RSR2_CNT        (BIT(14) | BIT(15))
+#define SLP_EN          RT_BIT(13)
+#define RSR2_CNT        (RT_BIT(14) | RT_BIT(15))
 #define RSR_CNT         (RSR1_CNT | RSR2_CNT)
 
-#define GPE0_BATTERY_INFO_CHANGED BIT (0)
+#define GPE0_BATTERY_INFO_CHANGED RT_BIT(0)
 
 enum
 {
@@ -138,15 +138,15 @@ enum
 #define BAT_TECH_PRIMARY                        1
 #define BAT_TECH_SECONDARY                      2
 
-#define BAT_STATUS_DISCHARGING_MASK             BIT (0)
-#define BAT_STATUS_CHARGING_MASK                BIT (1)
-#define BAT_STATUS_CRITICAL_MASK                BIT (2)
+#define BAT_STATUS_DISCHARGING_MASK             RT_BIT(0)
+#define BAT_STATUS_CHARGING_MASK                RT_BIT(1)
+#define BAT_STATUS_CRITICAL_MASK                RT_BIT(2)
 
-#define STA_DEVICE_PRESENT_MASK                 BIT (0)
-#define STA_DEVICE_ENABLED_MASK                 BIT (1)
-#define STA_DEVICE_SHOW_IN_UI_MASK              BIT (2)
-#define STA_DEVICE_FUNCTIONING_PROPERLY_MASK    BIT (3)
-#define STA_BATTERY_PRESENT_MASK                BIT (4)
+#define STA_DEVICE_PRESENT_MASK                 RT_BIT(0)
+#define STA_DEVICE_ENABLED_MASK                 RT_BIT(1)
+#define STA_DEVICE_SHOW_IN_UI_MASK              RT_BIT(2)
+#define STA_DEVICE_FUNCTIONING_PROPERLY_MASK    RT_BIT(3)
+#define STA_BATTERY_PRESENT_MASK                RT_BIT(4)
 
 struct ACPIState
 {
@@ -180,6 +180,7 @@ struct ACPIState
      *  acpiBatIndexWrite() for handling this. */
     uint8_t             u8IndexShift;
     uint8_t             u8UseIOApic;
+    bool                fPowerButtonHandled;
 
     /** ACPI port base interface. */
     PDMIBASE            IBase;
@@ -308,32 +309,32 @@ struct ACPITBLFADT
     uint8_t             u8MonAlarm;             /**< RTC CMOS RAM index of month-of-year alarm */
     uint8_t             u8Century;              /**< RTC CMOS RAM index of century */
     uint16_t            u16IAPCBOOTARCH;        /**< IA-PC boot architecture flags */
-#define IAPC_BOOT_ARCH_LEGACY_DEV       BIT(0)  /**< legacy devices present such as LPT
+#define IAPC_BOOT_ARCH_LEGACY_DEV       RT_BIT(0)  /**< legacy devices present such as LPT
                                                      (COM too?) */
-#define IAPC_BOOT_ARCH_8042             BIT(1)  /**< legacy keyboard device present */
-#define IAPC_BOOT_ARCH_NO_VGA           BIT(2)  /**< VGA not present */
+#define IAPC_BOOT_ARCH_8042             RT_BIT(1)  /**< legacy keyboard device present */
+#define IAPC_BOOT_ARCH_NO_VGA           RT_BIT(2)  /**< VGA not present */
     uint8_t             u8Must0_0;              /**< must be 0 */
     uint32_t            u32Flags;               /**< fixed feature flags */
-#define FADT_FL_WBINVD                  BIT(0)  /**< emulation of WBINVD available */
-#define FADT_FL_WBINVD_FLUSH            BIT(1)
-#define FADT_FL_PROC_C1                 BIT(2)  /**< 1=C1 supported on all processors */
-#define FADT_FL_P_LVL2_UP               BIT(3)  /**< 1=C2 works on SMP and UNI systems */
-#define FADT_FL_PWR_BUTTON              BIT(4)  /**< 1=power button handled as ctrl method dev */
-#define FADT_FL_SLP_BUTTON              BIT(5)  /**< 1=sleep button handled as ctrl method dev */
-#define FADT_FL_FIX_RTC                 BIT(6)  /**< 0=RTC wake status in fixed register */
-#define FADT_FL_RTC_S4                  BIT(7)  /**< 1=RTC can wake system from S4 */
-#define FADT_FL_TMR_VAL_EXT             BIT(8)  /**< 1=TMR_VAL implemented as 32 bit */
-#define FADT_FL_DCK_CAP                 BIT(9)  /**< 0=system cannot support docking */
-#define FADT_FL_RESET_REG_SUP           BIT(10) /**< 1=system supports system resets */
-#define FADT_FL_SEALED_CASE             BIT(11) /**< 1=case is sealed */
-#define FADT_FL_HEADLESS                BIT(12) /**< 1=system cannot detect moni/keyb/mouse */
-#define FADT_FL_CPU_SW_SLP              BIT(13)
-#define FADT_FL_PCI_EXT_WAK             BIT(14) /**< 1=system supports PCIEXP_WAKE_STS */
-#define FADT_FL_USE_PLATFORM_CLOCK      BIT(15) /**< 1=system has ACPI PM timer */
-#define FADT_FL_S4_RTC_STS_VALID        BIT(16) /**< 1=RTC_STS flag is valid when waking from S4 */
-#define FADT_FL_REMOVE_POWER_ON_CAPABLE BIT(17) /**< 1=platform can remote power on */
-#define FADT_FL_FORCE_APIC_CLUSTER_MODEL  BIT(18)
-#define FADT_FL_FORCE_APIC_PHYS_DEST_MODE BIT(19)
+#define FADT_FL_WBINVD                  RT_BIT(0)  /**< emulation of WBINVD available */
+#define FADT_FL_WBINVD_FLUSH            RT_BIT(1)
+#define FADT_FL_PROC_C1                 RT_BIT(2)  /**< 1=C1 supported on all processors */
+#define FADT_FL_P_LVL2_UP               RT_BIT(3)  /**< 1=C2 works on SMP and UNI systems */
+#define FADT_FL_PWR_BUTTON              RT_BIT(4)  /**< 1=power button handled as ctrl method dev */
+#define FADT_FL_SLP_BUTTON              RT_BIT(5)  /**< 1=sleep button handled as ctrl method dev */
+#define FADT_FL_FIX_RTC                 RT_BIT(6)  /**< 0=RTC wake status in fixed register */
+#define FADT_FL_RTC_S4                  RT_BIT(7)  /**< 1=RTC can wake system from S4 */
+#define FADT_FL_TMR_VAL_EXT             RT_BIT(8)  /**< 1=TMR_VAL implemented as 32 bit */
+#define FADT_FL_DCK_CAP                 RT_BIT(9)  /**< 0=system cannot support docking */
+#define FADT_FL_RESET_REG_SUP           RT_BIT(10) /**< 1=system supports system resets */
+#define FADT_FL_SEALED_CASE             RT_BIT(11) /**< 1=case is sealed */
+#define FADT_FL_HEADLESS                RT_BIT(12) /**< 1=system cannot detect moni/keyb/mouse */
+#define FADT_FL_CPU_SW_SLP              RT_BIT(13)
+#define FADT_FL_PCI_EXT_WAK             RT_BIT(14) /**< 1=system supports PCIEXP_WAKE_STS */
+#define FADT_FL_USE_PLATFORM_CLOCK      RT_BIT(15) /**< 1=system has ACPI PM timer */
+#define FADT_FL_S4_RTC_STS_VALID        RT_BIT(16) /**< 1=RTC_STS flag is valid when waking from S4 */
+#define FADT_FL_REMOVE_POWER_ON_CAPABLE RT_BIT(17) /**< 1=platform can remote power on */
+#define FADT_FL_FORCE_APIC_CLUSTER_MODEL  RT_BIT(18)
+#define FADT_FL_FORCE_APIC_PHYS_DEST_MODE RT_BIT(19)
     ACPIGENADDR         ResetReg;               /**< ext addr of reset register */
     uint8_t             u8ResetVal;             /**< ResetReg value to reset the system */
 #define ACPI_RESET_REG_VAL  0x10
@@ -541,8 +542,9 @@ static void acpiSetupFADT (ACPIState *s, RTGCPHYS addr, uint32_t facs_addr, uint
     fadt.u8Century            = 0;
     fadt.u16IAPCBOOTARCH      = RT_H2LE_U16(IAPC_BOOT_ARCH_LEGACY_DEV | IAPC_BOOT_ARCH_8042);
     /** @note WBINVD is required for ACPI versions newer than 1.0 */
-    fadt.u32Flags             = RT_H2LE_U32(  FADT_FL_WBINVD | FADT_FL_SLP_BUTTON
-                                            | FADT_FL_FIX_RTC | FADT_FL_TMR_VAL_EXT);
+    fadt.u32Flags             = RT_H2LE_U32(  FADT_FL_WBINVD
+                                            | FADT_FL_FIX_RTC
+                                            | FADT_FL_TMR_VAL_EXT);
     acpiWriteGenericAddr(&fadt.ResetReg,     1,  8, 0, 1, ACPI_RESET_BLK);
     fadt.u8ResetVal           = ACPI_RESET_REG_VAL;
     fadt.u64XFACS             = RT_H2LE_U64((uint64_t)facs_addr);
@@ -739,10 +741,31 @@ static int acpiPowerDown (ACPIState *s)
 static DECLCALLBACK(int) acpiPowerButtonPress(PPDMIACPIPORT pInterface)
 {
     ACPIState *s = IACPIPORT_2_ACPISTATE(pInterface);
+    s->fPowerButtonHandled = false;
     update_pm1a (s, s->pm1a_sts | PWRBTN_STS, s->pm1a_en);
     return VINF_SUCCESS;
 }
 
+static DECLCALLBACK(int) acpiGetPowerButtonHandled(PPDMIACPIPORT pInterface, bool *pfHandled)
+{
+    ACPIState *s = IACPIPORT_2_ACPISTATE(pInterface);
+    *pfHandled = s->fPowerButtonHandled;
+    return VINF_SUCCESS;
+}
+
+/**
+ * Send an ACPI sleep button event.
+ *
+ * @returns VBox status code
+ * @param   pInterface      Pointer to the interface structure containing the called function pointer.
+ */
+static DECLCALLBACK(int) acpiSleepButtonPress(PPDMIACPIPORT pInterface)
+{
+    ACPIState *s = IACPIPORT_2_ACPISTATE(pInterface);
+    update_pm1a (s, s->pm1a_sts | SLPBTN_STS, s->pm1a_en);
+    return VINF_SUCCESS;
+}
+
 /* PM1a_EVT_BLK enable */
 static uint32_t acpiPm1aEnReadw (ACPIState *s, uint32_t addr)
 {
@@ -769,6 +792,8 @@ static uint32_t acpiPm1aStsReadw (ACPIState *s, uint32_t addr)
 static void acpiPM1aStsWritew (ACPIState *s, uint32_t addr, uint32_t val)
 {
     Log (("acpi: acpiPM1aStsWritew <- %#x (%#x)\n", val, val & ~(RSR_STS | IGN_STS)));
+    if (val & PWRBTN_STS)
+        s->fPowerButtonHandled = true; /* Remember that the guest handled the last power button event */
     val = s->pm1a_sts & ~(val & ~(RSR_STS | IGN_STS));
     update_pm1a (s, val, s->pm1a_en);
 }
@@ -1048,7 +1073,6 @@ IO_READ_PROTO (acpiBatDataRead)
         default:
             return VERR_IOM_IOPORT_UNUSED;
     }
-//    LogRel(("Query %04x => %08x\n", s->uBatteryIndex, *pu32));
     return VINF_SUCCESS;
 }
 
@@ -1474,7 +1498,7 @@ static int acpiPlantTables (ACPIState *s)
     last_addr = RT_ALIGN_32 (dsdt_addr + sizeof(AmlCode), 16);
     if (last_addr > 0x10000)
         return PDMDEV_SET_ERROR(s->pDevIns, VERR_TOO_MUCH_DATA,
-                                N_("Error: ACPI tables > 64KB!"));
+                                N_("Error: ACPI tables > 64KB"));
 
     Log(("RSDP 0x%08X\n", find_rsdp_space()));
     addend = (uint32_t) s->u64RamSize - 0x10000;
@@ -1534,27 +1558,27 @@ static DECLCALLBACK(int) acpiConstruct (PPDMDEVINS pDevIns, int iInstance, PCFGM
         s->u8UseIOApic = 1;
     else if (VBOX_FAILURE (rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("Configuration error: Failed to read \"IOAPIC\"."));
+                                N_("Configuration error: Failed to read \"IOAPIC\""));
 
     rc = CFGMR3QueryBool (pCfgHandle, "GCEnabled", &fGCEnabled);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         fGCEnabled = true;
     else if (VBOX_FAILURE (rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("Configuration error: Failed to read \"GCEnabled\"."));
+                                N_("Configuration error: Failed to read \"GCEnabled\""));
 
     rc = CFGMR3QueryBool(pCfgHandle, "R0Enabled", &fR0Enabled);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         fR0Enabled = true;
     else if (VBOX_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("configuration error: failed to read R0Enabled as boolean."));
+                                N_("configuration error: failed to read R0Enabled as boolean"));
 
     /* */
     rsdp_addr = find_rsdp_space ();
     if (!rsdp_addr)
         return PDMDEV_SET_ERROR(pDevIns, VERR_NO_MEMORY,
-                                N_("Can not find space for RSDP. ACPI is disabled."));
+                                N_("Can not find space for RSDP. ACPI is disabled"));
 
     rc = acpiPlantTables (s);
     if (VBOX_FAILURE (rc))
@@ -1661,9 +1685,11 @@ static DECLCALLBACK(int) acpiConstruct (PPDMDEVINS pDevIns, int iInstance, PCFGM
      * Interfaces
      */
     /* IBase */
-    s->IBase.pfnQueryInterface         = acpiQueryInterface;
+    s->IBase.pfnQueryInterface            = acpiQueryInterface;
     /* IACPIPort */
-    s->IACPIPort.pfnPowerButtonPress   = acpiPowerButtonPress;
+    s->IACPIPort.pfnSleepButtonPress      = acpiSleepButtonPress;
+    s->IACPIPort.pfnPowerButtonPress      = acpiPowerButtonPress;
+    s->IACPIPort.pfnGetPowerButtonHandled = acpiGetPowerButtonHandled;
 
    /*
     * Get the corresponding connector interface
@@ -1675,7 +1701,7 @@ static DECLCALLBACK(int) acpiConstruct (PPDMDEVINS pDevIns, int iInstance, PCFGM
                                                                      PDMINTERFACE_ACPI_CONNECTOR);
        if (!s->pDrv)
            return PDMDEV_SET_ERROR(pDevIns, VERR_PDM_MISSING_INTERFACE,
-                                   N_("LUN #0 doesn't have an ACPI connector interface!\n"));
+                                   N_("LUN #0 doesn't have an ACPI connector interface"));
    }
    else if (rc == VERR_PDM_NO_ATTACHED_DRIVER)
    {
@@ -1685,7 +1711,7 @@ static DECLCALLBACK(int) acpiConstruct (PPDMDEVINS pDevIns, int iInstance, PCFGM
    }
    else
        return PDMDEV_SET_ERROR(pDevIns, rc,
-                               N_("Failed to attach LUN #0!"));
+                               N_("Failed to attach LUN #0"));
 
     return rc;
 }
