@@ -1,5 +1,5 @@
 #ifdef VBOX
-/* $Id: DevAPIC.cpp 4071 2007-08-07 17:07:59Z vboxsync $ */
+/* $Id: DevAPIC.cpp 27168 2008-01-09 16:58:25Z frank $ */
 /** @file
  * Advanced Programmable Interrupt Controller (APIC) Device and
  * I/O Advanced Programmable Interrupt Controller (IO-APIC) Device.
@@ -1650,7 +1650,7 @@ static DECLCALLBACK(int) apicConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMN
         fIOAPIC = true;
     else if (VBOX_FAILURE (rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("Configuration error: Failed to read \"IOAPIC\"."));
+                                N_("Configuration error: Failed to read \"IOAPIC\""));
 
     rc = CFGMR3QueryBool(pCfgHandle, "GCEnabled", &fGCEnabled);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
@@ -1736,12 +1736,12 @@ static DECLCALLBACK(int) apicConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMN
     if (u32Eax >= 1)
     {
         if (   fIOAPIC                       /* If IOAPIC is enabled, enable Local APIC in any case */
-            ||    u32Ebx == X86_CPUID_VENDOR_INTEL_EBX
-               && u32Ecx == X86_CPUID_VENDOR_INTEL_ECX
-               && u32Edx == X86_CPUID_VENDOR_INTEL_EDX /* GenuineIntel */
-            ||    u32Ebx == X86_CPUID_VENDOR_AMD_EBX
-               && u32Ecx == X86_CPUID_VENDOR_AMD_ECX
-               && u32Edx == X86_CPUID_VENDOR_AMD_EDX   /* AuthenticAMD */)
+            || (   u32Ebx == X86_CPUID_VENDOR_INTEL_EBX
+                && u32Ecx == X86_CPUID_VENDOR_INTEL_ECX
+                && u32Edx == X86_CPUID_VENDOR_INTEL_EDX /* GenuineIntel */)
+            || (   u32Ebx == X86_CPUID_VENDOR_AMD_EBX
+                && u32Ecx == X86_CPUID_VENDOR_AMD_ECX
+                && u32Edx == X86_CPUID_VENDOR_AMD_EDX   /* AuthenticAMD */))
         {
             LogRel(("Activating Local APIC\n"));
             pData->pApicHlpR3->pfnChangeFeature(pDevIns, true);
@@ -1992,7 +1992,7 @@ static DECLCALLBACK(int) ioapicConstruct(PPDMDEVINS pDevIns, int iInstance, PCFG
         fGCEnabled = true;
     else if (VBOX_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("Configuration error: Failed to query boolean value \"GCEnabled\"!"));
+                                N_("Configuration error: Failed to query boolean value \"GCEnabled\""));
     Log(("IOAPIC: fGCEnabled=%d\n", fGCEnabled));
 
     rc = CFGMR3QueryBool(pCfgHandle, "R0Enabled", &fR0Enabled);
@@ -2000,7 +2000,7 @@ static DECLCALLBACK(int) ioapicConstruct(PPDMDEVINS pDevIns, int iInstance, PCFG
         fR0Enabled = true;
     else if (VBOX_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("Configuration error: Failed to query boolean value \"R0Enabled\"!"));
+                                N_("Configuration error: Failed to query boolean value \"R0Enabled\""));
     Log(("IOAPIC: fR0Enabled=%d\n", fR0Enabled));
 
     /*
