@@ -1,4 +1,4 @@
-/* $Id: DevPcBios.cpp 4391 2007-08-27 17:12:05Z vboxsync $ */
+/* $Id: DevPcBios.cpp 27168 2008-01-09 16:58:25Z frank $ */
 /** @file
  * PC BIOS Device.
  */
@@ -92,6 +92,8 @@ typedef struct DEVPCBIOS
     uint8_t        uBootDelay;
     /** I/O-APIC enabled? */
     uint8_t        u8IOAPIC;
+    /** PXE debug logging enabled? */
+    uint8_t        u8PXEDebug;
 } DEVPCBIOS, *PDEVPCBIOS;
 
 
@@ -402,6 +404,11 @@ static DECLCALLBACK(int) pcbiosInitComplete(PPDMDEVINS pDevIns)
     pcbiosCmosWrite(pDevIns, 0x38, reg38);
     pcbiosCmosWrite(pDevIns, 0x3c, reg3c);
 
+    /*
+     * PXE debug option.
+     */
+    pcbiosCmosWrite(pDevIns, 0x3f, pData->u8PXEDebug);
+
     /*
      * Floppy drive type.
      */
@@ -445,9 +452,9 @@ static DECLCALLBACK(int) pcbiosInitComplete(PPDMDEVINS pDevIns)
         case 2: u32 = 0x41; break;      /* floppy installed, 2 drives. */
         default:u32 = 0;    break;      /* floppy not installed. */
     }
-    u32 |= BIT(1);                      /* math coprocessor installed  */
-    u32 |= BIT(2);                      /* keyboard enabled (or mouse?) */
-    u32 |= BIT(3);                      /* display enabled (monitory type is 0, i.e. vga) */
+    u32 |= RT_BIT(1);                      /* math coprocessor installed  */
+    u32 |= RT_BIT(2);                      /* keyboard enabled (or mouse?) */
+    u32 |= RT_BIT(3);                      /* display enabled (monitory type is 0, i.e. vga) */
     pcbiosCmosWrite(pDevIns, 0x14, u32);                                        /* 14h - Equipment Byte */
 
     /*
@@ -599,7 +606,7 @@ static DECLCALLBACK(int) pcbiosIOPortWrite(PPDMDEVINS pDevIns, void *pvUser, RTI
         }
         else
         {
-            if (pData->iMsg >= sizeof(pData->szMsg))
+            if (pData->iMsg >= sizeof(pData->szMsg)-1)
             {
                 pData->szMsg[pData->iMsg] = '\0';
                 Log(("pcbios: %s\n", pData->szMsg));
@@ -733,7 +740,7 @@ static DECLCALLBACK(int) logoIOPortWrite(PPDMDEVINS pDevIns, void *pvUser, RTIOP
  * @param   table           pointer to DMI table.
  */
 #define STRCPY(p, s) do { memcpy (p, s, sizeof(s)); p += sizeof(s); } while (0)
-static void pcbiosPlantDMITable(uint8_t *pTable)
+static void pcbiosPlantDMITable(uint8_t *pTable, PRTUUID puuid)
 {
     char *pszStr = (char*)pTable;
     int iStrNr;
@@ -752,15 +759,15 @@ static void pcbiosPlantDMITable(uint8_t *pTable)
     pBIOSInf->u8Release          = iStrNr++;
     STRCPY(pszStr, "12/01/2006");
     pBIOSInf->u8ROMSize          = 1; /* 128K */
-    pBIOSInf->u64Characteristics = BIT(4)   /* ISA is supported */
-                                 | BIT(7)   /* PCI is supported */
-                                 | BIT(15)  /* Boot from CD is supported */
-                                 | BIT(16)  /* Selectable Boot is supported */
-                                 | BIT(27)  /* Int 9h, 8042 Keyboard services supported */
-                                 | BIT(30)  /* Int 10h, CGA/Mono Video Services supported */
+    pBIOSInf->u64Characteristics = RT_BIT(4)   /* ISA is supported */
+                                 | RT_BIT(7)   /* PCI is supported */
+                                 | RT_BIT(15)  /* Boot from CD is supported */
+                                 | RT_BIT(16)  /* Selectable Boot is supported */
+                                 | RT_BIT(27)  /* Int 9h, 8042 Keyboard services supported */
+                                 | RT_BIT(30)  /* Int 10h, CGA/Mono Video Services supported */
                                  /* any more?? */
                                  ;
-    pBIOSInf->u8CharacteristicsByte1 = BIT(0)   /* ACPI is supported */
+    pBIOSInf->u8CharacteristicsByte1 = RT_BIT(0)   /* ACPI is supported */
                                      /* any more?? */
                                      ;
     pBIOSInf->u8CharacteristicsByte2 = 0
@@ -783,6 +790,7 @@ static void pcbiosPlantDMITable(uint8_t *pTable)
     STRCPY(pszStr, "1.2");
     pSystemInf->u8SerialNumber   = iStrNr++;
     STRCPY(pszStr, "0");
+    memcpy(pSystemInf->au8Uuid, puuid, sizeof(RTUUID));
     pSystemInf->u8WakeupType     = 6; /* Power Switch */
     pSystemInf->u8SKUNumber      = 0;
     pSystemInf->u8Family         = iStrNr++;
@@ -1043,7 +1051,7 @@ static int pcbiosBootFromCfg(PPDMDEVINS pDevIns, PCFGMNODE pCfgHandle, const cha
     else
     {
         PDMDevHlpVMSetError(pDevIns, rc, RT_SRC_POS,
-                            N_("Configuration error: The \"%s\" value \"%s\" is unknown.\n"),
+                            N_("Configuration error: The \"%s\" value \"%s\" is unknown"),
                             pszParam, psz);
         rc = VERR_INTERNAL_ERROR;
     }
@@ -1091,6 +1099,8 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
                               "ShowBootMenu\0"
                               "DelayBoot\0"
                               "LanBootRom\0"
+                              "PXEDebug\0"
+                              "UUID\0"
                               "IOAPIC\0"))
         return PDMDEV_SET_ERROR(pDevIns, VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES,
                                 N_("Invalid configuraton for  device pcbios device"));
@@ -1108,7 +1118,7 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
         pData->u8IOAPIC = 1;
     else if (VBOX_FAILURE (rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("Configuration error: Failed to read \"IOAPIC\"."));
+                                N_("Configuration error: Failed to read \"IOAPIC\""));
 
     static const char * const s_apszBootDevices[] = { "BootDevice0", "BootDevice1", "BootDevice2", "BootDevice3" };
     Assert(ELEMENTS(s_apszBootDevices) == ELEMENTS(pData->aenmBootDevice));
@@ -1141,7 +1151,21 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
     if (VBOX_FAILURE(rc))
         return rc;
 
-    pcbiosPlantDMITable(pData->au8DMIPage);
+    /*
+     * Query the machine's UUID for SMBIOS/DMI use.
+     */
+    RTUUID  uuid;
+    rc = CFGMR3QueryBytes(pCfgHandle, "UUID", &uuid, sizeof(uuid));
+    if (VBOX_FAILURE(rc))
+        return PDMDEV_SET_ERROR(pDevIns, rc,
+                                N_("Configuration error: Querying \"UUID\" failed"));
+
+
+    /* Convert the UUID to network byte order. Not entirely straightforward as parts are MSB already... */
+    uuid.Gen.u32TimeLow = RT_H2BE_U32(uuid.Gen.u32TimeLow);
+    uuid.Gen.u16TimeMid = RT_H2BE_U16(uuid.Gen.u16TimeMid);
+    uuid.Gen.u16TimeHiAndVersion = RT_H2BE_U16(uuid.Gen.u16TimeHiAndVersion);
+    pcbiosPlantDMITable(pData->au8DMIPage, &uuid);
     if (pData->u8IOAPIC)
         pcbiosPlantMPStable(pDevIns, pData->au8DMIPage + 0x100);
 
@@ -1149,6 +1173,16 @@ static DECLCALLBACK(int)  pcbiosConstruct(PPDMDEVINS pDevIns, int iInstance, PCF
     if (VBOX_FAILURE(rc))
         return rc;
 
+    /*
+     * Read the PXE debug logging option.
+     */
+    rc = CFGMR3QueryU8(pCfgHandle, "PXEDebug", &pData->u8PXEDebug);
+    if (rc == VERR_CFGM_VALUE_NOT_FOUND)
+        pData->u8PXEDebug = 0;
+    else if (VBOX_FAILURE(rc))
+        return PDMDEV_SET_ERROR(pDevIns, rc,
+                                N_("Configuration error: Querying \"PXEDebug\" as integer failed"));
+
     /*
      * Map the BIOS into memory.
      * There are two mappings:
