@@ -1,4 +1,4 @@
-/* $Id: DevParallel.cpp 4088 2007-08-08 09:12:59Z vboxsync $ */
+/* $Id: DevParallel.cpp 27168 2008-01-09 16:58:25Z frank $ */
 /** @file
  * VirtualBox Parallel Device Emulation.
  *
@@ -542,7 +542,7 @@ static DECLCALLBACK(int) parallelConstruct(PPDMDEVINS pDevIns,
     {
         AssertMsgFailed(("Parallel%d: Failed to attach to host driver. rc=%Vrc\n", iInstance, rc));
         return PDMDevHlpVMSetError(pDevIns, rc, RT_SRC_POS,
-                                   N_("Parallel device %d cannot attach to host driver\n"), iInstance);
+                                   N_("Parallel device %d cannot attach to host driver"), iInstance);
     }
 
     rc = PDMDevHlpSSMRegister(
