@@ -63,7 +63,7 @@
 #include <VBox/pci.h>
 #endif /* VBOX_SERIAL_PCI */
 
-#define SERIAL_SAVED_STATE_VERSION  2
+#define SERIAL_SAVED_STATE_VERSION  3
 
 #define UART_LCR_DLAB	            0x80	/* Divisor latch access bit */
 
@@ -116,20 +116,20 @@ struct SerialState
     PDMCRITSECT                     CritSect;
 
     /** Pointer to the device instance. */
-    HCPTRTYPE(PPDMDEVINS)           pDevInsHC;
+    R3PTRTYPE(PPDMDEVINS)           pDevInsHC;
     /** Pointer to the device instance. */
     GCPTRTYPE(PPDMDEVINS)           pDevInsGC;
 #if HC_ARCH_BITS == 64 && GC_ARCH_BITS != 64
     RTGCPTR                         Alignment0;
 #endif
     /** The base interface. */
-    HCPTRTYPE(PDMIBASE)             IBase;
+    PDMIBASE                        IBase;
     /** The character port interface. */
-    HCPTRTYPE(PDMICHARPORT)         ICharPort;
+    PDMICHARPORT                    ICharPort;
     /** Pointer to the attached base driver. */
-    HCPTRTYPE(PPDMIBASE)            pDrvBase;
+    R3PTRTYPE(PPDMIBASE)            pDrvBase;
     /** Pointer to the attached character driver. */
-    HCPTRTYPE(PPDMICHAR)            pDrvChar;
+    R3PTRTYPE(PPDMICHAR)            pDrvChar;
 
     uint16_t                        divider;
     uint16_t                        auAlignment[3];
@@ -145,10 +145,11 @@ struct SerialState
        it can be reset while reading iir */
     int                             thr_ipending;
     int                             irq;
+    bool                            msr_changed;
 
     bool                            fGCEnabled;
     bool                            fR0Enabled;
-    bool                            afAlignment[6];
+    bool                            afAlignment[5];
 
     RTSEMEVENT                      ReceiveSem;
     int                             last_break_enable;
@@ -181,6 +182,8 @@ static void serial_update_irq(SerialState *s)
         s->iir = UART_IIR_RDI;
     } else if (s->thr_ipending && (s->ier & UART_IER_THRI)) {
         s->iir = UART_IIR_THRI;
+    } else if (s->msr_changed && (s->ier & UART_IER_RLSI)) {
+        s->iir = UART_IIR_RLSI;
     } else {
         s->iir = UART_IIR_NO_INT;
     }
@@ -280,8 +283,11 @@ static int serial_ioport_write(void *opaque, uint32_t addr, uint32_t val)
     case 3:
         {
             int break_enable;
-            s->lcr = val;
-            serial_update_parameters(s);
+            if (s->lcr != val)
+            {
+                s->lcr = val;
+                serial_update_parameters(s);
+            }
             break_enable = (val >> 6) & 1;
             if (break_enable != s->last_break_enable) {
                 s->last_break_enable = break_enable;
@@ -290,6 +296,11 @@ static int serial_ioport_write(void *opaque, uint32_t addr, uint32_t val)
         break;
     case 4:
         s->mcr = val & 0x1f;
+        if (RT_LIKELY(s->pDrvChar))
+        {
+            int rc = s->pDrvChar->pfnSetModemLines(s->pDrvChar, !!(s->mcr & UART_MCR_RTS), !!(s->mcr & UART_MCR_DTR));
+            AssertRC(rc);
+        }
         break;
     case 5:
         break;
@@ -346,6 +357,8 @@ static uint32_t serial_ioport_read(void *opaque, uint32_t addr, int *pRC)
         /* reset THR pending bit */
         if ((ret & 0x7) == UART_IIR_THRI)
             s->thr_ipending = 0;
+        /* reset msr changed bit */
+        s->msr_changed = false;
         serial_update_irq(s);
 #endif
         break;
@@ -367,6 +380,8 @@ static uint32_t serial_ioport_read(void *opaque, uint32_t addr, int *pRC)
             ret |= (s->mcr & 0x01) << 5;
         } else {
             ret = s->msr;
+            /* Reset delta bits. */
+            s->msr &= ~UART_MSR_ANY_DELTA;
         }
         break;
     case 7:
@@ -412,6 +427,45 @@ static DECLCALLBACK(int) serialNotifyRead(PPDMICHARPORT pInterface, const void *
 
     return rc;
 }
+
+static DECLCALLBACK(int) serialNotifyStatusLinesChanged(PPDMICHARPORT pInterface, uint32_t newStatusLines)
+{
+    SerialState *pData = PDMICHARPORT_2_SERIALSTATE(pInterface);
+    uint8_t newMsr = 0;
+
+    Log(("%s: pInterface=%p newStatusLines=%u\n", __FUNCTION__, pInterface, newStatusLines));
+
+    PDMCritSectEnter(&pData->CritSect, VERR_PERMISSION_DENIED);
+
+    /* Set new states. */
+    if (newStatusLines & PDM_ICHAR_STATUS_LINES_DCD)
+        newMsr |= UART_MSR_DCD;
+    if (newStatusLines & PDM_ICHAR_STATUS_LINES_RI)
+        newMsr |= UART_MSR_RI;
+    if (newStatusLines & PDM_ICHAR_STATUS_LINES_DSR)
+        newMsr |= UART_MSR_DSR;
+    if (newStatusLines & PDM_ICHAR_STATUS_LINES_CTS)
+        newMsr |= UART_MSR_CTS;
+
+    /* Compare the old and the new states and set the delta bits accordingly. */
+    if ((newMsr & UART_MSR_DCD) != (pData->msr & UART_MSR_DCD))
+        newMsr |= UART_MSR_DDCD;
+    if ((newMsr & UART_MSR_RI) == 1 && (pData->msr & UART_MSR_RI) == 0)
+        newMsr |= UART_MSR_TERI;
+    if ((newMsr & UART_MSR_DSR) != (pData->msr & UART_MSR_DSR))
+        newMsr |= UART_MSR_DDSR;
+    if ((newMsr & UART_MSR_CTS) != (pData->msr & UART_MSR_CTS))
+        newMsr |= UART_MSR_DCTS;
+
+    pData->msr = newMsr;
+    pData->msr_changed = true;
+    serial_update_irq(pData);
+
+    PDMCritSectLeave(&pData->CritSect);
+
+    return VINF_SUCCESS;
+}
+
 #endif /* IN_RING3 */
 
 /**
@@ -505,6 +559,7 @@ static DECLCALLBACK(int) serialSaveExec(PPDMDEVINS pDevIns,
     SSMR3PutS32(pSSMHandle, pData->irq);
     SSMR3PutS32(pSSMHandle, pData->last_break_enable);
     SSMR3PutU32(pSSMHandle, pData->base);
+    SSMR3PutBool(pSSMHandle, pData->msr_changed);
     return SSMR3PutU32(pSSMHandle, ~0); /* sanity/terminator */
 }
 
@@ -542,6 +597,7 @@ static DECLCALLBACK(int) serialLoadExec(PPDMDEVINS pDevIns,
     SSMR3GetS32(pSSMHandle, &pData->irq);
     SSMR3GetS32(pSSMHandle, &pData->last_break_enable);
     SSMR3GetU32(pSSMHandle, &pData->base);
+    SSMR3GetBool(pSSMHandle, &pData->msr_changed);
 
     rc = SSMR3GetU32(pSSMHandle, &u32);
     if (VBOX_FAILURE(rc))
@@ -669,7 +725,10 @@ static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
      * Validate configuration.
      */
     if (!CFGMR3AreValuesValid(pCfgHandle, "IRQ\0IOBase\0"))
+    {
+        AssertMsgFailed(("serialConstruct Invalid configuration values\n"));
         return VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES;
+    }
 
     rc = CFGMR3QueryBool(pCfgHandle, "GCEnabled", &pData->fGCEnabled);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
@@ -690,6 +749,7 @@ static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
 
     /* ICharPort */
     pData->ICharPort.pfnNotifyRead = serialNotifyRead;
+    pData->ICharPort.pfnNotifyStatusLinesChanged = serialNotifyStatusLinesChanged;
 
     rc = RTSemEventCreate(&pData->ReceiveSem);
     AssertRC(rc);
@@ -704,16 +764,30 @@ static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
     if (VBOX_FAILURE(rc))
         return rc;
 
-/** @todo r=bird: Check for VERR_CFGM_VALUE_NOT_FOUND and provide sensible defaults.
- * Also do AssertMsgFailed(("Configuration error:....)) in the failure cases of CFGMR3Query*()
- * and CFGR3AreValuesValid() like we're doing in the other devices.  */
     rc = CFGMR3QueryU8 (pCfgHandle, "IRQ", &irq_lvl);
-    if (VBOX_FAILURE (rc))
-        return rc;
+    if (rc == VERR_CFGM_VALUE_NOT_FOUND)
+    {
+        /* Provide sensible defaults. */
+        if (iInstance == 0)
+            irq_lvl = 4;
+        else if (iInstance == 1)
+            irq_lvl = 3;
+    }
+    else if (VBOX_FAILURE(rc))
+        return PDMDEV_SET_ERROR(pDevIns, rc,
+                                N_("Configuration error: Failed to get the \"IRQ\" value"));
 
     rc = CFGMR3QueryU16 (pCfgHandle, "IOBase", &io_base);
-    if (VBOX_FAILURE (rc))
-        return rc;
+    if (rc == VERR_CFGM_VALUE_NOT_FOUND)
+    {
+        if (iInstance == 0)
+            io_base = 0x3f8;
+        else if (iInstance == 1)
+            io_base = 0x2f8;
+    }
+    else if (VBOX_FAILURE(rc))
+        return PDMDEV_SET_ERROR(pDevIns, rc,
+                                N_("Configuration error: Failed to get the \"IOBase\" value"));
 
     Log(("serialConstruct instance %d iobase=%04x irq=%d\n", iInstance, io_base, irq_lvl));
 
@@ -783,8 +857,8 @@ static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
     else
     {
         AssertMsgFailed(("Serial%d: Failed to attach to char driver. rc=%Vrc\n", iInstance, rc));
-        return PDMDevHlpVMSetError(pDevIns, rc, RT_SRC_POS,
-                                   N_("Serial device %d cannot attach to char driver\n"), iInstance);
+        /* Don't call VMSetError here as we assume that the driver already set an appropriate error */
+        return rc;
     }
 
     rc = PDMDevHlpSSMRegister (
