@@ -788,10 +788,10 @@ static uint32_t fdctrl_read_dor (fdctrl_t *fdctrl)
 #else
     /* bit4: 0 = drive 0 motor off/1 = on */
     if (drv0(fdctrl)->drflags & FDRIVE_MOTOR_ON)
-        retval |= BIT(4);
+        retval |= RT_BIT(4);
     /* bit5: 0 = drive 1 motor off/1 = on */
     if (drv1(fdctrl)->drflags & FDRIVE_MOTOR_ON)
-        retval |= BIT(5);
+        retval |= RT_BIT(5);
 #endif
     /* DMA enable */
     retval |= fdctrl->dma_en << 3;
@@ -2651,12 +2651,27 @@ static int fdConfig (fdrive_t *drv, PPDMDEVINS pDevIns)
             rc = VERR_PDM_MISSING_INTERFACE;
         }
     }
-    else {
+    else
+    {
         AssertMsg (rc == VERR_PDM_NO_ATTACHED_DRIVER,
                    ("Failed to attach LUN#%d. rc=%Vrc\n", drv->iLUN, rc));
+        switch (rc)
+        {
+            case VERR_ACCESS_DENIED:
+                /* Error already catched by DrvHostBase */
+                break;
+            case VERR_PDM_NO_ATTACHED_DRIVER:
+                /* Legal on architectures without a floppy controller */
+                break;
+            default:
+                rc = PDMDevHlpVMSetError(pDevIns, rc, RT_SRC_POS,
+                                         N_("The floppy controller cannot attach to the floppy drive"));
+                break;
+        }
     }
 
-    if (VBOX_FAILURE (rc)) {
+    if (VBOX_FAILURE (rc))
+    {
         drv->pDrvBase = NULL;
         drv->pDrvBlock = NULL;
         drv->pDrvBlockBios = NULL;
