@@ -643,6 +643,12 @@ static DECLCALLBACK(int) vmmdevRequestHandler(PPDMDEVINS pDevIns, void *pvUser,
                 Log(("VMMDevReq_SetPointerShape: visible: %d, alpha: %d, shape = %d, width: %d, height: %d\n",
                      fVisible, fAlpha, fShape, pointerShape->width, pointerShape->height));
 
+                if (requestHeader->size == sizeof(VMMDevReqMousePointer))
+                {
+                    /* The guest did not provide the shape actually. */
+                    fShape = false;
+                }
+
                 /* forward call to driver */
                 if (fShape)
                 {
