@@ -35,24 +35,27 @@ using namespace com;
 
 #include <VBox/types.h>
 #include <VBox/err.h>
+#include <VBox/log.h>
 #include <VBox/param.h>
 #include <VBox/pdm.h>
 #include <VBox/version.h>
 #ifdef VBOXBFE_WITH_USB
 # include <VBox/vusb.h>
 #endif
-#include <VBox/log.h>
-#include <iprt/path.h>
-#include <iprt/string.h>
-#include <iprt/runtime.h>
+#ifdef VBOX_HGCM
+# include <VBox/shflsvc.h>
+#endif
+#include <iprt/alloca.h>
 #include <iprt/assert.h>
+#include <iprt/ctype.h>
+#include <iprt/file.h>
+#include <iprt/path.h>
+#include <iprt/runtime.h>
 #include <iprt/semaphore.h>
 #include <iprt/stream.h>
+#include <iprt/string.h>
 #include <iprt/thread.h>
 #include <iprt/uuid.h>
-#include <iprt/file.h>
-#include <iprt/alloca.h>
-#include <iprt/ctype.h>
 
 #include "VBoxBFE.h"
 
@@ -67,10 +70,6 @@ using namespace com;
 #include <linux/if_tun.h>
 #endif
 
-#ifndef RT_OS_L4
-#include <vector>
-#endif
-
 #include "ConsoleImpl.h"
 #include "DisplayImpl.h"
 #include "MouseImpl.h"
@@ -91,12 +90,11 @@ using namespace com;
 #ifdef RT_OS_L4
 #include "L4Console.h"
 #include "L4Framebuffer.h"
+#include "L4IDLInterface.h"
 #endif
 
 #ifdef RT_OS_L4
-# ifndef L4API_l4v2onv4
-#  include <l4/sys/ktrace.h>
-# endif
+# include <l4/sys/ktrace.h>
 # include <l4/vboxserver/file.h>
 #endif
 
@@ -122,61 +120,68 @@ static DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser);
 *   Global Variables                                                           *
 *******************************************************************************/
 
-PVM              pVM              = NULL;
-Mouse           *gMouse           = NULL;
-VMDisplay       *gDisplay         = NULL;
-Keyboard        *gKeyboard        = NULL;
-VMMDev          *gVMMDev          = NULL;
-Framebuffer     *gFramebuffer     = NULL;
-MachineDebugger *gMachineDebugger = NULL;
-VMStatus        *gStatus          = NULL;
-Console         *gConsole         = NULL;
+PVM                pVM              = NULL;
+Mouse             *gMouse           = NULL;
+VMDisplay         *gDisplay         = NULL;
+Keyboard          *gKeyboard        = NULL;
+VMMDev            *gVMMDev          = NULL;
+Framebuffer       *gFramebuffer     = NULL;
+MachineDebugger   *gMachineDebugger = NULL;
+VMStatus          *gStatus          = NULL;
+Console           *gConsole         = NULL;
 #ifdef VBOXBFE_WITH_USB
-HostUSB         *gHostUSB         = NULL;
+HostUSB           *gHostUSB         = NULL;
 #endif
 
 VMSTATE machineState = VMSTATE_CREATING;
 
-PPDMLED     mapFDLeds[2]      = {0};
-PPDMLED     mapIDELeds[4]     = {0};
+static PPDMLED     mapFDLeds[2]   = {0};
 
 /** flag whether keyboard/mouse events are grabbed */
 #ifdef RT_OS_L4
 /** see <l4/input/macros.h> for key definitions */
-int gHostKey; /* not used */
-int gHostKeySym = KEY_RIGHTCTRL;
+int                gHostKey; /* not used */
+int                gHostKeySym = KEY_RIGHTCTRL;
 #elif defined (DEBUG_dmik)
 // my mini kbd doesn't have RCTRL...
-int gHostKey    = KMOD_RSHIFT;
-int gHostKeySym = SDLK_RSHIFT;
+int                gHostKey    = KMOD_RSHIFT;
+int                gHostKeySym = SDLK_RSHIFT;
 #else
-int gHostKey    = KMOD_RCTRL;
-int gHostKeySym = SDLK_RCTRL;
+int                gHostKey    = KMOD_RCTRL;
+int                gHostKeySym = SDLK_RCTRL;
 #endif
 bool gfAllowFullscreenToggle = true;
 
-static bool g_fIOAPIC = false;
-static bool g_fACPI   = true;
-static bool g_fAudio  = false;
+static bool        g_fIOAPIC = false;
+static bool        g_fACPI   = true;
+static bool        g_fAudio  = false;
 #ifdef VBOXBFE_WITH_USB
-static bool g_fUSB    = false;
+static bool        g_fUSB    = false;
 #endif
-static char *g_pszHdaFile   = NULL;
-static char *g_pszHdbFile   = NULL;
-static char *g_pszCdromFile = NULL;
-static char *g_pszFdaFile   = NULL;
-static const char *pszBootDevice = "IDE";
-static uint32_t g_u32MemorySizeMB = 128;
-static uint32_t g_u32VRamSizeMB = 4;
+static char       *g_pszHdaFile   = NULL;
+static char       *g_pszHdbFile   = NULL;
+static char       *g_pszCdromFile = NULL;
+static char       *g_pszFdaFile   = NULL;
+       const char *g_pszStateFile = NULL;
+static const char *g_pszBootDevice = "IDE";
+static uint32_t    g_u32MemorySizeMB = 128;
+static uint32_t    g_u32VRamSize = 4 * _1M;
 #ifdef VBOXSDL_ADVANCED_OPTIONS
-static bool g_fRawR0 = true;
-static bool g_fRawR3 = true;
-static bool g_fPATM  = true;
-static bool g_fCSAM  = true;
+static bool        g_fRawR0 = true;
+static bool        g_fRawR3 = true;
+static bool        g_fPATM  = true;
+static bool        g_fCSAM  = true;
 #endif
-static bool g_fPreAllocRam = false;
-static int  g_iBootMenu = 2;
-static bool g_fReleaseLog = true; /**< Set if we should open the release. */
+static bool        g_fRestoreState = false;
+static const char *g_pszShareDir[MaxSharedFolders];
+static const char *g_pszShareName[MaxSharedFolders];
+static bool        g_fShareReadOnly[MaxSharedFolders];
+static unsigned    g_uNumShares;
+static bool        g_fPreAllocRam = false;
+static int         g_iBootMenu = 2;
+static bool        g_fReleaseLog = true; /**< Set if we should open the release. */
+       const char *g_pszProgressString;
+       unsigned    g_uProgressPercent = ~0U;
 
 
 /**
@@ -196,10 +201,8 @@ typedef struct BFENetworkDevice
     const char *pszSniff;   /**< Output file for the network sniffer. */
     PDMMAC      Mac;        /**< The mac address for the device. */
     const char *pszName;     /**< The device name of a HIF device. The name of the internal network. */
-#if 1//defined(RT_OS_LINUX)
     bool        fHaveFd;    /**< Set if fd is valid. */
     int32_t     fd;         /**< The file descriptor of a HIF device.*/
-#endif
 } BFENETDEV, *PBFENETDEV;
 
 /** Array of network device configurations. */
@@ -210,6 +213,13 @@ static BFENETDEV g_aNetDevs[NetworkAdapterCount];
 static char szError[512];
 
 
+/**
+ */
+bool fActivateHGCM()
+{
+    return !!(g_uNumShares > 0);
+}
+
 /**
  * Converts the passed in network option
  *
@@ -243,7 +253,7 @@ static int networkArg2Index(const char *pszArg, int cchRoot)
  * @returns iprt status code
  * @param pAddress An array into which to store the newly generated address
  */
-int GenerateMACAddress(char pAddress[MAC_STRING_LEN + 1])
+int GenerateMACAddress(char pszAddress[MAC_STRING_LEN + 1])
 {
     /*
      * Our strategy is as follows: the first three bytes are our fixed
@@ -258,11 +268,11 @@ int GenerateMACAddress(char pAddress[MAC_STRING_LEN + 1])
         LogFlowFunc(("RTUuidCreate failed, returning %Vrc\n", rc));
         return rc;
     }
-    RTStrPrintf (pAddress, MAC_STRING_LEN + 1, "080027%02X%02X%02X",
-                 uuid.au8[0], uuid.au8[1], uuid.au8[2]);
-    LogFlowFunc(("generated MAC: '%s'\n", pAddress));
+    RTStrPrintf(pszAddress, MAC_STRING_LEN + 1, "080027%02X%02X%02X",
+                uuid.au8[0], uuid.au8[1], uuid.au8[2]);
+    LogFlowFunc(("generated MAC: '%s'\n", pszAddress));
     return VINF_SUCCESS;
-}                         
+}
 
 /**
  * Print a syntax error.
@@ -299,6 +309,32 @@ static int FatalError(const char *pszMsg, ...)
     return 1;
 }
 
+/**
+ * Start progress display.
+ */
+void startProgressInfo(const char *pszStr)
+{
+    g_pszProgressString = pszStr;
+    g_uProgressPercent = 0;
+}
+
+/**
+ * Update progress display.
+ */
+void callProgressInfo(PVM pVM, unsigned uPercent, void *pvUser)
+{
+    if (gConsole)
+        gConsole->progressInfo(pVM, uPercent, pvUser);
+}
+
+/**
+ * End progress display.
+ */
+void endProgressInfo(void)
+{
+    g_uProgressPercent = ~0U;
+}
+
 
 /**
  * Print program usage.
@@ -316,7 +352,11 @@ static void show_usage()
              "  -vram <size>       Set size of video memory in megabytes\n"
              "  -prealloc          Force RAM pre-allocation\n"
              "  -fullscreen        Start VM in fullscreen mode\n"
+             "  -statefile <file>  Define the file name for VM save/restore\n"
+             "  -restore           Restore the VM if the statefile exists, normal start otherwise\n"
              "  -nofstoggle        Forbid switching to/from fullscreen mode\n"
+             "  -share <dir> <name> [readonly]\n"
+             "                     Share directory <dir> as name <name>. Optionally read-only.\n"
              "  -nohostkey         Disable hostkey\n"
              "  -[no]acpi          Enable or disable ACPI (default: enabled)\n"
              "  -[no]ioapic        Enable or disable the IO-APIC (default: disabled)\n"
@@ -398,6 +438,10 @@ int main(int argc, char **argv)
     uint32_t secureLabelPointSize = 12;
     char *secureLabelFontFile = NULL;
 #endif
+#ifdef RT_OS_L4
+    uint32_t u32MaxVRAM;
+#endif
+
     RTPrintf("VirtualBox Simple SDL GUI built %s %s\n", __DATE__, __TIME__);
 
     // less than one parameter is not possible
@@ -424,19 +468,19 @@ int main(int argc, char **argv)
             {
                 case 'a':
                 {
-                    pszBootDevice = "FLOPPY";
+                    g_pszBootDevice = "FLOPPY";
                     break;
                 }
 
                 case 'c':
                 {
-                    pszBootDevice = "IDE";
+                    g_pszBootDevice = "IDE";
                     break;
                 }
 
                 case 'd':
                 {
-                    pszBootDevice = "DVD";
+                    g_pszBootDevice = "DVD";
                     break;
                 }
 
@@ -466,11 +510,38 @@ int main(int argc, char **argv)
         {
             if (++curArg >= argc)
                 return SyntaxError("missing argument for vram size!\n");
-            rc = RTStrToUInt32Ex(argv[curArg], NULL, 0, &g_u32VRamSizeMB);
+            uint32_t uVRAMMB;
+            rc = RTStrToUInt32Ex(argv[curArg], NULL, 0, &uVRAMMB);
+            g_u32VRamSize = uVRAMMB * _1M;
             if (VBOX_FAILURE(rc))
                 return SyntaxError("bad video ram size: %s (error %Vrc)\n",
                                    argv[curArg], rc);
         }
+        else if (strcmp(pszArg, "-statefile") == 0)
+        {
+            if (++curArg >= argc)
+                return SyntaxError("missing argument for restore!\n");
+            g_pszStateFile = argv[curArg];
+        }
+        else if (strcmp(pszArg, "-restore") == 0)
+            g_fRestoreState = true;
+        else if (strcmp(pszArg, "-share") == 0)
+        {
+            if (g_uNumShares >= MaxSharedFolders)
+                return SyntaxError("too many shared folders specified!\n");
+            if (++curArg >= argc)
+                return SyntaxError("missing 1s argument for share!\n");
+            g_pszShareDir[g_uNumShares] = argv[curArg];
+            if (++curArg >= argc)
+                return SyntaxError("missing 2nd argument for share!\n");
+            g_pszShareName[g_uNumShares] = argv[curArg];
+            if (curArg < argc-1 && strcmp(argv[curArg+1], "readonly") == 0)
+            {
+                g_fShareReadOnly[g_uNumShares] = true;
+                curArg++;
+            }
+            g_uNumShares++;
+        }
         else if (strcmp(pszArg, "-fullscreen") == 0)
             fFullscreen = true;
         else if (strcmp(pszArg, "-nofstoggle") == 0)
@@ -724,6 +795,7 @@ int main(int argc, char **argv)
      * the meantime, we have to service the SDL event loop.
      */
 
+
     RTTHREAD thread;
     rc = RTThreadCreate(&thread, VMPowerUpThread, 0, 0, RTTHREADTYPE_MAIN_WORKER, 0, "PowerUp");
     if (VBOX_FAILURE(rc))
@@ -732,6 +804,11 @@ int main(int argc, char **argv)
         return -1;
     }
 
+#ifdef RT_OS_L4
+    /* Start the external IDL interface */
+    L4CtrlInit();
+#endif
+
     /* loop until the powerup processing is done */
     do
     {
@@ -750,6 +827,10 @@ int main(int argc, char **argv)
                 gDisplay->ResizeCompleted();
                 break;
 
+            case CONEVENT_USR_TITLEBARUPDATE:
+                gConsole->updateTitlebar();
+                break;
+
             case CONEVENT_USR_QUIT:
                 RTPrintf("Error: failed to power up VM! No error text available.\n");
                 goto leave;
@@ -776,12 +857,24 @@ int main(int argc, char **argv)
 
 #ifdef RT_OS_L4
     /* The L4 console provides (currently) a fixed resolution. */
-    if (g_u32VRamSizeMB * _1M >=   gFramebuffer->getHostXres() 
-                          * gFramebuffer->getHostYres()
-                          * (gDisplay->getBitsPerPixel() / 8))
+    if (g_u32VRamSize >= gFramebuffer->getHostXres()
+                       * gFramebuffer->getHostYres()
+                       * (gDisplay->getBitsPerPixel() / 8))
         gDisplay->SetVideoModeHint(gFramebuffer->getHostXres(), gFramebuffer->getHostYres(), 0, 0);
+
+    /* Limit the VRAM of the guest to the amount of memory we got actually
+     * mapped from the L4 console. */
+    u32MaxVRAM = (gFramebuffer->getHostYres() + 18) /* don't omit the status bar */
+               * gFramebuffer->getHostXres()
+               * (gFramebuffer->getHostBitsPerPixel() / 8);
+    if (g_u32VRamSize > u32MaxVRAM)
+    {
+        RTPrintf("Limiting the video memory to %u bytes\n", u32MaxVRAM);
+        g_u32VRamSize = u32MaxVRAM;
+    }
 #endif
 
+
     /*
      * Main event loop
      */
@@ -946,9 +1039,12 @@ DECLCALLBACK(void) setVMErrorCallback(PVM pVM, void *pvUser, int rc, RT_SRC_POS_
         szError[0] = '\0';
     else
     {
-        RTStrPrintf(szError, sizeof(szError), 
-                    "%N!\nVBox status code: %d (%Vrc)", pszFormat, &args, rc, rc);
+        va_list va2;
+        va_copy(va2, args); /* Have to make a copy here or GCC will break. */
+        RTStrPrintf(szError, sizeof(szError),
+                    "%N!\nVBox status code: %d (%Vrc)", pszFormat, &va2, rc, rc);
         RTPrintf("%s\n", szError);
+        va_end(va2);
     }
 }
 
@@ -968,7 +1064,10 @@ DECLCALLBACK(void) setVMRuntimeErrorCallback(PVM pVM, void *pvUser, bool fFatal,
                                              const char *pszErrorId,
                                              const char *pszFormat, va_list args)
 {
-    RTPrintf("%s: %s!\n%N!\n", fFatal ? "Error" : "Warning", pszErrorId, pszFormat, &args);
+    va_list va2;
+    va_copy(va2, args); /* Have to make a copy here or GCC will break. */
+    RTPrintf("%s: %s!\n%N!\n", fFatal ? "Error" : "Warning", pszErrorId, pszFormat, &va2);
+    va_end(va2);
 }
 
 
@@ -1023,7 +1122,6 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
         goto failure;
     }
 
-
     /*
      * Register VM state change handler
      */
@@ -1034,6 +1132,64 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
         goto failure;
     }
 
+#ifdef VBOX_HGCM
+    /*
+     * Add shared folders to the VM
+     */
+    if (fActivateHGCM() && gVMMDev->isShFlActive())
+    {
+        for (unsigned i=0; i<g_uNumShares; i++)
+        {
+            VBOXHGCMSVCPARM  parms[SHFL_CPARMS_ADD_MAPPING];
+            SHFLSTRING      *pFolderName, *pMapName;
+            int              cbString;
+            PRTUCS2          aHostPath, aMapName;
+            int              rc;
+
+            rc = RTStrUtf8ToUcs2(&aHostPath, g_pszShareDir[i]);
+            AssertRC(rc);
+            rc = RTStrUtf8ToUcs2(&aMapName, g_pszShareName[i]);
+            AssertRC(rc);
+
+            cbString = (RTStrUcs2Len (aHostPath) + 1) * sizeof (RTUCS2);
+            pFolderName = (SHFLSTRING *) RTMemAllocZ (sizeof (SHFLSTRING) + cbString);
+            Assert (pFolderName);
+            memcpy (pFolderName->String.ucs2, aHostPath, cbString);
+
+            pFolderName->u16Size   = cbString;
+            pFolderName->u16Length = cbString - sizeof(RTUCS2);
+
+            parms[0].type = VBOX_HGCM_SVC_PARM_PTR;
+            parms[0].u.pointer.addr = pFolderName;
+            parms[0].u.pointer.size = sizeof (SHFLSTRING) + cbString;
+
+            cbString = (RTStrUcs2Len (aMapName) + 1) * sizeof (RTUCS2);
+            pMapName = (SHFLSTRING *) RTMemAllocZ (sizeof(SHFLSTRING) + cbString);
+            Assert (pMapName);
+            memcpy (pMapName->String.ucs2, aMapName, cbString);
+
+            pMapName->u16Size   = cbString;
+            pMapName->u16Length = cbString - sizeof (RTUCS2);
+
+            parms[1].type = VBOX_HGCM_SVC_PARM_PTR;
+            parms[1].u.pointer.addr = pMapName;
+            parms[1].u.pointer.size = sizeof (SHFLSTRING) + cbString;
+
+            parms[2].type = VBOX_HGCM_SVC_PARM_32BIT;
+            parms[2].u.uint32 = !g_fShareReadOnly[i];
+
+            rc = gVMMDev->hgcmHostCall ("VBoxSharedFolders",
+                                        SHFL_FN_ADD_MAPPING, SHFL_CPARMS_ADD_MAPPING, &parms[0]);
+            AssertRC(rc);
+            LogRel(("Added share %s: (%s)\n", g_pszShareName[i], g_pszShareDir[i]));
+            RTMemFree (pFolderName);
+            RTMemFree (pMapName);
+            RTStrUcs2Free (aHostPath);
+            RTStrUcs2Free (aMapName);
+        }
+    }
+#endif
+
 #ifdef VBOXBFE_WITH_USB
     /*
      * Capture USB devices.
@@ -1058,15 +1214,43 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
     if (VBOX_SUCCESS(rc))
     {
         PVMREQ pReq;
-        rc = VMR3ReqCall(pVM, &pReq, RT_INDEFINITE_WAIT, (PFNRT)VMR3PowerOn, 1, pVM);
-        if (VBOX_SUCCESS(rc))
+
+        if (   g_fRestoreState
+            && g_pszStateFile
+            && *g_pszStateFile
+            && RTPathExists(g_pszStateFile))
         {
-            rc = pReq->iStatus;
-            AssertRC(rc);
-            VMR3ReqFree(pReq);
+            startProgressInfo("Restoring");
+            rc = VMR3ReqCall(pVM, &pReq, RT_INDEFINITE_WAIT,
+                             (PFNRT)VMR3Load, 4, pVM, g_pszStateFile, &callProgressInfo, NULL);
+            endProgressInfo();
+            if (VBOX_SUCCESS(rc))
+            {
+                VMR3ReqFree(pReq);
+                rc = VMR3ReqCall(pVM, &pReq, RT_INDEFINITE_WAIT,
+                                 (PFNRT)VMR3Resume, 1, pVM);
+                if (VBOX_SUCCESS(rc))
+                {
+                    rc = pReq->iStatus;
+                    VMR3ReqFree(pReq);
+                }
+                gDisplay->setRunning();
+            }
+            else
+                AssertMsgFailed(("VMR3Load failed, rc=%Vrc\n", rc));
         }
         else
-            AssertMsgFailed(("VMR3PowerOn failed, rc=%Vrc\n", rc));
+        {
+            rc = VMR3ReqCall(pVM, &pReq, RT_INDEFINITE_WAIT, (PFNRT)VMR3PowerOn, 1, pVM);
+            if (VBOX_SUCCESS(rc))
+            {
+                rc = pReq->iStatus;
+                AssertRC(rc);
+                VMR3ReqFree(pReq);
+            }
+            else
+                AssertMsgFailed(("VMR3PowerOn failed, rc=%Vrc\n", rc));
+        }
     }
 
     /*
@@ -1199,13 +1383,16 @@ static DECLCALLBACK(int) vboxbfeConfigConstructor(PVM pVM, void *pvUser)
     rc = CFGMR3InsertInteger(pInst, "Trusted",              1);     /* boolean */   UPDATE_RC();
     rc = CFGMR3InsertNode(pInst,    "Config",         &pCfg);                       UPDATE_RC();
     rc = CFGMR3InsertInteger(pCfg,  "RamSize",        g_u32MemorySizeMB * _1M);     UPDATE_RC();
-    rc = CFGMR3InsertString(pCfg,   "BootDevice0",    pszBootDevice);               UPDATE_RC();
+    rc = CFGMR3InsertString(pCfg,   "BootDevice0",    g_pszBootDevice);             UPDATE_RC();
     rc = CFGMR3InsertString(pCfg,   "BootDevice1",    "NONE");                      UPDATE_RC();
     rc = CFGMR3InsertString(pCfg,   "BootDevice2",    "NONE");                      UPDATE_RC();
     rc = CFGMR3InsertString(pCfg,   "BootDevice3",    "NONE");                      UPDATE_RC();
     rc = CFGMR3InsertString(pCfg,   "HardDiskDevice", "piix3ide");                  UPDATE_RC();
     rc = CFGMR3InsertString(pCfg,   "FloppyDevice",   "i82078");                    UPDATE_RC();
     rc = CFGMR3InsertInteger(pCfg,  "IOAPIC",         g_fIOAPIC);                   UPDATE_RC();
+    RTUUID Uuid;
+    RTUuidClear(&Uuid);
+    rc = CFGMR3InsertBytes(pCfg,    "UUID", &Uuid, sizeof(Uuid));                   UPDATE_RC();
 
     /* Default: no bios logo. */
     rc = CFGMR3InsertInteger(pCfg,  "FadeIn",         1);                           UPDATE_RC();
@@ -1390,7 +1577,7 @@ static DECLCALLBACK(int) vboxbfeConfigConstructor(PVM pVM, void *pvUser)
     rc = CFGMR3InsertInteger(pInst, "PCIDeviceNo",    2);                           UPDATE_RC();
     rc = CFGMR3InsertInteger(pInst, "PCIFunctionNo",  0);                           UPDATE_RC();
     rc = CFGMR3InsertNode(pInst,    "Config",         &pCfg);                       UPDATE_RC();
-    rc = CFGMR3InsertInteger(pCfg,  "VRamSize",       g_u32VRamSizeMB * _1M);       UPDATE_RC();
+    rc = CFGMR3InsertInteger(pCfg,  "VRamSize",       g_u32VRamSize);               UPDATE_RC();
 
 #ifdef RT_OS_L4
     /* XXX hard-coded */
