@@ -40,6 +40,7 @@
 
 class QAction;
 class QLabel;
+class QToolButton;
 
 // Auxiliary types
 ////////////////////////////////////////////////////////////////////////////////
@@ -130,11 +131,23 @@ public:
     const QUuid snapshotId;
 };
 
+class VBoxCanShowRegDlgEvent : public QEvent
+{
+public:
+    VBoxCanShowRegDlgEvent (bool aCanShow)
+        : QEvent ((QEvent::Type) VBoxDefs::CanShowRegDlgEventType)
+        , mCanShow (aCanShow)
+        {}
+
+    const bool mCanShow;
+};
+
 // VBoxGlobal
 ////////////////////////////////////////////////////////////////////////////////
 
 class VBoxSelectorWnd;
 class VBoxConsoleWnd;
+class VBoxRegistrationDlg;
 
 class VBoxGlobal : public QObject
 {
@@ -144,11 +157,11 @@ public:
 
     static VBoxGlobal &instance();
 
-    bool isValid() { return valid; }
+    bool isValid() { return mValid; }
 
     QString versionString() { return verString; }
 
-    CVirtualBox virtualBox() const { return vbox; }
+    CVirtualBox virtualBox() const { return mVBox; }
 
     const VBoxGlobalSettings &settings() const { return gset; }
     bool setSettings (const VBoxGlobalSettings &gs);
@@ -251,6 +264,20 @@ public:
         return CEnums::ClipboardMode (it - clipboardTypes.begin());
     }
 
+    QString toString (CEnums::IDEControllerType t) const
+    {
+        AssertMsg (!ideControllerTypes [t].isNull(), ("No text for %d", t));
+        return ideControllerTypes [t];
+    }
+
+    CEnums::IDEControllerType toIDEControllerType (const QString &s) const
+    {
+        QStringVector::const_iterator it =
+            qFind (ideControllerTypes.begin(), ideControllerTypes.end(), s);
+        AssertMsg (it != ideControllerTypes.end(), ("No value for {%s}", s.latin1()));
+        return CEnums::IDEControllerType (it - ideControllerTypes.begin());
+    }
+
     CEnums::VRDPAuthType toVRDPAuthType (const QString &s) const
     {
         QStringVector::const_iterator it =
@@ -321,6 +348,20 @@ public:
         return CEnums::AudioDriverType (it - audioDriverTypes.begin());
     }
 
+    QString toString (CEnums::NetworkAdapterType t) const
+    {
+        AssertMsg (!networkAdapterTypes [t].isNull(), ("No text for %d", t));
+        return networkAdapterTypes [t];
+    }
+
+    CEnums::NetworkAdapterType toNetworkAdapterType (const QString &s) const
+    {
+        QStringVector::const_iterator it =
+            qFind (networkAdapterTypes.begin(), networkAdapterTypes.end(), s);
+        AssertMsg (it != networkAdapterTypes.end(), ("No value for {%s}", s.latin1()));
+        return CEnums::NetworkAdapterType (it - networkAdapterTypes.begin());
+    }
+
     QString toString (CEnums::NetworkAttachmentType t) const
     {
         AssertMsg (!networkAttachmentTypes [t].isNull(), ("No text for %d", t));
@@ -363,7 +404,11 @@ public:
 
     /* VirtualBox helpers */
 
-    CSession openSession (const QUuid &id);
+#ifdef Q_WS_X11
+    bool showVirtualBoxLicense();
+#endif
+
+    CSession openSession (const QUuid &id, bool aExisting = false);
 
     bool startMachine (const QUuid &id);
 
@@ -388,8 +433,6 @@ public:
 
     /* various helpers */
 
-    bool openURL (const QString &aURL);
-
     QString languageName() const;
     QString languageCountry() const;
     QString languageNameEnglish() const;
@@ -417,6 +460,8 @@ public:
                                const char *aDisabled = 0, const char *aSmallDisabled = 0,
                                const char *aActive = 0, const char *aSmallActive = 0);
 
+    static void setTextLabel (QToolButton *aToolButton, const QString &aTextLabel);
+
     static QRect normalizeGeometry (const QRect &aRect, const QRect &aBoundRect,
                                     bool aCanResize = true);
 
@@ -494,6 +539,14 @@ signals:
     void sessionStateChanged (const VBoxSessionStateChangeEvent &e);
     void snapshotChanged (const VBoxSnapshotEvent &e);
 
+    void canShowRegDlg (bool aCanShow);
+
+public slots:
+
+    bool openURL (const QString &aURL);
+
+    void showRegistrationDialog (bool aForce = true);
+
 protected:
 
     bool event (QEvent *e);
@@ -506,14 +559,18 @@ private:
 
     void init();
 
-    bool valid;
+    bool mValid;
 
-    CVirtualBox vbox;
+    CVirtualBox mVBox;
 
     VBoxGlobalSettings gset;
 
-    VBoxSelectorWnd *selector_wnd;
-    VBoxConsoleWnd *console_wnd;
+    VBoxSelectorWnd *mSelectorWnd;
+    VBoxConsoleWnd *mConsoleWnd;
+
+#ifdef VBOX_WITH_REGISTRATION
+    VBoxRegistrationDlg *mRegDlg;
+#endif
 
     QUuid vmUuid;
 
@@ -556,8 +613,10 @@ private:
     QStringVector usbFilterActionTypes;
     QStringVector diskControllerDevices;
     QStringVector audioDriverTypes;
+    QStringVector networkAdapterTypes;
     QStringVector networkAttachmentTypes;
     QStringVector clipboardTypes;
+    QStringVector ideControllerTypes;
     QStringVector USBDeviceStates;
 
     QString mUserDefinedCOMPortName;
