@@ -66,6 +66,8 @@
 #include <cstdio>
 #include <cstring>
 #include <cstdarg>
+#include <cctype>
+#include <locale>
 #include <assert.h>
 
 #include <string>
@@ -92,6 +94,19 @@ const char* GetWinsockErrorString( int err );
 //---------------------------------------------------------------------
 
 
+/*
+ * See: http://gcc.gnu.org/onlinedocs/libstdc++/22_locale/howto.html#7
+ */
+struct ToLower
+{
+    ToLower(std::locale const& l) : loc(l) {}
+    char operator() (char c) const
+    {
+        return std::tolower(c,loc);
+    }
+  private:
+    std::locale const& loc;
+};
 
 void BailOnSocketError( const char* context )
 {
@@ -383,9 +398,7 @@ void Connection::putrequest( const char* method, const char* url )
 
     m_State = REQ_STARTED;
 
-    char req[ 512 ];
-    sprintf( req, "%s %s HTTP/1.1", method, url );
-    m_Buffer.push_back( req );
+    m_Buffer.push_back (string (method) + " " + string (url) + " HTTP/1.1");
 
     putheader( "Host", m_Host.c_str() );    // required for HTTP1.1
 
@@ -537,6 +550,8 @@ Response::Response( const char* method, Connection& conn ) :
 
 const char* Response::getheader( const char* name ) const
 {
+    ToLower tolower(std::locale::classic());
+
     std::string lname( name );
     std::transform( lname.begin(), lname.end(), lname.begin(), tolower );
 
