@@ -89,6 +89,7 @@ QIRichLabel::~QIRichLabel()
 void QIRichLabel::init()
 {
    mMaxHeightMode = false;
+   mIsMainTip = true;
    baseheight = 0;
    lpixmap = 0;
    lmovie = 0;
@@ -296,13 +297,12 @@ QSize QIRichLabel::sizeForWidth (int w) const
    else if ( mov )
       br = mov->framePixmap().rect();
    else if (doc) {
-      int focusIndent = hasFocus() ? 3 : 0;
       int oldW = doc->width();
       if ( align & WordBreak ) {
          if (w < 0)
             doc->adjustSize();
          else
-            doc->setWidth (w-hextra - 2*focusIndent);
+            doc->setWidth (w-hextra - 2*3);
       }
       br = QRect (0, 0, doc->widthUsed(), doc->height());
       doc->setWidth (oldW);
@@ -381,9 +381,26 @@ void QIRichLabel::mouseMoveEvent (QMouseEvent *aEvent)
 
     QString link = doc->anchorAt (aEvent->pos());
     if (!link.isEmpty()) /* Mouse cursor above link */
+    {
+        if (mIsMainTip)
+        {
+            mTipText = QToolTip::textFor (this);
+            QToolTip::remove (this);
+            QToolTip::add (this, link);
+            mIsMainTip = false;
+        }
         setCursor (QCursor (Qt::PointingHandCursor));
+    }
     else /* Mouse cursor above non-link */
+    {
+        if (!mIsMainTip)
+        {
+            QToolTip::remove (this);
+            QToolTip::add (this, mTipText);
+            mIsMainTip = true;
+        }
         setCursor (QCursor (Qt::ArrowCursor));
+    }
 }
 
 
@@ -594,8 +611,7 @@ void QIRichLabel::drawContents (QPainter *p)
          QString filteredText = compressText();
          doc = new QSimpleRichText (filteredText, font());
          /* focus indent */
-         int focusIndent = hasFocus() ? 3 : 0;
-         doc->setWidth (p, cr.width() - 2*focusIndent);
+         doc->setWidth (p, cr.width() - 2*3);
          int rh = doc->height();
          int yo = 0;
          if (align & AlignVCenter)
@@ -629,7 +645,7 @@ void QIRichLabel::drawContents (QPainter *p)
                cg.setColor (QColorGroup::Text, paletteForegroundColor());
          }
 
-         doc->draw (p, cr.x()+focusIndent, cr.y()+yo, cr, cg, &paper);
+         doc->draw (p, cr.x()+3, cr.y()+yo, cr, cg, &paper);
          if (hasFocus())
             style().drawPrimitive (QStyle::PE_FocusRect, p, cr, cg,
             QStyle::Style_FocusAtBorder,
