@@ -24,6 +24,8 @@
 
 #include "VBoxAboutDlg.h"
 
+#include "QIHotKeyEdit.h"
+
 #include <qmessagebox.h>
 #include <qprogressdialog.h>
 #include <qcursor.h>
@@ -232,7 +234,7 @@ bool VBoxProblemReporter::isValid()
  *
  *  When all button arguments are zero, a single 'Ok' button is shown.
  *
- *  If autoConfirmId is not null, then the message box will contain a
+ *  If aAutoConfirmId is not null, then the message box will contain a
  *  checkbox "Don't show this message again" below the message text and its
  *  state will be saved in the global config. When the checkbox is in the
  *  checked state, this method doesn't actually show the message box but
@@ -241,56 +243,65 @@ bool VBoxProblemReporter::isValid()
  *  AutoConfirmed bit set (AutoConfirmed alone is returned when no default
  *  button is defined in button arguments).
  *
- *  @param  parent
- *      parent widget or 0 to use the desktop as the parent. Also,
+ *  @param  aParent
+ *      Parent widget or 0 to use the desktop as the parent. Also,
  *      #mainWindowShown can be used to determine the currently shown VBox
- *      main window (Selector or Console)
- *  @param  type
- *      one of values of the Type enum, that defines the message box
- *      title and icon
- *  @param  msg
- *      message text to display (can contain sinmple Qt-html tags)
- *  @param  details
- *      detailed message description displayed under the main message text using
+ *      main window (Selector or Console).
+ *  @param  aType
+ *      One of values of the Type enum, that defines the message box
+ *      title and icon.
+ *  @param  aMessage
+ *      Message text to display (can contain sinmple Qt-html tags).
+ *  @param  aDetails
+ *      Detailed message description displayed under the main message text using
  *      QTextEdit (that provides rich text support and scrollbars when necessary).
- *      If equals to QString::null, no details text box is shown
- *  @param  autoConfirmId
+ *      If equals to QString::null, no details text box is shown.
+ *  @param  aAutoConfirmId
  *      ID used to save the auto confirmation state across calls. If null,
  *      the auto confirmation feature is turned off (and no checkbox is shown)
- *  @param  b1
- *      first button code or 0, see QIMessageBox
- *  @param  b2
- *      second button code or 0, see QIMessageBox
- *  @param  b3
- *      third button code or 0, see QIMessageBox
- *  @param  name
- *      name of the underlying QIMessageBox object. If NULL, a value of
- *      autoConfirmId is used.
+ *  @param  aButton1
+ *      First button code or 0, see QIMessageBox for a list of codes.
+ *  @param  aButton2
+ *      Second button code or 0, see QIMessageBox for a list of codes.
+ *  @param  aButton3
+ *      Third button code or 0, see QIMessageBox for a list of codes.
+ *  @param  aText1
+ *      Optional custom text for the first button.
+ *  @param  aText2
+ *      Optional custom text for the second button.
+ *  @param  aText3
+ *      Optional custom text for the third button.
  *
  *  @return
  *      code of the button pressed by the user
  */
-int VBoxProblemReporter::message (QWidget *parent, Type type, const QString &msg,
-                                  const QString &details,
-                                  const char *autoConfirmId,
-                                  int b1, int b2, int b3,
-                                  const char *name)
+int VBoxProblemReporter::message (QWidget *aParent, Type aType, const QString &aMessage,
+                                  const QString &aDetails /* = QString::null */,
+                                  const char *aAutoConfirmId /* = 0 */,
+                                  int aButton1 /* = 0 */, int aButton2 /* = 0 */,
+                                  int aButton3 /* = 0 */,
+                                  const QString &aText1 /* = QString::null */,
+                                  const QString &aText2 /* = QString::null */,
+                                  const QString &aText3 /* = QString::null */)
 {
-    if (b1 == 0 && b2 == 0 && b3 == 0)
-        b1 = QIMessageBox::Ok | QIMessageBox::Default;
+    if (aButton1 == 0 && aButton2 == 0 && aButton3 == 0)
+        aButton1 = QIMessageBox::Ok | QIMessageBox::Default;
 
     CVirtualBox vbox;
     QStringList msgs;
 
-    if (autoConfirmId)
+    if (aAutoConfirmId)
     {
         vbox = vboxGlobal().virtualBox();
         msgs = QStringList::split (',', vbox.GetExtraData (VBoxDefs::GUI_SuppressMessages));
-        if (msgs.findIndex (autoConfirmId) >= 0) {
+        if (msgs.findIndex (aAutoConfirmId) >= 0) {
             int rc = AutoConfirmed;
-            if (b1 & QIMessageBox::Default) rc |= (b1 & QIMessageBox::ButtonMask);
-            if (b2 & QIMessageBox::Default) rc |= (b2 & QIMessageBox::ButtonMask);
-            if (b3 & QIMessageBox::Default) rc |= (b3 & QIMessageBox::ButtonMask);
+            if (aButton1 & QIMessageBox::Default)
+                rc |= (aButton1 & QIMessageBox::ButtonMask);
+            if (aButton2 & QIMessageBox::Default)
+                rc |= (aButton2 & QIMessageBox::ButtonMask);
+            if (aButton3 & QIMessageBox::Default)
+                rc |= (aButton3 & QIMessageBox::ButtonMask);
             return rc;
         }
     }
@@ -298,7 +309,7 @@ int VBoxProblemReporter::message (QWidget *parent, Type type, const QString &msg
     QString title;
     QIMessageBox::Icon icon;
 
-    switch (type)
+    switch (aType)
     {
         default:
         case Info:
@@ -323,21 +334,27 @@ int VBoxProblemReporter::message (QWidget *parent, Type type, const QString &msg
             break;
         case GuruMeditation:
             title = "VirtualBox - Guru Meditation"; /* don't translate this */
-            icon = QIMessageBox::Critical;
+            icon = QIMessageBox::GuruMeditation;
             break;
     }
 
-    if (!name)
-        name = autoConfirmId;
-    QIMessageBox *box = new QIMessageBox (title, msg, icon, b1, b2, b3, parent, name);
+    QIMessageBox *box = new QIMessageBox (title, aMessage, icon, aButton1, aButton2,
+                                          aButton3, aParent, aAutoConfirmId);
 
-    if (details)
+    if (!aText1.isNull())
+        box->setButtonText (0, aText1);
+    if (!aText2.isNull())
+        box->setButtonText (1, aText2);
+    if (!aText3.isNull())
+        box->setButtonText (2, aText3);
+
+    if (aDetails)
     {
-        box->setDetailsText (details);
+        box->setDetailsText (aDetails);
         box->setDetailsShown (true);
     }
 
-    if (autoConfirmId)
+    if (aAutoConfirmId)
     {
         box->setFlagText (tr ("Do not show this message again", "msg box flag"));
         box->setFlagChecked (false);
@@ -345,11 +362,11 @@ int VBoxProblemReporter::message (QWidget *parent, Type type, const QString &msg
 
     int rc = box->exec();
 
-    if (autoConfirmId)
+    if (aAutoConfirmId)
     {
         if (box->isFlagChecked())
         {
-            msgs << autoConfirmId;
+            msgs << aAutoConfirmId;
             vbox.SetExtraData (VBoxDefs::GUI_SuppressMessages, msgs.join (","));
         }
     }
@@ -359,30 +376,20 @@ int VBoxProblemReporter::message (QWidget *parent, Type type, const QString &msg
     return rc;
 }
 
-/** @fn message (QWidget *, Type, const QString &, const char *, int, int, int, const char *)
+/** @fn message (QWidget *, Type, const QString &, const char *, int, int,
+ *               int, const QString &, const QString &, const QString &)
  *
  *  A shortcut to #message() that doesn't require to specify the details
  *  text (QString::null is assumed).
  */
 
-/**
+/** @fn messageYesNo (QWidget *, Type, const QString &, const QString &, const char *)
+ *
  *  A shortcut to #message() that shows 'Yes' and 'No' buttons ('Yes' is the
  *  default) and returns true when the user selects Yes.
  */
-bool VBoxProblemReporter::messageYesNo (
-    QWidget *parent, Type type, const QString &msg,
-    const QString &details,
-    const char *autoConfirmId,
-    const char *name
-)
-{
-    return (message (parent, type, msg, details, autoConfirmId,
-                     QIMessageBox::Yes | QIMessageBox::Default,
-                     QIMessageBox::No | QIMessageBox::Escape,
-                     0, name) & QIMessageBox::ButtonMask) == QIMessageBox::Yes;
-}
 
-/** @fn messageYesNo (QWidget *, Type, const QString &, const char *, const char *)
+/** @fn messageYesNo (QWidget *, Type, const QString &, const char *)
  *
  *  A shortcut to #messageYesNo() that doesn't require to specify the details
  *  text (QString::null is assumed).
@@ -449,6 +456,27 @@ QWidget *VBoxProblemReporter::mainWindowShown()
 // Problem handlers
 /////////////////////////////////////////////////////////////////////////////
 
+#ifdef Q_WS_X11
+void VBoxProblemReporter::cannotFindLicenseFiles (const QString &aPath)
+{
+    message
+        (0, VBoxProblemReporter::Error,
+         tr ("Failed to find license files in "
+             "<nobr><b>%1</b></nobr>.")
+             .arg (aPath));
+}
+
+void VBoxProblemReporter::cannotOpenLicenseFile (QWidget *aParent,
+                                                 const QString &aPath)
+{
+    message
+        (aParent, VBoxProblemReporter::Error,
+         tr ("Failed to open the license file <nobr><b>%1</b></nobr>. "
+             "Check file permissions.")
+             .arg (aPath));
+}
+#endif
+
 void VBoxProblemReporter::cannotOpenURL (const QString &aURL)
 {
     message
@@ -670,6 +698,18 @@ void VBoxProblemReporter::cannotResumeMachine (const CConsole &console)
         formatErrorInfo (res));
 }
 
+void VBoxProblemReporter::cannotACPIShutdownMachine (const CConsole &console)
+{
+    /* preserve the current error info before calling the object again */
+    COMResult res (console);
+
+    message (mainWindowShown(),  Error,
+        tr ("Failed to send the ACPI Power Button press event to the "
+            "virtual machine <b>%1</b>.")
+            .arg (console.GetMachine().GetName()),
+        formatErrorInfo (res));
+}
+
 void VBoxProblemReporter::cannotSaveMachineState (const CConsole &console)
 {
     /* preserve the current error info before calling the object again */
@@ -873,6 +913,7 @@ void VBoxProblemReporter::cannotEnterSeamlessMode (ULONG aWidth,
 bool VBoxProblemReporter::confirmMachineDeletion (const CMachine &machine)
 {
     QString msg;
+    QString button;
     QString name;
 
     if (machine.GetAccessible())
@@ -882,6 +923,7 @@ bool VBoxProblemReporter::confirmMachineDeletion (const CMachine &machine)
                   "the virtual machine <b>%1</b>?</p>"
                   "<p>This operation cannot be undone.</p>")
                   .arg (name);
+        button = tr ("Delete", "machine");
     }
     else
     {
@@ -894,53 +936,47 @@ bool VBoxProblemReporter::confirmMachineDeletion (const CMachine &machine)
                   "<p>You will no longer be able to register it back from "
                   "GUI.</p>")
                   .arg (name);
+        button = tr ("Unregister", "machine");
     }
 
-    return messageYesNo (&vboxGlobal().selectorWnd(), Question, msg);
+    return messageOkCancel (&vboxGlobal().selectorWnd(), Question, msg,
+                            0 /* aAutoConfirmId */, button);
 }
 
 bool VBoxProblemReporter::confirmDiscardSavedState (const CMachine &machine)
 {
-    return messageYesNo (
-        &vboxGlobal().selectorWnd(),
-        Question,
+    return messageOkCancel (&vboxGlobal().selectorWnd(), Question,
         tr ("<p>Are you sure you want to discard the saved state of "
             "the virtual machine <b>%1</b>?</p>"
             "<p>This operation is equivalent to resetting or powering off "
             "the machine without doing a proper shutdown by means of the "
             "guest OS.</p>")
-            .arg (machine.GetName())
-    );
+            .arg (machine.GetName()),
+        0 /* aAutoConfirmId */,
+        tr ("Discard", "saved state"));
 }
 
 bool VBoxProblemReporter::confirmReleaseImage (QWidget *parent,
                                                const QString &usage)
 {
-    return messageYesNo (
-        parent,
-        Question,
+    return messageOkCancel (parent, Question,
         tr ("<p>Releasing this media image will detach it from the "
             "following virtual machine(s): <b>%1</b>.</p>"
             "<p>Continue?</p>")
             .arg (usage),
-        "confirmReleaseImage"
-    );
+        "confirmReleaseImage",
+        tr ("Continue", "detach image"));
 }
 
 void VBoxProblemReporter::sayCannotOverwriteHardDiskImage (QWidget *parent,
                                                            const QString &src)
 {
-    message (
-        parent,
-        Info,
-        tr (
-            "<p>The image file <b>%1</b> already exists. "
+    message (parent, Info,
+        tr ("<p>The image file <b>%1</b> already exists. "
             "You cannot create a new virtual hard disk that uses this file, "
             "because it can be already used by another virtual hard disk.</p>"
-            "<p>Please specify a different image file name.</p>"
-        )
-            .arg (src)
-    );
+            "<p>Please specify a different image file name.</p>")
+            .arg (src));
 }
 
 int VBoxProblemReporter::confirmHardDiskImageDeletion (QWidget *parent,
@@ -949,17 +985,19 @@ int VBoxProblemReporter::confirmHardDiskImageDeletion (QWidget *parent,
     return message (parent, Question,
         tr ("<p>Do you want to delete this hard disk's image file "
             "<nobr><b>%1</b>?</nobr></p>"
-            "<p>If you select <b>No</b> then the virtual hard disk will be "
-            "unregistered and removed from the collection, but the image file "
-            "will be left on your physical disk.</p>"
-            "<p>If you select <b>Yes</b> then the image file will be permanently "
+            "<p>If you select <b>Delete</b> then the image file will be permanently "
             "deleted after unregistering the hard disk. This operation "
-            "cannot be undone.</p>")
+            "cannot be undone.</p>"
+            "<p>If you select <b>Unregister</b> then the virtual hard disk will be "
+            "unregistered and removed from the collection, but the image file "
+            "will be left on your physical disk.</p>")
             .arg (src),
-        0, /* autoConfirmId */
+        0, /* aAutoConfirmId */
         QIMessageBox::Yes,
         QIMessageBox::No | QIMessageBox::Default,
-        QIMessageBox::Cancel | QIMessageBox::Escape);
+        QIMessageBox::Cancel | QIMessageBox::Escape,
+        tr ("Delete", "hard disk"),
+        tr ("Unregister", "hard disk"));
 }
 
 void VBoxProblemReporter::cannotDeleteHardDiskImage (QWidget *parent,
@@ -974,24 +1012,22 @@ void VBoxProblemReporter::cannotDeleteHardDiskImage (QWidget *parent,
         formatErrorInfo (vdi));
 }
 
-int VBoxProblemReporter::confirmHardDiskUnregister (QWidget *parent,
+bool VBoxProblemReporter::confirmHardDiskUnregister (QWidget *parent,
                                                     const QString &src)
 {
-    return message (parent, Question,
+    return messageOkCancel (parent, Question,
         tr ("<p>Do you want to remove (unregister) the virtual hard disk "
             "<nobr><b>%1</b>?</nobr></p>")
             .arg (src),
-        0, /* autoConfirmId */
-        QIMessageBox::Ok | QIMessageBox::Default,
-        QIMessageBox::Cancel | QIMessageBox::Escape);
+        0 /* aAutoConfirmId */,
+        tr ("Unregister", "hard disk"));
 }
 
-
 void VBoxProblemReporter::cannotCreateHardDiskImage (
     QWidget *parent, const CVirtualBox &vbox, const QString &src,
-    const CVirtualDiskImage &vdi, const CProgress &progress
-) {
-    message (parent,Error,
+    const CVirtualDiskImage &vdi, const CProgress &progress)
+{
+    message (parent, Error,
         tr ("Failed to create the virtual hard disk image <nobr><b>%1</b>.</nobr>")
             .arg (src),
         !vbox.isOk() ? formatErrorInfo (vbox) :
@@ -1290,7 +1326,7 @@ int VBoxProblemReporter::cannotFindGuestAdditions (const QString &aSrc1,
                         "<nobr><b>%2</b>.</nobr></p><p>Do you want to "
                         "download this CD image from the Internet?</p>")
                         .arg (aSrc1).arg (aSrc2),
-                    0, /* autoConfirmId */
+                    0, /* aAutoConfirmId */
                     QIMessageBox::Yes | QIMessageBox::Default,
                     QIMessageBox::No | QIMessageBox::Escape);
 }
@@ -1304,33 +1340,31 @@ void VBoxProblemReporter::cannotDownloadGuestAdditions (const QString &aURL,
                  .arg (aURL).arg (aURL).arg (aReason));
 }
 
-int VBoxProblemReporter::confirmDownloadAdditions (const QString &aURL,
+bool VBoxProblemReporter::confirmDownloadAdditions (const QString &aURL,
                                                    ulong aSize)
 {
-    return message (&vboxGlobal().consoleWnd(), Question,
-                    tr ("<p>Are you sure you want to download the VirtualBox "
-                        "Guest Additions CD image from "
-                        "<nobr><a href=\"%1\">%2</a></nobr> "
-                        "(size %3 bytes)?</p>").arg (aURL).arg (aURL).arg (aSize),
-                    0, /* autoConfirmId */
-                    QIMessageBox::Yes | QIMessageBox::Default,
-                    QIMessageBox::No | QIMessageBox::Escape);
+    return messageOkCancel (&vboxGlobal().consoleWnd(), Question,
+        tr ("<p>Are you sure you want to download the VirtualBox "
+            "Guest Additions CD image from "
+            "<nobr><a href=\"%1\">%2</a></nobr> "
+            "(size %3 bytes)?</p>").arg (aURL).arg (aURL).arg (aSize),
+        0, /* aAutoConfirmId */
+        tr ("Download", "additions"));
 }
 
-int VBoxProblemReporter::confirmMountAdditions (const QString &aURL,
+bool VBoxProblemReporter::confirmMountAdditions (const QString &aURL,
                                                 const QString &aSrc)
 {
-    return message (&vboxGlobal().consoleWnd(), Question,
-                    tr ("<p>The VirtualBox Guest Additions CD image has been "
-                        "successfully downloaded from "
-                        "<nobr><a href=\"%1\">%2</a></nobr> "
-                        "and saved locally as <nobr><b>%3</b>.</nobr></p>"
-                        "<p>Do you want to register this CD image and mount it "
-                        "on the virtual CD/DVD drive?</p>")
-                        .arg (aURL).arg (aURL).arg (aSrc),
-                    0, /* autoConfirmId */
-                    QIMessageBox::Yes | QIMessageBox::Default,
-                    QIMessageBox::No | QIMessageBox::Escape);
+    return messageOkCancel (&vboxGlobal().consoleWnd(), Question,
+        tr ("<p>The VirtualBox Guest Additions CD image has been "
+            "successfully downloaded from "
+            "<nobr><a href=\"%1\">%2</a></nobr> "
+            "and saved locally as <nobr><b>%3</b>.</nobr></p>"
+            "<p>Do you want to register this CD image and mount it "
+            "on the virtual CD/DVD drive?</p>")
+            .arg (aURL).arg (aURL).arg (aSrc),
+        0, /* aAutoConfirmId */
+        tr ("Mount", "additions"));
 }
 
 void VBoxProblemReporter::warnAboutTooOldAdditions (QWidget *aParent,
@@ -1382,8 +1416,40 @@ void VBoxProblemReporter::warnAboutNewAdditions (QWidget *aParent,
         "warnAboutNewAdditions");
 }
 
-/** @return false if the dialog wasn't actually shown (i.e. it was autoconfirmed) */
-bool VBoxProblemReporter::remindAboutInputCapture()
+void VBoxProblemReporter::cannotConnectRegister (QWidget *aParent,
+                                                 const QString &aURL,
+                                                 const QString &aReason)
+{
+    /* we don't want to expose the registration script URL to the user
+     * if he simply doesn't have an internet connection */
+    Q_UNUSED (aURL);
+
+    message (aParent, Error,
+             tr ("<p>Failed to connect to the VirtualBox online "
+                 "registration service.</p><p>%1</p>")
+                 .arg (aReason));
+}
+
+void VBoxProblemReporter::showRegisterResult (QWidget *aParent,
+                                              const QString &aResult)
+{
+    aResult == "OK" ?
+        message (aParent, Info,
+                 tr ("<p>Congratulations! You have been successfully registered "
+                     "as a user of VirtualBox.</p>"
+                     "<p>Thank you for finding time to fill out the "
+                     "registration form!</p>")) :
+        message (aParent, Error,
+                 tr ("<p>Failed to register the VirtualBox product</p><p>%1</p>")
+                 .arg (aResult));
+}
+
+/**
+ *  @return @c true if the user has confirmed input capture (this is always
+ *  the case if the dialog was autoconfirmed). @a aAutoConfirmed, when not
+ *  NULL, will receive @c true if the dialog wasn't actually shown.
+ */
+bool VBoxProblemReporter::confirmInputCapture (bool *aAutoConfirmed /* = NULL */)
 {
     int rc = message (&vboxGlobal().consoleWnd(), Info,
         tr ("<p>You have <b>clicked the mouse</b> inside the Virtual Machine display "
@@ -1400,16 +1466,25 @@ bool VBoxProblemReporter::remindAboutInputCapture()
             "<img src=hostkey_16px.png/>&nbsp;icon. This icon, together "
             "with the mouse icon placed nearby, indicate the current keyboard "
             "and mouse capture state."
-            "</p>"),
-        "remindAboutInputCapture");
+            "</p>") +
+        tr ("<p>The host key is currently defined as <b>%1</b>.</p>",
+            "additional message box paragraph")
+            .arg (QIHotKeyEdit::keyName (vboxGlobal().settings().hostKey())),
+        "confirmInputCapture",
+        QIMessageBox::Ok | QIMessageBox::Default,
+        QIMessageBox::Cancel | QIMessageBox::Escape,
+        0,
+        tr ("Capture", "do input capture"));
 
-    return !(rc & AutoConfirmed);
+    if (aAutoConfirmed)
+        *aAutoConfirmed = (rc & AutoConfirmed);
+
+    return (rc & QIMessageBox::ButtonMask) == QIMessageBox::Ok;
 }
 
-/** @return false if the dialog wasn't actually shown (i.e. it was autoconfirmed) */
-bool VBoxProblemReporter::remindAboutAutoCapture()
+void VBoxProblemReporter::remindAboutAutoCapture()
 {
-    int rc = message ( &vboxGlobal().consoleWnd(), Info,
+    message (&vboxGlobal().consoleWnd(), Info,
         tr ("<p>You have the <b>Auto capture keyboard</b> option turned on. "
             "This will cause the Virtual Machine to automatically <b>capture</b> "
             "the keyboard every time the VM window is activated and make it "
@@ -1424,14 +1499,14 @@ bool VBoxProblemReporter::remindAboutAutoCapture()
             "<img src=hostkey_16px.png/>&nbsp;icon. This icon, together "
             "with the mouse icon placed nearby, indicate the current keyboard "
             "and mouse capture state."
-            "</p>"),
+            "</p>") +
+        tr ("<p>The host key is currently defined as <b>%1</b>.</p>",
+            "additional message box paragraph")
+            .arg (QIHotKeyEdit::keyName (vboxGlobal().settings().hostKey())),
         "remindAboutAutoCapture");
-
-    return !(rc & AutoConfirmed);
 }
 
-/** @return false if the dialog wasn't actually shown (i.e. it was autoconfirmed) */
-bool VBoxProblemReporter::remindAboutMouseIntegration (bool supportsAbsolute)
+void VBoxProblemReporter::remindAboutMouseIntegration (bool aSupportsAbsolute)
 {
     static const char *kNames [2] =
     {
@@ -1440,18 +1515,18 @@ bool VBoxProblemReporter::remindAboutMouseIntegration (bool supportsAbsolute)
     };
 
     /* Close the previous (outdated) window if any. We use kName as
-     * autoConfirmId which is also used as the widget name by default. */
+     * aAutoConfirmId which is also used as the widget name by default. */
     {
         QWidget *outdated =
-            VBoxGlobal::findWidget (NULL, kNames [int (!supportsAbsolute)],
+            VBoxGlobal::findWidget (NULL, kNames [int (!aSupportsAbsolute)],
                                     "QIMessageBox");
         if (outdated)
             outdated->close();
     }
 
-    if (supportsAbsolute)
+    if (aSupportsAbsolute)
     {
-        int rc = message (&vboxGlobal().consoleWnd(), Info,
+        message (&vboxGlobal().consoleWnd(), Info,
             tr ("<p>The Virtual Machine reports that the guest OS supports "
                 "<b>mouse pointer integration</b>. This means that you do not "
                 "need to <i>capture</i> the mouse pointer to be able to use it "
@@ -1471,25 +1546,24 @@ bool VBoxProblemReporter::remindAboutMouseIntegration (bool supportsAbsolute)
                 "the current session (and enable it again) by selecting the "
                 "corresponding action from the menu bar."
                 "</p>"),
-            kNames [1] /* autoConfirmId */);
-
-        return !(rc & AutoConfirmed);
+            kNames [1] /* aAutoConfirmId */);
     }
     else
     {
-        int rc = message (&vboxGlobal().consoleWnd(), Info,
+        message (&vboxGlobal().consoleWnd(), Info,
             tr ("<p>The Virtual Machine reports that the guest OS does not "
                 "support <b>mouse pointer integration</b> in the current video "
                 "mode. You need to capture the mouse (by clicking over the VM "
                 "display or pressing the host key) in order to use the "
                 "mouse inside the guest OS.</p>"),
-            kNames [0] /* autoConfirmId */);
-
-        return !(rc & AutoConfirmed);
+            kNames [0] /* aAutoConfirmId */);
     }
 }
 
-/** @return false if the dialog wasn't actually shown (i.e. it was autoconfirmed) */
+/**
+ * @return @c false if the dialog wasn't actually shown (i.e. it was
+ * autoconfirmed).
+ */
 bool VBoxProblemReporter::remindAboutPausedVMInput()
 {
     int rc = message (
@@ -1509,33 +1583,32 @@ bool VBoxProblemReporter::remindAboutPausedVMInput()
 /** @return true if the user has chosen to show the Disk Manager Window */
 bool VBoxProblemReporter::remindAboutInaccessibleMedia()
 {
-    int rc = message (
-        &vboxGlobal().selectorWnd(),
-        Warning,
+    int rc = message (&vboxGlobal().selectorWnd(), Warning,
         tr ("<p>One or more of the registered virtual hard disks, CD/DVD or "
             "floppy media are not currently accessible. As a result, you will "
             "not be able to operate virtual machines that use these media until "
             "they become accessible later.</p>"
-            "<p>Press <b>OK</b> to open the Virtual Disk Manager window and "
+            "<p>Press <b>Check</b> to open the Virtual Disk Manager window and "
             "see what media are inaccessible, or press <b>Ignore</b> to "
             "ignore this message.</p>"),
         "remindAboutInaccessibleMedia",
         QIMessageBox::Ok | QIMessageBox::Default,
-        QIMessageBox::Ignore | QIMessageBox::Escape
-    );
+        QIMessageBox::Ignore | QIMessageBox::Escape,
+        0,
+        tr ("Check", "inaccessible media message box"));
 
-    return rc == QIMessageBox::Ok && !(rc & AutoConfirmed);
+    return rc == QIMessageBox::Ok; /* implies !AutoConfirmed */
 }
 
 /**
- *  @param  fullscreen hot key as defined in the menu
- *  @param  current host key as in the global settings
- *  @return true if the user has chosen to go fullscreen.
+ *  @param aHotKey Fullscreen hot key as defined in the menu.
+ *
+ *  @return @c true if the user has chosen to go fullscreen (this is always
+ *  the case if the dialog was autoconfirmed).
  */
-void VBoxProblemReporter::remindAboutGoingFullscreen (const QString &hotKey,
-                                                      const QString &hostKey)
+bool VBoxProblemReporter::confirmGoingFullscreen (const QString &aHotKey)
 {
-    int rc = message (&vboxGlobal().consoleWnd(), Info,
+    return messageOkCancel (&vboxGlobal().consoleWnd(), Info,
         tr ("<p>The virtual machine window will be now switched to "
             "<b>fullscreen</b> mode. "
             "You can go back to windowed mode at any time by pressing "
@@ -1543,20 +1616,21 @@ void VBoxProblemReporter::remindAboutGoingFullscreen (const QString &hotKey,
             "defined as <b>%1</b>.</p>"
             "<p>Note that the main menu bar is hidden fullscreen mode. You "
             "can access it by pressing <b>Host+Home</b>.</p>")
-            .arg (hotKey).arg (hostKey),
-        "remindAboutGoingFullscreen");
-    NOREF(rc);
+            .arg (aHotKey)
+            .arg (QIHotKeyEdit::keyName (vboxGlobal().settings().hostKey())),
+        "confirmGoingFullscreen",
+        tr ("Switch", "fullscreen"));
 }
 
 /**
- *  @param  fullscreen hot key as defined in the menu
- *  @param  current host key as in the global settings
- *  @return true if the user has chosen to go fullscreen.
+ *  @param aHotKey Seamless hot key as defined in the menu.
+ *
+ *  @return @c true if the user has chosen to go seamless (this is always
+ *  the case if the dialog was autoconfirmed).
  */
-void VBoxProblemReporter::remindAboutGoingSeamless (const QString &hotKey,
-                                                    const QString &hostKey)
+bool VBoxProblemReporter::confirmGoingSeamless (const QString &aHotKey)
 {
-    int rc = message (&vboxGlobal().consoleWnd(), Info,
+    return messageOkCancel (&vboxGlobal().consoleWnd(), Info,
         tr ("<p>The virtual machine window will be now switched to "
             "<b>Seamless</b> mode. "
             "You can go back to windowed mode at any time by pressing "
@@ -1564,9 +1638,10 @@ void VBoxProblemReporter::remindAboutGoingSeamless (const QString &hotKey,
             "defined as <b>%1</b>.</p>"
             "<p>Note that the main menu bar is hidden seamless mode. You "
             "can access it by pressing <b>Host+Home</b>.</p>")
-            .arg (hotKey).arg (hostKey),
-        "remindAboutGoingSeamless");
-    NOREF(rc);
+            .arg (aHotKey)
+            .arg (QIHotKeyEdit::keyName (vboxGlobal().settings().hostKey())),
+        "confirmGoingSeamless",
+        tr ("Switch", "seamless"));
 }
 
 void VBoxProblemReporter::remindAboutWrongColorDepth (ulong aRealBPP,
@@ -1575,7 +1650,7 @@ void VBoxProblemReporter::remindAboutWrongColorDepth (ulong aRealBPP,
     const char *kName = "remindAboutWrongColorDepth";
 
     /* Close the previous (outdated) window if any. We use kName as
-     * autoConfirmId which is also used as the widget name by default. */
+     * aAutoConfirmId which is also used as the widget name by default. */
     {
         QWidget *outdated = VBoxGlobal::findWidget (NULL, kName, "QIMessageBox");
         if (outdated)
@@ -1600,7 +1675,7 @@ void VBoxProblemReporter::remindAboutWrongColorDepth (ulong aRealBPP,
     NOREF(rc);
 }
 
-/** 
+/**
  *  Returns @c true if the user has selected to power off the machine.
  */
 bool VBoxProblemReporter::remindAboutGuruMeditation (const CConsole &aConsole,
@@ -1629,34 +1704,52 @@ bool VBoxProblemReporter::remindAboutGuruMeditation (const CConsole &aConsole,
             "Please note that debugging requires special knowledge and tools, so "
             "it is recommended to press <b>OK</b> now.</p>")
             .arg (aLogFolder),
-        0, /* autoConfirmId */
+        0, /* aAutoConfirmId */
         QIMessageBox::Ok | QIMessageBox::Default,
         QIMessageBox::Ignore | QIMessageBox::Escape);
 
     return rc == QIMessageBox::Ok;
 }
 
-int VBoxProblemReporter::remindAboutUnsetHD (QWidget *aParent)
+/**
+ *  @return @c true if the user has selected to reset the machine.
+ */
+bool VBoxProblemReporter::confirmVMReset (QWidget *aParent)
 {
-    return message (
-        aParent,
-        Warning,
+    return messageOkCancel (aParent, Question,
+        tr ("<p>Do you really want to reset the virtual machine?</p>"
+            "<p>When the machine is reset, unsaved data of all applications "
+            "running inside it will be lost.</p>"),
+        "confirmVMReset" /* aAutoConfirmId */,
+        tr ("Reset", "machine"));
+}
+
+/**
+ *  @return @c true if the user has selected to continue without attaching a
+ *  hard disk.
+ */
+bool VBoxProblemReporter::confirmHardDisklessMachine (QWidget *aParent)
+{
+    return message (aParent, Warning,
         tr ("<p>You didn't attach a hard disk to the new virtual machine. "
             "The machine will not be able to boot unless you attach "
             "a hard disk with a guest operating system or some other bootable "
             "media to it later using the machine settings dialog or the First "
             "Run Wizard.</p><p>Do you want to continue?</p>"),
-        0, /* autoConfirmId */
-        QIMessageBox::Yes,
-        QIMessageBox::No | QIMessageBox::Default | QIMessageBox::Escape);
+        0, /* aAutoConfirmId */
+        QIMessageBox::Ok,
+        QIMessageBox::Cancel | QIMessageBox::Default | QIMessageBox::Escape,
+        0,
+        tr ("Continue", "no hard disk attached"),
+        tr ("Go Back", "no hard disk attached")) == QIMessageBox::Ok;
 }
 
 void VBoxProblemReporter::cannotRunInSelectorMode()
 {
     message (mainWindowShown(), Critical,
-             tr ("<p>Cannot run VirtualBox in <i>VM Selector</i> "
-                 "mode due to local restrictions.</p>"
-                 "<p>The application will now terminate.</p>"));
+        tr ("<p>Cannot run VirtualBox in <i>VM Selector</i> "
+            "mode due to local restrictions.</p>"
+            "<p>The application will now terminate.</p>"));
 }
 
 void VBoxProblemReporter::showRuntimeError (const CConsole &aConsole, bool fatal,
@@ -1670,6 +1763,8 @@ void VBoxProblemReporter::showRuntimeError (const CConsole &aConsole, bool fatal
     //  - make warning messages modeless
     //  - add common buttons like Retry/Save/PowerOff/whatever
 
+    QCString autoConfimId = "showRuntimeError.";
+
     CConsole console = aConsole;
     CEnums::MachineState state = console.GetState();
     Type type;
@@ -1683,18 +1778,23 @@ void VBoxProblemReporter::showRuntimeError (const CConsole &aConsole, bool fatal
             console.Pause();
         type = Critical;
         severity = tr ("<nobr>Fatal Error</nobr>", "runtime error info");
+        autoConfimId += "fatal.";
     }
     else if (state == CEnums::Paused)
     {
         type = Error;
         severity = tr ("<nobr>Non-Fatal Error</nobr>", "runtime error info");
+        autoConfimId += "error.";
     }
     else
     {
         type = Warning;
         severity = tr ("<nobr>Warning</nobr>", "runtime error info");
+        autoConfimId += "warning.";
     }
 
+    autoConfimId += errorID.utf8();
+
     QString formatted;
 
     if (!errorMsg.isEmpty())
@@ -1727,7 +1827,7 @@ void VBoxProblemReporter::showRuntimeError (const CConsole &aConsole, bool fatal
                 "The virtual machine will be powered off. It is suggested to "
                 "use the clipboard to copy the following error message for "
                 "further examination:</p>"),
-            formatted);
+            formatted, autoConfimId.data());
 
         /* always power down after a fatal error */
         console.PowerDown();
@@ -1739,7 +1839,7 @@ void VBoxProblemReporter::showRuntimeError (const CConsole &aConsole, bool fatal
                 "The error details are shown below. You can try to correct "
                 "the described error and resume the virtual machine "
                 "execution.</p>"),
-            formatted);
+            formatted, autoConfimId.data());
     }
     else
     {
@@ -1749,10 +1849,10 @@ void VBoxProblemReporter::showRuntimeError (const CConsole &aConsole, bool fatal
                 "You may ignore this message, but it is suggested to perform "
                 "an appropriate action to make sure the described error will "
                 "not happen.</p>"),
-            formatted);
+            formatted, autoConfimId.data());
     }
 
-    NOREF(rc);
+    NOREF (rc);
 }
 
 /* static */
