@@ -43,6 +43,10 @@
 #include <qlayout.h>
 #include <qvbox.h>
 
+#ifdef Q_WS_X11
+#include <iprt/env.h>
+#endif
+
 
 // VBoxVMDetailsView class
 ////////////////////////////////////////////////////////////////////////////////
@@ -231,33 +235,42 @@ private:
     VBoxSelectorWnd *mParent;
     QToolButton *mBtnEdit;
     QTextBrowser *mBrowser;
-    QBrush mBrowserPaper;
+    QLabel *mLabel;
 };
 
 VBoxVMDescriptionPage::VBoxVMDescriptionPage (VBoxSelectorWnd *aParent,
                                               const char *aName)
     : QWidget (aParent, aName)
     , mItem (NULL), mParent (aParent)
-    , mBtnEdit (0), mBrowser (0)
+    , mBtnEdit (0), mBrowser (0), mLabel (0)
 {
-    /* main layout creation */
+    /* main layout */
     QVBoxLayout *mainLayout = new QVBoxLayout (this, 0, 10, "mainLayout");
 
-    /* mBrowser creation */
+    /* mBrowser */
     mBrowser = new QTextBrowser (this, "mBrowser");
-    mBrowserPaper = mBrowser->paper();
     mBrowser->setSizePolicy (QSizePolicy::Expanding, QSizePolicy::Expanding);
     mBrowser->setFocusPolicy (QWidget::StrongFocus);
     mBrowser->setLinkUnderline (false);
     mainLayout->addWidget (mBrowser);
+    /* hidden by default */
+    mBrowser->setHidden (true);
 
-    /* button layout creation */
+    mLabel = new QLabel (this, "mLabel");
+    mLabel->setFrameStyle (mBrowser->frameStyle());
+    mLabel->setSizePolicy (QSizePolicy::Expanding, QSizePolicy::Expanding);
+    mLabel->setAlignment (AlignCenter | WordBreak);
+    mainLayout->addWidget (mLabel);
+    /* always disabled */
+    mLabel->setEnabled (false);
+
+    /* button layout */
     QHBoxLayout *btnLayout = new QHBoxLayout (mainLayout, 10, "btnLayout");
     btnLayout->addItem (new QSpacerItem (0, 0,
                                          QSizePolicy::Expanding,
                                          QSizePolicy::Minimum));
 
-    /* button creation */
+    /* button */
     mBtnEdit = new QToolButton (this, "mBtnEdit");
     mBtnEdit->setSizePolicy (QSizePolicy::Preferred, QSizePolicy::Fixed);
     mBtnEdit->setFocusPolicy (QWidget::StrongFocus);
@@ -290,15 +303,15 @@ void VBoxVMDescriptionPage::setMachineItem (VBoxVMListBoxItem *aItem)
 
     if (!text.isEmpty())
     {
+        mLabel->setHidden (true);
         mBrowser->setText (text);
-        mBrowser->setPaper (mBrowserPaper);
-        mBrowser->setEnabled (true);
+        mBrowser->setShown (true);
     }
     else
     {
-        mBrowser->setText (tr ("No description. Press the Edit button below to add it."));
-        mBrowser->setEnabled (false);
-        mBrowser->setPaper (backgroundBrush());
+        mBrowser->setHidden (true);
+        mBrowser->clear();
+        mLabel->setShown (true);
     }
 
     /* check initial machine and session states */
@@ -307,6 +320,8 @@ void VBoxVMDescriptionPage::setMachineItem (VBoxVMListBoxItem *aItem)
 
 void VBoxVMDescriptionPage::languageChange()
 {
+    mLabel->setText (tr ("No description. Press the Edit button below to add it."));
+
     mBtnEdit->setTextLabel (tr ("Edit"));
     mBtnEdit->setAccel (tr ("Ctrl+E"));
     QToolTip::add (mBtnEdit, tr ("Edit (Ctrl+E)"));
@@ -402,6 +417,10 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent, const char* aName,
     vmDiscardAction->setIconSet (VBoxGlobal::iconSetEx (
         "vm_discard.png", "discard_16px.png",
         "vm_discard_dis.png", "discard_dis_16px.png"));
+    vmPauseAction = new QAction (this, "vmPauseAction");
+    vmPauseAction->setToggleAction (true);
+    vmPauseAction->setIconSet (VBoxGlobal::iconSet (
+        "pause_16px.png", "pause_disabled_16px.png"));
     vmRefreshAction = new QAction (this, "vmRefreshAction");
     vmRefreshAction->setIconSet (VBoxGlobal::iconSet (
         "refresh_16px.png", "refresh_disabled_16px.png"));
@@ -413,6 +432,9 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent, const char* aName,
     helpContentsAction->setIconSet (VBoxGlobal::iconSet ("help_16px.png"));
     helpWebAction = new QAction (this, "helpWebAction");
     helpWebAction->setIconSet (VBoxGlobal::iconSet ("site_16px.png"));
+    helpRegisterAction = new QAction (this, "helpRegisterAction");
+    helpRegisterAction->setIconSet (VBoxGlobal::iconSet ("register_16px.png",
+                                                         "register_disabled_16px.png"));
     helpAboutAction = new QAction (this, "helpAboutAction");
     helpAboutAction->setIconSet (VBoxGlobal::iconSet ("about_16px.png"));
     helpResetMessagesAction = new QAction( this, "helpResetMessagesAction" );
@@ -472,6 +494,7 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent, const char* aName,
     vmTools->setSizePolicy (QSizePolicy::Fixed, QSizePolicy::Preferred);
 
     vmNewAction->addTo (vmTools);
+    vmTools->addSeparator();
     vmConfigAction->addTo (vmTools);
     vmDeleteAction->addTo (vmTools);
     vmTools->addSeparator();
@@ -494,23 +517,44 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent, const char* aName,
 
     QPopupMenu *vmMenu = new QPopupMenu (this, "vmMenu");
     vmNewAction->addTo (vmMenu);
+    vmMenu->insertSeparator();
     vmConfigAction->addTo (vmMenu);
     vmDeleteAction->addTo (vmMenu);
     vmMenu->insertSeparator();
     vmStartAction->addTo (vmMenu);
     vmDiscardAction->addTo (vmMenu);
     vmMenu->insertSeparator();
+    vmPauseAction->addTo (vmMenu);
+    vmMenu->insertSeparator();
     vmRefreshAction->addTo (vmMenu);
     vmMenu->insertSeparator();
     vmShowLogsAction->addTo (vmMenu);
 
     menuBar()->insertItem (QString::null, vmMenu, 2);
 
+    mVMCtxtMenu = new QPopupMenu (this, "mVMCtxtMenu");
+    vmConfigAction->addTo (mVMCtxtMenu);
+    vmDeleteAction->addTo (mVMCtxtMenu);
+    mVMCtxtMenu->insertSeparator();
+    vmStartAction->addTo (mVMCtxtMenu);
+    vmDiscardAction->addTo (mVMCtxtMenu);
+    mVMCtxtMenu->insertSeparator();
+    vmPauseAction->addTo (mVMCtxtMenu);
+    mVMCtxtMenu->insertSeparator();
+    vmRefreshAction->addTo (mVMCtxtMenu);
+    mVMCtxtMenu->insertSeparator();
+    vmShowLogsAction->addTo (mVMCtxtMenu);
+
     QPopupMenu *helpMenu = new QPopupMenu( this, "helpMenu" );
-    helpContentsAction->addTo ( helpMenu );
-    helpWebAction->addTo( helpMenu );
+    helpContentsAction->addTo (helpMenu);
+    helpWebAction->addTo (helpMenu);
     helpMenu->insertSeparator();
-    helpAboutAction->addTo( helpMenu );
+#ifdef VBOX_WITH_REGISTRATION
+    helpRegisterAction->addTo (helpMenu);
+    helpRegisterAction->setEnabled (vboxGlobal().virtualBox().
+        GetExtraData (VBoxDefs::GUI_RegistrationDlgWinID).isEmpty());
+#endif
+    helpAboutAction->addTo (helpMenu);
     helpMenu->insertSeparator();
     helpResetMessagesAction->addTo (helpMenu);
 
@@ -546,11 +590,21 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent, const char* aName,
         }
         else
         {
-            resize (QSize (700, 500).expandedTo (minimumSizeHint())
+            resize (QSize (770, 550).expandedTo (minimumSizeHint())
                 .boundedTo (ar.size()));
         }
     }
 
+    /* restore the position of vm selector */
+    {
+        CVirtualBox vbox = vboxGlobal().virtualBox();
+        QString prevVMId = vbox.GetExtraData (VBoxDefs::GUI_LastVMSelected);
+
+        VBoxVMListBoxItem *item = vmListBox->item (QUuid (prevVMId));
+        if (item)
+            vmListBox->setSelected (item, true);
+    }
+
     clearWState (WState_Polished);
 
     /* signals and slots connections */
@@ -562,12 +616,18 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent, const char* aName,
     connect (vmDeleteAction, SIGNAL (activated()), this, SLOT (vmDelete()));
     connect (vmStartAction, SIGNAL (activated()), this, SLOT (vmStart()));
     connect (vmDiscardAction, SIGNAL (activated()), this, SLOT (vmDiscard()));
+    connect (vmPauseAction, SIGNAL (toggled (bool)), this, SLOT (vmPause (bool)));
     connect (vmRefreshAction, SIGNAL (activated()), this, SLOT (vmRefresh()));
     connect (vmShowLogsAction, SIGNAL (activated()), this, SLOT (vmShowLogs()));
 
-    connect (helpContentsAction, SIGNAL (activated()), this, SLOT(showHelpContents()));
+    connect (helpContentsAction, SIGNAL (activated()),
+             &vboxProblem(), SLOT (showHelpHelpDialog()));
     connect (helpWebAction, SIGNAL (activated()),
              &vboxProblem(), SLOT (showHelpWebDialog()));
+    connect (helpRegisterAction, SIGNAL (activated()),
+             &vboxGlobal(), SLOT (showRegistrationDialog()));
+    connect (&vboxGlobal(), SIGNAL (canShowRegDlg (bool)),
+             helpRegisterAction, SLOT (setEnabled (bool)));
     connect (helpAboutAction, SIGNAL (activated()),
              &vboxProblem(), SLOT (showHelpAboutDialog()));
     connect (helpResetMessagesAction, SIGNAL (activated()),
@@ -577,6 +637,8 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent, const char* aName,
              this, SLOT (vmListBoxCurrentChanged()));
     connect (vmListBox, SIGNAL (selected (QListBoxItem *)),
              this, SLOT (vmStart()));
+    connect (vmListBox, SIGNAL (contextMenuRequested (QListBoxItem *, const QPoint &)),
+             this, SLOT (showContextMenu (QListBoxItem *, const QPoint &)));
 
     connect (vmDetailsView, SIGNAL (linkClicked (const QString &)),
             this, SLOT (vmSettings (const QString &)));
@@ -605,9 +667,10 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent, const char* aName,
 
 VBoxSelectorWnd::~VBoxSelectorWnd()
 {
+    CVirtualBox vbox = vboxGlobal().virtualBox();
+
     /* save the position of the window */
     {
-        CVirtualBox vbox = vboxGlobal().virtualBox();
         QString winPos = QString ("%1,%2,%3,%4")
                                  .arg (normal_pos.x()).arg (normal_pos.y())
                                  .arg (normal_size.width())
@@ -617,6 +680,14 @@ VBoxSelectorWnd::~VBoxSelectorWnd()
 
         vbox.SetExtraData (VBoxDefs::GUI_LastWindowPosition, winPos);
     }
+    /* save vm selector position */
+    {
+        QListBoxItem *item = vmListBox->selectedItem();
+        QString curVMId = item ?
+            QString (static_cast<VBoxVMListBoxItem*> (item)->id()) :
+            QString::null;
+        vbox.SetExtraData (VBoxDefs::GUI_LastVMSelected, curVMId);
+    }
 }
 
 //
@@ -847,7 +918,7 @@ void VBoxSelectorWnd::vmStart()
 #if defined (Q_WS_X11)
     /* make sure the VM process will start on the same display as the Selector */
     {
-        const char *display = getenv ("DISPLAY");
+        const char *display = RTEnvGet ("DISPLAY");
         if (display)
             env.sprintf ("DISPLAY=%s", display);
     }
@@ -912,6 +983,37 @@ void VBoxSelectorWnd::vmDiscard()
     session.Close();
 }
 
+void VBoxSelectorWnd::vmPause (bool aPause)
+{
+    VBoxVMListBoxItem *item = (VBoxVMListBoxItem *) vmListBox->selectedItem();
+
+    AssertMsgReturn (item, ("Item must be always selected here"), (void) 0);
+
+    CSession session = vboxGlobal().openSession (item->id(), true);
+    if (session.isNull())
+        return;
+
+    CConsole console = session.GetConsole();
+    if (console.isNull())
+        return;
+
+    if (aPause)
+        console.Pause();
+    else
+        console.Resume();
+
+    bool ok = console.isOk();
+    if (!ok)
+    {
+        if (aPause)
+            vboxProblem().cannotPauseMachine (console);
+        else
+            vboxProblem().cannotResumeMachine (console);
+    }
+
+    session.Close();
+}
+
 void VBoxSelectorWnd::vmRefresh()
 {
     VBoxVMListBoxItem *item = (VBoxVMListBoxItem *) vmListBox->selectedItem();
@@ -947,9 +1049,10 @@ void VBoxSelectorWnd::refreshVMItem (const QUuid &aID, bool aDetails,
         vmListBoxCurrentChanged (aDetails, aSnapshots, aDescription);
 }
 
-void VBoxSelectorWnd::showHelpContents()
+void VBoxSelectorWnd::showContextMenu (QListBoxItem *aItem, const QPoint &aPoint)
 {
-    vboxProblem().showHelpHelpDialog();
+    if (aItem)
+        mVMCtxtMenu->exec (aPoint);
 }
 
 // Protected members
@@ -1056,14 +1159,12 @@ void VBoxSelectorWnd::languageChange()
 
     vmDeleteAction->setMenuText (tr ("&Delete"));
     vmDeleteAction->setText (tr ("Delete"));
-    vmDeleteAction->setAccel (QString::null);
     vmDeleteAction->setStatusTip (tr ("Delete the selected virtual machine"));
 
     /* Note: vmStartAction text is set up in vmListBoxCurrentChanged() */
 
     vmDiscardAction->setMenuText (tr ("D&iscard"));
     vmDiscardAction->setText (tr ("Discard"));
-    vmDiscardAction->setAccel (QString::null);
     vmDiscardAction->setStatusTip (
         tr ("Discard the saved state of the selected virtual machine"));
 
@@ -1084,16 +1185,17 @@ void VBoxSelectorWnd::languageChange()
     helpContentsAction->setStatusTip (tr ("Show the online help contents"));
 
     helpWebAction->setMenuText (tr ("&VirtualBox Web Site..."));
-    helpWebAction->setAccel( QString::null );
     helpWebAction->setStatusTip (
         tr ("Open the browser and go to the VirtualBox product web site"));
 
+    helpRegisterAction->setMenuText (tr ("R&egister VirtualBox..."));
+    helpRegisterAction->setStatusTip (
+        tr ("Open VirtualBox registration form"));
+
     helpAboutAction->setMenuText (tr ("&About VirtualBox..."));
-    helpAboutAction->setAccel( QString::null );
     helpAboutAction->setStatusTip (tr ("Show a dialog with product information"));
 
     helpResetMessagesAction->setMenuText (tr ("&Reset All Warnings"));
-    helpResetMessagesAction->setAccel (QString::null);
     helpResetMessagesAction->setStatusTip (
         tr ("Cause all suppressed warnings and messages to be shown again"));
 
@@ -1168,13 +1270,14 @@ void VBoxSelectorWnd::vmListBoxCurrentChanged (bool aRefreshDetails,
         vmConfigAction->setEnabled (modifyEnabled);
         vmDeleteAction->setEnabled (modifyEnabled);
         vmDiscardAction->setEnabled (state == CEnums::Saved && !running);
+        vmPauseAction->setEnabled (state == CEnums::Running ||
+                                   state == CEnums::Paused);
 
         /* change the Start button text accordingly */
         if (state >= CEnums::Running)
         {
             vmStartAction->setMenuText (tr ("S&how"));
             vmStartAction->setText (tr ("Show"));
-            vmStartAction->setAccel (QString::null);
             vmStartAction->setStatusTip (
                 tr ("Switch to the window of the selected virtual machine"));
 
@@ -1184,13 +1287,36 @@ void VBoxSelectorWnd::vmListBoxCurrentChanged (bool aRefreshDetails,
         {
             vmStartAction->setMenuText (tr ("S&tart"));
             vmStartAction->setText (tr ("Start"));
-            vmStartAction->setAccel (QString::null);
             vmStartAction->setStatusTip (
                 tr ("Start the selected virtual machine"));
 
             vmStartAction->setEnabled (!running);
         }
 
+        /* change the Pause/Resume button text accordingly */
+        if (state == CEnums::Paused)
+        {
+            vmPauseAction->setMenuText (tr ("R&esume"));
+            vmPauseAction->setText (tr ("Resume"));
+            vmPauseAction->setAccel (tr ("Ctrl+P"));
+            vmPauseAction->setStatusTip (
+                tr ("Resume the execution of the virtual machine"));
+            vmPauseAction->blockSignals (true);
+            vmPauseAction->setOn (true);
+            vmPauseAction->blockSignals (false);
+        }
+        else
+        {
+            vmPauseAction->setMenuText (tr ("&Pause"));
+            vmPauseAction->setText (tr ("Pause"));
+            vmPauseAction->setAccel (tr ("Ctrl+P"));
+            vmPauseAction->setStatusTip (
+                tr ("Suspend the execution of the virtual machine"));
+            vmPauseAction->blockSignals (true);
+            vmPauseAction->setOn (false);
+            vmPauseAction->blockSignals (false);
+        }
+
         /* disable Refresh for accessible machines */
         vmRefreshAction->setEnabled (false);
 
@@ -1246,11 +1372,11 @@ void VBoxSelectorWnd::vmListBoxCurrentChanged (bool aRefreshDetails,
         vmConfigAction->setEnabled (false);
         vmDeleteAction->setEnabled (item != NULL);
         vmDiscardAction->setEnabled (false);
+        vmPauseAction->setEnabled (false);
 
         /* change the Start button text accordingly */
         vmStartAction->setMenuText (tr ("S&tart"));
         vmStartAction->setText (tr ("Start"));
-        vmStartAction->setAccel (QString::null);
         vmStartAction->setStatusTip (
             tr ("Start the selected virtual machine"));
         vmStartAction->setEnabled (false);
