@@ -39,45 +39,50 @@ public:
         QListViewItem (parent), mStatus (VBoxMedia::Unknown) {}
 
     void setMedia (const VBoxMedia &aMedia) { mMedia = aMedia; }
-    VBoxMedia &getMedia() { return mMedia; }
+    const VBoxMedia &getMedia() const { return mMedia; }
 
-    QString getName() { return mName; }
+    void setPath (const QString &aPath) { mPath = aPath; }
+    const QString &getPath() const { return mPath; }
 
-    void setPath (QString aPath) { mPath = aPath; }
-    const QString &getPath() { return mPath; }
+    void setUsage (const QString &aUsage) { mUsage = aUsage; }
+    const QString &getUsage() const { return mUsage; }
 
-    void setUsage (QString aUsage) { mUsage = aUsage; }
-    const QString &getUsage() { return mUsage; }
+    void setSnapshotUsage (const QString &aSnapshotUsage) { mSnapshotUsage = aSnapshotUsage; }
+    const QString &getSnapshotUsage() const { return mSnapshotUsage; }
 
-    void setSnapshotName (QString aSnapshotName) { mSnapshotName = aSnapshotName; }
-    const QString &getSnapshotName() { return mSnapshotName; }
+    QString getTotalUsage() const
+    {
+        /* should correlate with VBoxDiskImageManagerDlg::compose[Cd/Fd]Tooltip */
+        return mSnapshotUsage.isNull() ? mUsage :
+            QString ("%1 (%2)").arg (mUsage, mSnapshotUsage);
+    }
 
-    void setDiskType (QString aDiskType) { mDiskType = aDiskType; }
-    const QString &getDiskType() { return mDiskType; }
+    void setSnapshotName (const QString &aSnapshotName) { mSnapshotName = aSnapshotName; }
+    const QString &getSnapshotName() const { return mSnapshotName; }
 
-    void setStorageType (QString aStorageType) { mStorageType = aStorageType; }
-    const QString &getStorageType() { return mStorageType; }
+    void setDiskType (const QString &aDiskType) { mDiskType = aDiskType; }
+    const QString &getDiskType() const { return mDiskType; }
 
-    void setVirtualSize (QString aVirtualSize) { mVirtualSize = aVirtualSize; }
-    const QString &getVirtualSize() { return mVirtualSize; }
+    void setStorageType (const QString &aStorageType) { mStorageType = aStorageType; }
+    const QString &getStorageType() const { return mStorageType; }
 
-    void setActualSize (QString aActualSize) { mActualSize = aActualSize; }
-    const QString &getActualSize() { return mActualSize; }
+    void setVirtualSize (const QString &aVirtualSize) { mVirtualSize = aVirtualSize; }
+    const QString &getVirtualSize() const { return mVirtualSize; }
 
+    void setActualSize (const QString &aActualSize) { mActualSize = aActualSize; }
+    const QString &getActualSize() const { return mActualSize; }
 
-    void setUuid (QUuid aUuid) { mUuid = aUuid; }
-    const QString &getUuid() { return mUuid; }
-
-    void setMachineId (QString aMachineId) { mMachineId = aMachineId; }
-    const QString &getMachineId() { return mMachineId; }
+    void setUuid (const QUuid &aUuid) { mUuid = aUuid; }
+    const QUuid &getUuid() const { return mUuid; }
 
+    void setMachineId (const QUuid &aMachineId) { mMachineId = aMachineId; }
+    const QUuid &getMachineId() const { return mMachineId; }
 
     void setStatus (VBoxMedia::Status aStatus) { mStatus = aStatus; }
-    VBoxMedia::Status getStatus() { return mStatus; }
-
+    VBoxMedia::Status getStatus() const { return mStatus; }
 
     void setToolTip (QString aToolTip) { mToolTip = aToolTip; }
-    const QString &getToolTip() { return mToolTip; }
+    const QString &getToolTip() const { return mToolTip; }
 
     QString getInformation (const QString &aInfo, bool aCompact = true,
                             const QString &aElipsis = "middle")
@@ -132,14 +137,15 @@ protected:
     QString mName;
     QString mPath;
     QString mUsage;
+    QString mSnapshotUsage;
     QString mSnapshotName;
     QString mDiskType;
     QString mStorageType;
     QString mVirtualSize;
     QString mActualSize;
 
-    QString mUuid;
-    QString mMachineId;
+    QUuid mUuid;
+    QUuid mMachineId;
 
     QString mToolTip;
 
@@ -348,7 +354,7 @@ void VBoxDiskImageManagerDlg::init()
     imRefreshAction->addTo (toolBar);
 #ifdef Q_WS_MAC
     toolBar->setMacStyle();
-#endif 
+#endif
 
 
     /* menu bar */
@@ -856,7 +862,8 @@ void VBoxDiskImageManagerDlg::invokePopup (QListViewItem *aItem, const QPoint &
 }
 
 
-QString VBoxDiskImageManagerDlg::getDVDImageUsage (const QUuid &aId)
+QString VBoxDiskImageManagerDlg::getDVDImageUsage (const QUuid &aId,
+                                                   QString &aSnapshotUsage)
 {
     CVirtualBox vbox = vboxGlobal().virtualBox();
 
@@ -873,7 +880,11 @@ QString VBoxDiskImageManagerDlg::getDVDImageUsage (const QUuid &aId)
     {
         if (usage)
             usage += ", ";
-        usage += vbox.GetMachine (QUuid (*it)).GetName();
+        CMachine machine = vbox.GetMachine (QUuid (*it));
+        usage += machine.GetName();
+
+        getDVDImageSnapshotUsage (aId, machine.GetSnapshot (QUuid()),
+                                  aSnapshotUsage);
     }
 
     for (QStringList::Iterator it = tempMachines.begin();
@@ -887,14 +898,19 @@ QString VBoxDiskImageManagerDlg::getDVDImageUsage (const QUuid &aId)
                 usage += ", [";
             else
                 usage += "[";
-            usage += vbox.GetMachine (QUuid (*it)).GetName() + "]";
+            CMachine machine = vbox.GetMachine (QUuid (*it));
+            usage += machine.GetName() + "]";
+
+            getDVDImageSnapshotUsage (aId, machine.GetSnapshot (QUuid()),
+                                      aSnapshotUsage);
         }
     }
 
     return usage;
 }
 
-QString VBoxDiskImageManagerDlg::getFloppyImageUsage (const QUuid &aId)
+QString VBoxDiskImageManagerDlg::getFloppyImageUsage (const QUuid &aId,
+                                                      QString &aSnapshotUsage)
 {
     CVirtualBox vbox = vboxGlobal().virtualBox();
 
@@ -911,7 +927,11 @@ QString VBoxDiskImageManagerDlg::getFloppyImageUsage (const QUuid &aId)
     {
         if (usage)
             usage += ", ";
-        usage += vbox.GetMachine (QUuid (*it)).GetName();
+        CMachine machine = vbox.GetMachine (QUuid (*it));
+        usage += machine.GetName();
+
+        getFloppyImageSnapshotUsage (aId, machine.GetSnapshot (QUuid()),
+                                     aSnapshotUsage);
     }
 
     for (QStringList::Iterator it = tempMachines.begin();
@@ -925,7 +945,11 @@ QString VBoxDiskImageManagerDlg::getFloppyImageUsage (const QUuid &aId)
                 usage += ", [";
             else
                 usage += "[";
-            usage += vbox.GetMachine (QUuid (*it)).GetName() + "]";
+            CMachine machine = vbox.GetMachine (QUuid (*it));
+            usage += machine.GetName() + "]";
+
+            getFloppyImageSnapshotUsage (aId, machine.GetSnapshot (QUuid()),
+                                         aSnapshotUsage);
         }
     }
 
@@ -933,26 +957,74 @@ QString VBoxDiskImageManagerDlg::getFloppyImageUsage (const QUuid &aId)
 }
 
 
+void VBoxDiskImageManagerDlg::getDVDImageSnapshotUsage (const QUuid &aImageId,
+                                                        const CSnapshot &aSnapshot,
+                                                        QString &aUsage)
+{
+    if (aSnapshot.isNull())
+        return;
+
+    if (!aSnapshot.GetMachine().GetDVDDrive().GetImage().isNull() &&
+        aSnapshot.GetMachine().GetDVDDrive().GetImage().GetId() == aImageId)
+    {
+        if (aUsage)
+            aUsage += ", ";
+        aUsage += aSnapshot.GetName();
+    }
+
+    CSnapshotEnumerator en = aSnapshot.GetChildren().Enumerate();
+    while (en.HasMore())
+        getDVDImageSnapshotUsage (aImageId, en.GetNext(), aUsage);
+}
+
+void VBoxDiskImageManagerDlg::getFloppyImageSnapshotUsage (const QUuid &aImageId,
+                                                           const CSnapshot &aSnapshot,
+                                                           QString &aUsage)
+{
+    if (aSnapshot.isNull())
+        return;
+
+    if (!aSnapshot.GetMachine().GetFloppyDrive().GetImage().isNull() &&
+        aSnapshot.GetMachine().GetFloppyDrive().GetImage().GetId() == aImageId)
+    {
+        if (aUsage)
+            aUsage += ", ";
+        aUsage += aSnapshot.GetName();
+    }
+
+    CSnapshotEnumerator en = aSnapshot.GetChildren().Enumerate();
+    while (en.HasMore())
+        getFloppyImageSnapshotUsage (aImageId, en.GetNext(), aUsage);
+}
+
+
 QString VBoxDiskImageManagerDlg::composeHdToolTip (CHardDisk &aHd,
-                                                   VBoxMedia::Status aStatus)
+                                                   VBoxMedia::Status aStatus,
+                                                   DiskImageItem *aItem)
 {
     CVirtualBox vbox = vboxGlobal().virtualBox();
-    QUuid machineId = aHd.GetMachineId();
+    QUuid machineId = aItem ? aItem->getMachineId() : aHd.GetMachineId();
 
-    QString src = aHd.GetLocation();
-    QFileInfo fi (src);
-    QString location = aHd.GetStorageType() == CEnums::ISCSIHardDisk ? src :
-                       QDir::convertSeparators (fi.absFilePath());
+    QString src = aItem ? aItem->getPath() : aHd.GetLocation();
+    QString location = aItem || aHd.GetStorageType() == CEnums::ISCSIHardDisk ? src :
+        QDir::convertSeparators (QFileInfo (src).absFilePath());
 
-    QString storageType = vboxGlobal().toString (aHd.GetStorageType());
-    QString hardDiskType = vboxGlobal().hardDiskTypeString (aHd);
+    QString storageType = aItem ? aItem->getStorageType() :
+        vboxGlobal().toString (aHd.GetStorageType());
+    QString hardDiskType = aItem ? aItem->getDiskType() :
+        vboxGlobal().hardDiskTypeString (aHd);
 
     QString usage;
-    if (!machineId.isNull())
+    if (aItem)
+        usage = aItem->getUsage();
+    else if (!machineId.isNull())
         usage = vbox.GetMachine (machineId).GetName();
 
+    QUuid snapshotId = aItem ? aItem->getUuid() : aHd.GetSnapshotId();
     QString snapshotName;
-    if (!machineId.isNull() && !aHd.GetSnapshotId().isNull())
+    if (aItem)
+        snapshotName = aItem->getSnapshotName();
+    else if (!machineId.isNull() && !snapshotId.isNull())
     {
         CSnapshot snapshot = vbox.GetMachine (machineId).
                                   GetSnapshot (aHd.GetSnapshotId());
@@ -1012,13 +1084,23 @@ QString VBoxDiskImageManagerDlg::composeHdToolTip (CHardDisk &aHd,
 }
 
 QString VBoxDiskImageManagerDlg::composeCdToolTip (CDVDImage &aCd,
-                                                   VBoxMedia::Status aStatus)
+                                                   VBoxMedia::Status aStatus,
+                                                   DiskImageItem *aItem)
 {
-    QString src = aCd.GetFilePath();
-    QFileInfo fi (src);
-    QString location = QDir::convertSeparators (fi.absFilePath ());
-    QUuid uuid = aCd.GetId();
-    QString usage = getDVDImageUsage (uuid);
+    QString location = aItem ? aItem->getPath() :
+        QDir::convertSeparators (QFileInfo (aCd.GetFilePath()).absFilePath());
+    QUuid uuid = aItem ? aItem->getUuid() : aCd.GetId();
+    QString usage;
+    if (aItem)
+        usage = aItem->getTotalUsage();
+    else
+    {
+        QString snapshotUsage;
+        usage = getDVDImageUsage (uuid, snapshotUsage);
+        /* should correlate with DiskImageItem::getTotalUsage() */
+        if (!snapshotUsage.isNull())
+            usage = QString ("%1 (%2)").arg (usage, snapshotUsage);
+    }
 
     /* compose tool-tip information */
     QString tip;
@@ -1068,15 +1150,24 @@ QString VBoxDiskImageManagerDlg::composeCdToolTip (CDVDImage &aCd,
 }
 
 QString VBoxDiskImageManagerDlg::composeFdToolTip (CFloppyImage &aFd,
-                                                   VBoxMedia::Status aStatus)
+                                                   VBoxMedia::Status aStatus,
+                                                   DiskImageItem *aItem)
 {
-    QString src = aFd.GetFilePath();
-    QFileInfo fi (src);
-    QString location = QDir::convertSeparators (fi.absFilePath ());
-    QUuid uuid = aFd.GetId();
-    QString usage = getFloppyImageUsage (uuid);
+    QString location = aItem ? aItem->getPath() :
+        QDir::convertSeparators (QFileInfo (aFd.GetFilePath()).absFilePath());
+    QUuid uuid = aItem ? aItem->getUuid() : aFd.GetId();
+    QString usage;
+    if (aItem)
+        usage = aItem->getTotalUsage();
+    else
+    {
+        QString snapshotUsage;
+        usage = getFloppyImageUsage (uuid, snapshotUsage);
+        /* should correlate with DiskImageItem::getTotalUsage() */
+        if (!snapshotUsage.isNull())
+            usage = QString ("%1 (%2)").arg (usage, snapshotUsage);
+    }
 
-    /* compose tool-tip information */
     /* compose tool-tip information */
     QString tip;
     switch (aStatus)
@@ -1128,7 +1219,9 @@ QString VBoxDiskImageManagerDlg::composeFdToolTip (CFloppyImage &aFd,
 void VBoxDiskImageManagerDlg::updateHdItem (DiskImageItem   *aItem,
                                             const VBoxMedia &aMedia)
 {
-    if (!aItem) return;
+    if (!aItem)
+        return;
+
     CHardDisk hd = aMedia.disk;
     VBoxMedia::Status status = aMedia.status;
 
@@ -1158,7 +1251,7 @@ void VBoxDiskImageManagerDlg::updateHdItem (DiskImageItem   *aItem,
     aItem->setText (1, virtualSize);
     aItem->setText (2, actualSize);
     aItem->setPath (hd.GetStorageType() == CEnums::ISCSIHardDisk ? src :
-                   QDir::convertSeparators (fi.absFilePath()));
+                    QDir::convertSeparators (fi.absFilePath()));
     aItem->setUsage (usage);
     aItem->setSnapshotName (snapshotName);
     aItem->setDiskType (hardDiskType);
@@ -1167,7 +1260,7 @@ void VBoxDiskImageManagerDlg::updateHdItem (DiskImageItem   *aItem,
     aItem->setActualSize (actualSize);
     aItem->setUuid (uuid);
     aItem->setMachineId (machineId);
-    aItem->setToolTip (composeHdToolTip (hd, status));
+    aItem->setToolTip (composeHdToolTip (hd, status, aItem));
     aItem->setStatus (status);
 
     makeWarningMark (aItem, aMedia.status, VBoxDefs::HD);
@@ -1176,13 +1269,16 @@ void VBoxDiskImageManagerDlg::updateHdItem (DiskImageItem   *aItem,
 void VBoxDiskImageManagerDlg::updateCdItem (DiskImageItem   *aItem,
                                             const VBoxMedia &aMedia)
 {
-    if (!aItem) return;
+    if (!aItem)
+        return;
+
     CDVDImage cd = aMedia.disk;
     VBoxMedia::Status status = aMedia.status;
 
     QUuid uuid = cd.GetId();
     QString src = cd.GetFilePath();
-    QString usage = getDVDImageUsage (uuid);
+    QString snapshotUsage;
+    QString usage = getDVDImageUsage (uuid, snapshotUsage);
     QString size = status == VBoxMedia::Ok ?
         vboxGlobal().formatSize (cd.GetSize()) : QString ("--");
     QFileInfo fi (src);
@@ -1191,9 +1287,10 @@ void VBoxDiskImageManagerDlg::updateCdItem (DiskImageItem   *aItem,
     aItem->setText (1, size);
     aItem->setPath (QDir::convertSeparators (fi.absFilePath ()));
     aItem->setUsage (usage);
+    aItem->setSnapshotUsage (snapshotUsage);
     aItem->setActualSize (size);
     aItem->setUuid (uuid);
-    aItem->setToolTip (composeCdToolTip (cd, status));
+    aItem->setToolTip (composeCdToolTip (cd, status, aItem));
     aItem->setStatus (status);
 
     makeWarningMark (aItem, aMedia.status, VBoxDefs::CD);
@@ -1202,13 +1299,16 @@ void VBoxDiskImageManagerDlg::updateCdItem (DiskImageItem   *aItem,
 void VBoxDiskImageManagerDlg::updateFdItem (DiskImageItem   *aItem,
                                             const VBoxMedia &aMedia)
 {
-    if (!aItem) return;
+    if (!aItem)
+        return;
+
     CFloppyImage fd = aMedia.disk;
     VBoxMedia::Status status = aMedia.status;
 
     QUuid uuid = fd.GetId();
     QString src = fd.GetFilePath();
-    QString usage = getFloppyImageUsage (uuid);
+    QString snapshotUsage;
+    QString usage = getFloppyImageUsage (uuid, snapshotUsage);
     QString size = status == VBoxMedia::Ok ?
         vboxGlobal().formatSize (fd.GetSize()) : QString ("--");
     QFileInfo fi (src);
@@ -1217,9 +1317,10 @@ void VBoxDiskImageManagerDlg::updateFdItem (DiskImageItem   *aItem,
     aItem->setText (1, size);
     aItem->setPath (QDir::convertSeparators (fi.absFilePath ()));
     aItem->setUsage (usage);
+    aItem->setSnapshotUsage (snapshotUsage);
     aItem->setActualSize (size);
     aItem->setUuid (uuid);
-    aItem->setToolTip (composeFdToolTip (fd, status));
+    aItem->setToolTip (composeFdToolTip (fd, status, aItem));
     aItem->setStatus (status);
 
     makeWarningMark (aItem, aMedia.status, VBoxDefs::FD);
@@ -1307,9 +1408,15 @@ DiskImageItem* VBoxDiskImageManagerDlg::searchItem (QListView *aList,
 void VBoxDiskImageManagerDlg::setup (int aType, bool aDoSelect,
                                      const QUuid *aTargetVMId /* = NULL */,
                                      bool aRefresh /* = true */,
-                                     CMachine machine /* = NULL */)
+                                     CMachine machine /* = NULL */,
+                                     const QUuid &aHdId,
+                                     const QUuid &aCdId,
+                                     const QUuid &aFdId)
 {
     cmachine = machine;
+    hdSelectedId = aHdId;
+    cdSelectedId = aCdId;
+    fdSelectedId = aFdId;
 
     type = aType;
     twImages->setTabEnabled (twImages->page(0), type & VBoxDefs::HD);
@@ -1318,7 +1425,7 @@ void VBoxDiskImageManagerDlg::setup (int aType, bool aDoSelect,
 
     doSelect = aDoSelect;
     if (aTargetVMId)
-        targetVMId = aTargetVMId->toString();
+        targetVMId = *aTargetVMId;
 
     if (doSelect)
         buttonOk->setText (tr ("&Select"));
@@ -1369,9 +1476,12 @@ void VBoxDiskImageManagerDlg::setup (int aType, bool aDoSelect,
     }
 
     /* for a newly opened dialog, select the first item */
-    setCurrentItem (hdsView, hdsView->firstChild());
-    setCurrentItem (cdsView, cdsView->firstChild());
-    setCurrentItem (fdsView, fdsView->firstChild());
+    if (!hdsView->selectedItem())
+        setCurrentItem (hdsView, hdsView->firstChild());
+    if (!cdsView->selectedItem())
+        setCurrentItem (cdsView, cdsView->firstChild());
+    if (!fdsView->selectedItem())
+        setCurrentItem (fdsView, fdsView->firstChild());
 }
 
 
@@ -1609,17 +1719,20 @@ void VBoxDiskImageManagerDlg::prepareToRefresh (int aTotal)
     item = hdsView->currentItem();
     di = (item && item->rtti() == DiskImageItem::TypeId) ?
         static_cast <DiskImageItem *> (item) : 0;
-    hdSelectedId = di ? di->getUuid() : QString::null;
+    if (hdSelectedId.isNull())
+        hdSelectedId = di ? di->getUuid() : QUuid();
 
     item = cdsView->currentItem();
     di = (item && item->rtti() == DiskImageItem::TypeId) ?
         static_cast <DiskImageItem *> (item) : 0;
-    cdSelectedId = di ? di->getUuid() : QString::null;
+    if (cdSelectedId.isNull())
+        cdSelectedId = di ? di->getUuid() : QUuid();
 
     item = fdsView->currentItem();
     di = (item && item->rtti() == DiskImageItem::TypeId) ?
         static_cast <DiskImageItem *> (item) : 0;
-    fdSelectedId = di ? di->getUuid() : QString::null;
+    if (fdSelectedId.isNull())
+        fdSelectedId = di ? di->getUuid() : QUuid();
 
     /* finally, clear all lists */
     hdsView->clear();
@@ -1637,7 +1750,7 @@ void VBoxDiskImageManagerDlg::refreshAll()
 
 bool VBoxDiskImageManagerDlg::checkImage (DiskImageItem* aItem)
 {
-    QUuid itemId = aItem ? QUuid (aItem->getUuid()) : QUuid();
+    QUuid itemId = aItem ? aItem->getUuid() : QUuid();
     if (itemId.isNull()) return false;
 
     QListView* parentList = aItem->listView();
@@ -1646,13 +1759,12 @@ bool VBoxDiskImageManagerDlg::checkImage (DiskImageItem* aItem)
         CHardDisk hd = aItem->getMedia().disk;
         QUuid machineId = hd.GetMachineId();
         if (machineId.isNull() ||
-            vbox.GetMachine (machineId).GetState() != CEnums::PoweredOff &&
-            vbox.GetMachine (machineId).GetState() != CEnums::Aborted)
+            (vbox.GetMachine (machineId).GetState() != CEnums::PoweredOff &&
+             vbox.GetMachine (machineId).GetState() != CEnums::Aborted))
             return false;
     }
     else if (parentList == cdsView)
     {
-        QString usage = getDVDImageUsage (itemId);
         /* check if there is temporary usage: */
         QStringList tempMachines =
             QStringList::split (' ', vbox.GetDVDImageUsage (itemId,
@@ -1671,14 +1783,13 @@ bool VBoxDiskImageManagerDlg::checkImage (DiskImageItem* aItem)
     }
     else if (parentList == fdsView)
     {
-        QString usage = getFloppyImageUsage(itemId);
         /* check if there is temporary usage: */
         QStringList tempMachines =
             QStringList::split (' ', vbox.GetFloppyImageUsage (itemId,
                                              CEnums::TemporaryUsage));
         if (!tempMachines.isEmpty())
             return false;
-        /* only permamently mounted .iso could be released */
+        /* only permamently mounted floppies could be released */
         QStringList permMachines =
             QStringList::split (' ', vbox.GetFloppyImageUsage (itemId,
                                              CEnums::PermanentUsage));
@@ -1744,6 +1855,7 @@ void VBoxDiskImageManagerDlg::processCurrentChanged (QListViewItem *aItem)
                           item &&  item->getUsage().isNull() &&
                           !item->firstChild() && !item->getPath().isNull();
     bool releaseEnabled = item && !item->getUsage().isNull() &&
+                          item->getSnapshotUsage().isNull() &&
                           checkImage (item) &&
                           !item->parent() && !item->firstChild() &&
                           item->getSnapshotName().isNull();
@@ -1784,12 +1896,12 @@ void VBoxDiskImageManagerDlg::processCurrentChanged (QListViewItem *aItem)
         else if (item->listView() == cdsView)
         {
             cdsPane1->setText (item->getInformation (item->getPath(), true, "end"));
-            cdsPane2->setText (item->getInformation (item->getUsage()));
+            cdsPane2->setText (item->getInformation (item->getTotalUsage()));
         }
         else if (item->listView() == fdsView)
         {
             fdsPane1->setText (item->getInformation (item->getPath(), true, "end"));
-            fdsPane2->setText (item->getInformation (item->getUsage()));
+            fdsPane2->setText (item->getInformation (item->getTotalUsage()));
         }
     }
     else
@@ -1895,7 +2007,7 @@ void VBoxDiskImageManagerDlg::removeImage()
         static_cast<DiskImageItem*> (currentList->currentItem()) : 0;
     AssertMsg (item, ("Current item must not be null"));
 
-    QUuid uuid = QUuid (item->getUuid());
+    QUuid uuid = item->getUuid();
     AssertMsg (!uuid.isNull(), ("Current item must have uuid"));
 
     QString src = item->getPath().stripWhiteSpace();
@@ -1904,20 +2016,30 @@ void VBoxDiskImageManagerDlg::removeImage()
     if (currentList == hdsView)
     {
         type = VBoxDefs::HD;
-        int deleteImage;
+        bool deleteImage = false;
+
         /// @todo When creation of VMDK is implemented, we should
         /// enable image deletion for  them as well (use
         /// GetStorageType() to define the correct cast).
         CHardDisk disk = item->getMedia().disk;
         if (disk.GetStorageType() == CEnums::VirtualDiskImage &&
             item->getStatus() == VBoxMedia::Ok)
-            deleteImage = vboxProblem().confirmHardDiskImageDeletion (this, src);
+        {
+            int rc = vboxProblem().confirmHardDiskImageDeletion (this, src);
+            if (rc == QIMessageBox::Cancel)
+                return;
+            deleteImage = rc == QIMessageBox::Yes;
+        }
         else
-            deleteImage = vboxProblem().confirmHardDiskUnregister (this, src);
-        if (deleteImage == QIMessageBox::Cancel)
-            return;
+        {
+            if (!vboxProblem().confirmHardDiskUnregister (this, src))
+                return;
+        }
+
         CHardDisk hd = vbox.UnregisterHardDisk (uuid);
-        if (vbox.isOk() && deleteImage == QIMessageBox::Yes)
+        if (!vbox.isOk())
+            vboxProblem().cannotUnregisterMedia (this, vbox, type, src);
+        else if (deleteImage)
         {
             /// @todo When creation of VMDK is implemented, we should
             /// enable image deletion for  them as well (use
@@ -1956,7 +2078,7 @@ void VBoxDiskImageManagerDlg::releaseImage()
         static_cast<DiskImageItem*> (currentList->currentItem()) : 0;
     AssertMsg (item, ("Current item must not be null"));
 
-    QUuid itemId = QUuid (item->getUuid());
+    QUuid itemId = item->getUuid();
     AssertMsg (!itemId.isNull(), ("Current item must have uuid"));
 
     /* if it is a hard disk sub-item: */
@@ -1968,14 +2090,17 @@ void VBoxDiskImageManagerDlg::releaseImage()
             vbox.GetMachine (machineId).GetName()))
         {
             releaseDisk (machineId, itemId, VBoxDefs::HD);
-            vboxGlobal().updateMedia (item->getMedia());
+            VBoxMedia media (item->getMedia());
+            media.status = hd.GetAccessible() ? VBoxMedia::Ok :
+                           hd.isOk() ? VBoxMedia::Inaccessible :
+                           VBoxMedia::Error;
+            vboxGlobal().updateMedia (media);
         }
     }
     /* if it is a cd/dvd sub-item: */
     else if (currentList == cdsView)
     {
-        QString usage = getDVDImageUsage (itemId);
-        /* only permamently mounted .iso could be released */
+        QString usage = item->getTotalUsage();
         if (vboxProblem().confirmReleaseImage (this, usage))
         {
             QStringList permMachines =
@@ -1986,14 +2111,17 @@ void VBoxDiskImageManagerDlg::releaseImage()
                 releaseDisk (QUuid (*it), itemId, VBoxDefs::CD);
 
             CDVDImage cd = vbox.GetDVDImage (itemId);
-            vboxGlobal().updateMedia (item->getMedia());
+            VBoxMedia media (item->getMedia());
+            media.status = cd.GetAccessible() ? VBoxMedia::Ok :
+                           cd.isOk() ? VBoxMedia::Inaccessible :
+                           VBoxMedia::Error;
+            vboxGlobal().updateMedia (media);
         }
     }
     /* if it is a floppy sub-item: */
     else if (currentList == fdsView)
     {
-        QString usage = getFloppyImageUsage (itemId);
-        /* only permamently mounted .img could be released */
+        QString usage = item->getTotalUsage();
         if (vboxProblem().confirmReleaseImage (this, usage))
         {
             QStringList permMachines =
@@ -2004,7 +2132,11 @@ void VBoxDiskImageManagerDlg::releaseImage()
                 releaseDisk (QUuid (*it), itemId, VBoxDefs::FD);
 
             CFloppyImage fd = vbox.GetFloppyImage (itemId);
-            vboxGlobal().updateMedia (item->getMedia());
+            VBoxMedia media (item->getMedia());
+            media.status = fd.GetAccessible() ? VBoxMedia::Ok :
+                           fd.isOk() ? VBoxMedia::Inaccessible :
+                           VBoxMedia::Error;
+            vboxGlobal().updateMedia (media);
         }
     }
 }
@@ -2083,8 +2215,8 @@ QUuid VBoxDiskImageManagerDlg::getSelectedUuid()
 
     if (currentList->selectedItem() &&
         currentList->selectedItem()->rtti() == DiskImageItem::TypeId)
-        uuid = QUuid (static_cast<DiskImageItem *>(currentList->selectedItem())
-                      ->getUuid());
+        uuid = static_cast <DiskImageItem *> (
+            currentList->selectedItem())->getUuid();
 
     return uuid;
 }
