@@ -49,18 +49,18 @@ public:
     };
 
     VBoxRichListItem (FormatType aFormat, QListView *aParent,
-                      const QString& aName, const QString& aNull,
-                      const QString& aKey) :
-        QListViewItem (aParent, aName, aNull, aKey), mFormat (aFormat)
+                      const QString& aName, const QString& aNull1,
+                      const QString& aNull2, const QString& aKey) :
+        QListViewItem (aParent, aName, aNull1, aNull2, aKey), mFormat (aFormat)
     {
     }
 
     VBoxRichListItem (FormatType aFormat, QListViewItem *aParent,
                       const QString& aName, const QString& aPath,
-                      const QString& aEdited) :
-        QListViewItem (aParent, aName, aPath, aEdited), mFormat (aFormat)
+                      const QString& aAccess, const QString& aEdited) :
+        QListViewItem (aParent, aName, aPath, aAccess, aEdited), mFormat (aFormat)
     {
-        mTextList << aName << aPath << aEdited;
+        mTextList << aName << aPath << aAccess << aEdited;
     }
 
     int rtti() const { return QIRichListItemId; }
@@ -72,7 +72,7 @@ public:
             return QListViewItem::compare (aItem, 0, aAscending);
         /* Sorting the root items always by key: */
         else if (!parent() && !aItem->parent())
-            return QListViewItem::compare (aItem, 2, aAscending);
+            return QListViewItem::compare (aItem, 3, aAscending);
         else
             return QListViewItem::compare (aItem, aColumn, aAscending);
     }
@@ -84,7 +84,7 @@ public:
             static_cast<VBoxRichListItem*> (item) : 0;
     }
 
-    QString getText (int aIndex)
+    QString getText (int aIndex) const
     {
         return aIndex >= 0 && aIndex < (int)mTextList.size() ?
             mTextList [aIndex] : QString::null;
@@ -101,8 +101,41 @@ protected:
     void paintCell (QPainter *aPainter, const QColorGroup &aColorGroup,
                     int aColumn, int aWidth, int aAlign)
     {
-        processColumn (aColumn, aWidth);
-        QListViewItem::paintCell (aPainter, aColorGroup, aColumn, aWidth, aAlign);
+        if (!parent())
+        {
+            /* Make root items occupy the whole width */
+            aWidth = listView()->viewport()->width();
+
+            if (aColumn > 0)
+            {
+                /* For columns other than the first one, paint the overlapping
+                 * portion of the first column after correcting the window */
+                aPainter->save();
+                QRect wnd = aPainter->window();
+                int dx = -listView()->treeStepSize();
+                for (int i = 0; i < aColumn; ++ i)
+                    dx += listView()->columnWidth (i);
+                wnd.moveBy (dx, 0);
+                aPainter->setWindow (wnd);
+                QListViewItem::paintCell (aPainter, aColorGroup, 0, aWidth, aAlign);
+                aPainter->restore();
+                return;
+            }
+
+            QListViewItem::paintCell (aPainter, aColorGroup, aColumn, aWidth, aAlign);
+        }
+        else
+        {
+            processColumn (aColumn, aWidth);
+            QListViewItem::paintCell (aPainter, aColorGroup, aColumn, aWidth, aAlign);
+        }
+    }
+
+    int width (const QFontMetrics &aFontMetrics, const QListView *, int aColumn) const
+    {
+        return parent() ?
+               aFontMetrics.boundingRect (getText (aColumn)).width() +
+               aFontMetrics.width ("...x") /* indent size */ : 0;
     }
 
     void processColumn (int aColumn, int aWidth)
@@ -180,7 +213,7 @@ public:
                      bool aEnableSelector /* for "permanent" checkbox */,
                      const SFoldersNameList &aUsedNames)
         : QDialog (aParent, "VBoxAddSFDialog", true /* modal */)
-        , mLePath (0), mLeName (0), mCbPermanent (0)
+        , mLePath (0), mLeName (0), mCbPermanent (0), mCbReadonly (0)
         , mUsedNames (aUsedNames)
     {
         switch (aType)
@@ -222,11 +255,18 @@ public:
         inputLayout->addWidget (lbName,  1, 0);
         inputLayout->addMultiCellWidget (mLeName, 1, 1, 1, 2);
 
+        mCbReadonly = new QCheckBox (tr ("&Read-only"), this);
+        QWhatsThis::add (mCbReadonly,
+            tr ("When checked, the guest OS will not be able to write to the "
+                "specified shared folder."));
+        mCbReadonly->setChecked (false);
+        inputLayout->addMultiCellWidget (mCbReadonly, 2, 2, 0, 2);
+
         if (aEnableSelector)
         {
-            mCbPermanent = new QCheckBox ("&Make Permanent", this);
+            mCbPermanent = new QCheckBox (tr ("&Make Permanent"), this);
             mCbPermanent->setChecked (true);
-            inputLayout->addMultiCellWidget (mCbPermanent, 2, 2, 0, 2);
+            inputLayout->addMultiCellWidget (mCbPermanent, 3, 3, 0, 2);
             connect (mCbPermanent, SIGNAL (toggled (bool)),
                      this, SLOT (validate()));
         }
@@ -234,7 +274,7 @@ public:
         /* Setup Button layout */
         QHBoxLayout *buttonLayout = new QHBoxLayout (mainLayout, 10, "buttonLayout");
         mBtOk = new QPushButton (tr ("&OK"), this, "btOk");
-        QSpacerItem *spacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
+        QSpacerItem *spacer = new QSpacerItem (0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
         QPushButton *btCancel = new QPushButton (tr ("Cancel"), this, "btCancel");
         connect (mBtOk, SIGNAL (clicked()), this, SLOT (accept()));
         connect (btCancel, SIGNAL (clicked()), this, SLOT (reject()));
@@ -255,6 +295,7 @@ public:
     {
         return mCbPermanent ? mCbPermanent->isChecked() : true;
     }
+    bool getWritable() { return !mCbReadonly->isChecked(); }
 
     void setPath (const QString &aPath) { mLePath->setText (aPath); }
     void setName (const QString &aName) { mLeName->setText (aName); }
@@ -266,6 +307,7 @@ public:
             mCbPermanent->setEnabled (!aPermanent);
         }
     }
+    void setWritable (bool aWritable) { mCbReadonly->setChecked (!aWritable); }
 
 private slots:
 
@@ -330,6 +372,7 @@ private:
     QLineEdit *mLePath;
     QLineEdit *mLeName;
     QCheckBox *mCbPermanent;
+    QCheckBox *mCbReadonly;
     SFoldersNameList mUsedNames;
 };
 
@@ -341,6 +384,7 @@ void VBoxSharedFoldersSettings::init()
     new QIListViewSelectionPreserver (this, listView);
     listView->setShowToolTips (false);
     listView->setRootIsDecorated (true);
+    listView->header()->setMovingEnabled (false);
     tbAdd->setIconSet (VBoxGlobal::iconSet ("add_shared_folder_16px.png",
                                             "add_shared_folder_disabled_16px.png"));
     tbEdit->setIconSet (VBoxGlobal::iconSet ("edit_shared_folder_16px.png",
@@ -352,10 +396,91 @@ void VBoxSharedFoldersSettings::init()
     connect (tbRemove, SIGNAL (clicked()), this, SLOT (tbRemovePressed()));
     connect (listView, SIGNAL (currentChanged (QListViewItem *)),
              this, SLOT (processCurrentChanged (QListViewItem *)));
-    connect (listView, SIGNAL (onItem (QListViewItem *)),
-             this, SLOT (processOnItem (QListViewItem *)));
+
+    /* Make after-paining list update to ensure all columns repainted correctly. */
+    connect (listView->header(), SIGNAL (sizeChange (int, int, int)),
+             this, SLOT (updateList()));
 
     mIsListViewChanged = false;
+
+    listView->viewport()->installEventFilter (this);
+
+    mTrFull = tr ("Full");
+    mTrReadOnly = tr ("Read-only");
+}
+
+void VBoxSharedFoldersSettings::showEvent (QShowEvent *aEvent)
+{
+    QWidget::showEvent (aEvent);
+
+    /* Adjusting size after all pending show events are processed. */
+    QTimer::singleShot (0, this, SLOT (adjustList()));
+}
+
+
+void VBoxSharedFoldersSettings::updateList()
+{
+    /* Updating list after all pending cell-repaint enevts. */
+    QTimer::singleShot (0, listView, SLOT (updateContents()));
+}
+
+void VBoxSharedFoldersSettings::adjustList()
+{
+    /* Adjust two columns size.
+     * Watching columns 0&2 to feat 1/3 of total width. */
+    int total = listView->viewport()->width();
+
+    listView->adjustColumn (0);
+    int w0 = listView->columnWidth (0) < total / 3 ?
+             listView->columnWidth (0) : total / 3;
+
+    listView->adjustColumn (2);
+    int w2 = listView->columnWidth (2) < total / 3 ?
+             listView->columnWidth (2) : total / 3;
+
+    /* We are adjusting columns 0 and 2 and resizing column 1 to feat
+     * visible listView' width according two adjusted columns. Due to
+     * adjusting column 2 influent column 0 restoring all widths. */
+    listView->setColumnWidth (0, w0);
+    listView->setColumnWidth (1, total - w0 - w2);
+    listView->setColumnWidth (2, w2);
+}
+
+bool VBoxSharedFoldersSettings::eventFilter (QObject *aObject, QEvent *aEvent)
+{
+    /* Process & show auto Tool-Tip for partially hidden listview items. */
+    if (aObject == listView->viewport() && aEvent->type() == QEvent::MouseMove)
+    {
+        QMouseEvent *e = static_cast<QMouseEvent*> (aEvent);
+        QListViewItem *i = listView->itemAt (e->pos());
+        VBoxRichListItem *item = i && i->rtti() == VBoxRichListItem::QIRichListItemId ?
+            static_cast<VBoxRichListItem*> (i) : 0;
+        if (item)
+        {
+            int delta = e->pos().x();
+            int id = 0;
+            for (; id < listView->columns(); ++ id)
+            {
+                if (delta < listView->columnWidth (id))
+                    break;
+                delta -= listView->columnWidth (id);
+            }
+
+            QString curText = QToolTip::textFor (listView->viewport());
+            QString newText = item->text (id) != item->getText (id) ?
+                              item->getText (id) : QString::null;
+
+            if (newText != curText)
+            {
+                QToolTip::remove (listView->viewport());
+                QToolTip::add (listView->viewport(), newText);
+            }
+        }
+        else
+            QToolTip::remove (listView->viewport());
+    }
+
+    return QWidget::eventFilter (aObject, aEvent);
 }
 
 void VBoxSharedFoldersSettings::setDialogType (int aType)
@@ -403,6 +528,7 @@ void VBoxSharedFoldersSettings::removeSharedFolder (const QString & aName,
 
 void VBoxSharedFoldersSettings::createSharedFolder (const QString & aName,
                                                     const QString & aPath,
+                                                    bool aWritable,
                                                     SFDialogType aType)
 {
     switch (aType)
@@ -416,7 +542,7 @@ void VBoxSharedFoldersSettings::createSharedFolder (const QString & aName,
         case MachineType:
         {
             Assert (!mMachine.isNull());
-            mMachine.CreateSharedFolder (aName, aPath);
+            mMachine.CreateSharedFolder (aName, aPath, aWritable);
             if (!mMachine.isOk())
                 vboxProblem().cannotCreateSharedFolder (this, mMachine,
                                                         aName, aPath);
@@ -425,7 +551,7 @@ void VBoxSharedFoldersSettings::createSharedFolder (const QString & aName,
         case ConsoleType:
         {
             Assert (!mConsole.isNull());
-            mConsole.CreateSharedFolder (aName, aPath);
+            mConsole.CreateSharedFolder (aName, aPath, aWritable);
             if (!mConsole.isOk())
                 vboxProblem().cannotCreateSharedFolder (this, mConsole,
                                                         aName, aPath);
@@ -448,7 +574,7 @@ void VBoxSharedFoldersSettings::getFromGlobal()
     QString name = tr (" Global Folders");
     QString key (QString::number (GlobalType));
     VBoxRichListItem *root = new VBoxRichListItem (VBoxRichListItem::EllipsisEnd,
-                                                   listView, name, QString::null, key);
+        listView, name, QString::null, QString::null, key);
     getFrom (vboxGlobal().virtualBox().GetSharedFolders().Enumerate(), root);
     */
 }
@@ -459,7 +585,7 @@ void VBoxSharedFoldersSettings::getFromMachine (const CMachine &aMachine)
     QString name = tr (" Machine Folders");
     QString key (QString::number (MachineType));
     VBoxRichListItem *root = new VBoxRichListItem (VBoxRichListItem::EllipsisEnd,
-                                                   listView, name, QString::null, key);
+        listView, name, QString::null, QString::null, key);
     getFrom (mMachine.GetSharedFolders().Enumerate(), root);
 }
 
@@ -469,7 +595,7 @@ void VBoxSharedFoldersSettings::getFromConsole (const CConsole &aConsole)
     QString name = tr (" Transient Folders");
     QString key (QString::number (ConsoleType));
     VBoxRichListItem *root = new VBoxRichListItem (VBoxRichListItem::EllipsisEnd,
-                                                   listView, name, QString::null, key);
+        listView, name, QString::null, QString::null, key);
     getFrom (mConsole.GetSharedFolders().Enumerate(), root);
 }
 
@@ -481,7 +607,9 @@ void VBoxSharedFoldersSettings::getFrom (const CSharedFolderEnumerator &aEn,
     {
         CSharedFolder sf = aEn.GetNext();
         new VBoxRichListItem (VBoxRichListItem::EllipsisFile, aRoot,
-                              sf.GetName(), sf.GetHostPath(), "not edited");
+                              sf.GetName(), sf.GetHostPath(),
+                              sf.GetWritable() ? mTrFull : mTrReadOnly,
+                              "not edited");
     }
     listView->setOpen (aRoot, true);
     listView->setCurrentItem (aRoot->firstChild() ? aRoot->firstChild() : aRoot);
@@ -499,7 +627,7 @@ void VBoxSharedFoldersSettings::putBackToGlobal()
     // This function is only available for GlobalType dialog
     Assert (mDialogType == GlobalType);
     // Searching for GlobalType item's root
-    QListViewItem *root = listView->findItem (QString::number (GlobalType), 2);
+    QListViewItem *root = listView->findItem (QString::number (GlobalType), 3);
     Assert (root);
     CSharedFolderEnumerator en = vboxGlobal().virtualBox().GetSharedFolders().Enumerate();
     putBackTo (en, root);
@@ -514,7 +642,7 @@ void VBoxSharedFoldersSettings::putBackToMachine()
     /* This function is only available for MachineType dialog */
     Assert (mDialogType & MachineType);
     /* Searching for MachineType item's root */
-    QListViewItem *root = listView->findItem (QString::number (MachineType), 2);
+    QListViewItem *root = listView->findItem (QString::number (MachineType), 3);
     Assert (root);
     CSharedFolderEnumerator en = mMachine.GetSharedFolders().Enumerate();
     putBackTo (en, root);
@@ -528,7 +656,7 @@ void VBoxSharedFoldersSettings::putBackToConsole()
     /* This function is only available for ConsoleType dialog */
     Assert (mDialogType & ConsoleType);
     /* Searching for ConsoleType item's root */
-    QListViewItem *root = listView->findItem (QString::number (ConsoleType), 2);
+    QListViewItem *root = listView->findItem (QString::number (ConsoleType), 3);
     Assert (root);
     CSharedFolderEnumerator en = mConsole.GetSharedFolders().Enumerate();
     putBackTo (en, root);
@@ -537,8 +665,8 @@ void VBoxSharedFoldersSettings::putBackToConsole()
 void VBoxSharedFoldersSettings::putBackTo (CSharedFolderEnumerator &aEn,
                                            QListViewItem *aRoot)
 {
-    Assert (!aRoot->text (2).isNull());
-    SFDialogType type = (SFDialogType)aRoot->text (2).toInt();
+    Assert (!aRoot->text (3).isNull());
+    SFDialogType type = (SFDialogType)aRoot->text (3).toInt();
 
     /* deleting all changed folders of the list */
     while (aEn.HasMore())
@@ -554,7 +682,7 @@ void VBoxSharedFoldersSettings::putBackTo (CSharedFolderEnumerator &aEn,
         {
             if (item->getText (0) == sf.GetName() &&
                 item->getText (1) == sf.GetHostPath() &&
-                item->getText (2) == "not edited")
+                item->getText (3) == "not edited")
                 break;
             item = item->nextSibling();
         }
@@ -571,8 +699,9 @@ void VBoxSharedFoldersSettings::putBackTo (CSharedFolderEnumerator &aEn,
         if (iterator->rtti() == VBoxRichListItem::QIRichListItemId)
             item = static_cast<VBoxRichListItem*> (iterator);
         if (item && !item->getText (0).isNull() && !item->getText (1).isNull()
-            && item->getText (2) == "edited")
-            createSharedFolder (item->getText (0), item->getText (1), type);
+            && item->getText (3) == "edited")
+            createSharedFolder (item->getText (0), item->getText (1),
+                                item->getText (2) == mTrFull ? true : false, type);
         iterator = iterator->nextSibling();
     }
 }
@@ -581,11 +710,11 @@ void VBoxSharedFoldersSettings::putBackTo (CSharedFolderEnumerator &aEn,
 QListViewItem* VBoxSharedFoldersSettings::searchRoot (bool aIsPermanent)
 {
     if (!aIsPermanent)
-        return listView->findItem (QString::number (ConsoleType), 2);
+        return listView->findItem (QString::number (ConsoleType), 3);
     else if (mDialogType & MachineType)
-        return listView->findItem (QString::number (MachineType), 2);
+        return listView->findItem (QString::number (MachineType), 3);
     else
-        return listView->findItem (QString::number (GlobalType), 2);
+        return listView->findItem (QString::number (GlobalType), 3);
 }
 
 void VBoxSharedFoldersSettings::tbAddPressed()
@@ -598,7 +727,7 @@ void VBoxSharedFoldersSettings::tbAddPressed()
         if ((*it)->parent() && (*it)->rtti() == VBoxRichListItem::QIRichListItemId)
         {
             VBoxRichListItem *item = static_cast<VBoxRichListItem*> (*it);
-            SFDialogType type = (SFDialogType)item->parent()->text (2).toInt();
+            SFDialogType type = (SFDialogType)item->parent()->text (3).toInt();
             usedList << qMakePair (item->getText (0), type);
         }
         ++ it;
@@ -619,7 +748,8 @@ void VBoxSharedFoldersSettings::tbAddPressed()
     Assert (root);
     /* Appending a new listview item to the root */
     VBoxRichListItem *item = new VBoxRichListItem (
-        VBoxRichListItem::EllipsisFile, root, name, path, "edited");
+        VBoxRichListItem::EllipsisFile, root, name, path,
+        dlg.getWritable() ? mTrFull : mTrReadOnly, "edited");
     /* Make the created item selected */
     listView->ensureItemVisible (item);
     listView->setCurrentItem (item);
@@ -640,7 +770,7 @@ void VBoxSharedFoldersSettings::tbEditPressed()
             (*it)->rtti() == VBoxRichListItem::QIRichListItemId)
         {
             VBoxRichListItem *item = static_cast<VBoxRichListItem*> (*it);
-            SFDialogType type = (SFDialogType)item->parent()->text (2).toInt();
+            SFDialogType type = (SFDialogType)item->parent()->text (3).toInt();
             usedList << qMakePair (item->getText (0), type);
         }
         ++ it;
@@ -658,8 +788,9 @@ void VBoxSharedFoldersSettings::tbEditPressed()
                          mDialogType & ConsoleType, usedList);
     dlg.setPath (item->getText (1));
     dlg.setName (item->getText (0));
-    dlg.setPermanent ((SFDialogType)item->parent()->text (2).toInt()
+    dlg.setPermanent ((SFDialogType)item->parent()->text (3).toInt()
                       != ConsoleType);
+    dlg.setWritable (item->getText (2) == mTrFull);
     if (dlg.exec() != QDialog::Accepted)
         return;
     QString name = dlg.getName();
@@ -671,7 +802,8 @@ void VBoxSharedFoldersSettings::tbEditPressed()
     QListViewItem *root = searchRoot (isPermanent);
     Assert (root);
     /* Updating an edited listview item */
-    item->updateText (2, "edited");
+    item->updateText (3, "edited");
+    item->updateText (2, dlg.getWritable() ? mTrFull : mTrReadOnly);
     item->updateText (1, path);
     item->updateText (0, name);
     if (item->parent() != root)
@@ -686,6 +818,7 @@ void VBoxSharedFoldersSettings::tbEditPressed()
         processCurrentChanged (item);
         listView->setFocus();
     }
+    item->repaint();
 
     mIsListViewChanged = true;
 }
@@ -698,30 +831,15 @@ void VBoxSharedFoldersSettings::tbRemovePressed()
 }
 
 
-void VBoxSharedFoldersSettings::processOnItem (QListViewItem *aItem)
-{
-    VBoxRichListItem *item = 0;
-    if (aItem->rtti() == VBoxRichListItem::QIRichListItemId)
-        item = static_cast<VBoxRichListItem*> (aItem);
-    Assert (item);
-    QString tip = tr ("<nobr>Name:&nbsp;&nbsp;%1</nobr><br>"
-                      "<nobr>Path:&nbsp;&nbsp;%2</nobr>")
-                      .arg (item->getText (0)).arg (item->getText (1));
-    if (!item->getText (0).isNull() && !item->getText (1).isNull())
-        QToolTip::add (listView->viewport(), listView->itemRect (aItem), tip);
-    else
-        QToolTip::remove (listView->viewport());
-}
-
 void VBoxSharedFoldersSettings::processCurrentChanged (QListViewItem *aItem)
 {
     if (aItem && aItem->isSelectable() && listView->selectedItem() != aItem)
         listView->setSelected (aItem, true);
     bool addEnabled = aItem &&
-                      (isEditable (aItem->text (2)) ||
-                       aItem->parent() && isEditable (aItem->parent()->text (2)));
+                      (isEditable (aItem->text (3)) ||
+                       (aItem->parent() && isEditable (aItem->parent()->text (3))));
     bool removeEnabled = aItem && aItem->parent() &&
-                         isEditable (aItem->parent()->text (2));
+                         isEditable (aItem->parent()->text (3));
     tbAdd->setEnabled (addEnabled);
     tbEdit->setEnabled (removeEnabled);
     tbRemove->setEnabled (removeEnabled);
@@ -730,7 +848,7 @@ void VBoxSharedFoldersSettings::processCurrentChanged (QListViewItem *aItem)
 void VBoxSharedFoldersSettings::processDoubleClick (QListViewItem *aItem)
 {
     bool editEnabled = aItem && aItem->parent() &&
-        isEditable (aItem->parent()->text (2));
+        isEditable (aItem->parent()->text (3));
     if (editEnabled)
         tbEditPressed();
 }
