@@ -395,8 +395,8 @@ private slots:
     {
         bool upEnabled   = aItem && aItem->isSelected() && aItem->itemAbove();
         bool downEnabled = aItem && aItem->isSelected() && aItem->itemBelow();
-        if (mBtnUp->hasFocus() && !upEnabled ||
-            mBtnDown->hasFocus() && !downEnabled)
+        if ((mBtnUp->hasFocus() && !upEnabled) ||
+            (mBtnDown->hasFocus() && !downEnabled))
             mBootTable->setFocus();
         mBtnUp->setEnabled (upEnabled);
         mBtnDown->setEnabled (downEnabled);
@@ -473,7 +473,11 @@ void VBoxVMSettingsDlg::init()
 {
     polished = false;
 
-    mResetFirstRunFlag = false;
+    /* disallow resetting First Run Wizard flag until media enumeration
+     * process is finished and all data is finally loaded into ui */
+    mAllowResetFirstRunFlag = false;
+    connect (&vboxGlobal(), SIGNAL (mediaEnumFinished (const VBoxMediaList &)),
+             this, SLOT (onMediaEnumerationDone()));
 
     setIcon (QPixmap::fromMimeSource ("settings_16px.png"));
 
@@ -717,6 +721,9 @@ void VBoxVMSettingsDlg::init()
 
     /* USB Page */
 
+    connect (cbEnableUSBController, SIGNAL (toggled (bool)),
+             this, SLOT (usbAdapterToggled (bool)));
+
     lvUSBFilters->header()->hide();
     /* disable sorting */
     lvUSBFilters->setSorting (-1);
@@ -824,6 +831,7 @@ void VBoxVMSettingsDlg::init()
     tblBootOrder = new BootItemsList (groupBox12, "tblBootOrder");
     connect (tblBootOrder, SIGNAL (bootSequenceChanged()),
              this, SLOT (resetFirstRunFlag()));
+
     /* Fixing focus order for BootItemsList */
     setTabOrder (tbwGeneral, tblBootOrder);
     setTabOrder (tblBootOrder->focusProxy(), chbEnableACPI);
@@ -834,6 +842,9 @@ void VBoxVMSettingsDlg::init()
     cbSharedClipboard->insertItem (vboxGlobal().toString (CEnums::ClipHostToGuest));
     cbSharedClipboard->insertItem (vboxGlobal().toString (CEnums::ClipGuestToHost));
     cbSharedClipboard->insertItem (vboxGlobal().toString (CEnums::ClipBidirectional));
+    /* IDE Controller Type */
+    cbIdeController->insertItem (vboxGlobal().toString (CEnums::IDEControllerPIIX3));
+    cbIdeController->insertItem (vboxGlobal().toString (CEnums::IDEControllerPIIX4));
 
     /* HDD Images page */
 
@@ -852,6 +863,9 @@ void VBoxVMSettingsDlg::init()
 #ifdef VBOX_WITH_ALSA
     cbAudioDriver->insertItem (vboxGlobal().toString (CEnums::ALSAAudioDriver));
 #endif
+#ifdef VBOX_WITH_PULSE
+    cbAudioDriver->insertItem (vboxGlobal().toString (CEnums::PulseAudioDriver));
+#endif
 #elif defined Q_OS_MACX
     cbAudioDriver->insertItem (vboxGlobal().toString (CEnums::CoreAudioDriver));
 #endif
@@ -859,6 +873,7 @@ void VBoxVMSettingsDlg::init()
     /* Network Page */
 
     loadInterfacesList();
+    loadNetworksList();
 
     /*
      *  update the Ok button state for pages with validation
@@ -876,7 +891,7 @@ void VBoxVMSettingsDlg::init()
     cbVRDPAuthType->insertItem (vboxGlobal().toString (CEnums::VRDPAuthGuest));
 }
 
-/** 
+/**
  *  Returns a path to the given page of this settings dialog. See ::path() for
  *  details.
  */
@@ -1001,6 +1016,31 @@ void VBoxVMSettingsDlg::loadInterfacesList()
 #endif
 }
 
+void VBoxVMSettingsDlg::loadNetworksList()
+{
+    /* clear inner list */
+    mNetworksList.clear();
+    /* loading internal networks list */
+    CVirtualBox vbox = vboxGlobal().virtualBox();
+    ulong count = vbox.GetSystemProperties().GetNetworkAdapterCount();
+    CMachineEnumerator en = vbox.GetMachines().Enumerate();
+    while (en.HasMore())
+    {
+        CMachine machine = en.GetNext();
+        if (machine.GetAccessible())
+        {
+            for (ulong slot = 0; slot < count; ++ slot)
+            {
+                CNetworkAdapter adapter = machine.GetNetworkAdapter (slot);
+                if (adapter.GetAttachmentType() == CEnums::InternalNetworkAttachment &&
+                        !mNetworksList.contains (adapter.GetInternalNetwork()))
+                    mNetworksList << adapter.GetInternalNetwork();
+            }
+        }
+    }
+    mLockNetworkListUpdate = false;
+}
+
 void VBoxVMSettingsDlg::hostInterfaceAdd()
 {
 #if defined Q_WS_WIN
@@ -1124,15 +1164,22 @@ void VBoxVMSettingsDlg::networkPageUpdate (QWidget *aWidget)
     if (!aWidget) return;
 #if defined Q_WS_WIN
     VBoxVMNetworkSettings *set = static_cast<VBoxVMNetworkSettings*> (aWidget);
-    set->loadList (mInterfaceList, mNoInterfaces);
+    set->loadInterfaceList (mInterfaceList, mNoInterfaces);
     set->revalidate();
 #endif
 }
 
 
+void VBoxVMSettingsDlg::onMediaEnumerationDone()
+{
+    mAllowResetFirstRunFlag = true;
+}
+
+
 void VBoxVMSettingsDlg::resetFirstRunFlag()
 {
-    mResetFirstRunFlag = true;
+    if (mAllowResetFirstRunFlag)
+        mResetFirstRunFlag = true;
 }
 
 
@@ -1469,18 +1516,26 @@ void VBoxVMSettingsDlg::revalidate (QIWidgetValidator *wval)
     else if (pg == pageNetwork)
     {
         QWidget *tab = NULL;
+        VBoxVMNetworkSettings::CheckPageResult error =
+                VBoxVMNetworkSettings::CheckPage_Ok;
         for (int index = 0; index < tbwNetwork->count(); ++ index)
         {
             tab = tbwNetwork->page (index);
             VBoxVMNetworkSettings *page =
                 static_cast <VBoxVMNetworkSettings *> (tab);
-            valid = page->isPageValid (mInterfaceList);
+            error = page->checkPage (mInterfaceList);
+            valid = !error;
             if (!valid) break;
         }
         if (!valid)
         {
             Assert (tab);
-            warningText = tr ("Incorrect host network interface is selected");
+            warningText =
+                error == VBoxVMNetworkSettings::CheckPage_InvalidInterface ?
+                    tr ("Incorrect host network interface is selected") :
+                error == VBoxVMNetworkSettings::CheckPage_NoNetworkName ?
+                    tr ("Internal network name is not set") :
+                QString::null;
             pageTitle += ": " + tbwNetwork->tabLabel (tab);
         }
     }
@@ -1496,11 +1551,8 @@ void VBoxVMSettingsDlg::revalidate (QIWidgetValidator *wval)
             VBoxVMSerialPortSettings *page =
                 static_cast <VBoxVMSerialPortSettings *> (tab);
 
-            /* skip disabled ports */
-            if (!page->mSerialPortBox->isChecked())
-                continue;
             /* check the predefined port number unicity */
-            if (!page->isUserDefined())
+            if (page->mSerialPortBox->isChecked() && !page->isUserDefined())
             {
                 QString port = page->mPortNumCombo->currentText();
                 valid = !ports.contains (port);
@@ -1512,16 +1564,18 @@ void VBoxVMSettingsDlg::revalidate (QIWidgetValidator *wval)
                 }
                 ports << port;
             }
-            /* check the port path unicity */
+            /* check the port path emptiness & unicity */
             CEnums::PortMode mode =
                 vboxGlobal().toPortMode (page->mHostModeCombo->currentText());
             if (mode != CEnums::DisconnectedPort)
             {
                 QString path = page->mPortPathLine->text();
-                valid = !paths.contains (path);
+                valid = !path.isEmpty() && !paths.contains (path);
                 if (!valid)
                 {
-                    warningText = tr ("Duplicate port path is entered ");
+                    warningText = path.isEmpty() ?
+                        tr ("Port path is not specified ") :
+                        tr ("Duplicate port path is entered ");
                     pageTitle += ": " + tbwSerialPorts->tabLabel (tab);
                     break;
                 }
@@ -1584,6 +1638,9 @@ void VBoxVMSettingsDlg::getFromMachine (const CMachine &machine)
     /* Shared clipboard mode */
     cbSharedClipboard->setCurrentItem (machine.GetClipboardMode());
 
+    /* IDE controller type */
+    cbIdeController->setCurrentText (vboxGlobal().toString (biosSettings.GetIDEControllerType()));
+
     /* other features */
     QString saveRtimeImages = cmachine.GetExtraData (VBoxDefs::GUI_SaveMountedAtRuntime);
     chbRememberMedia->setChecked (saveRtimeImages != "no");
@@ -1834,6 +1891,8 @@ void VBoxVMSettingsDlg::getFromMachine (const CMachine &machine)
         else
         {
             cbEnableUSBController->setChecked (ctl.GetEnabled());
+            cbEnableUSBEhci->setChecked (ctl.GetEnabledEhci());
+            usbAdapterToggled (cbEnableUSBController->isChecked());
 
             CUSBDeviceFilterEnumerator en = ctl.GetDeviceFilters().Enumerate();
             while (en.HasMore())
@@ -1891,6 +1950,11 @@ void VBoxVMSettingsDlg::getFromMachine (const CMachine &machine)
     wvalDVD->revalidate();
     wvalFloppy->revalidate();
     wvalVRDP->revalidate();
+
+    /* finally set the reset First Run Wizard flag to "false" to make sure
+     * user will see this dialog if he hasn't change the boot-order
+     * and/or mounted images configuration */
+    mResetFirstRunFlag = false;
 }
 
 
@@ -1943,6 +2007,9 @@ COMResult VBoxVMSettingsDlg::putBackToMachine()
     /* Shared clipboard mode */
     cmachine.SetClipboardMode ((CEnums::ClipboardMode)cbSharedClipboard->currentItem());
 
+    /* IDE controller type */
+    biosSettings.SetIDEControllerType (vboxGlobal().toIDEControllerType (cbIdeController->currentText()));
+
     /* other features */
     cmachine.SetExtraData (VBoxDefs::GUI_SaveMountedAtRuntime,
                            chbRememberMedia->isChecked() ? "yes" : "no");
@@ -2098,6 +2165,7 @@ COMResult VBoxVMSettingsDlg::putBackToMachine()
             /* the USB controller may be unavailable (i.e. in VirtualBox OSE) */
 
             ctl.SetEnabled (cbEnableUSBController->isChecked());
+            ctl.SetEnabledEhci (cbEnableUSBEhci->isChecked());
 
             /*
              *  first, remove all old filters (only if the list is changed,
@@ -2174,7 +2242,11 @@ void VBoxVMSettingsDlg::showVDImageManager (QUuid *id, VBoxMediaComboBox *cbb, Q
     VBoxDiskImageManagerDlg dlg (this, "VBoxDiskImageManagerDlg",
                                  WType_Dialog | WShowModal);
     QUuid machineId = cmachine.GetId();
-    dlg.setup (type, true, &machineId, true /* aRefresh */, cmachine);
+    QUuid hdId = type == VBoxDefs::HD ? cbb->getId() : QUuid();
+    QUuid cdId = type == VBoxDefs::CD ? cbb->getId() : QUuid();
+    QUuid fdId = type == VBoxDefs::FD ? cbb->getId() : QUuid();
+    dlg.setup (type, true, &machineId, true /* aRefresh */, cmachine,
+               hdId, cdId, fdId);
     if (dlg.exec() == VBoxDiskImageManagerDlg::Accepted)
     {
         *id = dlg.getSelectedUuid();
@@ -2197,7 +2269,8 @@ void VBoxVMSettingsDlg::showVDImageManager (QUuid *id, VBoxMediaComboBox *cbb, Q
 void VBoxVMSettingsDlg::addNetworkAdapter (const CNetworkAdapter &aAdapter)
 {
     VBoxVMNetworkSettings *page = new VBoxVMNetworkSettings();
-    page->loadList (mInterfaceList, mNoInterfaces);
+    page->loadInterfaceList (mInterfaceList, mNoInterfaces);
+    page->loadNetworksList (mNetworksList);
     page->getFromAdapter (aAdapter);
     QString pageTitle = QString (tr ("Adapter %1", "network"))
                                  .arg (aAdapter.GetSlot());
@@ -2216,6 +2289,12 @@ void VBoxVMSettingsDlg::addNetworkAdapter (const CNetworkAdapter &aAdapter)
     connect (page->grbEnabled, SIGNAL (toggled (bool)), wval, SLOT (revalidate()));
     connect (page->cbNetworkAttachment, SIGNAL (activated (const QString &)),
              wval, SLOT (revalidate()));
+    connect (page->cbInternalNetworkName, SIGNAL (activated (const QString &)),
+             wval, SLOT (revalidate()));
+    connect (page->cbInternalNetworkName, SIGNAL (textChanged (const QString &)),
+             this, SLOT (updateNetworksList()));
+    connect (page->cbInternalNetworkName, SIGNAL (textChanged (const QString &)),
+             wval, SLOT (revalidate()));
     connect (wval, SIGNAL (validityChanged (const QIWidgetValidator *)),
              this, SLOT (enableOk (const QIWidgetValidator *)));
     connect (wval, SIGNAL (isValidRequested (QIWidgetValidator *)),
@@ -2236,6 +2315,35 @@ void VBoxVMSettingsDlg::addNetworkAdapter (const CNetworkAdapter &aAdapter)
 #endif
 }
 
+void VBoxVMSettingsDlg::updateNetworksList()
+{
+    if (mLockNetworkListUpdate)
+        return;
+    mLockNetworkListUpdate = true;
+
+    QStringList curList (mNetworksList);
+    for (int index = 0; index < tbwNetwork->count(); ++ index)
+    {
+        VBoxVMNetworkSettings *pg = tbwNetwork->page (index) ?
+            static_cast <VBoxVMNetworkSettings*> (tbwNetwork->page (index)) : 0;
+        if (pg)
+        {
+            QString curText = pg->cbInternalNetworkName->currentText();
+            if (!curText.isEmpty() && !curList.contains (curText))
+                curList << curText;
+        }
+    }
+
+    for (int index = 0; index < tbwNetwork->count(); ++ index)
+    {
+        VBoxVMNetworkSettings *pg = tbwNetwork->page (index) ?
+            static_cast <VBoxVMNetworkSettings*> (tbwNetwork->page (index)) : 0;
+        pg->loadNetworksList (curList);
+    }
+
+    mLockNetworkListUpdate = false;
+}
+
 void VBoxVMSettingsDlg::addSerialPort (const CSerialPort &aPort)
 {
     VBoxVMSerialPortSettings *page = new VBoxVMSerialPortSettings();
@@ -2338,6 +2446,13 @@ void VBoxVMSettingsDlg::tbSelectSavedStateFolder_clicked()
 // USB Filter stuff
 ////////////////////////////////////////////////////////////////////////////////
 
+void VBoxVMSettingsDlg::usbAdapterToggled (bool aOn)
+{
+    if (!aOn)
+        cbEnableUSBEhci->setChecked (aOn);
+    grbUSBFilters->setEnabled (aOn);
+}
+
 void VBoxVMSettingsDlg::addUSBFilter (const CUSBDeviceFilter &aFilter, bool isNew)
 {
     QListViewItem *currentItem = isNew
