@@ -1,4 +1,4 @@
-/* $Revision: 4161 $ */
+/* $Revision: 27237 $ */
 /** @file
  * VirtualBox Support Driver - Internal header.
  */
@@ -27,6 +27,9 @@
 #include <iprt/assert.h>
 #include <iprt/asm.h>
 #include <VBox/sup.h>
+# ifdef VBOX_WITHOUT_IDT_PATCHING
+#  define VBOX_WITH_FAST_IOCTL
+# endif
 #ifdef USE_NEW_OS_INTERFACE
 # define USE_NEW_OS_INTERFACE_FOR_MM
 # define USE_NEW_OS_INTERFACE_FOR_GIP
@@ -86,6 +89,11 @@
 #   include <asm/semaphore.h>
 #   include <linux/timer.h>
 
+#   if 0
+#    include <linux/hrtimer.h>
+#    define VBOX_HRTIMER
+#   endif
+
 #elif defined(RT_OS_DARWIN)
 #   include <libkern/libkern.h>
 #   include <iprt/string.h>
@@ -97,6 +105,7 @@
 #   include <iprt/string.h>
 
 #elif defined(RT_OS_SOLARIS)
+#   include <sys/cmn_err.h>
 #   include <iprt/string.h>
 
 #else
@@ -186,7 +195,7 @@ __END_DECLS
  * Solaris
  */
 #elif defined(RT_OS_SOLARIS)
-# define OSDBGPRINT(a) printf a
+# define OSDBGPRINT(a) uprintf a
 
 
 #else
@@ -249,6 +258,14 @@ __END_DECLS
 /** Pointer to the device extension. */
 typedef struct SUPDRVDEVEXT *PSUPDRVDEVEXT;
 
+#ifdef RT_OS_LINUX
+# ifdef VBOX_HRTIMER
+typedef struct hrtimer    VBOXKTIMER;
+# else
+typedef struct timer_list VBOXKTIMER;
+# endif
+#endif
+
 #ifndef VBOX_WITHOUT_IDT_PATCHING
 
 /**
@@ -355,6 +372,8 @@ typedef enum
     MEMREF_TYPE_LOW,
     /** Memory block (r3 and r0 mapping). */
     MEMREF_TYPE_MEM,
+    /** Locked memory (r3 mapping only) allocated by the support driver. */
+    MEMREF_TYPE_LOCKED_SUP,
     /** Blow the type up to 32-bit and mark the end. */
     MEMREG_TYPE_32BIT_HACK = 0x7fffffff
 } SUPDRVMEMREFTYPE, *PSUPDRVMEMREFTYPE;
@@ -561,7 +580,8 @@ typedef struct SUPDRVSESSION
 #ifndef VBOX_WITHOUT_IDT_PATCHING
     /** Patch usage records. (protected by SUPDRVDEVEXT::SpinLock) */
     PSUPDRVPATCHUSAGE volatile  pPatchUsage;
-#else
+#endif
+#ifdef VBOX_WITH_FAST_IOCTL
     /** The VM associated with the session. */
     PVM                         pVM;
 #endif
@@ -639,7 +659,7 @@ typedef struct SUPDRVDEVEXT
      * 0 if the code VMM isn't loaded and Idt are nops. */
     void * volatile         pvVMMR0;
     /** VMMR0Entry() pointer. */
-    DECLCALLBACKMEMBER(int, pfnVMMR0Entry)(PVM pVM, unsigned uOperation, void *pvArg);
+    DECLR0CALLBACKMEMBER(int, pfnVMMR0Entry, (PVM pVM, unsigned uOperation, void *pvArg));
 
     /** Linked list of loaded code. */
     PSUPDRVLDRIMAGE volatile pLdrImages;
@@ -693,7 +713,7 @@ typedef struct SUPDRVDEVEXT
         /** The Linux Process ID. */
         unsigned            iSmpProcessorId;
         /** The per cpu timer. */
-        struct timer_list   Timer;
+        VBOXKTIMER          Timer;
     }                       aCPUs[256];
 # endif
 #endif
@@ -733,7 +753,7 @@ bool VBOXCALL   supdrvOSGetForcedAsyncTscMode(void);
 *******************************************************************************/
 int  VBOXCALL   supdrvIOCtl(unsigned uIOCtl, PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession,
                             void *pvIn, unsigned cbIn, void *pvOut, unsigned cbOut, unsigned *pcbReturned);
-#ifdef VBOX_WITHOUT_IDT_PATCHING
+#ifdef VBOX_WITH_FAST_IOCTL
 int  VBOXCALL   supdrvIOCtlFast(unsigned uIOCtl, PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession);
 #endif
 int  VBOXCALL   supdrvInitDevExt(PSUPDRVDEVEXT pDevExt);
