@@ -1,4 +1,4 @@
-/* $Revision: 4249 $ */
+/* $Revision: 24538 $ */
 /** @file
  * VirtualBox Support Driver - IOCtl definitions.
  */
@@ -133,6 +133,10 @@
 #define SUP_IOCTL_GIP_UNMAP         SUP_CTL_CODE(19)
 /** Set the VM handle for doing fast call ioctl calls. */
 #define SUP_IOCTL_SET_VM_FOR_FAST   SUP_CTL_CODE(20)
+/** Allocate memory and map into the user process. */
+#define SUP_IOCTL_PAGE_ALLOC        SUP_CTL_CODE(21)
+/** Free memory allocated with SUP_IOCTL_PAGE_ALLOC. */
+#define SUP_IOCTL_PAGE_FREE         SUP_CTL_CODE(22)
 
 /** Fast path IOCtl: VMMR0_DO_RAW_RUN */
 #define SUP_IOCTL_FAST_DO_RAW_RUN   SUP_CTL_CODE_FAST(64)
@@ -174,7 +178,7 @@ typedef struct SUPDRVIOCTLDATA
 /** Current interface version.
  * The upper 16-bit is the major version, the the lower the minor version.
  * When incompatible changes are made, the upper major number has to be changed. */
-#define SUPDRVIOC_VERSION           0x00050001
+#define SUPDRVIOC_VERSION           0x00050002
 
 /** SUP_IOCTL_COOKIE Input. */
 typedef struct SUPCOOKIE_IN
@@ -597,6 +601,41 @@ typedef struct SUPSETVMFORFAST_IN
     PVMR0           pVMR0;
 } SUPSETVMFORFAST_IN, *PSUPSETVMFORFAST_IN;
 
+
+
+/** SUP_IOCTL_PAGE_ALLOC Input. */
+typedef struct SUPPAGEALLOC_IN
+{
+    /** Cookie. */
+    uint32_t        u32Cookie;
+    /** Session cookie. */
+    uint32_t        u32SessionCookie;
+    /** Number of pages to allocate */
+    uint32_t        cPages;
+} SUPPAGEALLOC_IN, *PSUPPAGEALLOC_IN;
+
+/** SUP_IOCTL_PAGE_ALLOC Output. */
+typedef struct SUPPAGEALLOC_OUT
+{
+    /** Returned ring-3 address */
+    RTR3PTR         pvR3;
+    /** Array of pages. */
+    SUPPAGE         aPages[1];
+} SUPPAGEALLOC_OUT, *PSUPPAGEALLOC_OUT;
+
+
+
+/** SUP_IOCTL_PAGE_FREE Input. */
+typedef struct SUPPAGEFREE_IN
+{
+    /** Cookie. */
+    uint32_t        u32Cookie;
+    /** Session cookie. */
+    uint32_t        u32SessionCookie;
+    /** Address of memory range to free */
+    RTR3PTR         pvR3;
+} SUPPAGEFREE_IN, *PSUPPAGEFREE_IN;
+
 #pragma pack()                          /* paranoia */
 
 #endif
