@@ -20,6 +20,7 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include "SUPDRV.h"
+#include "the-linux-kernel.h"
 #include "version-generated.h"
 
 #include <iprt/assert.h>
@@ -31,19 +32,7 @@
 #include <iprt/mem.h>
 
 #include <linux/module.h>
-#include <linux/kernel.h>
-#include <linux/init.h>
-#include <linux/fs.h>
-#include <linux/mm.h>
-#include <linux/pagemap.h>
 #include <linux/sched.h>
-#include <linux/slab.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
-# include <linux/jiffies.h>
-#endif
-#include <asm/mman.h>
-#include <asm/io.h>
-#include <asm/uaccess.h>
 #ifdef CONFIG_DEVFS_FS
 # include <linux/devfs_fs_kernel.h>
 #endif
@@ -58,9 +47,6 @@
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
-# ifndef page_to_pfn
-#  define page_to_pfn(page) ((page) - mem_map)
-# endif
 # include <asm/pgtable.h>
 # define global_flush_tlb __flush_tlb_global
 #endif
@@ -180,7 +166,7 @@
  * The number of jiffies that make up 1 millisecond. This is only actually used
  * when HZ is > 1000. */
 #if HZ <= 1000
-# define ONE_MSEC_IN_JIFFIES       0
+# define ONE_MSEC_IN_JIFFIES       1
 #elif !(HZ % 1000)
 # define ONE_MSEC_IN_JIFFIES       (HZ / 1000)
 #else
@@ -228,6 +214,8 @@ extern int nmi_active;
 
 #endif /* CONFIG_X86_LOCAL_APIC */
 
+#define xstr(s) str(s)
+#define str(s) #s
 
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
@@ -238,7 +226,7 @@ extern int nmi_active;
 static SUPDRVDEVEXT     g_DevExt;
 
 /** Timer structure for the GIP update. */
-static struct timer_list g_GipTimer;
+static VBOXKTIMER       g_GipTimer;
 /** Pointer to the page structure for the GIP. */
 struct page            *g_pGipPage;
 
@@ -258,6 +246,9 @@ static devfs_handle_t   g_hDevFsVBoxDrv = NULL;
 static int              g_iModuleMajor;
 #endif /* !CONFIG_VBOXDRV_AS_MISC */
 
+/** Module parameter */
+static int force_async_tsc = 0;
+
 /** The module name. */
 #define DEVICE_NAME    "vboxdrv"
 
@@ -280,20 +271,38 @@ __asm__(".section execmemory, \"awx\", @progbits\n\t"
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
+#ifdef VBOX_HRTIMER
+typedef enum hrtimer_restart (*VBOXTIMERFN)(struct hrtimer *);
+#else
+typedef void (*VBOXTIMERFN)(unsigned long);
+#endif
+
 static int      VBoxSupDrvInit(void);
 static void     VBoxSupDrvUnload(void);
 static int      VBoxSupDrvCreate(struct inode *pInode, struct file *pFilp);
 static int      VBoxSupDrvClose(struct inode *pInode, struct file *pFilp);
-static int      VBoxSupDrvDeviceControl(struct inode *pInode, struct file *pFilp,
-                                        unsigned int IOCmd, unsigned long IOArg);
+#ifdef HAVE_UNLOCKED_IOCTL
+static long     VBoxSupDrvIOCtl(struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
+#else
+static int      VBoxSupDrvIOCtl(struct inode *pInode, struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
+#endif
+static int      VBoxSupDrvIOCtlSlow(struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
 #ifndef USE_NEW_OS_INTERFACE_FOR_MM
 static RTR3PTR  VBoxSupDrvMapUser(struct page **papPages, unsigned cPages, unsigned fProt, pgprot_t pgFlags);
 #endif /* !USE_NEW_OS_INTERFACE_FOR_MM */
 static int      VBoxSupDrvInitGip(PSUPDRVDEVEXT pDevExt);
 static int      VBoxSupDrvTermGip(PSUPDRVDEVEXT pDevExt);
+#ifdef VBOX_HRTIMER
+static enum hrtimer_restart VBoxSupGipTimer(struct hrtimer *timer);
+#else
 static void     VBoxSupGipTimer(unsigned long ulUser);
+#endif
 #ifdef CONFIG_SMP
+# ifdef VBOX_HRTIMER
+static enum hrtimer_restart VBoxSupGipTimerPerCpu(struct hrtimer *timer);
+# else
 static void     VBoxSupGipTimerPerCpu(unsigned long ulUser);
+# endif
 static void     VBoxSupGipResumePerCpu(void *pvUser);
 #endif
 static int      VBoxSupDrvErr2LinuxErr(int);
@@ -305,7 +314,11 @@ static struct file_operations gFileOpsVBoxDrv =
     owner:      THIS_MODULE,
     open:       VBoxSupDrvCreate,
     release:    VBoxSupDrvClose,
-    ioctl:      VBoxSupDrvDeviceControl,
+#ifdef HAVE_UNLOCKED_IOCTL
+    unlocked_ioctl: VBoxSupDrvIOCtl,
+#else
+    ioctl:      VBoxSupDrvIOCtl,
+#endif
 };
 
 #ifdef CONFIG_VBOXDRV_AS_MISC
@@ -322,6 +335,39 @@ static struct miscdevice gMiscDevice =
 };
 #endif
 
+static inline void vbox_ktimer_init(VBOXKTIMER *timer, VBOXTIMERFN function, unsigned long data)
+{
+#ifdef VBOX_HRTIMER
+    hrtimer_init(timer, CLOCK_MONOTONIC, HRTIMER_MODE_ABS);
+    timer->function = function;
+#else
+    init_timer(timer);
+    timer->data     = data;
+    timer->function = function;
+    timer->expires  = jiffies;
+#endif
+}
+
+static inline void vbox_ktimer_start(VBOXKTIMER *timer)
+{
+#ifdef VBOX_HRTIMER
+    hrtimer_start(timer, ktime_add_ns(ktime_get(), 1000000), HRTIMER_MODE_ABS);
+#else
+    mod_timer(timer, jiffies);
+#endif
+}
+
+static inline void vbox_ktimer_stop(VBOXKTIMER *timer)
+{
+#ifdef VBOX_HRTIMER
+    hrtimer_cancel(timer);
+#else
+    if (timer_pending(timer))
+        del_timer_sync(timer);
+#endif
+}
+
+
 #ifdef CONFIG_X86_LOCAL_APIC
 # ifdef DO_DISABLE_NMI
 
@@ -620,6 +666,16 @@ nmi_activated:
                 if (!rc)
                 {
                     dprintf(("VBoxDrv::ModuleInit returning %#x\n", rc));
+                    printk(KERN_INFO DEVICE_NAME ": TSC mode is %s, kernel timer mode is "
+#ifdef VBOX_HRTIMER
+                            "'high-res'"
+#else
+                            "'normal'"
+#endif
+                            ".\n",
+                           g_DevExt.pGip->u32Mode == SUPGIPMODE_SYNC_TSC ? "'synchronous'" : "'asynchronous'");
+                    printk(KERN_INFO DEVICE_NAME ": Successfully loaded version "
+                           VBOX_VERSION_STRING " (interface " xstr(SUPDRVIOC_VERSION) ").\n");
                     return rc;
                 }
 
@@ -740,13 +796,51 @@ static int VBoxSupDrvClose(struct inode *pInode, struct file *pFilp)
 /**
  * Device I/O Control entry point.
  *
- * @param   pInode      Pointer to inode info structure.
+ * @param   pFilp       Associated file pointer.
+ * @param   uCmd        The function specified to ioctl().
+ * @param   ulArg       The argument specified to ioctl().
+ */
+#ifdef HAVE_UNLOCKED_IOCTL
+static long VBoxSupDrvIOCtl(struct file *pFilp, unsigned int uCmd, unsigned long ulArg)
+#else
+static int VBoxSupDrvIOCtl(struct inode *pInode, struct file *pFilp, unsigned int uCmd, unsigned long ulArg)
+#endif
+{
+    /*
+     * Deal with the two high-speed IOCtl that takes it's arguments from
+     * the session and iCmd, and only returns a VBox status code.
+     */
+#ifdef HAVE_UNLOCKED_IOCTL
+    if (RT_LIKELY(   uCmd == SUP_IOCTL_FAST_DO_RAW_RUN
+                  || uCmd == SUP_IOCTL_FAST_DO_HWACC_RUN
+                  || uCmd == SUP_IOCTL_FAST_DO_NOP))
+        return supdrvIOCtlFast(uCmd, &g_DevExt, (PSUPDRVSESSION)pFilp->private_data);
+    return VBoxSupDrvIOCtlSlow(pFilp, uCmd, ulArg);
+
+#else   /* !HAVE_UNLOCKED_IOCTL */
+
+    int rc;
+    unlock_kernel();
+    if (RT_LIKELY(   uCmd == SUP_IOCTL_FAST_DO_RAW_RUN
+                  || uCmd == SUP_IOCTL_FAST_DO_HWACC_RUN
+                  || uCmd == SUP_IOCTL_FAST_DO_NOP))
+        rc = supdrvIOCtlFast(uCmd, &g_DevExt, (PSUPDRVSESSION)pFilp->private_data);
+    else
+        rc = VBoxSupDrvIOCtlSlow(pFilp, uCmd, ulArg);
+    lock_kernel();
+    return rc;
+#endif  /* !HAVE_UNLOCKED_IOCTL */
+}
+
+
+/**
+ * Device I/O Control entry point.
+ *
  * @param   pFilp       Associated file pointer.
  * @param   IOCmd       The function specified to ioctl().
  * @param   IOArg       The argument specified to ioctl().
  */
-static int VBoxSupDrvDeviceControl(struct inode *pInode, struct file *pFilp,
-                                   unsigned int IOCmd, unsigned long IOArg)
+static int VBoxSupDrvIOCtlSlow(struct file *pFilp, unsigned int IOCmd, unsigned long IOArg)
 {
     int                 rc;
     SUPDRVIOCTLDATA     Args;
@@ -754,19 +848,19 @@ static int VBoxSupDrvDeviceControl(struct inode *pInode, struct file *pFilp,
     int                 cbBuf = 0;
     unsigned            cbOut = 0;
 
-    dprintf2(("VBoxSupDrvDeviceControl: pFilp=%p IOCmd=%x IOArg=%p\n", pFilp, IOCmd, (void *)IOArg));
+    dprintf2(("VBoxSupDrvIOCtl: pFilp=%p IOCmd=%x IOArg=%p\n", pFilp, IOCmd, (void *)IOArg));
 
     /*
      * Copy ioctl data structure from user space.
      */
     if (_IOC_SIZE(IOCmd) != sizeof(SUPDRVIOCTLDATA))
     {
-        dprintf(("VBoxSupDrvDeviceControl: incorrect input length! cbArgs=%d\n", _IOC_SIZE(IOCmd)));
+        dprintf(("VBoxSupDrvIOCtl: incorrect input length! cbArgs=%d\n", _IOC_SIZE(IOCmd)));
         return -EINVAL;
     }
     if (copy_from_user(&Args, (void *)IOArg, _IOC_SIZE(IOCmd)))
     {
-        dprintf(("VBoxSupDrvDeviceControl: copy_from_user(&Args) failed.\n"));
+        dprintf(("VBoxSupDrvIOCtl: copy_from_user(&Args) failed.\n"));
         return -EFAULT;
     }
 
@@ -779,13 +873,13 @@ static int VBoxSupDrvDeviceControl(struct inode *pInode, struct file *pFilp,
         pvBuf = vmalloc(cbBuf);
         if (pvBuf == NULL)
         {
-            dprintf(("VBoxSupDrvDeviceControl: failed to allocate buffer of %d bytes.\n", cbBuf));
+            dprintf(("VBoxSupDrvIOCtl: failed to allocate buffer of %d bytes.\n", cbBuf));
             return -ENOMEM;
         }
 
         if (copy_from_user(pvBuf, (void *)Args.pvIn, Args.cbIn))
         {
-            dprintf(("VBoxSupDrvDeviceControl: copy_from_user(pvBuf) failed.\n"));
+            dprintf(("VBoxSupDrvIOCtl: copy_from_user(pvBuf) failed.\n"));
             vfree(pvBuf);
             return -EFAULT;
         }
@@ -802,7 +896,7 @@ static int VBoxSupDrvDeviceControl(struct inode *pInode, struct file *pFilp,
      */
     if (rc)
     {
-        dprintf(("VBoxSupDrvDeviceControl: pFilp=%p IOCmd=%x IOArg=%p failed, rc=%d (linux rc=%d)\n",
+        dprintf(("VBoxSupDrvIOCtl: pFilp=%p IOCmd=%x IOArg=%p failed, rc=%d (linux rc=%d)\n",
                  pFilp, IOCmd, (void *)IOArg, rc, VBoxSupDrvErr2LinuxErr(rc)));
         rc = VBoxSupDrvErr2LinuxErr(rc);
     }
@@ -826,7 +920,7 @@ static int VBoxSupDrvDeviceControl(struct inode *pInode, struct file *pFilp,
     if (pvBuf)
         vfree(pvBuf);
 
-    dprintf2(("VBoxSupDrvDeviceControl: returns %d\n", rc));
+    dprintf2(("VBoxSupDrvIOCtl: returns %d\n", rc));
     return rc;
 }
 
@@ -1487,20 +1581,14 @@ static int VBoxSupDrvInitGip(PSUPDRVDEVEXT pDevExt)
     /*
      * Initialize the timer.
      */
-    init_timer(&g_GipTimer);
-    g_GipTimer.data = (unsigned long)pDevExt;
-    g_GipTimer.function = VBoxSupGipTimer;
-    g_GipTimer.expires = jiffies;
+    vbox_ktimer_init(&g_GipTimer, VBoxSupGipTimer, (unsigned long)pDevExt);
 #ifdef CONFIG_SMP
     for (i = 0; i < RT_ELEMENTS(pDevExt->aCPUs); i++)
     {
         pDevExt->aCPUs[i].u64LastMonotime = pDevExt->u64LastMonotime;
         pDevExt->aCPUs[i].ulLastJiffies   = pDevExt->ulLastJiffies;
         pDevExt->aCPUs[i].iSmpProcessorId = -512;
-        init_timer(&pDevExt->aCPUs[i].Timer);
-        pDevExt->aCPUs[i].Timer.data      = i;
-        pDevExt->aCPUs[i].Timer.function  = VBoxSupGipTimerPerCpu;
-        pDevExt->aCPUs[i].Timer.expires   = jiffies;
+        vbox_ktimer_init(&pDevExt->aCPUs[i].Timer, VBoxSupGipTimerPerCpu, i);
     }
 #endif
 
@@ -1526,12 +1614,10 @@ static int VBoxSupDrvTermGip(PSUPDRVDEVEXT pDevExt)
     /*
      * Delete the timer if it's pending.
      */
-    if (timer_pending(&g_GipTimer))
-        del_timer_sync(&g_GipTimer);
+    vbox_ktimer_stop(&g_GipTimer);
 #ifdef CONFIG_SMP
     for (i = 0; i < RT_ELEMENTS(pDevExt->aCPUs); i++)
-        if (timer_pending(&pDevExt->aCPUs[i].Timer))
-            del_timer_sync(&pDevExt->aCPUs[i].Timer);
+        vbox_ktimer_stop(&pDevExt->aCPUs[i].Timer);
 #endif
 
     /*
@@ -1564,7 +1650,11 @@ static int VBoxSupDrvTermGip(PSUPDRVDEVEXT pDevExt)
  *
  * @param   ulUser  The device extension pointer.
  */
-static void     VBoxSupGipTimer(unsigned long ulUser)
+#ifdef VBOX_HRTIMER
+static enum hrtimer_restart VBoxSupGipTimer(struct hrtimer* timer)
+#else
+static void VBoxSupGipTimer(unsigned long ulUser)
+#endif
 {
     PSUPDRVDEVEXT       pDevExt;
     PSUPGLOBALINFOPAGE  pGip;
@@ -1572,12 +1662,20 @@ static void     VBoxSupGipTimer(unsigned long ulUser)
     unsigned long       ulDiff;
     uint64_t            u64Monotime;
     unsigned long       SavedFlags;
+#ifdef VBOX_HRTIMER
+    ktime_t             kNow;
+#endif
 
     local_irq_save(SavedFlags);
 
-    pDevExt = (PSUPDRVDEVEXT)ulUser;
-    pGip    = pDevExt->pGip;
     ulNow   = jiffies;
+#ifdef VBOX_HRTIMER
+    kNow    = ktime_get();
+    pDevExt = &g_DevExt;
+#else
+    pDevExt = (PSUPDRVDEVEXT)ulUser;
+#endif
+    pGip    = pDevExt->pGip;
 
 #ifdef CONFIG_SMP
     if (pGip && pGip->u32Mode == SUPGIPMODE_ASYNC_TSC)
@@ -1607,9 +1705,19 @@ static void     VBoxSupGipTimer(unsigned long ulUser)
     if (RT_LIKELY(pGip))
         supdrvGipUpdate(pDevExt->pGip, u64Monotime);
     if (RT_LIKELY(!pDevExt->fGIPSuspended))
-        mod_timer(&g_GipTimer, ulNow + (HZ <= 1000 ? 0 : ONE_MSEC_IN_JIFFIES));
+    {
+#ifdef VBOX_HRTIMER
+        hrtimer_forward(&g_GipTimer, kNow, ktime_set(0, 1000000));
+#else
+        mod_timer(&g_GipTimer, ulNow + ONE_MSEC_IN_JIFFIES);
+#endif
+    }
 
     local_irq_restore(SavedFlags);
+
+#ifdef VBOX_HRTIMER
+    return pDevExt->fGIPSuspended ? HRTIMER_NORESTART : HRTIMER_RESTART;
+#endif
 }
 
 
@@ -1619,37 +1727,56 @@ static void     VBoxSupGipTimer(unsigned long ulUser)
  *
  * @param   iTimerCPU     The APIC ID of this timer.
  */
+# ifdef VBOX_HRTIMER
+static enum hrtimer_restart VBoxSupGipTimerPerCpu(struct hrtimer *timer)
+# else
 static void VBoxSupGipTimerPerCpu(unsigned long iTimerCPU)
+# endif
 {
     PSUPDRVDEVEXT       pDevExt;
     PSUPGLOBALINFOPAGE  pGip;
     uint8_t             iCPU;
     uint64_t            u64Monotime;
     unsigned long       SavedFlags;
+    unsigned long       ulNow;
+# ifdef VBOX_HRTIMER
+    unsigned long       iTimerCPU;
+    ktime_t             kNow;
+# endif
 
     local_irq_save(SavedFlags);
 
+    ulNow   = jiffies;
     pDevExt = &g_DevExt;
     pGip    = pDevExt->pGip;
     iCPU    = ASMGetApicId();
+# ifdef VBOX_HRTIMER
+    iTimerCPU = iCPU; /* XXX hrtimer does not support a 'data' field */
+    kNow    = ktime_get();
+# endif
 
     if (RT_LIKELY(iCPU < RT_ELEMENTS(pGip->aCPUs)))
     {
         if (RT_LIKELY(iTimerCPU == iCPU))
         {
-            unsigned long   ulNow  = jiffies;
             unsigned long   ulDiff = ulNow - pDevExt->aCPUs[iCPU].ulLastJiffies;
             pDevExt->aCPUs[iCPU].ulLastJiffies = ulNow;
-#ifdef TICK_NSEC
+# ifdef TICK_NSEC
             u64Monotime = pDevExt->aCPUs[iCPU].u64LastMonotime + ulDiff * TICK_NSEC;
-#else
+# else
             u64Monotime = pDevExt->aCPUs[iCPU].u64LastMonotime + ulDiff * (1000000 / HZ);
-#endif
+# endif
             pDevExt->aCPUs[iCPU].u64LastMonotime = u64Monotime;
             if (RT_LIKELY(pGip))
                 supdrvGipUpdatePerCpu(pGip, u64Monotime, iCPU);
             if (RT_LIKELY(!pDevExt->fGIPSuspended))
-                mod_timer(&pDevExt->aCPUs[iCPU].Timer, ulNow + (HZ <= 1000 ? 0 : ONE_MSEC_IN_JIFFIES));
+            {
+# ifdef VBOX_HRTIMER
+                hrtimer_forward(&pDevExt->aCPUs[iCPU].Timer, kNow, ktime_set(0, 1000000));
+# else
+                mod_timer(&pDevExt->aCPUs[iCPU].Timer, ulNow + ONE_MSEC_IN_JIFFIES);
+# endif
+            }
         }
         else
             printk("vboxdrv: error: GIP CPU update timer executing on the wrong CPU: apicid=%d != timer-apicid=%ld (cpuid=%d !=? timer-cpuid=%d)\n",
@@ -1660,6 +1787,10 @@ static void VBoxSupGipTimerPerCpu(unsigned long iTimerCPU)
                iCPU, (unsigned long)RT_ELEMENTS(pGip->aCPUs), smp_processor_id());
 
     local_irq_restore(SavedFlags);
+
+# ifdef VBOX_HRTIMER
+    return pDevExt->fGIPSuspended ? HRTIMER_NORESTART : HRTIMER_RESTART;
+# endif
 }
 #endif  /* CONFIG_SMP */
 
@@ -1772,12 +1903,14 @@ void  VBOXCALL  supdrvOSGipResume(PSUPDRVDEVEXT pDevExt)
     ASMAtomicXchgU8(&pDevExt->fGIPSuspended, false);
 #ifdef CONFIG_SMP
     if (pDevExt->pGip->u32Mode != SUPGIPMODE_ASYNC_TSC)
+    {
 #endif
-        mod_timer(&g_GipTimer, jiffies);
+        vbox_ktimer_start(&g_GipTimer);
 #ifdef CONFIG_SMP
+    }
     else
     {
-        mod_timer(&g_GipTimer, jiffies);
+        vbox_ktimer_start(&g_GipTimer);
         smp_call_function(VBoxSupGipResumePerCpu, pDevExt, 0 /* retry */, 1 /* wait */);
     }
 #endif
@@ -1805,7 +1938,7 @@ static void VBoxSupGipResumePerCpu(void *pvUser)
     }
 
     pDevExt->aCPUs[iCPU].iSmpProcessorId = smp_processor_id();
-    mod_timer(&pDevExt->aCPUs[iCPU].Timer, jiffies);
+    vbox_ktimer_start(&pDevExt->aCPUs[iCPU].Timer);
 }
 #endif /* CONFIG_SMP */
 
@@ -1823,12 +1956,10 @@ void  VBOXCALL  supdrvOSGipSuspend(PSUPDRVDEVEXT pDevExt)
     dprintf2(("supdrvOSGipSuspend:\n"));
     ASMAtomicXchgU8(&pDevExt->fGIPSuspended, true);
 
-    if (timer_pending(&g_GipTimer))
-        del_timer_sync(&g_GipTimer);
+    vbox_ktimer_stop(&g_GipTimer);
 #ifdef CONFIG_SMP
     for (i = 0; i < RT_ELEMENTS(pDevExt->aCPUs); i++)
-        if (timer_pending(&pDevExt->aCPUs[i].Timer))
-            del_timer_sync(&pDevExt->aCPUs[i].Timer);
+        vbox_ktimer_stop(&pDevExt->aCPUs[i].Timer);
 #endif
 }
 
@@ -1840,8 +1971,10 @@ void  VBOXCALL  supdrvOSGipSuspend(PSUPDRVDEVEXT pDevExt)
 unsigned VBOXCALL supdrvOSGetCPUCount(void)
 {
 #ifdef CONFIG_SMP
-# ifdef num_present_cpus
+# if defined(num_present_cpus)
     return num_present_cpus();
+# elif defined(num_online_cpus)
+    return num_online_cpus();
 # else
     return smp_num_cpus;
 # endif
@@ -1856,7 +1989,7 @@ unsigned VBOXCALL supdrvOSGetCPUCount(void)
  */
 bool VBOXCALL  supdrvOSGetForcedAsyncTscMode(void)
 {
-    return false;
+    return force_async_tsc != 0;
 }
 
 
@@ -1944,7 +2077,13 @@ MODULE_AUTHOR("innotek GmbH");
 MODULE_DESCRIPTION("VirtualBox Support Driver");
 MODULE_LICENSE("GPL");
 #ifdef MODULE_VERSION
-#define xstr(s) str(s)
-#define str(s) #s
 MODULE_VERSION(VBOX_VERSION_STRING " (" xstr(SUPDRVIOC_VERSION) ")");
 #endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
+module_param(force_async_tsc, int, 0444);
+#else
+MODULE_PARM(force_async_tsc, "i");
+#endif
+MODULE_PARM_DESC(force_async_tsc, "force the asynchronous TSC mode");
+
