@@ -239,6 +239,21 @@ int     suplibOsIOCtl(unsigned uFunction, void *pvIn, size_t cbIn, void *pvOut,
 }
 
 
+/**
+ * Fast I/O Control path, no buffers.
+ *
+ * @returns VBox status code.
+ * @param   uFunction   The operation.
+ */
+int suplibOSIOCtlFast(unsigned uFunction)
+{
+    int rc = ioctl(g_hDevice, uFunction, NULL);
+    if (rc == -1)
+        rc = -errno;
+    return rc;
+}
+
+
 /**
  * Allocate a number of zero-filled pages in user space.
  *
