@@ -1,4 +1,4 @@
-/** $Id: SUPLib-win.cpp 4071 2007-08-07 17:07:59Z vboxsync $ */
+/** $Id: SUPLib-win.cpp 24812 2007-09-25 13:34:55Z sandervl $ */
 /** @file
  * VirtualBox Support Library - Windows NT specific parts.
  */
@@ -142,6 +142,7 @@ int     suplibOsInit(size_t cbReserve)
      */
     /** @todo implement driver version checking. */
 
+#if 0 /* obsolete code and restricts our virtual address space for the new allocation method */
     /*
      * Reserve memory.
      */
@@ -226,6 +227,7 @@ int     suplibOsInit(size_t cbReserve)
         }
         /* ignore errors */
     }
+#endif /* end of obsolete memory reservation hack */
 
     /*
      * We're done.
