@@ -762,6 +762,9 @@ static void vboxClipboardGetProc(Widget, XtPointer pClientData, Atom * /* select
     /* The X Toolkit may have failed to get the clipboard selection for us. */
     if (*atomType == XT_CONVERT_FAIL)
         return;
+    /* The clipboard selection may have changed before we could get it. */
+    if (NULL == pValue)
+        return;
     /* We grab this mutex whenever an asynchronous clipboard operation completes and while
        disconnecting a client from the clipboard to stop these operations colliding. */
     RTSemMutexRequest(g_ctx.asyncMutex, RT_INDEFINITE_WAIT);
@@ -1013,7 +1016,6 @@ static int vboxClipboardThread(RTTHREAD self, void * /* pvUser */)
     g_ctx.formatList.clear();
     RTSemEventDestroy(g_ctx.waitForData);
     RTSemMutexDestroy(g_ctx.asyncMutex);
-    XtCloseDisplay(pDisplay);
     LogRel (("vboxClipboardThread: clipboard thread terminated successfully with return code %Vrc\n", rc));
     return rc;
 }
@@ -1050,9 +1052,11 @@ void vboxClipboardDestroy (void)
     ev.xclient.type = ClientMessage;
     ev.xclient.format = 8;
     XSendEvent(XtDisplay(g_ctx.widget), XtWindow(g_ctx.widget), false, 0, &ev);
+    XFlush(XtDisplay(g_ctx.widget));
     rc = RTThreadWait(g_ctx.thread, 2000, &rcThread);
     AssertRC(rc);
     AssertRC(rcThread);
+    XtCloseDisplay(XtDisplay(g_ctx.widget));
     LogFlowFunc(("returning.\n"));
 }
 
@@ -1303,20 +1307,20 @@ static Boolean vboxClipboardConvertUtf8(Atom *atomTypeReturn, XtPointer *pValRet
         LogRel(("vboxClipboardConvertUtf8: clipboard conversion failed.  vboxClipboardUtf16WinToLin() returned %Vrc.  Abandoning.\n", rc));
         RTMemFree(reinterpret_cast<void *>(pu16DestText));
         vboxClipboardEmptyGuestBuffer();
-        return rc;
+        return false;
     }
-    /* Allocate as much space as the Utf16 string takes up, as RTUtf16ToUtf8Ex may fail if the
+    /* Allocate enough space, as RTUtf16ToUtf8Ex may fail if the
        space is too tightly calculated. */
-    pu8DestText = XtMalloc(cwDestLen * 2);
+    pu8DestText = XtMalloc(cwDestLen * 4);
     if (pu8DestText == 0)
     {
-        LogRel (("vboxClipboardConvertUtf8: failed to allocate %d bytes\n", cwDestLen * 2));
+        LogRel (("vboxClipboardConvertUtf8: failed to allocate %d bytes\n", cwDestLen * 4));
         RTMemFree(reinterpret_cast<void *>(pu16DestText));
         vboxClipboardEmptyGuestBuffer();
         return false;
     }
     /* Convert the Utf16 string to Utf8. */
-    rc = RTUtf16ToUtf8Ex(pu16DestText + 1, cwDestLen - 1, &pu8DestText, cwDestLen * 2,
+    rc = RTUtf16ToUtf8Ex(pu16DestText + 1, cwDestLen - 1, &pu8DestText, cwDestLen * 4,
                          &cbDestLen);
     RTMemFree(reinterpret_cast<void *>(pu16DestText));
     if (RT_FAILURE(rc))
@@ -1324,7 +1328,7 @@ static Boolean vboxClipboardConvertUtf8(Atom *atomTypeReturn, XtPointer *pValRet
         LogRel(("vboxClipboardConvertUtf8: clipboard conversion failed.  RTUtf16ToUtf8Ex() returned %Vrc.  Abandoning.\n", rc));
         XtFree(pu8DestText);
         vboxClipboardEmptyGuestBuffer();
-        return rc;
+        return false;
     }
     LogFlowFunc (("converted string is %.*s. Returning.\n", cbDestLen, pu8DestText));
     vboxClipboardEmptyGuestBuffer();
