@@ -20,6 +20,7 @@
 
 #include <iprt/alloc.h>
 #include <iprt/string.h>
+#include <iprt/asm.h>
 #include <iprt/assert.h>
 #include <iprt/thread.h>
 #include <process.h>
@@ -41,8 +42,6 @@ struct _VBOXCLIPBOARDCONTEXT
     HANDLE hRenderEvent;
     
     VBOXCLIPBOARDCLIENTDATA *pClient;
-    
-    bool fAnnouncing;
 };
 
 /* Only one client is supported. There seems to be no need for more clients. */
@@ -223,8 +222,9 @@ static LRESULT CALLBACK vboxClipboardWndProc(HWND hwnd, UINT msg, WPARAM wParam,
         {
             Log(("WM_DRAWCLIPBOARD next %p\n", pCtx->hwndNextInChain));
 
-            if (!pCtx->fAnnouncing)
+            if (GetClipboardOwner () != hwnd)
             {
+                /* Clipboard was updated by another application. */
                 vboxClipboardChanged (pCtx);
             }
 
@@ -380,25 +380,25 @@ static LRESULT CALLBACK vboxClipboardWndProc(HWND hwnd, UINT msg, WPARAM wParam,
             
             Log(("WM_USER u32Formats = %02X\n", u32Formats));
             
-            pCtx->fAnnouncing = true;
-
             if (OpenClipboard (hwnd))
             {
                 EmptyClipboard();
                 
                 Log(("WM_USER emptied clipboard\n"));
             
-                HANDLE hClip;
+                HANDLE hClip = NULL;
 
                 if (u32Formats & VBOX_SHARED_CLIPBOARD_FMT_UNICODETEXT)
                 {
                     dprintf(("window proc WM_USER: VBOX_SHARED_CLIPBOARD_FMT_UNICODETEXT\n"));
+                    
                     hClip = SetClipboardData (CF_UNICODETEXT, NULL);
                 }
 
                 if (u32Formats & VBOX_SHARED_CLIPBOARD_FMT_BITMAP)
                 {
                     dprintf(("window proc WM_USER: VBOX_SHARED_CLIPBOARD_FMT_BITMAP\n"));
+                    
                     hClip = SetClipboardData (CF_DIB, NULL);
                 }
 
@@ -414,15 +414,12 @@ static LRESULT CALLBACK vboxClipboardWndProc(HWND hwnd, UINT msg, WPARAM wParam,
 
                 CloseClipboard();
 
-                dprintf(("window proc WM_USER: hClip %p\n", hClip));
+                dprintf(("window proc WM_USER: hClip %p, err %d\n", hClip, GetLastError ()));
             }
             else
             {
                 dprintf(("window proc WM_USER: failed to open clipboard\n"));
             }
-            
-            pCtx->fAnnouncing = false;
-            
         } break;
 
         default:
@@ -604,6 +601,8 @@ int vboxClipboardReadData (VBOXCLIPBOARDCLIENTDATA *pClient, uint32_t u32Format,
      */
     if (OpenClipboard (pClient->pCtx->hwnd))
     {
+        dprintf(("Clipboard opened.\n"));
+        
         if (u32Format & VBOX_SHARED_CLIPBOARD_FMT_BITMAP)
         {
             hClip = GetClipboardData (CF_DIB);
