@@ -1,4 +1,4 @@
-/** $Id: ConsoleImpl2.cpp 4427 2007-08-29 23:18:56Z vboxsync $ */
+/** $Id: ConsoleImpl2.cpp 28195 2008-02-18 15:57:41Z frank $ */
 /** @file
  * VBox Console COM Class implementation
  *
@@ -93,7 +93,6 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
     HRESULT         hrc;
     char           *psz = NULL;
     BSTR            str = NULL;
-    ULONG           cRamMBs;
     unsigned        i;
 
 #define STR_CONV()  do { rc = RTStrUcs2ToUtf8(&psz, str); RC_CHECK(); } while (0)
@@ -101,8 +100,9 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
 #define RC_CHECK()  do { if (VBOX_FAILURE(rc)) { AssertMsgFailed(("rc=%Vrc\n", rc)); STR_FREE(); return rc; } } while (0)
 #define H()         do { if (FAILED(hrc)) { AssertMsgFailed(("hrc=%#x\n", hrc)); STR_FREE(); return VERR_GENERAL_FAILURE; } } while (0)
 
-    /* Get necessary objects */
-
+    /*
+     * Get necessary objects and frequently used parameters.
+     */
     ComPtr<IVirtualBox> virtualBox;
     hrc = pMachine->COMGETTER(Parent)(virtualBox.asOutParam());                     H();
 
@@ -115,6 +115,13 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
     ComPtr<IBIOSSettings> biosSettings;
     hrc = pMachine->COMGETTER(BIOSSettings)(biosSettings.asOutParam());             H();
 
+    Guid uuid;
+    hrc = pMachine->COMGETTER(Id)(uuid.asOutParam());                               H();
+    PCRTUUID pUuid = uuid.raw();
+
+    ULONG cRamMBs;
+    hrc = pMachine->COMGETTER(MemorySize)(&cRamMBs);                                H();
+
 
     /*
      * Get root node first.
@@ -130,7 +137,7 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
     STR_CONV();
     rc = CFGMR3InsertString(pRoot,  "Name",                 psz);                   RC_CHECK();
     STR_FREE();
-    hrc = pMachine->COMGETTER(MemorySize)(&cRamMBs);                                H();
+    rc = CFGMR3InsertBytes(pRoot,   "UUID", pUuid, sizeof(*pUuid));                 RC_CHECK();
     rc = CFGMR3InsertInteger(pRoot, "RamSize",              cRamMBs * _1M);         RC_CHECK();
     rc = CFGMR3InsertInteger(pRoot, "TimerMillies",         10);                    RC_CHECK();
     rc = CFGMR3InsertInteger(pRoot, "RawR3Enabled",         1);     /* boolean */   RC_CHECK();
@@ -162,6 +169,28 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
     BOOL fIOAPIC;
     hrc = biosSettings->COMGETTER(IOAPICEnabled)(&fIOAPIC);                          H();
 
+    BOOL fPXEDebug;
+    hrc = biosSettings->COMGETTER(PXEDebugEnabled)(&fPXEDebug);                      H();
+
+    /*
+     * Virtual IDE controller type.
+     */
+    IDEControllerType_T controllerType;
+    BOOL fPIIX4;
+    hrc = biosSettings->COMGETTER(IDEControllerType)(&controllerType);               H();
+    switch (controllerType)
+    {
+        case IDEControllerType_IDEControllerPIIX3:
+            fPIIX4 = FALSE;
+            break;
+        case IDEControllerType_IDEControllerPIIX4:
+            fPIIX4 = TRUE;
+            break;
+        default:
+            AssertMsgFailed(("Invalid IDE controller type '%d'", controllerType));
+            return VERR_INVALID_PARAMETER;
+    }
+
     /*
      * PDM config.
      *  Load drivers in VBoxC.[so|dll]
@@ -214,6 +243,8 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
     rc = CFGMR3InsertString(pCfg,   "HardDiskDevice",       "piix3ide");            RC_CHECK();
     rc = CFGMR3InsertString(pCfg,   "FloppyDevice",         "i82078");              RC_CHECK();
     rc = CFGMR3InsertInteger(pCfg,  "IOAPIC",               fIOAPIC);               RC_CHECK();
+    rc = CFGMR3InsertInteger(pCfg,  "PXEDebug",             fPXEDebug);             RC_CHECK();
+    rc = CFGMR3InsertBytes(pCfg,    "UUID", pUuid, sizeof(*pUuid));                 RC_CHECK();
 
     DeviceType_T bootDevice;
     if (SchemaDefs::MaxBootPosition > 9)
@@ -540,6 +571,7 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
     rc = CFGMR3InsertInteger(pInst, "PCIDeviceNo",          1);                     RC_CHECK();
     rc = CFGMR3InsertInteger(pInst, "PCIFunctionNo",        1);                     RC_CHECK();
     rc = CFGMR3InsertNode(pInst,    "Config", &pCfg);                               RC_CHECK();
+    rc = CFGMR3InsertInteger(pCfg,  "PIIX4", fPIIX4);               /* boolean */   RC_CHECK();
 
     /* Attach the status driver */
     rc = CFGMR3InsertNode(pInst,    "LUN#999", &pLunL0);                            RC_CHECK();
@@ -773,8 +805,12 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
     /*
      * Network adapters
      */
-    rc = CFGMR3InsertNode(pDevices, "pcnet", &pDev);                                RC_CHECK();
-    //rc = CFGMR3InsertNode(pDevices, "ne2000", &pDev);                               RC_CHECK();
+    PCFGMNODE pDevPCNet = NULL;          /* PCNet-type devices */
+    rc = CFGMR3InsertNode(pDevices, "pcnet", &pDevPCNet);                           RC_CHECK();
+#ifdef VBOX_WITH_E1000
+    PCFGMNODE pDevE1000 = NULL;          /* E1000-type devices */
+    rc = CFGMR3InsertNode(pDevices, "e1000", &pDevE1000);                           RC_CHECK();
+#endif
     for (ULONG ulInstance = 0; ulInstance < SchemaDefs::NetworkAdapterCount; ulInstance++)
     {
         ComPtr<INetworkAdapter> networkAdapter;
@@ -784,6 +820,28 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
         if (!fEnabled)
             continue;
 
+        /*
+        * The virtual hardware type. Create appropriate device first.
+        */
+        NetworkAdapterType_T adapterType;
+        hrc = networkAdapter->COMGETTER(AdapterType)(&adapterType);                 H();
+        switch (adapterType)
+        {
+            case NetworkAdapterType_NetworkAdapterAm79C970A:
+            case NetworkAdapterType_NetworkAdapterAm79C973:
+                pDev = pDevPCNet;
+                break;
+#ifdef VBOX_WITH_E1000
+            case NetworkAdapterType_NetworkAdapter82540EM:
+                pDev = pDevE1000;
+                break;
+#endif
+            default:
+                AssertMsgFailed(("Invalid network adapter type '%d' for slot '%d'",
+                                 adapterType, ulInstance));
+                return VERR_GENERAL_FAILURE;
+        }
+
         char szInstance[4]; Assert(ulInstance <= 999);
         RTStrPrintf(szInstance, sizeof(szInstance), "%lu", ulInstance);
         rc = CFGMR3InsertNode(pDev, szInstance, &pInst);                            RC_CHECK();
@@ -796,8 +854,6 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
         /*
          * The virtual hardware type.
          */
-        NetworkAdapterType_T adapterType;
-        hrc = networkAdapter->COMGETTER(AdapterType)(&adapterType);                 H();
         switch (adapterType)
         {
             case NetworkAdapterType_NetworkAdapterAm79C970A:
@@ -806,10 +862,6 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
             case NetworkAdapterType_NetworkAdapterAm79C973:
                 rc = CFGMR3InsertInteger(pCfg, "Am79C973", 1);                      RC_CHECK();
                 break;
-            default:
-                AssertMsgFailed(("Invalid network adapter type '%d' for slot '%d'",
-                                 adapterType, ulInstance));
-                return VERR_GENERAL_FAILURE;
         }
 
         /*
@@ -1191,6 +1243,13 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
                 break;
             }
 # endif
+# ifdef VBOX_WITH_PULSE
+            case AudioDriverType_PulseAudioDriver:
+            {
+                rc = CFGMR3InsertString(pCfg, "AudioDriver", "pulse");                  RC_CHECK();
+                break;
+            }
+# endif
 #endif /* RT_OS_LINUX */
 #ifdef RT_OS_DARWIN
             case AudioDriverType_CoreAudioDriver:
@@ -1200,6 +1259,10 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
             }
 #endif
         }
+        hrc = pMachine->COMGETTER(Name)(&str);                                          H();
+        STR_CONV();
+        rc = CFGMR3InsertString(pCfg,  "StreamName", psz);                              RC_CHECK();
+        STR_FREE();
     }
 
     /*
@@ -1233,6 +1296,47 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
             rc = CFGMR3InsertInteger(pCfg,  "papLeds", (uintptr_t)&pConsole->mapUSBLed);RC_CHECK();
             rc = CFGMR3InsertInteger(pCfg,  "First",    0);                             RC_CHECK();
             rc = CFGMR3InsertInteger(pCfg,  "Last",     0);                             RC_CHECK();
+
+#ifdef VBOX_WITH_EHCI
+            hrc = USBCtlPtr->COMGETTER(EnabledEhci)(&fEnabled);                         H();
+            if (fEnabled)
+            {
+                rc = CFGMR3InsertNode(pDevices, "usb-ehci", &pDev);                         RC_CHECK();
+                rc = CFGMR3InsertNode(pDev,     "0", &pInst);                               RC_CHECK();
+                rc = CFGMR3InsertNode(pInst,    "Config", &pCfg);                           RC_CHECK();
+                rc = CFGMR3InsertInteger(pInst, "Trusted",              1); /* boolean */   RC_CHECK();
+                rc = CFGMR3InsertInteger(pInst, "PCIDeviceNo",          11);                RC_CHECK();
+                rc = CFGMR3InsertInteger(pInst, "PCIFunctionNo",        0);                 RC_CHECK();
+
+                rc = CFGMR3InsertNode(pInst,    "LUN#0", &pLunL0);                          RC_CHECK();
+                rc = CFGMR3InsertString(pLunL0, "Driver",               "VUSBRootHub");     RC_CHECK();
+                rc = CFGMR3InsertNode(pLunL0,   "Config", &pCfg);                           RC_CHECK();
+
+                /*
+                 * Attach the status driver.
+                 */
+                rc = CFGMR3InsertNode(pInst,    "LUN#999", &pLunL0);                        RC_CHECK();
+                rc = CFGMR3InsertString(pLunL0, "Driver",               "MainStatus");      RC_CHECK();
+                rc = CFGMR3InsertNode(pLunL0,   "Config", &pCfg);                           RC_CHECK();
+                rc = CFGMR3InsertInteger(pCfg,  "papLeds", (uintptr_t)&pConsole->mapUSBLed);RC_CHECK();
+                rc = CFGMR3InsertInteger(pCfg,  "First",    0);                             RC_CHECK();
+                rc = CFGMR3InsertInteger(pCfg,  "Last",     0);                             RC_CHECK();
+            }
+            else
+#endif
+            {
+                /*
+                 * Enable the 2.0 -> 1.1 device morphing of proxied devies to keep windows quiet.
+                 */
+                rc = CFGMR3InsertNode(pRoot, "USB", &pCfg);                                RC_CHECK();
+                rc = CFGMR3InsertNode(pCfg, "USBProxy", &pCfg);                            RC_CHECK();
+                rc = CFGMR3InsertNode(pCfg, "GlobalConfig", &pCfg);                        RC_CHECK();
+                rc = CFGMR3InsertInteger(pCfg, "Force11Device", true);                     RC_CHECK();
+                // The following breaks stuff, but it makes MSDs work in vista. (I include it here so
+                // that it's documented somewhere.) Users needing it can use:
+                //      VBoxManage setextradata "myvm" "VBoxInternal/USB/USBProxy/GlobalConfig/Force11PacketSize" 1
+                //rc = CFGMR3InsertInteger(pCfg, "Force11PacketSize", true);                  RC_CHECK();
+            }
         }
     }
 
@@ -1388,7 +1492,10 @@ DECLCALLBACK(int) Console::configConstructor(PVM pVM, void *pvConsole)
         {
             /* if it's a valid number, we'll insert it as such, otherwise string */
             uint64_t u64Value;
-            if (RTStrToUInt64Ex(pszCFGMValue, NULL, 0, &u64Value) == VINF_SUCCESS)
+            char *pszNext = NULL;
+            if (   RTStrToUInt64Ex(pszCFGMValue, &pszNext, 0, &u64Value) == VINF_SUCCESS
+                && (!pszNext || *pszNext == '\0') /* check if the _whole_ string is a valid number */
+                )
             {
                 rc = CFGMR3InsertInteger(pNode, pszCFGMValueName, u64Value);
             }
