@@ -28,6 +28,7 @@
 
 #include <iprt/asm.h>
 #include <iprt/ldr.h>
+#include <iprt/alloca.h>
 
 #include <VBox/err.h>
 
@@ -90,6 +91,10 @@ public:
 
     STDMETHOD(OnKeyboardLedsChange)(BOOL fNumLock, BOOL fCapsLock, BOOL fScrollLock)
     {
+        if (m_server)
+        {
+            m_server->NotifyKeyboardLedsChange (fNumLock, fCapsLock, fScrollLock);
+        }
         return S_OK;
     }
 
@@ -379,12 +384,12 @@ static void findTopLeftBorder (const uint8_t *pu8AndMask, const uint8_t *pu8XorM
 static void mousePointerGenerateANDMask (uint8_t *pu8DstAndMask, int cbDstAndMask, const uint8_t *pu8SrcAlpha, int w, int h)
 {
     memset (pu8DstAndMask, 0xFF, cbDstAndMask);
-    
+
     int y;
     for (y = 0; y < h; y++)
     {
         uint8_t bitmask = 0x80;
-            
+
         int x;
         for (x = 0; x < w; x++, bitmask >>= 1)
         {
@@ -392,14 +397,14 @@ static void mousePointerGenerateANDMask (uint8_t *pu8DstAndMask, int cbDstAndMas
             {
                 bitmask = 0x80;
             }
-            
+
             /* Whether alpha channel value is not transparent enough for the pixel to be seen. */
             if (pu8SrcAlpha[x * 4 + 3] > 0x7f)
             {
                 pu8DstAndMask[x / 8] &= ~bitmask;
             }
         }
-    
+
         /* Point to next source and dest scans. */
         pu8SrcAlpha += w * 4;
         pu8DstAndMask += (w + 7) / 8;
@@ -642,19 +647,44 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
 
         case VRDP_QP_NETWORK_ADDRESS:
         {
-            com::Bstr address;
-            server->mConsole->getVRDPServer ()->COMGETTER(NetAddress) (address.asOutParam());
+            com::Bstr bstr;
+            server->mConsole->getVRDPServer ()->COMGETTER(NetAddress) (bstr.asOutParam());
             
-            if (cbBuffer >= address.length ())
+            /* The server expects UTF8. */
+            com::Utf8Str address = bstr;
+
+            size_t cbAddress = address.length () + 1;
+            
+            if (cbAddress >= 0x10000)
             {
-                if (address.length () > 0)
-                {
-                   memcpy (pvBuffer, address.raw(), address.length ());
-                }
-                rc = VINF_SUCCESS;
+                /* More than 64K seems to be an  invalid address. */
+                rc = VERR_TOO_MUCH_DATA;
+                break;
             }
             
-            *pcbOut = address.length ();
+            if ((size_t)cbBuffer >= cbAddress)
+            {
+                if (cbAddress > 0)
+                {
+                    if (address.raw())
+                    {
+                        memcpy (pvBuffer, address.raw(), cbAddress);
+                    }
+                    else
+                    {
+                        /* The value is an empty string. */
+                        *(uint8_t *)pvBuffer = 0;
+                    }
+                }
+
+                rc = VINF_SUCCESS;
+            }
+            else
+            {
+                rc = VINF_BUFFER_OVERFLOW;
+            }
+
+            *pcbOut = (uint32_t)cbAddress;
         } break;
 
         case VRDP_QP_NUMBER_MONITORS:
@@ -662,7 +692,7 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
             ULONG cMonitors = 1;
 
             server->mConsole->machine ()->COMGETTER(MonitorCount)(&cMonitors);
-    
+
             if (cbBuffer >= sizeof (uint32_t))
             {
                 *(uint32_t *)pvBuffer = (uint32_t)cMonitors;
@@ -686,28 +716,28 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackQueryProperty (void *pvCallbac
 DECLCALLBACK(int) ConsoleVRDPServer::VRDPCallbackClientLogon (void *pvCallback, uint32_t u32ClientId, const char *pszUser, const char *pszPassword, const char *pszDomain)
 {
     ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
-    
+
     return server->mConsole->VRDPClientLogon (u32ClientId, pszUser, pszPassword, pszDomain);
 }
 
 DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackClientConnect (void *pvCallback, uint32_t u32ClientId)
 {
     ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
-    
+
     server->mConsole->VRDPClientConnect (u32ClientId);
 }
 
 DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackClientDisconnect (void *pvCallback, uint32_t u32ClientId, uint32_t fu32Intercepted)
 {
     ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
-    
+
     server->mConsole->VRDPClientDisconnect (u32ClientId, fu32Intercepted);
 }
 
 DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackIntercept (void *pvCallback, uint32_t u32ClientId, uint32_t fu32Intercept, void **ppvIntercept)
 {
     ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
-    
+
     LogFlowFunc(("%x\n", fu32Intercept));
 
     int rc = VERR_NOT_SUPPORTED;
@@ -749,7 +779,11 @@ DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackIntercept (void *pvCallback, u
 
 DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackUSB (void *pvCallback, void *pvIntercept, uint32_t u32ClientId, uint8_t u8Code, const void *pvRet, uint32_t cbRet)
 {
+#ifdef VBOX_WITH_USB
     return USBClientResponseCallback (pvIntercept, u32ClientId, u8Code, pvRet, cbRet);
+#else
+    return VERR_NOT_SUPPORTED;
+#endif
 }
 
 DECLCALLBACK(int)  ConsoleVRDPServer::VRDPCallbackClipboard (void *pvCallback, void *pvIntercept, uint32_t u32ClientId, uint32_t u32Function, uint32_t u32Format, const void *pvData, uint32_t cbData)
@@ -797,9 +831,9 @@ DECLCALLBACK(bool) ConsoleVRDPServer::VRDPCallbackFramebufferQuery (void *pvCall
         pInfo->cHeight = height;
         pInfo->cBitsPerPixel = bitsPerPixel;
         pInfo->cbLine = lineSize;
-        
+
         pfb->Unlock ();
-        
+
         fAvailable = true;
     }
 
@@ -815,7 +849,7 @@ DECLCALLBACK(bool) ConsoleVRDPServer::VRDPCallbackFramebufferQuery (void *pvCall
 DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackFramebufferLock (void *pvCallback, unsigned uScreenId)
 {
     ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
-    
+
     if (server->maFramebuffers[uScreenId])
     {
         server->maFramebuffers[uScreenId]->Lock ();
@@ -825,13 +859,31 @@ DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackFramebufferLock (void *pvCallb
 DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackFramebufferUnlock (void *pvCallback, unsigned uScreenId)
 {
     ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
-    
+
     if (server->maFramebuffers[uScreenId])
     {
         server->maFramebuffers[uScreenId]->Unlock ();
     }
 }
 
+static void fixKbdLockStatus (VRDPInputSynch *pInputSynch, IKeyboard *pKeyboard)
+{
+    if (   pInputSynch->cGuestNumLockAdaptions
+        && (pInputSynch->fGuestNumLock != pInputSynch->fClientNumLock))
+    {
+        pInputSynch->cGuestNumLockAdaptions--;
+        pKeyboard->PutScancode(0x45);
+        pKeyboard->PutScancode(0x45 | 0x80);
+    }
+    if (   pInputSynch->cGuestCapsLockAdaptions
+        && (pInputSynch->fGuestCapsLock != pInputSynch->fClientCapsLock))
+    {
+        pInputSynch->cGuestCapsLockAdaptions--;
+        pKeyboard->PutScancode(0x3a);
+        pKeyboard->PutScancode(0x3a | 0x80);
+    }
+}
+
 DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackInput (void *pvCallback, int type, const void *pvInput, unsigned cbInput)
 {
     ConsoleVRDPServer *server = static_cast <ConsoleVRDPServer *> (pvCallback);
@@ -843,8 +895,30 @@ DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackInput (void *pvCallback, int t
         {
             if (cbInput == sizeof (VRDPINPUTSCANCODE))
             {
+                IKeyboard *pKeyboard = pConsole->getKeyboard ();
+
                 const VRDPINPUTSCANCODE *pInputScancode = (VRDPINPUTSCANCODE *)pvInput;
-                pConsole->getKeyboard ()->PutScancode((LONG)pInputScancode->uScancode);
+
+                /* Track lock keys. */
+                if (pInputScancode->uScancode == 0x45)
+                {
+                    server->m_InputSynch.fClientNumLock = !server->m_InputSynch.fClientNumLock;
+                }
+                else if (pInputScancode->uScancode == 0x3a)
+                {
+                    server->m_InputSynch.fClientCapsLock = !server->m_InputSynch.fClientCapsLock;
+                }
+                else if (pInputScancode->uScancode == 0x46)
+                {
+                    server->m_InputSynch.fClientScrollLock = !server->m_InputSynch.fClientScrollLock;
+                }
+                else if ((pInputScancode->uScancode & 0x80) == 0)
+                {
+                    /* Key pressed. */
+                    fixKbdLockStatus (&server->m_InputSynch, pKeyboard);
+                }
+
+                pKeyboard->PutScancode((LONG)pInputScancode->uScancode);
             }
         } break;
 
@@ -904,6 +978,36 @@ DECLCALLBACK(void) ConsoleVRDPServer::VRDPCallbackInput (void *pvCallback, int t
             pConsole->Reset();
         } break;
 
+        case VRDP_INPUT_SYNCH:
+        {
+            if (cbInput == sizeof (VRDPINPUTSYNCH))
+            {
+                IKeyboard *pKeyboard = pConsole->getKeyboard ();
+
+                const VRDPINPUTSYNCH *pInputSynch = (VRDPINPUTSYNCH *)pvInput;
+
+                server->m_InputSynch.fClientNumLock    = (pInputSynch->uLockStatus & VRDP_INPUT_SYNCH_NUMLOCK) != 0;
+                server->m_InputSynch.fClientCapsLock   = (pInputSynch->uLockStatus & VRDP_INPUT_SYNCH_CAPITAL) != 0;
+                server->m_InputSynch.fClientScrollLock = (pInputSynch->uLockStatus & VRDP_INPUT_SYNCH_SCROLL) != 0;
+
+                /* The client initiated synchronization. Always make the guest to reflect the client state. 
+                 * Than means, when the guest changes the state itself, it is forced to return to the client
+                 * state.
+                 */
+                if (server->m_InputSynch.fClientNumLock != server->m_InputSynch.fGuestNumLock)
+                {
+                    server->m_InputSynch.cGuestNumLockAdaptions = 2;
+                }
+
+                if (server->m_InputSynch.fClientCapsLock != server->m_InputSynch.fGuestCapsLock)
+                {
+                    server->m_InputSynch.cGuestCapsLockAdaptions = 2;
+                }
+
+                fixKbdLockStatus (&server->m_InputSynch, pKeyboard);
+            }
+        } break;
+
         default:
             break;
     }
@@ -923,14 +1027,14 @@ ConsoleVRDPServer::ConsoleVRDPServer (Console *console)
 
     int rc = RTCritSectInit (&mCritSect);
     AssertRC (rc);
-    
+
     mcClipboardRefs = 0;
     mpfnClipboardCallback = NULL;
 
 #ifdef VBOX_WITH_USB
     mUSBBackends.pHead = NULL;
     mUSBBackends.pTail = NULL;
-    
+
     mUSBBackends.thread = NIL_RTTHREAD;
     mUSBBackends.fThreadRunning = false;
     mUSBBackends.event = 0;
@@ -938,14 +1042,25 @@ ConsoleVRDPServer::ConsoleVRDPServer (Console *console)
 
 #ifdef VBOX_VRDP
     mhServer = 0;
-    
+
 #ifdef VRDP_NO_COM
     m_fGuestWantsAbsolute = false;
     m_mousex = 0;
     m_mousey = 0;
+
+    m_InputSynch.cGuestNumLockAdaptions = 2;
+    m_InputSynch.cGuestCapsLockAdaptions = 2;
+
+    m_InputSynch.fGuestNumLock    = false;
+    m_InputSynch.fGuestCapsLock   = false;
+    m_InputSynch.fGuestScrollLock = false;
+    
+    m_InputSynch.fClientNumLock    = false;
+    m_InputSynch.fClientCapsLock   = false;
+    m_InputSynch.fClientScrollLock = false;
     
     memset (maFramebuffers, 0, sizeof (maFramebuffers));
-    
+
     mConsoleCallback = new VRDPConsoleCallback(this);
     mConsoleCallback->AddRef();
     console->RegisterCallback(mConsoleCallback);
@@ -996,27 +1111,34 @@ int ConsoleVRDPServer::Launch (void)
     HRESULT rc2 = vrdpserver->COMGETTER(Enabled) (&vrdpEnabled);
     AssertComRC(rc2);
 
-    if (SUCCEEDED (rc2)
-        && vrdpEnabled
-        && loadVRDPLibrary ())
+    if (SUCCEEDED (rc2) && vrdpEnabled)
     {
+        if (loadVRDPLibrary ())
+        {
 #ifdef VRDP_NO_COM
-        rc = mpfnVRDPCreateServer (&mCallbacks.header, this, (VRDPINTERFACEHDR **)&mpEntryPoints, &mhServer);
+            rc = mpfnVRDPCreateServer (&mCallbacks.header, this, (VRDPINTERFACEHDR **)&mpEntryPoints, &mhServer);
 #else
-        rc = mpfnVRDPStartServer(mConsole, vrdpserver, &mhServer);
+            rc = mpfnVRDPStartServer(mConsole, vrdpserver, &mhServer);
 #endif /* VRDP_NO_COM */
 
-        if (VBOX_SUCCESS(rc))
-        {
+            if (VBOX_SUCCESS(rc))
+            {
 #ifdef VBOX_WITH_USB
-            remoteUSBThreadStart ();
+                remoteUSBThreadStart ();
 #endif /* VBOX_WITH_USB */
+            }
+            else
+                AssertMsgFailed(("Could not start VRDP server: rc = %Vrc\n", rc));
         }
         else
-            AssertMsgFailed(("Could not start VRDP server: rc = %Vrc\n", rc));
+        {
+            AssertMsgFailed(("Could not load the VRDP library\n"));
+            rc = VERR_FILE_NOT_FOUND;
+        }
     }
 #else
     int rc = VERR_NOT_SUPPORTED;
+    LogRel(("VRDP: this version does not include the VRDP server.\n"));
 #endif /* VBOX_VRDP */
     return rc;
 }
@@ -1123,13 +1245,13 @@ static DECLCALLBACK(int) threadRemoteUSB (RTTHREAD self, void *pvUser)
     ConsoleVRDPServer *pOwner = (ConsoleVRDPServer *)pvUser;
 
     LogFlow(("Console::threadRemoteUSB: start. owner = %p.\n", pOwner));
-    
+
     pOwner->notifyRemoteUSBThreadRunning (self);
 
     while (pOwner->isRemoteUSBThreadRunning ())
     {
         RemoteUSBBackend *pRemoteUSBBackend = NULL;
-        
+
         while ((pRemoteUSBBackend = pOwner->usbBackendGetNext (pRemoteUSBBackend)) != NULL)
         {
             pRemoteUSBBackend->PollRemoteDevices ();
@@ -1150,12 +1272,12 @@ void ConsoleVRDPServer::notifyRemoteUSBThreadRunning (RTTHREAD thread)
     int rc = RTThreadUserSignal (thread);
     AssertRC (rc);
 }
-    
+
 bool ConsoleVRDPServer::isRemoteUSBThreadRunning (void)
 {
     return mUSBBackends.fThreadRunning;
 }
-    
+
 void ConsoleVRDPServer::waitRemoteUSBThreadEvent (unsigned cMillies)
 {
     int rc = RTSemEventWait (mUSBBackends.event, cMillies);
@@ -1172,13 +1294,13 @@ void ConsoleVRDPServer::remoteUSBThreadStart (void)
         AssertFailed ();
         mUSBBackends.event = 0;
     }
-    
+
     if (VBOX_SUCCESS (rc))
     {
         rc = RTThreadCreate (&mUSBBackends.thread, threadRemoteUSB, this, 65536,
                              RTTHREADTYPE_VRDP_IO, RTTHREADFLAGS_WAITABLE, "remote usb");
     }
-    
+
     if (VBOX_FAILURE (rc))
     {
         LogRel(("Warning: could not start the remote USB thread, rc = %Vrc!!!\n", rc));
@@ -1196,19 +1318,19 @@ void ConsoleVRDPServer::remoteUSBThreadStart (void)
 void ConsoleVRDPServer::remoteUSBThreadStop (void)
 {
     mUSBBackends.fThreadRunning = false;
-    
+
     if (mUSBBackends.thread != NIL_RTTHREAD)
     {
         Assert (mUSBBackends.event != 0);
-        
+
         RTSemEventSignal (mUSBBackends.event);
-        
+
         int rc = RTThreadWait (mUSBBackends.thread, 60000, NULL);
         AssertRC (rc);
-        
+
         mUSBBackends.thread = NIL_RTTHREAD;
     }
-    
+
     if (mUSBBackends.event)
     {
         RTSemEventDestroy (mUSBBackends.event);
@@ -1365,11 +1487,11 @@ DECLCALLBACK(int) ConsoleVRDPServer::ClipboardCallback (void *pvCallback,
                  pvCallback, u32ClientId, u32Function, u32Format, pvData, cbData));
 
     int rc = VINF_SUCCESS;
-    
+
     ConsoleVRDPServer *pServer = static_cast <ConsoleVRDPServer *>(pvCallback);
-    
+
     NOREF(u32ClientId);
-    
+
     switch (u32Function)
     {
         case VRDP_CLIPBOARD_FUNCTION_FORMAT_ANNOUNCE:
@@ -1381,8 +1503,8 @@ DECLCALLBACK(int) ConsoleVRDPServer::ClipboardCallback (void *pvCallback,
                                                 (void *)pvData,
                                                 cbData);
             }
-        } break; 
-        
+        } break;
+
         case VRDP_CLIPBOARD_FUNCTION_DATA_READ:
         {
             if (pServer->mpfnClipboardCallback)
@@ -1392,12 +1514,12 @@ DECLCALLBACK(int) ConsoleVRDPServer::ClipboardCallback (void *pvCallback,
                                                 (void *)pvData,
                                                 cbData);
             }
-        } break; 
+        } break;
 
-        default: 
+        default:
             rc = VERR_NOT_SUPPORTED;
     }
-    
+
     return rc;
 }
 
@@ -1408,14 +1530,14 @@ DECLCALLBACK(int) ConsoleVRDPServer::ClipboardServiceExtension (void *pvExtensio
 {
     LogFlowFunc(("pvExtension = %p, u32Function = %d, pvParms = %p, cbParms = %d\n",
                  pvExtension, u32Function, pvParms, cbParms));
-    
+
     int rc = VINF_SUCCESS;
-    
+
 #ifdef VBOX_VRDP
     ConsoleVRDPServer *pServer = static_cast <ConsoleVRDPServer *>(pvExtension);
-    
+
     VBOXCLIPBOARDEXTPARMS *pParms = (VBOXCLIPBOARDEXTPARMS *)pvParms;
-    
+
     switch (u32Function)
     {
         case VBOX_CLIPBOARD_EXT_FN_SET_CALLBACK:
@@ -1429,7 +1551,7 @@ DECLCALLBACK(int) ConsoleVRDPServer::ClipboardServiceExtension (void *pvExtensio
 #ifdef VRDP_NO_COM
             if (mpEntryPoints && pServer->mhServer)
             {
-                mpEntryPoints->VRDPClipboard (pServer->mhServer, 
+                mpEntryPoints->VRDPClipboard (pServer->mhServer,
                                               VRDP_CLIPBOARD_FUNCTION_FORMAT_ANNOUNCE,
                                               pParms->u32Format,
                                               NULL,
@@ -1439,7 +1561,7 @@ DECLCALLBACK(int) ConsoleVRDPServer::ClipboardServiceExtension (void *pvExtensio
 #else
             if (mpfnVRDPClipboard)
             {
-                mpfnVRDPClipboard (pServer->mhServer, 
+                mpfnVRDPClipboard (pServer->mhServer,
                                    VRDP_CLIPBOARD_FUNCTION_FORMAT_ANNOUNCE,
                                    pParms->u32Format,
                                    NULL,
@@ -1458,7 +1580,7 @@ DECLCALLBACK(int) ConsoleVRDPServer::ClipboardServiceExtension (void *pvExtensio
 #ifdef VRDP_NO_COM
             if (mpEntryPoints && pServer->mhServer)
             {
-                mpEntryPoints->VRDPClipboard (pServer->mhServer, 
+                mpEntryPoints->VRDPClipboard (pServer->mhServer,
                                               VRDP_CLIPBOARD_FUNCTION_DATA_READ,
                                               pParms->u32Format,
                                               pParms->pvData,
@@ -1468,7 +1590,7 @@ DECLCALLBACK(int) ConsoleVRDPServer::ClipboardServiceExtension (void *pvExtensio
 #else
             if (mpfnVRDPClipboard)
             {
-                mpfnVRDPClipboard (pServer->mhServer, 
+                mpfnVRDPClipboard (pServer->mhServer,
                                    VRDP_CLIPBOARD_FUNCTION_DATA_READ,
                                    pParms->u32Format,
                                    pParms->pvData,
@@ -1483,7 +1605,7 @@ DECLCALLBACK(int) ConsoleVRDPServer::ClipboardServiceExtension (void *pvExtensio
 #ifdef VRDP_NO_COM
             if (mpEntryPoints && pServer->mhServer)
             {
-                mpEntryPoints->VRDPClipboard (pServer->mhServer, 
+                mpEntryPoints->VRDPClipboard (pServer->mhServer,
                                               VRDP_CLIPBOARD_FUNCTION_DATA_WRITE,
                                               pParms->u32Format,
                                               pParms->pvData,
@@ -1493,7 +1615,7 @@ DECLCALLBACK(int) ConsoleVRDPServer::ClipboardServiceExtension (void *pvExtensio
 #else
             if (mpfnVRDPClipboard)
             {
-                mpfnVRDPClipboard (pServer->mhServer, 
+                mpfnVRDPClipboard (pServer->mhServer,
                                    VRDP_CLIPBOARD_FUNCTION_DATA_WRITE,
                                    pParms->u32Format,
                                    pParms->pvData,
@@ -1502,8 +1624,8 @@ DECLCALLBACK(int) ConsoleVRDPServer::ClipboardServiceExtension (void *pvExtensio
             }
 #endif /* VRDP_NO_COM */
         } break;
-        
-        default: 
+
+        default:
             rc = VERR_NOT_SUPPORTED;
     }
 #endif /* VBOX_VRDP */
@@ -1518,19 +1640,19 @@ void ConsoleVRDPServer::ClipboardCreate (uint32_t u32ClientId, PFNVRDPCLIPBOARDC
 #endif /* VRDP_NO_COM */
 {
     int rc = lockConsoleVRDPServer ();
-        
+
     if (VBOX_SUCCESS (rc))
     {
         if (mcClipboardRefs == 0)
         {
             rc = HGCMHostRegisterServiceExtension (&mhClipboard, "VBoxSharedClipboard", ClipboardServiceExtension, this);
-            
+
             if (VBOX_SUCCESS (rc))
             {
                 mcClipboardRefs++;
             }
         }
-        
+
 #ifdef VRDP_NO_COM
 #else
         if (VBOX_SUCCESS (rc))
@@ -1547,11 +1669,11 @@ void ConsoleVRDPServer::ClipboardCreate (uint32_t u32ClientId, PFNVRDPCLIPBOARDC
 void ConsoleVRDPServer::ClipboardDelete (uint32_t u32ClientId)
 {
     int rc = lockConsoleVRDPServer ();
-        
+
     if (VBOX_SUCCESS (rc))
     {
         mcClipboardRefs--;
-        
+
         if (mcClipboardRefs == 0)
         {
             HGCMHostUnregisterServiceExtension (mhClipboard);
@@ -1572,17 +1694,17 @@ void ConsoleVRDPServer::USBBackendCreate (uint32_t u32ClientId, PFNVRDPUSBCALLBA
 {
 #ifdef VBOX_WITH_USB
     LogFlow(("ConsoleVRDPServer::USBBackendCreate: u32ClientId = %d\n", u32ClientId));
-    
+
     /* Create a new instance of the USB backend for the new client. */
     RemoteUSBBackend *pRemoteUSBBackend = new RemoteUSBBackend (mConsole, this, u32ClientId);
 
     if (pRemoteUSBBackend)
     {
         pRemoteUSBBackend->AddRef (); /* 'Release' called in USBBackendDelete. */
-            
+
         /* Append the new instance in the list. */
         int rc = lockConsoleVRDPServer ();
-        
+
         if (VBOX_SUCCESS (rc))
         {
             pRemoteUSBBackend->pNext = mUSBBackends.pHead;
@@ -1594,11 +1716,11 @@ void ConsoleVRDPServer::USBBackendCreate (uint32_t u32ClientId, PFNVRDPUSBCALLBA
             {
                 mUSBBackends.pTail = pRemoteUSBBackend;
             }
-            
+
             mUSBBackends.pHead = pRemoteUSBBackend;
-            
+
             unlockConsoleVRDPServer ();
-            
+
 #ifdef VRDP_NO_COM
             if (ppvIntercept)
             {
@@ -1623,23 +1745,23 @@ void ConsoleVRDPServer::USBBackendDelete (uint32_t u32ClientId)
     LogFlow(("ConsoleVRDPServer::USBBackendDelete: u32ClientId = %d\n", u32ClientId));
 
     RemoteUSBBackend *pRemoteUSBBackend = NULL;
-    
+
     /* Find the instance. */
     int rc = lockConsoleVRDPServer ();
-        
+
     if (VBOX_SUCCESS (rc))
     {
         pRemoteUSBBackend = usbBackendFind (u32ClientId);
-        
+
         if (pRemoteUSBBackend)
         {
             /* Notify that it will be deleted. */
             pRemoteUSBBackend->NotifyDelete ();
         }
-        
+
         unlockConsoleVRDPServer ();
     }
-    
+
     if (pRemoteUSBBackend)
     {
         /* Here the instance has been excluded from the list and can be dereferenced. */
@@ -1652,19 +1774,19 @@ void *ConsoleVRDPServer::USBBackendRequestPointer (uint32_t u32ClientId, const G
 {
 #ifdef VBOX_WITH_USB
     RemoteUSBBackend *pRemoteUSBBackend = NULL;
-    
+
     /* Find the instance. */
     int rc = lockConsoleVRDPServer ();
-        
+
     if (VBOX_SUCCESS (rc))
     {
         pRemoteUSBBackend = usbBackendFind (u32ClientId);
-        
+
         if (pRemoteUSBBackend)
         {
             /* Inform the backend instance that it is referenced by the Guid. */
             bool fAdded = pRemoteUSBBackend->addUUID (pGuid);
-            
+
             if (fAdded)
             {
                 /* Reference the instance because its pointer is being taken. */
@@ -1675,15 +1797,15 @@ void *ConsoleVRDPServer::USBBackendRequestPointer (uint32_t u32ClientId, const G
                 pRemoteUSBBackend = NULL;
             }
         }
-        
+
         unlockConsoleVRDPServer ();
     }
-    
+
     if (pRemoteUSBBackend)
     {
         return pRemoteUSBBackend->GetBackendCallbackPointer ();
     }
-    
+
 #endif
     return NULL;
 }
@@ -1692,21 +1814,21 @@ void ConsoleVRDPServer::USBBackendReleasePointer (const Guid *pGuid)
 {
 #ifdef VBOX_WITH_USB
     RemoteUSBBackend *pRemoteUSBBackend = NULL;
-    
+
     /* Find the instance. */
     int rc = lockConsoleVRDPServer ();
-        
+
     if (VBOX_SUCCESS (rc))
     {
         pRemoteUSBBackend = usbBackendFindByUUID (pGuid);
-        
+
         if (pRemoteUSBBackend)
         {
             pRemoteUSBBackend->removeUUID (pGuid);
         }
-        
+
         unlockConsoleVRDPServer ();
-        
+
         if (pRemoteUSBBackend)
         {
             pRemoteUSBBackend->Release ();
@@ -1721,9 +1843,9 @@ RemoteUSBBackend *ConsoleVRDPServer::usbBackendGetNext (RemoteUSBBackend *pRemot
 
     RemoteUSBBackend *pNextRemoteUSBBackend = NULL;
 #ifdef VBOX_WITH_USB
-        
+
     int rc = lockConsoleVRDPServer ();
-        
+
     if (VBOX_SUCCESS (rc))
     {
         if (pRemoteUSBBackend == NULL)
@@ -1736,21 +1858,21 @@ RemoteUSBBackend *ConsoleVRDPServer::usbBackendGetNext (RemoteUSBBackend *pRemot
             /* Get pointer to the next backend. */
             pNextRemoteUSBBackend = (RemoteUSBBackend *)pRemoteUSBBackend->pNext;
         }
-        
+
         if (pNextRemoteUSBBackend)
         {
             pNextRemoteUSBBackend->AddRef ();
         }
-        
+
         unlockConsoleVRDPServer ();
-        
+
         if (pRemoteUSBBackend)
         {
             pRemoteUSBBackend->Release ();
         }
     }
 #endif
-    
+
     return pNextRemoteUSBBackend;
 }
 
@@ -1759,17 +1881,17 @@ RemoteUSBBackend *ConsoleVRDPServer::usbBackendGetNext (RemoteUSBBackend *pRemot
 RemoteUSBBackend *ConsoleVRDPServer::usbBackendFind (uint32_t u32ClientId)
 {
     RemoteUSBBackend *pRemoteUSBBackend = mUSBBackends.pHead;
-        
+
     while (pRemoteUSBBackend)
     {
         if (pRemoteUSBBackend->ClientId () == u32ClientId)
         {
             break;
         }
-            
+
         pRemoteUSBBackend = (RemoteUSBBackend *)pRemoteUSBBackend->pNext;
     }
-    
+
     return pRemoteUSBBackend;
 }
 
@@ -1777,17 +1899,17 @@ RemoteUSBBackend *ConsoleVRDPServer::usbBackendFind (uint32_t u32ClientId)
 RemoteUSBBackend *ConsoleVRDPServer::usbBackendFindByUUID (const Guid *pGuid)
 {
     RemoteUSBBackend *pRemoteUSBBackend = mUSBBackends.pHead;
-        
+
     while (pRemoteUSBBackend)
     {
         if (pRemoteUSBBackend->findUUID (pGuid))
         {
             break;
         }
-            
+
         pRemoteUSBBackend = (RemoteUSBBackend *)pRemoteUSBBackend->pNext;
     }
-    
+
     return pRemoteUSBBackend;
 }
 #endif
@@ -1798,7 +1920,7 @@ void ConsoleVRDPServer::usbBackendRemoveFromList (RemoteUSBBackend *pRemoteUSBBa
 #ifdef VBOX_WITH_USB
     int rc = lockConsoleVRDPServer ();
     AssertRC (rc);
-        
+
     /* Exclude the found instance from the list. */
     if (pRemoteUSBBackend->pNext)
     {
@@ -1808,7 +1930,7 @@ void ConsoleVRDPServer::usbBackendRemoveFromList (RemoteUSBBackend *pRemoteUSBBa
     {
         mUSBBackends.pTail = (RemoteUSBBackend *)pRemoteUSBBackend->pPrev;
     }
-                
+
     if (pRemoteUSBBackend->pPrev)
     {
         pRemoteUSBBackend->pPrev->pNext = pRemoteUSBBackend->pNext;
@@ -1817,9 +1939,9 @@ void ConsoleVRDPServer::usbBackendRemoveFromList (RemoteUSBBackend *pRemoteUSBBa
     {
         mUSBBackends.pHead = (RemoteUSBBackend *)pRemoteUSBBackend->pNext;
     }
-                
+
     pRemoteUSBBackend->pNext = pRemoteUSBBackend->pPrev = NULL;
-    
+
     unlockConsoleVRDPServer ();
 #endif
 }
