@@ -200,6 +200,9 @@ STDMETHODIMP NetworkAdapter::COMSETTER(AdapterType) (NetworkAdapterType_T aAdapt
     {
         case NetworkAdapterType_NetworkAdapterAm79C970A:
         case NetworkAdapterType_NetworkAdapterAm79C973:
+#ifdef VBOX_WITH_E1000
+        case NetworkAdapterType_NetworkAdapter82540EM:
+#endif
             break;
         default:
             return setError (E_FAIL,
@@ -325,15 +328,26 @@ STDMETHODIMP NetworkAdapter::COMSETTER(MACAddress)(INPTR BSTR aMACAddress)
              * Verify given MAC address
              */
             Utf8Str macAddressUtf = aMACAddress;
-            char *macAddressStr = (char*)macAddressUtf.raw();
+            char *macAddressStr = macAddressUtf.mutableRaw();
             int i = 0;
-            while ((i < 12) && macAddressStr && (rc == S_OK))
+            while ((i < 13) && macAddressStr && *macAddressStr && (rc == S_OK))
             {
                 char c = *macAddressStr;
+                /* canonicalize hex digits to capital letters */
+                if (c >= 'a' && c <= 'f')
+                {
+                    /** @todo the runtime lacks an ascii lower/upper conv */
+                    c &= 0xdf;
+                    *macAddressStr = c;
+                }
                 /* we only accept capital letters */
                 if (((c < '0') || (c > '9')) &&
                     ((c < 'A') || (c > 'F')))
                     rc = setError(E_INVALIDARG, tr("Invalid MAC address format"));
+                /* the second digit must have even value for unicast addresses */
+                if ((i == 1) && (!!(c & 1) == (c >= '0' && c <= '9')))
+                    rc = setError(E_INVALIDARG, tr("Invalid MAC address format"));
+
                 macAddressStr++;
                 i++;
             }
@@ -345,7 +359,7 @@ STDMETHODIMP NetworkAdapter::COMSETTER(MACAddress)(INPTR BSTR aMACAddress)
             {
                 mData.backup();
 
-                mData->mMACAddress = aMACAddress;
+                mData->mMACAddress = macAddressUtf;
                 emitChangeEvent = true;
             }
         }
@@ -729,9 +743,6 @@ STDMETHODIMP NetworkAdapter::COMGETTER(TraceFile) (BSTR *aTraceFile)
 
 STDMETHODIMP NetworkAdapter::COMSETTER(TraceFile) (INPTR BSTR aTraceFile)
 {
-    if (!aTraceFile)
-        return E_INVALIDARG;
-
     AutoCaller autoCaller (this);
     CheckComRCReturnRC (autoCaller.rc());
 
@@ -884,7 +895,7 @@ STDMETHODIMP NetworkAdapter::Detach()
 // public methods only for internal purposes
 ////////////////////////////////////////////////////////////////////////////////
 
-/** 
+/**
  *  @note Locks this object for writing.
  */
 bool NetworkAdapter::rollback()
@@ -908,7 +919,7 @@ bool NetworkAdapter::rollback()
     return changed;
 }
 
-/** 
+/**
  *  @note Locks this object for writing, together with the peer object (also
  *  for writing) if there is one.
  */
@@ -936,7 +947,7 @@ void NetworkAdapter::commit()
     }
 }
 
-/** 
+/**
  *  @note Locks this object for writing, together with the peer object
  *  represented by @a aThat (locked for reading).
  */
