@@ -209,8 +209,6 @@ STDMETHODIMP USBController::COMGETTER(Enabled) (BOOL *aEnabled)
 
     *aEnabled = mData->mEnabled;
 
-    LogFlowThisFunc (("returns %RTbool\n", *aEnabled));
-
     return S_OK;
 }
 
@@ -242,6 +240,48 @@ STDMETHODIMP USBController::COMSETTER(Enabled) (BOOL aEnabled)
     return S_OK;
 }
 
+STDMETHODIMP USBController::COMGETTER(EnabledEhci) (BOOL *aEnabled)
+{
+    if (!aEnabled)
+        return E_POINTER;
+
+    AutoCaller autoCaller (this);
+    CheckComRCReturnRC (autoCaller.rc());
+
+    AutoReaderLock alock (this);
+
+    *aEnabled = mData->mEnabledEhci;
+
+    return S_OK;
+}
+
+STDMETHODIMP USBController::COMSETTER(EnabledEhci) (BOOL aEnabled)
+{
+    LogFlowThisFunc (("aEnabled=%RTbool\n", aEnabled));
+
+    AutoCaller autoCaller (this);
+    CheckComRCReturnRC (autoCaller.rc());
+
+    /* the machine needs to be mutable */
+    Machine::AutoMutableStateDependency adep (mParent);
+    CheckComRCReturnRC (adep.rc());
+
+    AutoLock alock (this);
+
+    if (mData->mEnabledEhci != aEnabled)
+    {
+        mData.backup();
+        mData->mEnabledEhci = aEnabled;
+
+        /* leave the lock for safety */
+        alock.leave();
+
+        mParent->onUSBControllerChange();
+    }
+
+    return S_OK;
+}
+
 STDMETHODIMP USBController::COMGETTER(USBStandard) (USHORT *aUSBStandard)
 {
     if (!aUSBStandard)
@@ -252,10 +292,9 @@ STDMETHODIMP USBController::COMGETTER(USBStandard) (USHORT *aUSBStandard)
 
     /* not accessing data -- no need to lock */
 
+    /** Note: This is no longer correct */
     *aUSBStandard = 0x0101;
 
-    LogFlowThisFunc (("returns %04hX\n", *aUSBStandard));
-
     return S_OK;
 }
 
@@ -464,6 +503,10 @@ HRESULT USBController::loadSettings (CFGNODE aMachine)
     CFGLDRQueryBool (controller, "enabled", &enabled);
     mData->mEnabled = enabled;
 
+    /* enabledEhci */
+    CFGLDRQueryBool (controller, "enabledEhci", &enabled);
+    mData->mEnabledEhci = enabled;
+
     HRESULT rc = S_OK;
 
     unsigned filterCount = 0;
@@ -495,13 +538,16 @@ HRESULT USBController::loadSettings (CFGNODE aMachine)
         CFGLDRQueryBSTR (filter, "port", port.asOutParam());
         Bstr remote;
         CFGLDRQueryBSTR (filter, "remote", remote.asOutParam());
+        uint32_t maskedIfs;
+        if (RT_FAILURE(CFGLDRQueryUInt32 (filter, "maskedInterfaces", &maskedIfs)))
+            maskedIfs = 0;
 
         ComObjPtr <USBDeviceFilter> filterObj;
         filterObj.createObject();
         rc = filterObj->init (this,
                               name, active, vendorId, productId, revision,
                               manufacturer, product, serialNumber,
-                              port, remote);
+                              port, remote, maskedIfs);
         /* error info is set by init() when appropriate */
         if (SUCCEEDED (rc))
         {
@@ -546,6 +592,9 @@ HRESULT USBController::saveSettings (CFGNODE aMachine)
     /* enabled */
     CFGLDRSetBool (controller, "enabled", !!mData->mEnabled);
 
+    /* enabledEhci */
+    CFGLDRSetBool (controller, "enabledEhci", !!mData->mEnabledEhci);
+
     DeviceFilterList::const_iterator it = mDeviceFilters->begin();
     while (it != mDeviceFilters->end())
     {
@@ -611,6 +660,9 @@ HRESULT USBController::saveSettings (CFGNODE aMachine)
             CFGLDRSetBSTR (filter, "remote", data.mRemote.string());
 #endif /* VBOX_WITH_USBFILTER */
 
+        if (data.mMaskedIfs)
+            CFGLDRSetUInt32 (filter, "maskedInterfaces", data.mMaskedIfs);
+
         CFGLDRReleaseNode (filter);
 
         ++ it;
@@ -1055,7 +1107,7 @@ HRESULT USBController::onDeviceFilterChange (USBDeviceFilter *aFilter,
  *
  *  @note Locks this object for reading.
  */
-bool USBController::hasMatchingFilter (ComObjPtr <HostUSBDevice> &aDevice)
+bool USBController::hasMatchingFilter (const ComObjPtr <HostUSBDevice> &aDevice, ULONG *aMaskedIfs)
 {
     AutoCaller autoCaller (this);
     AssertComRCReturn (autoCaller.rc(), false);
@@ -1066,18 +1118,20 @@ bool USBController::hasMatchingFilter (ComObjPtr <HostUSBDevice> &aDevice)
     if (!mData->mEnabled)
         return false;
 
-    bool match = false;
-
     /* apply self filters */
     for (DeviceFilterList::const_iterator it = mDeviceFilters->begin();
-         !match && it != mDeviceFilters->end();
+         it != mDeviceFilters->end();
          ++ it)
     {
         AutoLock filterLock (*it);
-        match = aDevice->isMatch ((*it)->data());
+        if (aDevice->isMatch ((*it)->data()))
+        {
+            *aMaskedIfs = (*it)->data().mMaskedIfs;
+            return true;
+        }
     }
 
-    return match;
+    return false;
 }
 
 /**
@@ -1092,7 +1146,7 @@ bool USBController::hasMatchingFilter (ComObjPtr <HostUSBDevice> &aDevice)
  *
  *  @note Locks this object for reading.
  */
-bool USBController::hasMatchingFilter (IUSBDevice *aUSBDevice)
+bool USBController::hasMatchingFilter (IUSBDevice *aUSBDevice, ULONG *aMaskedIfs)
 {
     LogFlowThisFuncEnter();
 
@@ -1218,6 +1272,7 @@ bool USBController::hasMatchingFilter (IUSBDevice *aUSBDevice)
 #endif /* VBOX_WITH_USBFILTER */
 
         match = true;
+        *aMaskedIfs = aData.mMaskedIfs;
         break;
     }
 
