@@ -125,13 +125,15 @@ public:
     STDMETHOD(Pause)();
     STDMETHOD(Resume)();
     STDMETHOD(PowerButton)();
+    STDMETHOD(SleepButton)();
+    STDMETHOD(GetPowerButtonHandled)(BOOL *aHandled);
     STDMETHOD(SaveState) (IProgress **aProgress);
     STDMETHOD(DiscardSavedState)();
     STDMETHOD(GetDeviceActivity) (DeviceType_T aDeviceType,
                                  DeviceActivity_T *aDeviceActivity);
     STDMETHOD(AttachUSBDevice) (INPTR GUIDPARAM aId);
     STDMETHOD(DetachUSBDevice) (INPTR GUIDPARAM aId, IUSBDevice **aDevice);
-    STDMETHOD(CreateSharedFolder) (INPTR BSTR aName, INPTR BSTR aHostPath);
+    STDMETHOD(CreateSharedFolder) (INPTR BSTR aName, INPTR BSTR aHostPath, BOOL aWritable);
     STDMETHOD(RemoveSharedFolder) (INPTR BSTR aName);
     STDMETHOD(TakeSnapshot) (INPTR BSTR aName, INPTR BSTR aDescription,
                              IProgress **aProgress);
@@ -172,7 +174,7 @@ public:
     HRESULT onVRDPServerChange();
     HRESULT onUSBControllerChange();
     HRESULT onSharedFolderChange (BOOL aGlobal);
-    HRESULT onUSBDeviceAttach (IUSBDevice *aDevice, IVirtualBoxErrorInfo *aError);
+    HRESULT onUSBDeviceAttach (IUSBDevice *aDevice, IVirtualBoxErrorInfo *aError, ULONG aMaskedIfs);
     HRESULT onUSBDeviceDetach (INPTR GUIDPARAM aId, IVirtualBoxErrorInfo *aError);
 
     VMMDev *getVMMDev() { return mVMMDev; }
@@ -354,8 +356,21 @@ public:
      */
     typedef SafeVMPtrBase <true> SafeVMPtrQuiet;
 
+    class SharedFolderData
+    {
+    public:
+        SharedFolderData() {}
+        SharedFolderData(Bstr aHostPath, BOOL aWritable)
+           : mHostPath (aHostPath)
+           , mWritable (aWritable) {}
+        SharedFolderData(const SharedFolderData& aThat)
+           : mHostPath (aThat.mHostPath)
+           , mWritable (aThat.mWritable) {}
+        Bstr mHostPath;
+        BOOL mWritable;
+    };
     typedef std::map <Bstr, ComObjPtr <SharedFolder> > SharedFolderMap;
-    typedef std::map <Bstr, Bstr> SharedFolderDataMap;
+    typedef std::map <Bstr, SharedFolderData> SharedFolderDataMap;
 
 private:
 
@@ -384,10 +399,10 @@ private:
                               bool aSetError = false);
 
     HRESULT fetchSharedFolders (BOOL aGlobal);
-    bool findOtherSharedFolder (INPTR BSTR aName, 
+    bool findOtherSharedFolder (INPTR BSTR aName,
                                 SharedFolderDataMap::const_iterator &aIt);
 
-    HRESULT createSharedFolder (INPTR BSTR aName, INPTR BSTR aHostPath);
+    HRESULT createSharedFolder (INPTR BSTR aName, SharedFolderData aData);
     HRESULT removeSharedFolder (INPTR BSTR aName);
 
     static DECLCALLBACK(int) configConstructor(PVM pVM, void *pvConsole);
@@ -402,25 +417,16 @@ private:
                                           DriveState_T eState, DriveState_T *peState,
                                           const char *pszPath, bool fPassthrough);
 
-#if 1
-    HRESULT attachUSBDevice (IUSBDevice *aHostDevice, PVUSBIRHCONFIG aConfig);
-    HRESULT detachUSBDevice (USBDeviceList::iterator &aIt);
-
-    static DECLCALLBACK(int)
-    usbAttachCallback (Console *that, IUSBDevice *aHostDevice,
-                       PVUSBIRHCONFIG aConfig, PCRTUUID aUuid, bool aRemote,
-                       const char *aAddress, void *aRemoteBackend);
-#else /* PDMUsb coding. */
-    HRESULT attachUSBDevice (IUSBDevice *aHostDevice);
+#ifdef VBOX_WITH_USB
+    HRESULT attachUSBDevice (IUSBDevice *aHostDevice, ULONG aMaskedIfs);
     HRESULT detachUSBDevice (USBDeviceList::iterator &aIt);
 
     static DECLCALLBACK(int)
     usbAttachCallback (Console *that, IUSBDevice *aHostDevice, PCRTUUID aUuid,
-                       bool aRemote, const char *aAddress, void *aRemoteBackend);
-#endif /* PDMUsb coding. */
+                       bool aRemote, const char *aAddress, ULONG aMaskedIfs);
     static DECLCALLBACK(int)
-    usbDetachCallback (Console *that, USBDeviceList::iterator *aIt,
-                       PVUSBIRHCONFIG aConfig, PCRTUUID aUuid);
+    usbDetachCallback (Console *that, USBDeviceList::iterator *aIt, PCRTUUID aUuid);
+#endif
 
     static DECLCALLBACK (int)
     stateProgressCallback (PVM pVM, unsigned uPercent, void *pvUser);
