@@ -59,6 +59,7 @@ class MachineDebugger;
 class USBController;
 class Snapshot;
 class SharedFolder;
+class HostUSBDevice;
 
 class SessionMachine;
 
@@ -354,7 +355,7 @@ public:
         DECLARE_CLS_NEW_DELETE_NOOP (AutoStateDependency)
     };
 
-    /** 
+    /**
      *  Shortcut to AutoStateDependency <AnyStateDep>.
      *  See AutoStateDependency to get the usage pattern.
      *
@@ -367,7 +368,7 @@ public:
      */
     typedef AutoStateDependency <AnyStateDep> AutoAnyStateDependency;
 
-    /** 
+    /**
      *  Shortcut to AutoStateDependency <MutableStateDep>.
      *  See AutoStateDependency to get the usage pattern.
      *
@@ -384,7 +385,7 @@ public:
      */
     typedef AutoStateDependency <MutableStateDep> AutoMutableStateDependency;
 
-    /** 
+    /**
      *  Shortcut to AutoStateDependency <MutableOrSavedStateDep>.
      *  See AutoStateDependency to get the usage pattern.
      *
@@ -489,7 +490,7 @@ public:
     STDMETHOD(GetSnapshot) (INPTR GUIDPARAM aId, ISnapshot **aSnapshot);
     STDMETHOD(FindSnapshot) (INPTR BSTR aName, ISnapshot **aSnapshot);
     STDMETHOD(SetCurrentSnapshot) (INPTR GUIDPARAM aId);
-    STDMETHOD(CreateSharedFolder) (INPTR BSTR aName, INPTR BSTR aHostPath);
+    STDMETHOD(CreateSharedFolder) (INPTR BSTR aName, INPTR BSTR aHostPath, BOOL aWritable);
     STDMETHOD(RemoveSharedFolder) (INPTR BSTR aName);
     STDMETHOD(CanShowConsoleWindow) (BOOL *aCanShow);
     STDMETHOD(ShowConsoleWindow) (ULONG64 *aWinId);
@@ -503,25 +504,34 @@ public:
 
     ComObjPtr <SessionMachine> sessionMachine();
 
-    // Note: the below methods are intended to be called only after adding
-    // a caller to the Machine instance and, when necessary, from under
-    // the Machine lock in appropriate mode
+    /**
+     *  Returns the VirtualBox object this machine belongs to.
+     *
+     *  @note This method doesn't check this object's readiness as it is
+     *  intended to be used only by Machine children where it is guaranteed
+     *  that this object still exists in memory.
+     */
+    const ComObjPtr <VirtualBox, ComWeakRef> &virtualBox() const { return mParent; }
 
-    /// @todo (dmik) revise code using these methods: improving incapsulation
-    //  should make them not necessary
+    /**
+     *  Returns this machine's name.
+     *
+     *  @note This method doesn't check this object's readiness as it is
+     *  intended to be used only after adding a caller to this object (that
+     *  guarantees that the object is ready or at least limited).
+     */
+    const Guid &uuid() const { return mData->mUuid; }
 
-    const ComObjPtr <VirtualBox, ComWeakRef> &virtualBox() { return mParent; }
-
-    const Shareable <Data> &data() const { return mData; }
-    const Backupable <UserData> &userData() const { return mUserData; }
-    const Backupable <HDData> &hdData() const { return mHDData; }
-
-    const Shareable <SSData> &ssData() const { return mSSData; }
-
-    const ComObjPtr <DVDDrive> &dvdDrive() { return mDVDDrive; }
-    const ComObjPtr <FloppyDrive> &floppyDrive() { return mFloppyDrive; }
-    const ComObjPtr <USBController> &usbController() { return mUSBController; }
+    /**
+     *  Returns this machine's name.
+     *
+     *  @note This method doesn't lock this object or check its readiness as
+     *  it is intended to be used only after adding a caller to this object
+     *  (that guarantees that the object is ready) and locking it for reading.
+     */
+    const Bstr &name() const { return mUserData->mName; }
 
+    // callback handlers
     virtual HRESULT onDVDDriveChange() { return S_OK; }
     virtual HRESULT onFloppyDriveChange() { return S_OK; }
     virtual HRESULT onNetworkAdapterChange(INetworkAdapter *networkAdapter) { return S_OK; }
@@ -531,11 +541,16 @@ public:
     virtual HRESULT onUSBControllerChange() { return S_OK; }
     virtual HRESULT onSharedFolderChange() { return S_OK; }
 
+    HRESULT saveRegistryEntry (CFGNODE aEntryNode);
+
     int calculateFullPath (const char *aPath, Utf8Str &aResult);
     void calculateRelativePath (const char *aPath, Utf8Str &aResult);
 
     void getLogFolder (Utf8Str &aLogFolder);
 
+    bool isDVDImageUsed (const Guid &aId, ResourceUsage_T aUsage);
+    bool isFloppyImageUsed (const Guid &aId, ResourceUsage_T aUsage);
+
     HRESULT openSession (IInternalSessionControl *aControl);
     HRESULT openRemoteSession (IInternalSessionControl *aControl,
                                INPTR BSTR aType, INPTR BSTR aEnvironment,
@@ -570,6 +585,7 @@ protected:
 
     inline Machine *machine();
 
+    HRESULT initDataAndChildObjects();
     void uninitDataAndChildObjects();
 
     void checkStateDependencies (AutoLock &aLock);
@@ -739,7 +755,7 @@ public:
     // IInternalMachineControl methods
     STDMETHOD(UpdateState)(MachineState_T machineState);
     STDMETHOD(GetIPCId)(BSTR *id);
-    STDMETHOD(RunUSBDeviceFilters) (IUSBDevice *aUSBDevice, BOOL *aMatched);
+    STDMETHOD(RunUSBDeviceFilters) (IUSBDevice *aUSBDevice, BOOL *aMatched, ULONG *aMaskedIfs);
     STDMETHOD(CaptureUSBDevice) (INPTR GUIDPARAM aId);
     STDMETHOD(DetachUSBDevice) (INPTR GUIDPARAM aId, BOOL aDone);
     STDMETHOD(AutoCaptureUSBDevices)();
@@ -763,7 +779,7 @@ public:
 
     bool checkForDeath();
 
-#if defined (RT_OS_WINDOWS) 
+#if defined (RT_OS_WINDOWS)
     HANDLE ipcSem() { return mIPCSem; }
 #elif defined (RT_OS_OS2)
     HMTX ipcSem() { return mIPCSem; }
@@ -777,11 +793,14 @@ public:
     HRESULT onVRDPServerChange();
     HRESULT onUSBControllerChange();
     HRESULT onUSBDeviceAttach (IUSBDevice *aDevice,
-                               IVirtualBoxErrorInfo *aError);
+                               IVirtualBoxErrorInfo *aError,
+                               ULONG aMaskedIfs);
     HRESULT onUSBDeviceDetach (INPTR GUIDPARAM aId,
                                IVirtualBoxErrorInfo *aError);
     HRESULT onSharedFolderChange();
 
+    bool hasMatchingUSBFilter (const ComObjPtr <HostUSBDevice> &aDevice, ULONG *aMaskedIfs);
+
 private:
 
     struct SnapshotData
@@ -897,6 +916,8 @@ public:
 private:
 
     Guid mSnapshotId;
+
+    friend class Snapshot;
 };
 
 ////////////////////////////////////////////////////////////////////////////////
