@@ -162,7 +162,7 @@ public:
     STDMETHOD(UnregisterFloppyImage) (INPTR GUIDPARAM aId, IFloppyImage **aFloppyImage);
 
     STDMETHOD(GetGuestOSType) (INPTR BSTR aId, IGuestOSType **aType);
-    STDMETHOD(CreateSharedFolder) (INPTR BSTR aName, INPTR BSTR aHostPath);
+    STDMETHOD(CreateSharedFolder) (INPTR BSTR aName, INPTR BSTR aHostPath, BOOL aWritable);
     STDMETHOD(RemoveSharedFolder) (INPTR BSTR aName);
     STDMETHOD(GetNextExtraDataKey) (INPTR BSTR aKey, BSTR *aNextKey, BSTR *aNextValue);
     STDMETHOD(GetExtraData) (INPTR BSTR aKey, BSTR *aValue);
@@ -230,10 +230,10 @@ public:
         return findHardDisk (&aId, NULL, true /* aDoSetError */, &aHardDisk);
     }
 
-    BOOL getDVDImageUsage (const Guid &id, ResourceUsage_T usage,
-                           Bstr *machineIDs = NULL);
-    BOOL getFloppyImageUsage (const Guid &id, ResourceUsage_T usage,
-                              Bstr *machineIDs = NULL);
+    bool getDVDImageUsage (const Guid &aId, ResourceUsage_T aUsage,
+                           Bstr *aMachineIDs = NULL);
+    bool getFloppyImageUsage (const Guid &aId, ResourceUsage_T aUsage,
+                              Bstr *aMachineIDs = NULL);
 
     const ComObjPtr <Host> &host() { return mData.mHost; }
     const ComObjPtr <SystemProperties> &systemProperties() { return mData.mSystemProperties; }
