@@ -325,10 +325,20 @@ int main(int argc, char *argv[])
     // after the session is closed)
     EventQueue eventQ;
 
+#if 0
+    // the simplest COM API test
+    ////////////////////////////////////////////////////////////////////////////
+    {
+        Bstr version;
+        CHECK_ERROR_BREAK (virtualBox, COMGETTER(Version) (version.asOutParam()));
+        printf ("VirtualBox version = %ls\n", version.raw());
+    }
+#endif
+
+#if 0
     // some outdated stuff
     ////////////////////////////////////////////////////////////////////////////
 
-#if 0
     printf("Getting IHost interface...\n");
     IHost *host;
     rc = virtualBox->GetHost(&host);
