@@ -1,4 +1,4 @@
-/* $Revision: 4160 $ */
+/* $Revision: 27227 $ */
 /** @file
  * innotek Portable Runtime - Ring-0 Memory Objects, Linux.
  */
@@ -29,6 +29,13 @@
 #include <iprt/process.h>
 #include "internal/memobj.h"
 
+/* early 2.6 kernels */
+#ifndef PAGE_SHARED_EXEC
+#define PAGE_SHARED_EXEC PAGE_SHARED
+#endif
+#ifndef PAGE_READONLY_EXEC
+#define PAGE_READONLY_EXEC PAGE_READONLY
+#endif
 
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
@@ -683,7 +690,7 @@ int rtR0MemObjNativeLockUser(PPRTR0MEMOBJINTERNAL ppMem, RTR3PTR R3Ptr, size_t c
     struct task_struct *pTask = rtR0ProcessToLinuxTask(R0Process);
     struct vm_area_struct **papVMAs;
     PRTR0MEMOBJLNX pMemLnx;
-    int rc;
+    int rc = VERR_NO_MEMORY;
 
     /*
      * Check for valid task and size overflows.
