@@ -1,4 +1,4 @@
-/* $Id: the-linux-kernel.h 4155 2007-08-15 19:41:26Z vboxsync $ */
+/* $Id: the-linux-kernel.h 27986 2008-02-07 20:36:57Z frank $ */
 /** @file
  * innotek Portable Runtime - Include all necessary headers for the Linux kernel.
  */
@@ -54,6 +54,9 @@
 #include <linux/slab.h>
 #include <asm/semaphore.h>
 #include <linux/module.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
+# include <linux/moduleparam.h>
+#endif
 #include <linux/kernel.h>
 #include <linux/init.h>
 #include <linux/fs.h>
@@ -70,6 +73,10 @@
 #include <linux/delay.h>
 #include <linux/interrupt.h>
 #include <linux/completion.h>
+#include <linux/compiler.h>
+#ifndef HAVE_UNLOCKED_IOCTL /* linux/fs.h defines this */
+# include <linux/smp_lock.h>
+#endif
 /* For the shared folders module */
 #include <linux/vmalloc.h>
 #define wchar_t linux_wchar_t
@@ -108,7 +115,7 @@ DECLINLINE(unsigned int) jiffies_to_msecs(unsigned long cJiffies)
 #  elif HZ > 1000 && !(HZ % 1000)
     return (cJiffies + (HZ / 1000) - 1) / (HZ / 1000);
 #  else
-    return (j * 1000) / HZ;
+    return (cJiffies * 1000) / HZ;
 #  endif
 }
 
@@ -217,5 +224,13 @@ DECLINLINE(unsigned long) msecs_to_jiffies(unsigned int cMillies)
  */
 #undef bool
 
+/*
+ * There are post-2.6.24 kernels (confusingly with unchanged version number)
+ * which eliminate macros which were marked as deprecated.
+ */
+#ifndef __attribute_used__
+#define __attribute_used__ __used
+#endif
+
 #endif
 
