@@ -1,4 +1,4 @@
-/* $Id: init.cpp 4071 2007-08-07 17:07:59Z vboxsync $ */
+/* $Id: init.cpp 26312 2007-11-27 13:59:55Z frank $ */
 /** @file
  * innotek Portable Runtime - Init Ring-3.
  */
@@ -35,7 +35,7 @@
 #include <iprt/err.h>
 #include <iprt/string.h>
 #include <iprt/param.h>
-#ifndef IN_GUEST
+#if !defined(IN_GUEST) && !defined(RT_NO_GIP)
 # include <iprt/file.h>
 # include <VBox/sup.h>
 # include <stdlib.h>
@@ -101,7 +101,7 @@ RTR3DECL(int) RTR3Init(bool fInitSUPLib, size_t cbReserve)
 {
     /* no entry log flow, because prefixes and thread may freak out. */
 
-#ifndef IN_GUEST
+#if !defined(IN_GUEST) && !defined(RT_NO_GIP)
 # ifdef VBOX
     /*
      * This MUST be done as the very first thing, before any file is opened.
@@ -117,7 +117,7 @@ RTR3DECL(int) RTR3Init(bool fInitSUPLib, size_t cbReserve)
         RTFileSetForceFlags(RTFILE_O_READWRITE, RTFILE_O_WRITE_THROUGH, 0);
     }
 # endif  /* VBOX */
-#endif /* !IN_GUEST */
+#endif /* !IN_GUEST && !RT_NO_GIP */
 
     /*
      * Thread Thread database and adopt the caller thread as 'main'.
@@ -131,7 +131,7 @@ RTR3DECL(int) RTR3Init(bool fInitSUPLib, size_t cbReserve)
         return rc;
     }
 
-#ifndef IN_GUEST
+#if !defined(IN_GUEST) && !defined(RT_NO_GIP)
     if (fInitSUPLib)
     {
         /*
@@ -149,7 +149,7 @@ RTR3DECL(int) RTR3Init(bool fInitSUPLib, size_t cbReserve)
     g_u64ProgramStartMicroTS = g_u64ProgramStartNanoTS / 1000;
     g_u64ProgramStartMilliTS = g_u64ProgramStartNanoTS / 1000000;
 
-#ifndef IN_GUEST
+#if !defined(IN_GUEST) && !defined(RT_NO_GIP)
     /*
      * The threading is initialized we can safely sleep a bit if GIP
      * needs some time to update itself updating.
