@@ -1,4 +1,4 @@
-/* $Id: strformatrt.cpp 4071 2007-08-07 17:07:59Z vboxsync $ */
+/* $Id: strformatrt.cpp 25896 2007-11-01 16:13:17Z frank $ */
 /** @file
  * innotek Portable Runtime - IPRT String Formatter Extensions.
  */
@@ -644,7 +644,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                     RTST_TIMESPEC
                 } RTST;
 /** Set if it's a pointer */
-#define RTST_FLAGS_POINTER  BIT(0)
+#define RTST_FLAGS_POINTER  RT_BIT(0)
                 static const struct
                 {
                     uint8_t     cch;        /**< the length of the string. */
