@@ -1,4 +1,4 @@
-/* $Id: strtonum.cpp 4071 2007-08-07 17:07:59Z vboxsync $ */
+/* $Id: strtonum.cpp 26583 2007-12-06 21:54:43Z bird $ */
 /** @file
  * innotek Portable Runtime - String To Number Convertion.
  */
@@ -38,12 +38,11 @@ static const unsigned char g_auchDigits[256] =
     255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
     255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255
 };
-/** Approximated overflow shift checks.
- * @todo make the overflow stuff work for real. */
+/** Approximated overflow shift checks. */
 static const char g_auchShift[36] =
 {
   /*  0   1   2   3   4   5   6   7   8   9  10  11  12  13  14  15  16  17  18  19  20  21  22  23  24  25  26  27  28  29  30  31  32  33  34  35 */
-     64, 64, 63, 63, 62, 62, 61, 61, 60, 60, 59, 59, 58, 58, 57, 57, 56, 56, 55, 55, 54, 54, 53, 53, 52, 52, 51, 51, 50, 50, 49, 49, 50, 50, 51, 51
+     64, 64, 63, 63, 62, 62, 62, 62, 61, 61, 61, 61, 61, 61, 61, 61, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 60, 59, 59, 59, 59
 };
 
 /*
@@ -82,6 +81,13 @@ int main()
  *
  * @returns iprt status code.
  *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_NEGATIVE_UNSIGNED
+ * @retval  VWRN_TRAILING_CHARS
+ * @retval  VWRN_TRAILING_SPACES
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ *
  * @param   pszValue    Pointer to the string value.
  * @param   ppszNext    Where to store the pointer to the first char following the number. (Optional)
  * @param   uBase       The base of the representation used.
@@ -172,6 +178,54 @@ RTDECL(int) RTStrToUInt64Ex(const char *pszValue, char **ppszNext, unsigned uBas
     if (ppszNext)
         *ppszNext = (char *)psz;
 
+    /*
+     * Warn about trailing chars/spaces.
+     */
+    if (    rc == VINF_SUCCESS
+        &&  *psz)
+    {
+        while (*psz == ' ' || *psz == '\t')
+            psz++;
+        rc = *psz ? VWRN_TRAILING_CHARS : VWRN_TRAILING_SPACES;
+    }
+
+    return rc;
+}
+
+
+/**
+ * Converts a string representation of a number to a 64-bit unsigned number,
+ * making sure the full string is converted.
+ *
+ * @returns iprt status code.
+ *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_NEGATIVE_UNSIGNED
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ * @retval  VERR_TRAILING_SPACES
+ * @retval  VERR_TRAILING_CHARS
+ *
+ * @param   pszValue    Pointer to the string value.
+ * @param   uBase       The base of the representation used.
+ *                      If the function will look for known prefixes before defaulting to 10.
+ * @param   pu64        Where to store the converted number. (optional)
+ */
+RTDECL(int) RTStrToUInt64Full(const char *pszValue, unsigned uBase, uint64_t *pu64)
+{
+    char *psz;
+    int rc = RTStrToUInt64Ex(pszValue, &psz, uBase, pu64);
+    if (RT_SUCCESS(rc) && *psz)
+    {
+        if (rc == VWRN_TRAILING_CHARS || rc == VWRN_TRAILING_SPACES)
+            rc = -rc;
+        else
+        {
+            while (*psz == ' ' || *psz == '\t')
+                psz++;
+            rc = *psz ? VERR_TRAILING_CHARS : VERR_TRAILING_SPACES;
+        }
+    }
     return rc;
 }
 
@@ -199,6 +253,13 @@ RTDECL(uint64_t) RTStrToUInt64(const char *pszValue)
  *
  * @returns iprt status code.
  *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_NEGATIVE_UNSIGNED
+ * @retval  VWRN_TRAILING_CHARS
+ * @retval  VWRN_TRAILING_SPACES
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ *
  * @param   pszValue    Pointer to the string value.
  * @param   ppszNext    Where to store the pointer to the first char following the number. (Optional)
  * @param   uBase       The base of the representation used.
@@ -220,6 +281,39 @@ RTDECL(int) RTStrToUInt32Ex(const char *pszValue, char **ppszNext, unsigned uBas
 }
 
 
+/**
+ * Converts a string representation of a number to a 32-bit unsigned number,
+ * making sure the full string is converted.
+ *
+ * @returns iprt status code.
+ *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_NEGATIVE_UNSIGNED
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ * @retval  VERR_TRAILING_SPACES
+ * @retval  VERR_TRAILING_CHARS
+ *
+ * @param   pszValue    Pointer to the string value.
+ * @param   uBase       The base of the representation used.
+ *                      If the function will look for known prefixes before defaulting to 10.
+ * @param   pu32        Where to store the converted number. (optional)
+ */
+RTDECL(int) RTStrToUInt32Full(const char *pszValue, unsigned uBase, uint32_t *pu32)
+{
+    uint64_t u64;
+    int rc = RTStrToUInt64Full(pszValue, uBase, &u64);
+    if (rc == VINF_SUCCESS)
+    {
+        if (u64 & ~0xffffffffULL)
+            rc = VWRN_NUMBER_TOO_BIG;
+    }
+    if (pu32)
+        *pu32 = (uint32_t)u64;
+    return rc;
+}
+
+
 /**
  * Converts a string representation of a number to a 64-bit unsigned number.
  * The base is guessed.
@@ -243,6 +337,13 @@ RTDECL(uint32_t) RTStrToUInt32(const char *pszValue)
  *
  * @returns iprt status code.
  *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_NEGATIVE_UNSIGNED
+ * @retval  VWRN_TRAILING_CHARS
+ * @retval  VWRN_TRAILING_SPACES
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ *
  * @param   pszValue    Pointer to the string value.
  * @param   ppszNext    Where to store the pointer to the first char following the number. (Optional)
  * @param   uBase       The base of the representation used.
@@ -264,6 +365,39 @@ RTDECL(int) RTStrToUInt16Ex(const char *pszValue, char **ppszNext, unsigned uBas
 }
 
 
+/**
+ * Converts a string representation of a number to a 16-bit unsigned number,
+ * making sure the full string is converted.
+ *
+ * @returns iprt status code.
+ *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_NEGATIVE_UNSIGNED
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ * @retval  VERR_TRAILING_SPACES
+ * @retval  VERR_TRAILING_CHARS
+ *
+ * @param   pszValue    Pointer to the string value.
+ * @param   uBase       The base of the representation used.
+ *                      If the function will look for known prefixes before defaulting to 10.
+ * @param   pu16        Where to store the converted number. (optional)
+ */
+RTDECL(int) RTStrToUInt16Full(const char *pszValue, unsigned uBase, uint16_t *pu16)
+{
+    uint64_t u64;
+    int rc = RTStrToUInt64Full(pszValue, uBase, &u64);
+    if (rc == VINF_SUCCESS)
+    {
+        if (u64 & ~0xffffULL)
+            rc = VWRN_NUMBER_TOO_BIG;
+    }
+    if (pu16)
+        *pu16 = (uint16_t)u64;
+    return rc;
+}
+
+
 /**
  * Converts a string representation of a number to a 16-bit unsigned number.
  * The base is guessed.
@@ -287,6 +421,13 @@ RTDECL(uint16_t) RTStrToUInt16(const char *pszValue)
  *
  * @returns iprt status code.
  *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_NEGATIVE_UNSIGNED
+ * @retval  VWRN_TRAILING_CHARS
+ * @retval  VWRN_TRAILING_SPACES
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ *
  * @param   pszValue    Pointer to the string value.
  * @param   ppszNext    Where to store the pointer to the first char following the number. (Optional)
  * @param   uBase       The base of the representation used.
@@ -308,6 +449,39 @@ RTDECL(int) RTStrToUInt8Ex(const char *pszValue, char **ppszNext, unsigned uBase
 }
 
 
+/**
+ * Converts a string representation of a number to a 8-bit unsigned number,
+ * making sure the full string is converted.
+ *
+ * @returns iprt status code.
+ *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_NEGATIVE_UNSIGNED
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ * @retval  VERR_TRAILING_SPACES
+ * @retval  VERR_TRAILING_CHARS
+ *
+ * @param   pszValue    Pointer to the string value.
+ * @param   uBase       The base of the representation used.
+ *                      If the function will look for known prefixes before defaulting to 10.
+ * @param   pu8         Where to store the converted number. (optional)
+ */
+RTDECL(int) RTStrToUInt8Full(const char *pszValue, unsigned uBase, uint8_t *pu8)
+{
+    uint64_t u64;
+    int rc = RTStrToUInt64Full(pszValue, uBase, &u64);
+    if (rc == VINF_SUCCESS)
+    {
+        if (u64 & ~0xffULL)
+            rc = VWRN_NUMBER_TOO_BIG;
+    }
+    if (pu8)
+        *pu8 = (uint8_t)u64;
+    return rc;
+}
+
+
 /**
  * Converts a string representation of a number to a 8-bit unsigned number.
  * The base is guessed.
@@ -336,6 +510,12 @@ RTDECL(uint8_t) RTStrToUInt8(const char *pszValue)
  *
  * @returns iprt status code.
  *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_TRAILING_CHARS
+ * @retval  VWRN_TRAILING_SPACES
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ *
  * @param   pszValue    Pointer to the string value.
  * @param   ppszNext    Where to store the pointer to the first char following the number. (Optional)
  * @param   uBase       The base of the representation used.
@@ -391,7 +571,7 @@ RTDECL(int) RTStrToInt64Ex(const char *pszValue, char **ppszNext, unsigned uBase
      * Interpret the value.
      * Note: We only support ascii digits at this time... :-)
      */
-    //int             iShift = g_auchShift[uBase];
+    int             iShift = g_auchShift[uBase]; /** @todo test this, it's probably not 100% right yet. */
     pszValue = psz; /* (Prefix and sign doesn't count in the digit counting.) */
     int             rc = VINF_SUCCESS;
     int64_t         i64 = 0;
@@ -405,7 +585,7 @@ RTDECL(int) RTStrToInt64Ex(const char *pszValue, char **ppszNext, unsigned uBase
         int64_t i64Prev = i64;
         i64 *= uBase;
         i64 += chDigit;
-        if (i64Prev > i64/* || (i64Prev >> iShift)*/)
+        if (i64Prev > i64 || (i64Prev >> iShift))
             rc = VWRN_NUMBER_TOO_BIG;
         psz++;
     }
@@ -422,6 +602,53 @@ RTDECL(int) RTStrToInt64Ex(const char *pszValue, char **ppszNext, unsigned uBase
     if (ppszNext)
         *ppszNext = (char *)psz;
 
+    /*
+     * Warn about trailing chars/spaces.
+     */
+    if (    rc == VINF_SUCCESS
+        &&  *psz)
+    {
+        while (*psz == ' ' || *psz == '\t')
+            psz++;
+        rc = *psz ? VWRN_TRAILING_CHARS : VWRN_TRAILING_SPACES;
+    }
+
+    return rc;
+}
+
+
+/**
+ * Converts a string representation of a number to a 64-bit signed number,
+ * making sure the full string is converted.
+ *
+ * @returns iprt status code.
+ *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_TRAILING_CHARS
+ * @retval  VERR_TRAILING_SPACES
+ * @retval  VERR_NO_DIGITS
+ *
+ * @param   pszValue    Pointer to the string value.
+ * @param   uBase       The base of the representation used.
+ *                      If the function will look for known prefixes before defaulting to 10.
+ * @param   pi64        Where to store the converted number. (optional)
+ */
+RTDECL(int) RTStrToInt64Full(const char *pszValue, unsigned uBase, int64_t *pi64)
+{
+    char *psz;
+    int rc = RTStrToInt64Ex(pszValue, &psz, uBase, pi64);
+    if (RT_SUCCESS(rc) && *psz)
+    {
+        if (rc == VWRN_TRAILING_CHARS || rc == VWRN_TRAILING_SPACES)
+            rc = -rc;
+        else
+        {
+            while (*psz == ' ' || *psz == '\t')
+                psz++;
+            rc = *psz ? VERR_TRAILING_CHARS : VERR_TRAILING_SPACES;
+        }
+    }
     return rc;
 }
 
@@ -449,6 +676,12 @@ RTDECL(int64_t) RTStrToInt64(const char *pszValue)
  *
  * @returns iprt status code.
  *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_TRAILING_CHARS
+ * @retval  VWRN_TRAILING_SPACES
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ *
  * @param   pszValue    Pointer to the string value.
  * @param   ppszNext    Where to store the pointer to the first char following the number. (Optional)
  * @param   uBase       The base of the representation used.
@@ -471,6 +704,39 @@ RTDECL(int) RTStrToInt32Ex(const char *pszValue, char **ppszNext, unsigned uBase
 }
 
 
+/**
+ * Converts a string representation of a number to a 32-bit signed number,
+ * making sure the full string is converted.
+ *
+ * @returns iprt status code.
+ *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_TRAILING_CHARS
+ * @retval  VERR_TRAILING_SPACES
+ * @retval  VERR_NO_DIGITS
+ *
+ * @param   pszValue    Pointer to the string value.
+ * @param   uBase       The base of the representation used.
+ *                      If the function will look for known prefixes before defaulting to 10.
+ * @param   pi32        Where to store the converted number. (optional)
+ */
+RTDECL(int) RTStrToInt32Full(const char *pszValue, unsigned uBase, int32_t *pi32)
+{
+    int64_t i64;
+    int rc = RTStrToInt64Full(pszValue, uBase, &i64);
+    if (rc == VINF_SUCCESS)
+    {
+        int32_t i32 = (int32_t)i64;
+        if (i64 != (int64_t)i32)
+            rc = VWRN_NUMBER_TOO_BIG;
+    }
+    if (pi32)
+        *pi32 = (int32_t)i64;
+    return rc;
+}
+
+
 /**
  * Converts a string representation of a number to a 32-bit signed number.
  * The base is guessed.
@@ -494,6 +760,12 @@ RTDECL(int32_t) RTStrToInt32(const char *pszValue)
  *
  * @returns iprt status code.
  *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_TRAILING_CHARS
+ * @retval  VWRN_TRAILING_SPACES
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ *
  * @param   pszValue    Pointer to the string value.
  * @param   ppszNext    Where to store the pointer to the first char following the number. (Optional)
  * @param   uBase       The base of the representation used.
@@ -516,6 +788,39 @@ RTDECL(int) RTStrToInt16Ex(const char *pszValue, char **ppszNext, unsigned uBase
 }
 
 
+/**
+ * Converts a string representation of a number to a 16-bit signed number,
+ * making sure the full string is converted.
+ *
+ * @returns iprt status code.
+ *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_TRAILING_CHARS
+ * @retval  VERR_TRAILING_SPACES
+ * @retval  VERR_NO_DIGITS
+ *
+ * @param   pszValue    Pointer to the string value.
+ * @param   uBase       The base of the representation used.
+ *                      If the function will look for known prefixes before defaulting to 10.
+ * @param   pi16        Where to store the converted number. (optional)
+ */
+RTDECL(int) RTStrToInt16Full(const char *pszValue, unsigned uBase, int16_t *pi16)
+{
+    int64_t i64;
+    int rc = RTStrToInt64Full(pszValue, uBase, &i64);
+    if (rc == VINF_SUCCESS)
+    {
+        int16_t i16 = (int16_t)i64;
+        if (i64 != (int64_t)i16)
+            rc = VWRN_NUMBER_TOO_BIG;
+    }
+    if (pi16)
+        *pi16 = (int16_t)i64;
+    return rc;
+}
+
+
 /**
  * Converts a string representation of a number to a 16-bit signed number.
  * The base is guessed.
@@ -539,6 +844,12 @@ RTDECL(int16_t) RTStrToInt16(const char *pszValue)
  *
  * @returns iprt status code.
  *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VWRN_TRAILING_CHARS
+ * @retval  VWRN_TRAILING_SPACES
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_NO_DIGITS
+ *
  * @param   pszValue    Pointer to the string value.
  * @param   ppszNext    Where to store the pointer to the first char following the number. (Optional)
  * @param   uBase       The base of the representation used.
@@ -561,6 +872,39 @@ RTDECL(int) RTStrToInt8Ex(const char *pszValue, char **ppszNext, unsigned uBase,
 }
 
 
+/**
+ * Converts a string representation of a number to a 8-bit signed number,
+ * making sure the full string is converted.
+ *
+ * @returns iprt status code.
+ *          Warnings are used to indicate convertion problems.
+ * @retval  VWRN_NUMBER_TOO_BIG
+ * @retval  VINF_SUCCESS
+ * @retval  VERR_TRAILING_CHARS
+ * @retval  VERR_TRAILING_SPACES
+ * @retval  VERR_NO_DIGITS
+ *
+ * @param   pszValue    Pointer to the string value.
+ * @param   uBase       The base of the representation used.
+ *                      If the function will look for known prefixes before defaulting to 10.
+ * @param   pi64        Where to store the converted number. (optional)
+ */
+RTDECL(int) RTStrToInt8Full(const char *pszValue, unsigned uBase, int8_t *pi8)
+{
+    int64_t i64;
+    int rc = RTStrToInt64Full(pszValue, uBase, &i64);
+    if (rc == VINF_SUCCESS)
+    {
+        int8_t i8 = (int8_t)i64;
+        if (i64 != (int64_t)i8)
+            rc = VWRN_NUMBER_TOO_BIG;
+    }
+    if (pi8)
+        *pi8 = (int8_t)i64;
+    return rc;
+}
+
+
 /**
  * Converts a string representation of a number to a 8-bit signed number.
  * The base is guessed.
