@@ -1,4 +1,4 @@
-/* $Id: CFGM.cpp 4071 2007-08-07 17:07:59Z vboxsync $ */
+/* $Id: CFGM.cpp 26504 2007-12-04 11:07:02Z frank $ */
 /** @file
  * CFGM - Configuration Manager.
  *
@@ -66,6 +66,7 @@
 #include <VBox/log.h>
 #include <iprt/assert.h>
 #include <iprt/string.h>
+#include <iprt/uuid.h>
 
 
 /*******************************************************************************
@@ -767,6 +768,10 @@ static int cfgmR3CreateDefault(PVM pVM)
     UPDATERC();
     rc = CFGMR3InsertString(pCfg,   "FloppyDevice",         "");
     UPDATERC();
+    RTUUID Uuid;
+    RTUuidClear(&Uuid);
+    rc = CFGMR3InsertBytes(pCfg,    "UUID", &Uuid, sizeof(Uuid));
+    UPDATERC();
     /* Bios logo. */
     rc = CFGMR3InsertInteger(pCfg,  "FadeIn",               1);
     UPDATERC();
@@ -1072,9 +1077,9 @@ CFGMR3DECL(int) CFGMR3InsertSubTree(PCFGMNODE pNode, const char *pszName, PCFGMN
     int rc = CFGMR3InsertNode(pNode, pszName, &pNewChild);
     if (RT_SUCCESS(rc))
     {
-        Assert(pNewChild->pFirstChild);
+        Assert(!pNewChild->pFirstChild);
         pNewChild->pFirstChild = pSubTree->pFirstChild;
-        Assert(pNewChild->pFirstLeaf);
+        Assert(!pNewChild->pFirstLeaf);
         pNewChild->pFirstLeaf = pSubTree->pFirstLeaf;
         if (ppChild)
             *ppChild = pNewChild;
@@ -1380,7 +1385,7 @@ CFGMR3DECL(void) CFGMR3RemoveNode(PCFGMNODE pNode)
         {
             if (pNode->pParent)
                 pNode->pParent->pFirstChild = pNode->pNext;
-            else if (pNode == pNode->pVM->cfgm.s.pRoot)
+            else if (pNode == pNode->pVM->cfgm.s.pRoot) /* might be a different tree */
                 pNode->pVM->cfgm.s.pRoot = NULL;
         }
         if (pNode->pNext)
@@ -1394,7 +1399,6 @@ CFGMR3DECL(void) CFGMR3RemoveNode(PCFGMNODE pNode)
         pNode->pPrev = NULL;
         pNode->pParent = NULL;
         MMR3HeapFree(pNode);
-
     }
 }
 
@@ -1539,7 +1543,7 @@ CFGMR3DECL(int) CFGMR3InsertString(PCFGMNODE pNode, const char *pszName, const c
  * @param   pvBytes         The value.
  * @param   cbBytes         The value size.
  */
-CFGMR3DECL(int) CFGMR3InsertBytes(PCFGMNODE pNode, const char *pszName, void *pvBytes, size_t cbBytes)
+CFGMR3DECL(int) CFGMR3InsertBytes(PCFGMNODE pNode, const char *pszName, const void *pvBytes, size_t cbBytes)
 {
     int rc;
     if (pNode)
@@ -1673,8 +1677,6 @@ CFGMR3DECL(int) CFGMR3QueryS32(PCFGMNODE pNode, const char *pszName, int32_t *pi
     {
         if (   !(u64 & 0xffffffff80000000ULL)
             ||  (u64 & 0xffffffff80000000ULL) == 0xffffffff80000000ULL)
-
-        if (((uint32_t)(u64 >> 32) + 1) <= 1)
             *pi32 = (int32_t)u64;
         else
             rc = VERR_CFGM_INTEGER_TOO_BIG;
