@@ -1,4 +1,4 @@
-/* $Id: CPUM.cpp 4425 2007-08-29 21:34:18Z vboxsync $ */
+/* $Id: CPUM.cpp 26935 2007-12-21 21:37:51Z frank $ */
 /** @file
  * CPUM - CPU Monitor(/Manager)
  */
@@ -193,7 +193,7 @@ static int cpumR3CpuIdInit(PVM pVM)
      * Get the host CPUIDs.
      */
     for (i = 0; i < ELEMENTS(pVM->cpum.s.aGuestCpuIdStd); i++)
-        ASMCpuId(i,
+        ASMCpuId_Idx_ECX(i, 0,
                  &pCPUM->aGuestCpuIdStd[i].eax, &pCPUM->aGuestCpuIdStd[i].ebx,
                  &pCPUM->aGuestCpuIdStd[i].ecx, &pCPUM->aGuestCpuIdStd[i].edx);
     for (i = 0; i < ELEMENTS(pCPUM->aGuestCpuIdExt); i++)
@@ -223,7 +223,7 @@ static int cpumR3CpuIdInit(PVM pVM)
                                        //| X86_CPUID_FEATURE_EDX_PAT   - not virtualized.
                                        //| X86_CPUID_FEATURE_EDX_PSE36 - not virtualized.
                                        //| X86_CPUID_FEATURE_EDX_PSN   - no serial number.
-                                       //| X86_CPUID_FEATURE_EDX_CLFSH - no CLFLUSH instruction.
+                                       | X86_CPUID_FEATURE_EDX_CLFSH
                                        //| X86_CPUID_FEATURE_EDX_DS    - no debug store.
                                        //| X86_CPUID_FEATURE_EDX_ACPI  - not virtualized yet.
                                        | X86_CPUID_FEATURE_EDX_MMX
@@ -268,7 +268,7 @@ static int cpumR3CpuIdInit(PVM pVM)
                                        | X86_CPUID_AMD_FEATURE_EDX_MMX
                                        | X86_CPUID_AMD_FEATURE_EDX_FXSR
                                        | X86_CPUID_AMD_FEATURE_EDX_FFXSR
-                                       //| X86_CPUID_AMD_FEATURE_EDX_LONG_MODE - definintly not.
+                                       //| X86_CPUID_AMD_FEATURE_EDX_LONG_MODE - definitely not.
                                        | X86_CPUID_AMD_FEATURE_EDX_3DNOW_EX
                                        | X86_CPUID_AMD_FEATURE_EDX_3DNOW
                                        | 0;
@@ -293,7 +293,7 @@ static int cpumR3CpuIdInit(PVM pVM)
      * Hide HTT, multicode, SMP, whatever.
      * (APIC-ID := 0 and #LogCpus := 0)
      */
-    pCPUM->aGuestCpuIdStd[1].ebx = 0x0000ffff;
+    pCPUM->aGuestCpuIdStd[1].ebx &= 0x0000ffff;
 
     /*
      * Determin the default value and limit it the number of entries.
@@ -319,14 +319,21 @@ static int cpumR3CpuIdInit(PVM pVM)
     for (i = pCPUM->aGuestCpuIdExt[0].eax - 0x80000000 + 1; i < ELEMENTS(pCPUM->aGuestCpuIdExt); i++)
         pCPUM->aGuestCpuIdExt[i] = pCPUM->GuestCpuIdDef;
 
+    /*
+     * Workaround for missing cpuid(0) patches: If we miss to patch a cpuid(0).eax then
+     * Linux tries to determine the number of processors from (cpuid(4).eax >> 26) + 1.
+     * We don't support more than 1 processor.
+     */
+    pCPUM->aGuestCpuIdStd[4].eax = 0;
+
     /*
      * Load CPUID overrides from configuration.
      */
     PCPUMCPUID  pCpuId = &pCPUM->aGuestCpuIdStd[0];
     uint32_t    cElements = ELEMENTS(pCPUM->aGuestCpuIdStd);
-    for (;;)
+    for (i=0;; )
     {
-        while (cElements-- < 0)
+        while (cElements-- > 0)
         {
             PCFGMNODE pNode = CFGMR3GetChildF(CFGMR3GetRoot(pVM), "CPUM/CPUID/%RX32", i);
             if (pNode)
@@ -356,6 +363,8 @@ static int cpumR3CpuIdInit(PVM pVM)
                 else
                     AssertReturn(rc == VERR_CFGM_VALUE_NOT_FOUND, rc);
             }
+            pCpuId++;
+            i++;
         }
 
         /* next */
@@ -1031,10 +1040,10 @@ static DECLCALLBACK(void) cpumR3CpuIdInfo(PVM pVM, PCDBGFINFOHLP pHlp, const cha
     pHlp->pfnPrintf(pHlp,
                     "         RAW Standard CPUIDs\n"
                     "     Function  eax      ebx      ecx      edx\n");
-    for (unsigned i = 0; i <= ELEMENTS(pVM->cpum.s.aGuestCpuIdStd); i++)
+    for (unsigned i = 0; i < ELEMENTS(pVM->cpum.s.aGuestCpuIdStd); i++)
     {
         Guest = pVM->cpum.s.aGuestCpuIdStd[i];
-        ASMCpuId(i, &Host.eax, &Host.ebx, &Host.ecx, &Host.edx);
+        ASMCpuId_Idx_ECX(i, 0, &Host.eax, &Host.ebx, &Host.ecx, &Host.edx);
 
         pHlp->pfnPrintf(pHlp,
                         "Gst: %08x  %08x %08x %08x %08x%s\n"
@@ -1083,58 +1092,58 @@ static DECLCALLBACK(void) cpumR3CpuIdInfo(PVM pVM, PCDBGFINFOHLP pHlp, const cha
         {
             uint32_t uEDX = Guest.edx;
             pHlp->pfnPrintf(pHlp, "Features EDX:                   ");
-            if (uEDX & BIT(0))   pHlp->pfnPrintf(pHlp, " FPU");
-            if (uEDX & BIT(1))   pHlp->pfnPrintf(pHlp, " VME");
-            if (uEDX & BIT(2))   pHlp->pfnPrintf(pHlp, " DE");
-            if (uEDX & BIT(3))   pHlp->pfnPrintf(pHlp, " PSE");
-            if (uEDX & BIT(4))   pHlp->pfnPrintf(pHlp, " TSC");
-            if (uEDX & BIT(5))   pHlp->pfnPrintf(pHlp, " MSR");
-            if (uEDX & BIT(6))   pHlp->pfnPrintf(pHlp, " PAE");
-            if (uEDX & BIT(7))   pHlp->pfnPrintf(pHlp, " MCE");
-            if (uEDX & BIT(8))   pHlp->pfnPrintf(pHlp, " CX8");
-            if (uEDX & BIT(9))   pHlp->pfnPrintf(pHlp, " APIC");
-            if (uEDX & BIT(10))  pHlp->pfnPrintf(pHlp, " 10");
-            if (uEDX & BIT(11))  pHlp->pfnPrintf(pHlp, " SEP");
-            if (uEDX & BIT(12))  pHlp->pfnPrintf(pHlp, " MTRR");
-            if (uEDX & BIT(13))  pHlp->pfnPrintf(pHlp, " PGE");
-            if (uEDX & BIT(14))  pHlp->pfnPrintf(pHlp, " MCA");
-            if (uEDX & BIT(15))  pHlp->pfnPrintf(pHlp, " CMOV");
-            if (uEDX & BIT(16))  pHlp->pfnPrintf(pHlp, " PAT");
-            if (uEDX & BIT(17))  pHlp->pfnPrintf(pHlp, " PSE36");
-            if (uEDX & BIT(18))  pHlp->pfnPrintf(pHlp, " PSN");
-            if (uEDX & BIT(19))  pHlp->pfnPrintf(pHlp, " CLFSH");
-            if (uEDX & BIT(20))  pHlp->pfnPrintf(pHlp, " 20");
-            if (uEDX & BIT(21))  pHlp->pfnPrintf(pHlp, " DS");
-            if (uEDX & BIT(22))  pHlp->pfnPrintf(pHlp, " ACPI");
-            if (uEDX & BIT(23))  pHlp->pfnPrintf(pHlp, " MMX");
-            if (uEDX & BIT(24))  pHlp->pfnPrintf(pHlp, " FXSR");
-            if (uEDX & BIT(25))  pHlp->pfnPrintf(pHlp, " SSE");
-            if (uEDX & BIT(26))  pHlp->pfnPrintf(pHlp, " SSE2");
-            if (uEDX & BIT(27))  pHlp->pfnPrintf(pHlp, " SS");
-            if (uEDX & BIT(28))  pHlp->pfnPrintf(pHlp, " HTT");
-            if (uEDX & BIT(29))  pHlp->pfnPrintf(pHlp, " TM");
-            if (uEDX & BIT(30))  pHlp->pfnPrintf(pHlp, " 30");
-            if (uEDX & BIT(31))  pHlp->pfnPrintf(pHlp, " PBE");
+            if (uEDX & RT_BIT(0))   pHlp->pfnPrintf(pHlp, " FPU");
+            if (uEDX & RT_BIT(1))   pHlp->pfnPrintf(pHlp, " VME");
+            if (uEDX & RT_BIT(2))   pHlp->pfnPrintf(pHlp, " DE");
+            if (uEDX & RT_BIT(3))   pHlp->pfnPrintf(pHlp, " PSE");
+            if (uEDX & RT_BIT(4))   pHlp->pfnPrintf(pHlp, " TSC");
+            if (uEDX & RT_BIT(5))   pHlp->pfnPrintf(pHlp, " MSR");
+            if (uEDX & RT_BIT(6))   pHlp->pfnPrintf(pHlp, " PAE");
+            if (uEDX & RT_BIT(7))   pHlp->pfnPrintf(pHlp, " MCE");
+            if (uEDX & RT_BIT(8))   pHlp->pfnPrintf(pHlp, " CX8");
+            if (uEDX & RT_BIT(9))   pHlp->pfnPrintf(pHlp, " APIC");
+            if (uEDX & RT_BIT(10))  pHlp->pfnPrintf(pHlp, " 10");
+            if (uEDX & RT_BIT(11))  pHlp->pfnPrintf(pHlp, " SEP");
+            if (uEDX & RT_BIT(12))  pHlp->pfnPrintf(pHlp, " MTRR");
+            if (uEDX & RT_BIT(13))  pHlp->pfnPrintf(pHlp, " PGE");
+            if (uEDX & RT_BIT(14))  pHlp->pfnPrintf(pHlp, " MCA");
+            if (uEDX & RT_BIT(15))  pHlp->pfnPrintf(pHlp, " CMOV");
+            if (uEDX & RT_BIT(16))  pHlp->pfnPrintf(pHlp, " PAT");
+            if (uEDX & RT_BIT(17))  pHlp->pfnPrintf(pHlp, " PSE36");
+            if (uEDX & RT_BIT(18))  pHlp->pfnPrintf(pHlp, " PSN");
+            if (uEDX & RT_BIT(19))  pHlp->pfnPrintf(pHlp, " CLFSH");
+            if (uEDX & RT_BIT(20))  pHlp->pfnPrintf(pHlp, " 20");
+            if (uEDX & RT_BIT(21))  pHlp->pfnPrintf(pHlp, " DS");
+            if (uEDX & RT_BIT(22))  pHlp->pfnPrintf(pHlp, " ACPI");
+            if (uEDX & RT_BIT(23))  pHlp->pfnPrintf(pHlp, " MMX");
+            if (uEDX & RT_BIT(24))  pHlp->pfnPrintf(pHlp, " FXSR");
+            if (uEDX & RT_BIT(25))  pHlp->pfnPrintf(pHlp, " SSE");
+            if (uEDX & RT_BIT(26))  pHlp->pfnPrintf(pHlp, " SSE2");
+            if (uEDX & RT_BIT(27))  pHlp->pfnPrintf(pHlp, " SS");
+            if (uEDX & RT_BIT(28))  pHlp->pfnPrintf(pHlp, " HTT");
+            if (uEDX & RT_BIT(29))  pHlp->pfnPrintf(pHlp, " TM");
+            if (uEDX & RT_BIT(30))  pHlp->pfnPrintf(pHlp, " 30");
+            if (uEDX & RT_BIT(31))  pHlp->pfnPrintf(pHlp, " PBE");
             pHlp->pfnPrintf(pHlp, "\n");
 
             uint32_t uECX = Guest.ecx;
             pHlp->pfnPrintf(pHlp, "Features ECX:                   ");
-            if (uECX & BIT(0))   pHlp->pfnPrintf(pHlp, " SSE3");
-            if (uECX & BIT(1))   pHlp->pfnPrintf(pHlp, " 1");
-            if (uECX & BIT(2))   pHlp->pfnPrintf(pHlp, " 2");
-            if (uECX & BIT(3))   pHlp->pfnPrintf(pHlp, " MONITOR");
-            if (uECX & BIT(4))   pHlp->pfnPrintf(pHlp, " DS-CPL");
-            if (uECX & BIT(5))   pHlp->pfnPrintf(pHlp, " VMX");
-            if (uECX & BIT(6))   pHlp->pfnPrintf(pHlp, " 6");
-            if (uECX & BIT(7))   pHlp->pfnPrintf(pHlp, " EST");
-            if (uECX & BIT(8))   pHlp->pfnPrintf(pHlp, " TM2");
-            if (uECX & BIT(9))   pHlp->pfnPrintf(pHlp, " 9");
-            if (uECX & BIT(10))  pHlp->pfnPrintf(pHlp, " CNXT-ID");
-            if (uECX & BIT(11))  pHlp->pfnPrintf(pHlp, " 11");
-            if (uECX & BIT(12))  pHlp->pfnPrintf(pHlp, " 12");
-            if (uECX & BIT(13))  pHlp->pfnPrintf(pHlp, " CX16");
+            if (uECX & RT_BIT(0))   pHlp->pfnPrintf(pHlp, " SSE3");
+            if (uECX & RT_BIT(1))   pHlp->pfnPrintf(pHlp, " 1");
+            if (uECX & RT_BIT(2))   pHlp->pfnPrintf(pHlp, " 2");
+            if (uECX & RT_BIT(3))   pHlp->pfnPrintf(pHlp, " MONITOR");
+            if (uECX & RT_BIT(4))   pHlp->pfnPrintf(pHlp, " DS-CPL");
+            if (uECX & RT_BIT(5))   pHlp->pfnPrintf(pHlp, " VMX");
+            if (uECX & RT_BIT(6))   pHlp->pfnPrintf(pHlp, " 6");
+            if (uECX & RT_BIT(7))   pHlp->pfnPrintf(pHlp, " EST");
+            if (uECX & RT_BIT(8))   pHlp->pfnPrintf(pHlp, " TM2");
+            if (uECX & RT_BIT(9))   pHlp->pfnPrintf(pHlp, " 9");
+            if (uECX & RT_BIT(10))  pHlp->pfnPrintf(pHlp, " CNXT-ID");
+            if (uECX & RT_BIT(11))  pHlp->pfnPrintf(pHlp, " 11");
+            if (uECX & RT_BIT(12))  pHlp->pfnPrintf(pHlp, " 12");
+            if (uECX & RT_BIT(13))  pHlp->pfnPrintf(pHlp, " CX16");
             for (unsigned iBit = 14; iBit < 32; iBit++)
-                if (uECX & BIT(iBit))
+                if (uECX & RT_BIT(iBit))
                     pHlp->pfnPrintf(pHlp, " %d", iBit);
             pHlp->pfnPrintf(pHlp, "\n");
         }
@@ -1212,7 +1221,7 @@ static DECLCALLBACK(void) cpumR3CpuIdInfo(PVM pVM, PCDBGFINFOHLP pHlp, const cha
                     "\n"
                     "         RAW Extended CPUIDs\n"
                     "     Function  eax      ebx      ecx      edx\n");
-    for (unsigned i = 0; i <= ELEMENTS(pVM->cpum.s.aGuestCpuIdExt); i++)
+    for (unsigned i = 0; i < ELEMENTS(pVM->cpum.s.aGuestCpuIdExt); i++)
     {
         Guest = pVM->cpum.s.aGuestCpuIdExt[i];
         ASMCpuId(0x80000000 | i, &Host.eax, &Host.ebx, &Host.ecx, &Host.edx);
@@ -1255,56 +1264,56 @@ static DECLCALLBACK(void) cpumR3CpuIdInfo(PVM pVM, PCDBGFINFOHLP pHlp, const cha
         {
             uint32_t uEDX = Guest.edx;
             pHlp->pfnPrintf(pHlp, "Features EDX:                   ");
-            if (uEDX & BIT(0))   pHlp->pfnPrintf(pHlp, " FPU");
-            if (uEDX & BIT(1))   pHlp->pfnPrintf(pHlp, " VME");
-            if (uEDX & BIT(2))   pHlp->pfnPrintf(pHlp, " DE");
-            if (uEDX & BIT(3))   pHlp->pfnPrintf(pHlp, " PSE");
-            if (uEDX & BIT(4))   pHlp->pfnPrintf(pHlp, " TSC");
-            if (uEDX & BIT(5))   pHlp->pfnPrintf(pHlp, " MSR");
-            if (uEDX & BIT(6))   pHlp->pfnPrintf(pHlp, " PAE");
-            if (uEDX & BIT(7))   pHlp->pfnPrintf(pHlp, " MCE");
-            if (uEDX & BIT(8))   pHlp->pfnPrintf(pHlp, " CX8");
-            if (uEDX & BIT(9))   pHlp->pfnPrintf(pHlp, " APIC");
-            if (uEDX & BIT(10))  pHlp->pfnPrintf(pHlp, " 10");
-            if (uEDX & BIT(11))  pHlp->pfnPrintf(pHlp, " SCR");
-            if (uEDX & BIT(12))  pHlp->pfnPrintf(pHlp, " MTRR");
-            if (uEDX & BIT(13))  pHlp->pfnPrintf(pHlp, " PGE");
-            if (uEDX & BIT(14))  pHlp->pfnPrintf(pHlp, " MCA");
-            if (uEDX & BIT(15))  pHlp->pfnPrintf(pHlp, " CMOV");
-            if (uEDX & BIT(16))  pHlp->pfnPrintf(pHlp, " PAT");
-            if (uEDX & BIT(17))  pHlp->pfnPrintf(pHlp, " PSE36");
-            if (uEDX & BIT(18))  pHlp->pfnPrintf(pHlp, " 18");
-            if (uEDX & BIT(19))  pHlp->pfnPrintf(pHlp, " 19");
-            if (uEDX & BIT(20))  pHlp->pfnPrintf(pHlp, " NX");
-            if (uEDX & BIT(21))  pHlp->pfnPrintf(pHlp, " 21");
-            if (uEDX & BIT(22))  pHlp->pfnPrintf(pHlp, " ExtMMX");
-            if (uEDX & BIT(23))  pHlp->pfnPrintf(pHlp, " MMX");
-            if (uEDX & BIT(24))  pHlp->pfnPrintf(pHlp, " FXSR");
-            if (uEDX & BIT(25))  pHlp->pfnPrintf(pHlp, " FastFXSR");
-            if (uEDX & BIT(26))  pHlp->pfnPrintf(pHlp, " Page1GB");
-            if (uEDX & BIT(27))  pHlp->pfnPrintf(pHlp, " RDTSCP");
-            if (uEDX & BIT(28))  pHlp->pfnPrintf(pHlp, " 28");
-            if (uEDX & BIT(29))  pHlp->pfnPrintf(pHlp, " LongMode");
-            if (uEDX & BIT(30))  pHlp->pfnPrintf(pHlp, " Ext3DNow");
-            if (uEDX & BIT(31))  pHlp->pfnPrintf(pHlp, " 3DNow");
+            if (uEDX & RT_BIT(0))   pHlp->pfnPrintf(pHlp, " FPU");
+            if (uEDX & RT_BIT(1))   pHlp->pfnPrintf(pHlp, " VME");
+            if (uEDX & RT_BIT(2))   pHlp->pfnPrintf(pHlp, " DE");
+            if (uEDX & RT_BIT(3))   pHlp->pfnPrintf(pHlp, " PSE");
+            if (uEDX & RT_BIT(4))   pHlp->pfnPrintf(pHlp, " TSC");
+            if (uEDX & RT_BIT(5))   pHlp->pfnPrintf(pHlp, " MSR");
+            if (uEDX & RT_BIT(6))   pHlp->pfnPrintf(pHlp, " PAE");
+            if (uEDX & RT_BIT(7))   pHlp->pfnPrintf(pHlp, " MCE");
+            if (uEDX & RT_BIT(8))   pHlp->pfnPrintf(pHlp, " CX8");
+            if (uEDX & RT_BIT(9))   pHlp->pfnPrintf(pHlp, " APIC");
+            if (uEDX & RT_BIT(10))  pHlp->pfnPrintf(pHlp, " 10");
+            if (uEDX & RT_BIT(11))  pHlp->pfnPrintf(pHlp, " SCR");
+            if (uEDX & RT_BIT(12))  pHlp->pfnPrintf(pHlp, " MTRR");
+            if (uEDX & RT_BIT(13))  pHlp->pfnPrintf(pHlp, " PGE");
+            if (uEDX & RT_BIT(14))  pHlp->pfnPrintf(pHlp, " MCA");
+            if (uEDX & RT_BIT(15))  pHlp->pfnPrintf(pHlp, " CMOV");
+            if (uEDX & RT_BIT(16))  pHlp->pfnPrintf(pHlp, " PAT");
+            if (uEDX & RT_BIT(17))  pHlp->pfnPrintf(pHlp, " PSE36");
+            if (uEDX & RT_BIT(18))  pHlp->pfnPrintf(pHlp, " 18");
+            if (uEDX & RT_BIT(19))  pHlp->pfnPrintf(pHlp, " 19");
+            if (uEDX & RT_BIT(20))  pHlp->pfnPrintf(pHlp, " NX");
+            if (uEDX & RT_BIT(21))  pHlp->pfnPrintf(pHlp, " 21");
+            if (uEDX & RT_BIT(22))  pHlp->pfnPrintf(pHlp, " ExtMMX");
+            if (uEDX & RT_BIT(23))  pHlp->pfnPrintf(pHlp, " MMX");
+            if (uEDX & RT_BIT(24))  pHlp->pfnPrintf(pHlp, " FXSR");
+            if (uEDX & RT_BIT(25))  pHlp->pfnPrintf(pHlp, " FastFXSR");
+            if (uEDX & RT_BIT(26))  pHlp->pfnPrintf(pHlp, " Page1GB");
+            if (uEDX & RT_BIT(27))  pHlp->pfnPrintf(pHlp, " RDTSCP");
+            if (uEDX & RT_BIT(28))  pHlp->pfnPrintf(pHlp, " 28");
+            if (uEDX & RT_BIT(29))  pHlp->pfnPrintf(pHlp, " LongMode");
+            if (uEDX & RT_BIT(30))  pHlp->pfnPrintf(pHlp, " Ext3DNow");
+            if (uEDX & RT_BIT(31))  pHlp->pfnPrintf(pHlp, " 3DNow");
             pHlp->pfnPrintf(pHlp, "\n");
 
             uint32_t uECX = Guest.ecx;
             pHlp->pfnPrintf(pHlp, "Features ECX:                   ");
-            if (uECX & BIT(0))   pHlp->pfnPrintf(pHlp, " LAHF/SAHF");
-            if (uECX & BIT(1))   pHlp->pfnPrintf(pHlp, " CMPL");
-            if (uECX & BIT(2))   pHlp->pfnPrintf(pHlp, " SVM");
-            if (uECX & BIT(3))   pHlp->pfnPrintf(pHlp, " ExtAPIC");
-            if (uECX & BIT(4))   pHlp->pfnPrintf(pHlp, " CR8L");
-            if (uECX & BIT(5))   pHlp->pfnPrintf(pHlp, " ABM");
-            if (uECX & BIT(6))   pHlp->pfnPrintf(pHlp, " SSE4A");
-            if (uECX & BIT(7))   pHlp->pfnPrintf(pHlp, " MISALNSSE");
-            if (uECX & BIT(8))   pHlp->pfnPrintf(pHlp, " 3DNOWPRF");
-            if (uECX & BIT(9))   pHlp->pfnPrintf(pHlp, " OSVW");
-            if (uECX & BIT(12))  pHlp->pfnPrintf(pHlp, " SKINIT");
-            if (uECX & BIT(13))  pHlp->pfnPrintf(pHlp, " WDT");
+            if (uECX & RT_BIT(0))   pHlp->pfnPrintf(pHlp, " LAHF/SAHF");
+            if (uECX & RT_BIT(1))   pHlp->pfnPrintf(pHlp, " CMPL");
+            if (uECX & RT_BIT(2))   pHlp->pfnPrintf(pHlp, " SVM");
+            if (uECX & RT_BIT(3))   pHlp->pfnPrintf(pHlp, " ExtAPIC");
+            if (uECX & RT_BIT(4))   pHlp->pfnPrintf(pHlp, " CR8L");
+            if (uECX & RT_BIT(5))   pHlp->pfnPrintf(pHlp, " ABM");
+            if (uECX & RT_BIT(6))   pHlp->pfnPrintf(pHlp, " SSE4A");
+            if (uECX & RT_BIT(7))   pHlp->pfnPrintf(pHlp, " MISALNSSE");
+            if (uECX & RT_BIT(8))   pHlp->pfnPrintf(pHlp, " 3DNOWPRF");
+            if (uECX & RT_BIT(9))   pHlp->pfnPrintf(pHlp, " OSVW");
+            if (uECX & RT_BIT(12))  pHlp->pfnPrintf(pHlp, " SKINIT");
+            if (uECX & RT_BIT(13))  pHlp->pfnPrintf(pHlp, " WDT");
             for (unsigned iBit = 5; iBit < 32; iBit++)
-                if (uECX & BIT(iBit))
+                if (uECX & RT_BIT(iBit))
                     pHlp->pfnPrintf(pHlp, " %d", iBit);
             pHlp->pfnPrintf(pHlp, "\n");
         }
@@ -1315,54 +1324,54 @@ static DECLCALLBACK(void) cpumR3CpuIdInfo(PVM pVM, PCDBGFINFOHLP pHlp, const cha
             uint32_t uEdxGst = Guest.edx;
             uint32_t uEdxHst = Host.edx;
             pHlp->pfnPrintf(pHlp, "Mnemonic - Description                 = guest (host)\n");
-            pHlp->pfnPrintf(pHlp, "FPU - x87 FPU on Chip                  = %d (%d)\n",  !!(uEdxGst & BIT( 0)),  !!(uEdxHst & BIT( 0)));
-            pHlp->pfnPrintf(pHlp, "VME - Virtual 8086 Mode Enhancements   = %d (%d)\n",  !!(uEdxGst & BIT( 1)),  !!(uEdxHst & BIT( 1)));
-            pHlp->pfnPrintf(pHlp, "DE - Debugging extensions              = %d (%d)\n",  !!(uEdxGst & BIT( 2)),  !!(uEdxHst & BIT( 2)));
-            pHlp->pfnPrintf(pHlp, "PSE - Page Size Extension              = %d (%d)\n",  !!(uEdxGst & BIT( 3)),  !!(uEdxHst & BIT( 3)));
-            pHlp->pfnPrintf(pHlp, "TSC - Time Stamp Counter               = %d (%d)\n",  !!(uEdxGst & BIT( 4)),  !!(uEdxHst & BIT( 4)));
-            pHlp->pfnPrintf(pHlp, "MSR - K86 Model Specific Registers     = %d (%d)\n",  !!(uEdxGst & BIT( 5)),  !!(uEdxHst & BIT( 5)));
-            pHlp->pfnPrintf(pHlp, "PAE - Physical Address Extension       = %d (%d)\n",  !!(uEdxGst & BIT( 6)),  !!(uEdxHst & BIT( 6)));
-            pHlp->pfnPrintf(pHlp, "MCE - Machine Check Exception          = %d (%d)\n",  !!(uEdxGst & BIT( 7)),  !!(uEdxHst & BIT( 7)));
-            pHlp->pfnPrintf(pHlp, "CX8 - CMPXCHG8B instruction            = %d (%d)\n",  !!(uEdxGst & BIT( 8)),  !!(uEdxHst & BIT( 8)));
-            pHlp->pfnPrintf(pHlp, "APIC - APIC On-Chip                    = %d (%d)\n",  !!(uEdxGst & BIT( 9)),  !!(uEdxHst & BIT( 9)));
-            pHlp->pfnPrintf(pHlp, "10 - Reserved                          = %d (%d)\n",  !!(uEdxGst & BIT(10)),  !!(uEdxHst & BIT(10)));
-            pHlp->pfnPrintf(pHlp, "SEP - SYSCALL and SYSRET               = %d (%d)\n",  !!(uEdxGst & BIT(11)),  !!(uEdxHst & BIT(11)));
-            pHlp->pfnPrintf(pHlp, "MTRR - Memory Type Range Registers     = %d (%d)\n",  !!(uEdxGst & BIT(12)),  !!(uEdxHst & BIT(12)));
-            pHlp->pfnPrintf(pHlp, "PGE - PTE Global Bit                   = %d (%d)\n",  !!(uEdxGst & BIT(13)),  !!(uEdxHst & BIT(13)));
-            pHlp->pfnPrintf(pHlp, "MCA - Machine Check Architecture       = %d (%d)\n",  !!(uEdxGst & BIT(14)),  !!(uEdxHst & BIT(14)));
-            pHlp->pfnPrintf(pHlp, "CMOV - Conditional Move Instructions   = %d (%d)\n",  !!(uEdxGst & BIT(15)),  !!(uEdxHst & BIT(15)));
-            pHlp->pfnPrintf(pHlp, "PAT - Page Attribute Table             = %d (%d)\n",  !!(uEdxGst & BIT(16)),  !!(uEdxHst & BIT(16)));
-            pHlp->pfnPrintf(pHlp, "PSE-36 - 36-bit Page Size Extention    = %d (%d)\n",  !!(uEdxGst & BIT(17)),  !!(uEdxHst & BIT(17)));
-            pHlp->pfnPrintf(pHlp, "18 - Reserved                          = %d (%d)\n",  !!(uEdxGst & BIT(18)),  !!(uEdxHst & BIT(18)));
-            pHlp->pfnPrintf(pHlp, "19 - Reserved                          = %d (%d)\n",  !!(uEdxGst & BIT(19)),  !!(uEdxHst & BIT(19)));
-            pHlp->pfnPrintf(pHlp, "NX - No-Execute Page Protection        = %d (%d)\n",  !!(uEdxGst & BIT(20)),  !!(uEdxHst & BIT(20)));
-            pHlp->pfnPrintf(pHlp, "DS - Debug Store                       = %d (%d)\n",  !!(uEdxGst & BIT(21)),  !!(uEdxHst & BIT(21)));
-            pHlp->pfnPrintf(pHlp, "AXMMX - AMD Extensions to MMX Instr.   = %d (%d)\n",  !!(uEdxGst & BIT(22)),  !!(uEdxHst & BIT(22)));
-            pHlp->pfnPrintf(pHlp, "MMX - Intel MMX Technology             = %d (%d)\n",  !!(uEdxGst & BIT(23)),  !!(uEdxHst & BIT(23)));
-            pHlp->pfnPrintf(pHlp, "FXSR - FXSAVE and FXRSTOR Instructions = %d (%d)\n",  !!(uEdxGst & BIT(24)),  !!(uEdxHst & BIT(24)));
-            pHlp->pfnPrintf(pHlp, "25 - AMD fast FXSAVE and FXRSTOR Instr.= %d (%d)\n",  !!(uEdxGst & BIT(25)),  !!(uEdxHst & BIT(25)));
-            pHlp->pfnPrintf(pHlp, "26 - Reserved                          = %d (%d)\n",  !!(uEdxGst & BIT(26)),  !!(uEdxHst & BIT(26)));
-            pHlp->pfnPrintf(pHlp, "27 - Reserved                          = %d (%d)\n",  !!(uEdxGst & BIT(27)),  !!(uEdxHst & BIT(27)));
-            pHlp->pfnPrintf(pHlp, "28 - Reserved                          = %d (%d)\n",  !!(uEdxGst & BIT(28)),  !!(uEdxHst & BIT(28)));
-            pHlp->pfnPrintf(pHlp, "29 - AMD Long Mode                     = %d (%d)\n",  !!(uEdxGst & BIT(29)),  !!(uEdxHst & BIT(29)));
-            pHlp->pfnPrintf(pHlp, "30 - AMD Extensions to 3DNow           = %d (%d)\n",  !!(uEdxGst & BIT(30)),  !!(uEdxHst & BIT(30)));
-            pHlp->pfnPrintf(pHlp, "31 - AMD 3DNow                         = %d (%d)\n",  !!(uEdxGst & BIT(31)),  !!(uEdxHst & BIT(31)));
+            pHlp->pfnPrintf(pHlp, "FPU - x87 FPU on Chip                  = %d (%d)\n",  !!(uEdxGst & RT_BIT( 0)),  !!(uEdxHst & RT_BIT( 0)));
+            pHlp->pfnPrintf(pHlp, "VME - Virtual 8086 Mode Enhancements   = %d (%d)\n",  !!(uEdxGst & RT_BIT( 1)),  !!(uEdxHst & RT_BIT( 1)));
+            pHlp->pfnPrintf(pHlp, "DE - Debugging extensions              = %d (%d)\n",  !!(uEdxGst & RT_BIT( 2)),  !!(uEdxHst & RT_BIT( 2)));
+            pHlp->pfnPrintf(pHlp, "PSE - Page Size Extension              = %d (%d)\n",  !!(uEdxGst & RT_BIT( 3)),  !!(uEdxHst & RT_BIT( 3)));
+            pHlp->pfnPrintf(pHlp, "TSC - Time Stamp Counter               = %d (%d)\n",  !!(uEdxGst & RT_BIT( 4)),  !!(uEdxHst & RT_BIT( 4)));
+            pHlp->pfnPrintf(pHlp, "MSR - K86 Model Specific Registers     = %d (%d)\n",  !!(uEdxGst & RT_BIT( 5)),  !!(uEdxHst & RT_BIT( 5)));
+            pHlp->pfnPrintf(pHlp, "PAE - Physical Address Extension       = %d (%d)\n",  !!(uEdxGst & RT_BIT( 6)),  !!(uEdxHst & RT_BIT( 6)));
+            pHlp->pfnPrintf(pHlp, "MCE - Machine Check Exception          = %d (%d)\n",  !!(uEdxGst & RT_BIT( 7)),  !!(uEdxHst & RT_BIT( 7)));
+            pHlp->pfnPrintf(pHlp, "CX8 - CMPXCHG8B instruction            = %d (%d)\n",  !!(uEdxGst & RT_BIT( 8)),  !!(uEdxHst & RT_BIT( 8)));
+            pHlp->pfnPrintf(pHlp, "APIC - APIC On-Chip                    = %d (%d)\n",  !!(uEdxGst & RT_BIT( 9)),  !!(uEdxHst & RT_BIT( 9)));
+            pHlp->pfnPrintf(pHlp, "10 - Reserved                          = %d (%d)\n",  !!(uEdxGst & RT_BIT(10)),  !!(uEdxHst & RT_BIT(10)));
+            pHlp->pfnPrintf(pHlp, "SEP - SYSCALL and SYSRET               = %d (%d)\n",  !!(uEdxGst & RT_BIT(11)),  !!(uEdxHst & RT_BIT(11)));
+            pHlp->pfnPrintf(pHlp, "MTRR - Memory Type Range Registers     = %d (%d)\n",  !!(uEdxGst & RT_BIT(12)),  !!(uEdxHst & RT_BIT(12)));
+            pHlp->pfnPrintf(pHlp, "PGE - PTE Global Bit                   = %d (%d)\n",  !!(uEdxGst & RT_BIT(13)),  !!(uEdxHst & RT_BIT(13)));
+            pHlp->pfnPrintf(pHlp, "MCA - Machine Check Architecture       = %d (%d)\n",  !!(uEdxGst & RT_BIT(14)),  !!(uEdxHst & RT_BIT(14)));
+            pHlp->pfnPrintf(pHlp, "CMOV - Conditional Move Instructions   = %d (%d)\n",  !!(uEdxGst & RT_BIT(15)),  !!(uEdxHst & RT_BIT(15)));
+            pHlp->pfnPrintf(pHlp, "PAT - Page Attribute Table             = %d (%d)\n",  !!(uEdxGst & RT_BIT(16)),  !!(uEdxHst & RT_BIT(16)));
+            pHlp->pfnPrintf(pHlp, "PSE-36 - 36-bit Page Size Extention    = %d (%d)\n",  !!(uEdxGst & RT_BIT(17)),  !!(uEdxHst & RT_BIT(17)));
+            pHlp->pfnPrintf(pHlp, "18 - Reserved                          = %d (%d)\n",  !!(uEdxGst & RT_BIT(18)),  !!(uEdxHst & RT_BIT(18)));
+            pHlp->pfnPrintf(pHlp, "19 - Reserved                          = %d (%d)\n",  !!(uEdxGst & RT_BIT(19)),  !!(uEdxHst & RT_BIT(19)));
+            pHlp->pfnPrintf(pHlp, "NX - No-Execute Page Protection        = %d (%d)\n",  !!(uEdxGst & RT_BIT(20)),  !!(uEdxHst & RT_BIT(20)));
+            pHlp->pfnPrintf(pHlp, "DS - Debug Store                       = %d (%d)\n",  !!(uEdxGst & RT_BIT(21)),  !!(uEdxHst & RT_BIT(21)));
+            pHlp->pfnPrintf(pHlp, "AXMMX - AMD Extensions to MMX Instr.   = %d (%d)\n",  !!(uEdxGst & RT_BIT(22)),  !!(uEdxHst & RT_BIT(22)));
+            pHlp->pfnPrintf(pHlp, "MMX - Intel MMX Technology             = %d (%d)\n",  !!(uEdxGst & RT_BIT(23)),  !!(uEdxHst & RT_BIT(23)));
+            pHlp->pfnPrintf(pHlp, "FXSR - FXSAVE and FXRSTOR Instructions = %d (%d)\n",  !!(uEdxGst & RT_BIT(24)),  !!(uEdxHst & RT_BIT(24)));
+            pHlp->pfnPrintf(pHlp, "25 - AMD fast FXSAVE and FXRSTOR Instr.= %d (%d)\n",  !!(uEdxGst & RT_BIT(25)),  !!(uEdxHst & RT_BIT(25)));
+            pHlp->pfnPrintf(pHlp, "26 - Reserved                          = %d (%d)\n",  !!(uEdxGst & RT_BIT(26)),  !!(uEdxHst & RT_BIT(26)));
+            pHlp->pfnPrintf(pHlp, "27 - Reserved                          = %d (%d)\n",  !!(uEdxGst & RT_BIT(27)),  !!(uEdxHst & RT_BIT(27)));
+            pHlp->pfnPrintf(pHlp, "28 - Reserved                          = %d (%d)\n",  !!(uEdxGst & RT_BIT(28)),  !!(uEdxHst & RT_BIT(28)));
+            pHlp->pfnPrintf(pHlp, "29 - AMD Long Mode                     = %d (%d)\n",  !!(uEdxGst & RT_BIT(29)),  !!(uEdxHst & RT_BIT(29)));
+            pHlp->pfnPrintf(pHlp, "30 - AMD Extensions to 3DNow           = %d (%d)\n",  !!(uEdxGst & RT_BIT(30)),  !!(uEdxHst & RT_BIT(30)));
+            pHlp->pfnPrintf(pHlp, "31 - AMD 3DNow                         = %d (%d)\n",  !!(uEdxGst & RT_BIT(31)),  !!(uEdxHst & RT_BIT(31)));
 
             uint32_t uEcxGst = Guest.ecx;
             uint32_t uEcxHst = Host.ecx;
-            pHlp->pfnPrintf(pHlp, "LahfSahf - LAHF/SAHF in 64-bit mode    = %d (%d)\n",  !!(uEcxGst & BIT( 0)),  !!(uEcxHst & BIT( 0)));
-            pHlp->pfnPrintf(pHlp, "CmpLegacy - Core MP legacy mode (depr) = %d (%d)\n",  !!(uEcxGst & BIT( 1)),  !!(uEcxHst & BIT( 1)));
-            pHlp->pfnPrintf(pHlp, "SVM - AMD VM Extensions                = %d (%d)\n",  !!(uEcxGst & BIT( 2)),  !!(uEcxHst & BIT( 2)));
-            pHlp->pfnPrintf(pHlp, "APIC registers starting at 0x400       = %d (%d)\n",  !!(uEcxGst & BIT( 3)),  !!(uEcxHst & BIT( 3)));
-            pHlp->pfnPrintf(pHlp, "AltMovCR8 - LOCK MOV CR0 means MOV CR8 = %d (%d)\n",  !!(uEcxGst & BIT( 4)),  !!(uEcxHst & BIT( 4)));
-            pHlp->pfnPrintf(pHlp, "Advanced bit manipulation              = %d (%d)\n",  !!(uEcxGst & BIT( 5)),  !!(uEcxHst & BIT( 5)));
-            pHlp->pfnPrintf(pHlp, "SSE4A instruction support              = %d (%d)\n",  !!(uEcxGst & BIT( 6)),  !!(uEcxHst & BIT( 6)));
-            pHlp->pfnPrintf(pHlp, "Misaligned SSE mode                    = %d (%d)\n",  !!(uEcxGst & BIT( 7)),  !!(uEcxHst & BIT( 7)));
-            pHlp->pfnPrintf(pHlp, "PREFETCH and PREFETCHW instruction     = %d (%d)\n",  !!(uEcxGst & BIT( 8)),  !!(uEcxHst & BIT( 8)));
-            pHlp->pfnPrintf(pHlp, "OS visible workaround                  = %d (%d)\n",  !!(uEcxGst & BIT( 9)),  !!(uEcxHst & BIT( 9)));
+            pHlp->pfnPrintf(pHlp, "LahfSahf - LAHF/SAHF in 64-bit mode    = %d (%d)\n",  !!(uEcxGst & RT_BIT( 0)),  !!(uEcxHst & RT_BIT( 0)));
+            pHlp->pfnPrintf(pHlp, "CmpLegacy - Core MP legacy mode (depr) = %d (%d)\n",  !!(uEcxGst & RT_BIT( 1)),  !!(uEcxHst & RT_BIT( 1)));
+            pHlp->pfnPrintf(pHlp, "SVM - AMD VM Extensions                = %d (%d)\n",  !!(uEcxGst & RT_BIT( 2)),  !!(uEcxHst & RT_BIT( 2)));
+            pHlp->pfnPrintf(pHlp, "APIC registers starting at 0x400       = %d (%d)\n",  !!(uEcxGst & RT_BIT( 3)),  !!(uEcxHst & RT_BIT( 3)));
+            pHlp->pfnPrintf(pHlp, "AltMovCR8 - LOCK MOV CR0 means MOV CR8 = %d (%d)\n",  !!(uEcxGst & RT_BIT( 4)),  !!(uEcxHst & RT_BIT( 4)));
+            pHlp->pfnPrintf(pHlp, "Advanced bit manipulation              = %d (%d)\n",  !!(uEcxGst & RT_BIT( 5)),  !!(uEcxHst & RT_BIT( 5)));
+            pHlp->pfnPrintf(pHlp, "SSE4A instruction support              = %d (%d)\n",  !!(uEcxGst & RT_BIT( 6)),  !!(uEcxHst & RT_BIT( 6)));
+            pHlp->pfnPrintf(pHlp, "Misaligned SSE mode                    = %d (%d)\n",  !!(uEcxGst & RT_BIT( 7)),  !!(uEcxHst & RT_BIT( 7)));
+            pHlp->pfnPrintf(pHlp, "PREFETCH and PREFETCHW instruction     = %d (%d)\n",  !!(uEcxGst & RT_BIT( 8)),  !!(uEcxHst & RT_BIT( 8)));
+            pHlp->pfnPrintf(pHlp, "OS visible workaround                  = %d (%d)\n",  !!(uEcxGst & RT_BIT( 9)),  !!(uEcxHst & RT_BIT( 9)));
             pHlp->pfnPrintf(pHlp, "11:10 - Reserved                       = %#x (%#x)\n",  (uEcxGst >> 10) & 3,    (uEcxHst >> 10) & 3);
-            pHlp->pfnPrintf(pHlp, "SKINIT, STGI, and DEV support          = %d (%d)\n",  !!(uEcxGst & BIT(12)),  !!(uEcxHst & BIT(12)));
-            pHlp->pfnPrintf(pHlp, "Watchdog timer support.                = %d (%d)\n",  !!(uEcxGst & BIT(13)),  !!(uEcxHst & BIT(13)));
+            pHlp->pfnPrintf(pHlp, "SKINIT, STGI, and DEV support          = %d (%d)\n",  !!(uEcxGst & RT_BIT(12)),  !!(uEcxHst & RT_BIT(12)));
+            pHlp->pfnPrintf(pHlp, "Watchdog timer support.                = %d (%d)\n",  !!(uEcxGst & RT_BIT(13)),  !!(uEcxHst & RT_BIT(13)));
             pHlp->pfnPrintf(pHlp, "31:14 - Reserved                       = %#x (%#x)\n",   uEcxGst >> 14,          uEcxHst >> 14);
         }
     }
@@ -1462,14 +1471,14 @@ static DECLCALLBACK(void) cpumR3CpuIdInfo(PVM pVM, PCDBGFINFOHLP pHlp, const cha
         uint32_t uEDX = pVM->cpum.s.aGuestCpuIdExt[7].edx;
 
         pHlp->pfnPrintf(pHlp, "APM Features:                   ");
-        if (uEDX & BIT(0))   pHlp->pfnPrintf(pHlp, " TS");
-        if (uEDX & BIT(1))   pHlp->pfnPrintf(pHlp, " FID");
-        if (uEDX & BIT(2))   pHlp->pfnPrintf(pHlp, " VID");
-        if (uEDX & BIT(3))   pHlp->pfnPrintf(pHlp, " TTP");
-        if (uEDX & BIT(4))   pHlp->pfnPrintf(pHlp, " TM");
-        if (uEDX & BIT(5))   pHlp->pfnPrintf(pHlp, " STC");
+        if (uEDX & RT_BIT(0))   pHlp->pfnPrintf(pHlp, " TS");
+        if (uEDX & RT_BIT(1))   pHlp->pfnPrintf(pHlp, " FID");
+        if (uEDX & RT_BIT(2))   pHlp->pfnPrintf(pHlp, " VID");
+        if (uEDX & RT_BIT(3))   pHlp->pfnPrintf(pHlp, " TTP");
+        if (uEDX & RT_BIT(4))   pHlp->pfnPrintf(pHlp, " TM");
+        if (uEDX & RT_BIT(5))   pHlp->pfnPrintf(pHlp, " STC");
         for (unsigned iBit = 6; iBit < 32; iBit++)
-            if (uEDX & BIT(iBit))
+            if (uEDX & RT_BIT(iBit))
                 pHlp->pfnPrintf(pHlp, " %d", iBit);
         pHlp->pfnPrintf(pHlp, "\n");
     }
@@ -1524,20 +1533,18 @@ typedef struct CPUMDISASSTATE
 /**
  * Instruction reader.
  *
- * @returns VBox status code. (Why this is a int32_t and not just an int is also beyond me.)
+ * @returns VBox status code.
  * @param   PtrSrc      Address to read from.
  *                      In our case this is relative to the selector pointed to by the 2nd user argument of uDisCpu.
  * @param   pu8Dst      Where to store the bytes.
  * @param   cbRead      Number of bytes to read.
- * @param   uDisCpu     Pointer to the disassembler cpu state. (Why this is a VBOXHUINTPTR is beyond me...)
+ * @param   uDisCpu     Pointer to the disassembler cpu state.
  *                      In this context it's always pointer to the Core of a DBGFDISASSTATE.
- * @todo r=bird: The status code should be an int. The PtrSrc should *NOT* be a RTHCUINTPTR. The uDisCpu could just as well be
- *               declared as what it actually is a PDISCPUSTATE.
  */
-static DECLCALLBACK(int32_t) cpumR3DisasInstrRead(RTHCUINTPTR PtrSrc, uint8_t *pu8Dst, uint32_t cbRead, RTHCUINTPTR uDisCpu)
+static DECLCALLBACK(int) cpumR3DisasInstrRead(RTHCUINTPTR PtrSrc, uint8_t *pu8Dst, uint32_t cbRead, void *uDisCpu)
 {
     PDISCPUSTATE pCpu = (PDISCPUSTATE)uDisCpu;
-    PCPUMDISASSTATE pState = (PCPUMDISASSTATE)pCpu->dwUserData[0]; /** @todo r=bird: Invalid prefix, dw='double word' which it isn't. Besides it's an array too. And btw. RTHCUINTPTR isn't the right thing either in a 32-bit host 64-bit guest situation */
+    PCPUMDISASSTATE pState = (PCPUMDISASSTATE)pCpu->apvUserData[0];
     Assert(cbRead > 0);
     for (;;)
     {
@@ -1661,15 +1668,17 @@ CPUMR3DECL(int) CPUMR3DisasmInstrCPU(PVM pVM, PCPUMCTX pCtx, RTGCPTR GCPtrPC, PD
      * Disassemble the instruction.
      */
     pCpu->pfnReadBytes    = cpumR3DisasInstrRead;
-    pCpu->dwUserData[0]   = (uintptr_t)&State;
+    pCpu->apvUserData[0]  = (void *)&State;
 
     uint32_t cbInstr;
 #ifdef LOG_ENABLED
-    if (DISInstr(pCpu, GCPtrPC, 0, &cbInstr, NULL))
+    rc = DISInstr(pCpu, GCPtrPC, 0, &cbInstr, NULL);
+    if (VBOX_SUCCESS(rc))
     {
 #else
     char szOutput[160];
-    if (DISInstr(pCpu, GCPtrPC, 0, &cbInstr, &szOutput[0]))
+    rc = DISInstr(pCpu, GCPtrPC, 0, &cbInstr, &szOutput[0]);
+    if (VBOX_SUCCESS(rc))
     {
         /* log it */
         if (pszPrefix)
@@ -1677,17 +1686,10 @@ CPUMR3DECL(int) CPUMR3DisasmInstrCPU(PVM pVM, PCPUMCTX pCtx, RTGCPTR GCPtrPC, PD
         else
             Log(("%s", szOutput));
 #endif
-        return VINF_SUCCESS;
+        rc = VINF_SUCCESS;
     }
-
-    /* DISInstr failure */
-    if (VBOX_FAILURE(State.rc))
-    {
-        Log(("CPUMR3DisasmInstrCPU: DISInstr failed for %04X:%VGv rc=%Vrc\n", pCtx->cs, GCPtrPC, State.rc));
-        return State.rc;
-    }
-    Log(("CPUMR3DisasmInstrCPU: DISInstr failed for %04X:%VGv\n", pCtx->cs, GCPtrPC));
-    rc = VERR_GENERAL_FAILURE;
+    else
+        Log(("CPUMR3DisasmInstrCPU: DISInstr failed for %04X:%VGv rc=%Vrc\n", pCtx->cs, GCPtrPC, rc));
     return rc;
 }
 
