@@ -1,4 +1,4 @@
-/* $Id: MM.cpp 4388 2007-08-27 14:26:05Z vboxsync $ */
+/* $Id: MM.cpp 25983 2007-11-09 09:47:39Z frank $ */
 /** @file
  * MM - Memory Monitor(/Manager).
  */
@@ -167,13 +167,11 @@ MMR3DECL(int) MMR3Init(PVM pVM)
                                        NULL, mmR3Load, NULL);
             if (VBOX_SUCCESS(rc))
                 return rc;
-        }
 
-        /* .... failure .... */
-        mmR3Term(pVM, true /* keep the heap */);
+            /* .... failure .... */
+        }
     }
-    else
-        mmr3HeapDestroy(pVM->mm.s.pHeap);
+    mmR3Term(pVM, true /* keep the heap */);
     return rc;
 }
 
@@ -191,7 +189,11 @@ MMR3DECL(int) MMR3InitPaging(PVM pVM)
     bool        fPreAlloc;
     int rc = CFGMR3QueryBool(CFGMR3GetRoot(pVM), "RamPreAlloc", &fPreAlloc);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
+#ifdef VBOX_WITH_PREALLOC_RAM_BY_DEFAULT
+        fPreAlloc = true;
+#else
         fPreAlloc = false;
+#endif
     else
         AssertMsgRCReturn(rc, ("Configuration error: Failed to query integer \"RamPreAlloc\", rc=%Vrc.\n", rc), rc);
 
@@ -282,6 +284,11 @@ MMR3DECL(int) MMR3Term(PVM pVM)
  */
 static int mmR3Term(PVM pVM, bool fKeepTheHeap)
 {
+    /*
+     * Destroy the page pool. (first as it used the hyper heap)
+     */
+    mmr3PagePoolTerm(pVM);
+
     /*
      * Release locked memory.
      * (Associated record are released by the heap.)
@@ -307,11 +314,6 @@ static int mmR3Term(PVM pVM, bool fKeepTheHeap)
         pLockedMem = pLockedMem->pNext;
     }
 
-    /*
-     * Destroy the page pool.
-     */
-    mmr3PagePoolTerm(pVM);
-
     /*
      * Destroy the heap if requested.
      */
