@@ -1,4 +1,4 @@
-/* $Id: MMInternal.h 4388 2007-08-27 14:26:05Z vboxsync $ */
+/* $Id: MMInternal.h 26504 2007-12-04 11:07:02Z frank $ */
 /** @file
  * MM - Internal header file.
  */
@@ -548,7 +548,7 @@ typedef enum MMLOOKUPHYPERTYPE
     /** Dynamic mapping area (MMR3HyperReserve).
      * A conversion will require to check what's in the page table for the pages. */
     MMLOOKUPHYPERTYPE_DYNAMIC
-};
+} MMLOOKUPHYPERTYPE;
 
 /**
  * Lookup record for the hypervisor memory area.
@@ -571,18 +571,18 @@ typedef struct MMLOOKUPHYPER
         struct
         {
             /** Host context pointer. */
-            HCPTRTYPE(void *)       pvHC;
+            R3PTRTYPE(void *)       pvHC;
             /** Host context ring-0 pointer. */
             RTR0PTR                 pvR0;
             /** Pointer to the locked mem record. */
-            HCPTRTYPE(PMMLOCKEDMEM) pLockedMem;
+            R3PTRTYPE(PMMLOCKEDMEM) pLockedMem;
         } Locked;
 
         /** Contiguous physical memory. */
         struct
         {
             /** Host context pointer. */
-            HCPTRTYPE(void *)       pvHC;
+            R3PTRTYPE(void *)       pvHC;
             /** HC physical address corresponding to pvHC. */
             RTHCPHYS                HCPhys;
         } HCPhys;
@@ -641,21 +641,21 @@ typedef struct MM
     /** The hypervisor heap (GC Ptr). */
     GCPTRTYPE(PMMHYPERHEAP)     pHyperHeapGC;
     /** The hypervisor heap (HC Ptr). */
-    HCPTRTYPE(PMMHYPERHEAP)     pHyperHeapHC;
+    R3R0PTRTYPE(PMMHYPERHEAP)   pHyperHeapHC;
 
     /** List of memory locks. (HC only) */
-    HCPTRTYPE(PMMLOCKEDMEM)     pLockedMem;
+    R3PTRTYPE(PMMLOCKEDMEM)     pLockedMem;
 
     /** Page pool. (HC only) */
-    HCPTRTYPE(PMMPAGEPOOL)      pPagePool;
+    R3R0PTRTYPE(PMMPAGEPOOL)    pPagePool;
     /** Page pool pages in low memory. (HC only) */
-    HCPTRTYPE(PMMPAGEPOOL)      pPagePoolLow;
+    R3R0PTRTYPE(PMMPAGEPOOL)    pPagePoolLow;
 
     /** Pointer to the dummy page.
      * The dummy page is a paranoia thingy used for instance for pure MMIO RAM ranges
      * to make sure any bugs will not harm whatever the system stores in the first
      * physical page. */
-    HCPTRTYPE(void *)           pvDummyPage;
+    R3PTRTYPE(void *)           pvDummyPage;
     /** Physical address of the dummy page. */
     RTHCPHYS                    HCPhysDummyPage;
 
@@ -665,12 +665,12 @@ typedef struct MM
     /** Size of the base RAM in bytes. */
     RTUINT                      cbRamBase;
     /** Pointer to the base RAM. */
-    HCPTRTYPE(void *)           pvRamBaseHC;
+    R3PTRTYPE(void *)           pvRamBaseHC;
     /** The head of the ROM ranges. */
     R3PTRTYPE(PMMROMRANGE)      pRomHead;
 
     /** Pointer to the MM R3 Heap. */
-    HCPTRTYPE(PMMHEAP)          pHeap;
+    R3PTRTYPE(PMMHEAP)          pHeap;
 
 } MM;
 /** Pointer to MM Data (part of VM). */
