@@ -1,4 +1,4 @@
-/* $Id: MMPagePool.cpp 4071 2007-08-07 17:07:59Z vboxsync $ */
+/* $Id: MMPagePool.cpp 27168 2008-01-09 16:58:25Z frank $ */
 /** @file
  * MM - Memory Monitor(/Manager) - Page Pool.
  */
@@ -60,9 +60,11 @@ int mmr3PagePoolInit(PVM pVM)
     /*
      * Allocate the pool structures.
      */
-    pVM->mm.s.pPagePool = (PMMPAGEPOOL)MMR3HeapAllocZ(pVM, MM_TAG_MM_PAGE, sizeof(MMPAGEPOOL));
-    if (!pVM->mm.s.pPagePool)
-        return VERR_NO_MEMORY;
+    AssertRelease(sizeof(*pVM->mm.s.pPagePool) + sizeof(*pVM->mm.s.pPagePoolLow) < PAGE_SIZE);
+    int rc = SUPPageAllocLocked(1, (void **)&pVM->mm.s.pPagePool);
+    if (VBOX_FAILURE(rc))
+        return rc;
+    memset(pVM->mm.s.pPagePool, 0, PAGE_SIZE);
     pVM->mm.s.pPagePool->pVM = pVM;
     STAM_REG(pVM, &pVM->mm.s.pPagePool->cPages,         STAMTYPE_U32,     "/MM/Page/Def/cPages",        STAMUNIT_PAGES, "Number of pages in the default pool.");
     STAM_REG(pVM, &pVM->mm.s.pPagePool->cFreePages,     STAMTYPE_U32,     "/MM/Page/Def/cFreePages",    STAMUNIT_PAGES, "Number of free pages in the default pool.");
@@ -73,9 +75,7 @@ int mmr3PagePoolInit(PVM pVM)
     STAM_REG(pVM, &pVM->mm.s.pPagePool->cToVirtCalls,   STAMTYPE_COUNTER, "/MM/Page/Def/cToVirtCalls",  STAMUNIT_CALLS, "Number of MMR3PagePhys2Page()+MMR3PageFreeByPhys() calls for the default pool.");
     STAM_REG(pVM, &pVM->mm.s.pPagePool->cErrors,        STAMTYPE_COUNTER, "/MM/Page/Def/cErrors",       STAMUNIT_ERRORS,"Number of errors for the default pool.");
 
-    pVM->mm.s.pPagePoolLow = (PMMPAGEPOOL)MMR3HeapAllocZ(pVM, MM_TAG_MM_PAGE, sizeof(MMPAGEPOOL));
-    if (!pVM->mm.s.pPagePoolLow)
-        return VERR_NO_MEMORY;
+    pVM->mm.s.pPagePoolLow = pVM->mm.s.pPagePool + 1;
     pVM->mm.s.pPagePoolLow->pVM = pVM;
     pVM->mm.s.pPagePoolLow->fLow = true;
     STAM_REG(pVM, &pVM->mm.s.pPagePoolLow->cPages,      STAMTYPE_U32,     "/MM/Page/Low/cPages",        STAMUNIT_PAGES, "Number of pages in the <4GB pool.");
@@ -119,7 +119,6 @@ void mmr3PagePoolTerm(PVM pVM)
             /* next */
             pSubPool = pSubPool->pNext;
         }
-
         pVM->mm.s.pPagePool = NULL;
     }
 
@@ -139,8 +138,7 @@ void mmr3PagePoolTerm(PVM pVM)
             /* next */
             pSubPool = pSubPool->pNext;
         }
-
-        pVM->mm.s.pPagePool = NULL;
+        pVM->mm.s.pPagePoolLow = NULL;
     }
 }
 
@@ -219,16 +217,19 @@ DECLINLINE(void *) mmr3PagePoolAlloc(PMMPAGEPOOL pPool)
      * Allocate new subpool.
      */
     unsigned        cPages = !pPool->fLow ? 128 : 32;
-    PMMPAGESUBPOOL  pSub = (PMMPAGESUBPOOL)MMR3HeapAlloc(pPool->pVM, MM_TAG_MM_PAGE,
-                                                         RT_OFFSETOF(MMPAGESUBPOOL, auBitmap[cPages / (sizeof(pSub->auBitmap[0] * 8))])
-                                                         + (sizeof(SUPPAGE) + sizeof(MMPPLOOKUPHCPHYS)) * cPages
-                                                         + sizeof(MMPPLOOKUPHCPTR));
-    if (!pSub)
+    PMMPAGESUBPOOL  pSub;
+    int rc = MMHyperAlloc(pPool->pVM, 
+                          RT_OFFSETOF(MMPAGESUBPOOL, auBitmap[cPages / (sizeof(pSub->auBitmap[0] * 8))])
+                          + (sizeof(SUPPAGE) + sizeof(MMPPLOOKUPHCPHYS)) * cPages
+                          + sizeof(MMPPLOOKUPHCPTR),
+                          0,
+                          MM_TAG_MM_PAGE,
+                          (void **)&pSub);
+    if (VBOX_FAILURE(rc))
         return NULL;
 
     PSUPPAGE paPhysPages = (PSUPPAGE)&pSub->auBitmap[cPages / (sizeof(pSub->auBitmap[0]) * 8)];
     Assert((uintptr_t)paPhysPages >= (uintptr_t)&pSub->auBitmap[1]);
-    int rc;
     if (!pPool->fLow)
     {
         /*
@@ -311,7 +312,7 @@ DECLINLINE(void *) mmr3PagePoolAlloc(PMMPAGEPOOL pPool)
     STAM_COUNTER_INC(&pPool->cErrors);
     if (pPool->fLow)
         VMSetError(pPool->pVM, rc, RT_SRC_POS,
-                   N_("Failed to expand page pool for memory below 4GB. current size: %d pages\n"), 
+                   N_("Failed to expand page pool for memory below 4GB. current size: %d pages"), 
                    pPool->cPages);
     AssertMsgFailed(("Failed to expand pool%s. rc=%Vrc poolsize=%d\n",
                      pPool->fLow ? " (<4GB)" : "", rc, pPool->cPages));
