@@ -1,4 +1,4 @@
-/* $Id: CSAM.cpp 4071 2007-08-07 17:07:59Z vboxsync $ */
+/* $Id: CSAM.cpp 25923 2007-11-03 09:35:04Z sandervl $ */
 /** @file
  * CSAM - Guest OS Code Scanning and Analysis Manager
  */
@@ -233,6 +233,9 @@ static int csamReinit(PVM pVM)
     pVM->csam.s.cDangerousInstr = 0;
     pVM->csam.s.iDangerousInstr  = 0;
 
+    memset(pVM->csam.s.pvCallInstruction, 0, sizeof(pVM->csam.s.pvCallInstruction));
+    pVM->csam.s.iCallInstruction = 0;
+
     /** @note never mess with the pgdir bitmap here! */
     return VINF_SUCCESS;
 }
@@ -547,10 +550,10 @@ static DECLCALLBACK(int) csamr3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Versio
  * @returns             Host context pointer or NULL in case of an error
  *
  */
-static HCPTRTYPE(void *) CSAMGCVirtToHCVirt(PVM pVM, PCSAMP2GLOOKUPREC pCacheRec, GCPTRTYPE(uint8_t *) pGCPtr)
+static R3PTRTYPE(void *) CSAMGCVirtToHCVirt(PVM pVM, PCSAMP2GLOOKUPREC pCacheRec, GCPTRTYPE(uint8_t *) pGCPtr)
 {
     int rc;
-    HCPTRTYPE(void *) pHCPtr;
+    R3PTRTYPE(void *) pHCPtr;
 
     STAM_PROFILE_START(&pVM->csam.s.StatTimeAddrConv, a);
 
@@ -574,9 +577,9 @@ static HCPTRTYPE(void *) CSAMGCVirtToHCVirt(PVM pVM, PCSAMP2GLOOKUPREC pCacheRec
         STAM_PROFILE_STOP(&pVM->csam.s.StatTimeAddrConv, a);
         return NULL;
     }
-////invalid?    Assert(sizeof(HCPTRTYPE(uint8_t*)) == sizeof(uint32_t));
+////invalid?    Assert(sizeof(R3PTRTYPE(uint8_t*)) == sizeof(uint32_t));
 
-    pCacheRec->pPageLocStartHC = (HCPTRTYPE(uint8_t*))((RTHCUINTPTR)pHCPtr & PAGE_BASE_HC_MASK);
+    pCacheRec->pPageLocStartHC = (R3PTRTYPE(uint8_t*))((RTHCUINTPTR)pHCPtr & PAGE_BASE_HC_MASK);
     pCacheRec->pGuestLoc      = pGCPtr & PAGE_BASE_GC_MASK;
     STAM_PROFILE_STOP(&pVM->csam.s.StatTimeAddrConv, a);
     return pHCPtr;
@@ -592,17 +595,14 @@ static HCPTRTYPE(void *) CSAMGCVirtToHCVirt(PVM pVM, PCSAMP2GLOOKUPREC pCacheRec
  * @param   dwUserdata  Callback specific user data (pCpu)
  *
  */
-int32_t CSAMR3ReadBytes(RTHCUINTPTR pSrc, uint8_t *pDest, uint32_t size, RTHCUINTPTR dwUserdata)
+static DECLCALLBACK(int) CSAMR3ReadBytes(RTHCUINTPTR pSrc, uint8_t *pDest, unsigned size, void *pvUserdata)
 {
-    DISCPUSTATE  *pCpu     = (DISCPUSTATE *)dwUserdata;
-    PVM           pVM      = (PVM)pCpu->dwUserData[0];
-    RTHCUINTPTR   pInstrHC = pCpu->dwUserData[1];
-    RTGCUINTPTR   pInstrGC = pCpu->dwUserData[2];
+    DISCPUSTATE  *pCpu     = (DISCPUSTATE *)pvUserdata;
+    PVM           pVM      = (PVM)pCpu->apvUserData[0];
+    RTHCUINTPTR   pInstrHC = (RTHCUINTPTR)pCpu->apvUserData[1];
+    RTGCUINTPTR   pInstrGC = (uintptr_t)pCpu->apvUserData[2];
     int           orgsize  = size;
 
-    Assert(sizeof(RTHCUINTPTR) <= sizeof(pCpu->dwUserData[0]));
-    Assert(sizeof(RTGCUINTPTR) <= sizeof(pCpu->dwUserData[0]));
-
     /* We are not interested in patched instructions, so read the original opcode bytes. */
     /** @note single instruction patches (int3) are checked in CSAMR3AnalyseCallback */
     for (int i=0;i<orgsize;i++)
@@ -637,12 +637,12 @@ int32_t CSAMR3ReadBytes(RTHCUINTPTR pSrc, uint8_t *pDest, uint32_t size, RTHCUIN
     return VINF_SUCCESS;
 }
 
-inline bool CSAMR3DISInstr(PVM pVM, DISCPUSTATE *pCpu, RTGCPTR InstrGC, uint8_t *InstrHC, uint32_t *pOpsize, char *pszOutput)
+inline int CSAMR3DISInstr(PVM pVM, DISCPUSTATE *pCpu, RTGCPTR InstrGC, uint8_t *InstrHC, uint32_t *pOpsize, char *pszOutput)
 {
     (pCpu)->pfnReadBytes  = CSAMR3ReadBytes;
-    (pCpu)->dwUserData[0] = (RTHCUINTPTR)pVM;
-    (pCpu)->dwUserData[1] = (RTHCUINTPTR)InstrHC;
-    (pCpu)->dwUserData[2] = (RTHCUINTPTR)InstrGC;
+    (pCpu)->apvUserData[0] = pVM;
+    (pCpu)->apvUserData[1] = InstrHC;
+    (pCpu)->apvUserData[2] = (void *)InstrGC; Assert(sizeof(InstrGC) <= sizeof(pCpu->apvUserData[0]));
 #ifdef DEBUG
     return DISInstrEx(pCpu, InstrGC, 0, pOpsize, pszOutput, OPTYPE_ALL);
 #else
@@ -717,7 +717,6 @@ static int CSAMR3AnalyseCallback(PVM pVM, DISCPUSTATE *pCpu, GCPTRTYPE(uint8_t *
         {
             DISCPUSTATE  cpu;
             uint8_t     *pCurInstrHC = 0;
-            bool         disret;
 
             if (cbInstr + opsize >= SIZEOF_NEARJUMP32)
                 break;
@@ -740,9 +739,9 @@ static int CSAMR3AnalyseCallback(PVM pVM, DISCPUSTATE *pCpu, GCPTRTYPE(uint8_t *
             Assert(VALID_PTR(pCurInstrHC));
 
             cpu.mode = (pPage->fCode32) ? CPUMODE_32BIT : CPUMODE_16BIT;
-            disret = CSAMR3DISInstr(pVM, &cpu, pCurInstrGC, pCurInstrHC, &opsize, NULL);
-            Assert(disret == true);
-            if (disret == false)
+            rc = CSAMR3DISInstr(pVM, &cpu, pCurInstrGC, pCurInstrHC, &opsize, NULL);
+            Assert(VBOX_SUCCESS(rc));
+            if (VBOX_FAILURE(rc))
                 break;
         }
         break;
@@ -888,7 +887,7 @@ static int csamAnalyseCallCodeStream(PVM pVM, GCPTRTYPE(uint8_t *) pInstrGC, GCP
             DISCPUSTATE  cpu;
             uint32_t     opsize;
             uint8_t     *pCurInstrHC = 0;
-            bool         disret;
+            int          rc2;
 #ifdef DEBUG
             char szOutput[256];
 #endif
@@ -923,15 +922,15 @@ static int csamAnalyseCallCodeStream(PVM pVM, GCPTRTYPE(uint8_t *) pInstrGC, GCP
                 cpu.mode = (fCode32) ? CPUMODE_32BIT : CPUMODE_16BIT;
                 STAM_PROFILE_START(&pVM->csam.s.StatTimeDisasm, a);
 #ifdef DEBUG
-                disret = CSAMR3DISInstr(pVM, &cpu, pCurInstrGC, pCurInstrHC, &opsize, szOutput);
-                if (disret == true) Log(("CSAM Call Analysis: %s", szOutput));
+                rc2 = CSAMR3DISInstr(pVM, &cpu, pCurInstrGC, pCurInstrHC, &opsize, szOutput);
+                if (VBOX_SUCCESS(rc2)) Log(("CSAM Call Analysis: %s", szOutput));
 #else
-                disret = CSAMR3DISInstr(pVM, &cpu, pCurInstrGC, pCurInstrHC, &opsize, NULL);
+                rc2 = CSAMR3DISInstr(pVM, &cpu, pCurInstrGC, pCurInstrHC, &opsize, NULL);
 #endif
                 STAM_PROFILE_STOP(&pVM->csam.s.StatTimeDisasm, a);
-                if (disret == false)
+                if (VBOX_FAILURE(rc2))
                 {
-                    Log(("Disassembly failed at %VGv (probably page not present) -> return to caller\n", pCurInstrGC));
+                    Log(("Disassembly failed at %VGv with %Vrc (probably page not present) -> return to caller\n", pCurInstrGC, rc2));
                     goto done;
                 }
 
@@ -1071,8 +1070,8 @@ static int csamAnalyseCodeStream(PVM pVM, GCPTRTYPE(uint8_t *) pInstrGC, GCPTRTY
     PCSAMPAGE pPage = (PCSAMPAGE)pUserData;
     int rc = VWRN_CONTINUE_ANALYSIS;
     uint32_t opsize;
-    HCPTRTYPE(uint8_t *) pCurInstrHC = 0;
-    bool disret;
+    R3PTRTYPE(uint8_t *) pCurInstrHC = 0;
+    int rc2;
 
 #ifdef DEBUG
     char szOutput[256];
@@ -1133,15 +1132,15 @@ static int csamAnalyseCodeStream(PVM pVM, GCPTRTYPE(uint8_t *) pInstrGC, GCPTRTY
         cpu.mode = (fCode32) ? CPUMODE_32BIT : CPUMODE_16BIT;
         STAM_PROFILE_START(&pVM->csam.s.StatTimeDisasm, a);
 #ifdef DEBUG
-        disret = CSAMR3DISInstr(pVM, &cpu, pCurInstrGC, pCurInstrHC, &opsize, szOutput);
-        if (disret == true) Log(("CSAM Analysis: %s", szOutput));
+        rc2 = CSAMR3DISInstr(pVM, &cpu, pCurInstrGC, pCurInstrHC, &opsize, szOutput);
+        if (VBOX_SUCCESS(rc2)) Log(("CSAM Analysis: %s", szOutput));
 #else
-        disret = CSAMR3DISInstr(pVM, &cpu, pCurInstrGC, pCurInstrHC, &opsize, NULL);
+        rc2 = CSAMR3DISInstr(pVM, &cpu, pCurInstrGC, pCurInstrHC, &opsize, NULL);
 #endif
         STAM_PROFILE_STOP(&pVM->csam.s.StatTimeDisasm, a);
-        if (disret == false)
+        if (VBOX_FAILURE(rc2))
         {
-            Log(("Disassembly failed at %VGv (probably page not present) -> return to caller\n", pCurInstrGC));
+            Log(("Disassembly failed at %VGv with %Vrc (probably page not present) -> return to caller\n", pCurInstrGC, rc2));
             rc = VINF_SUCCESS;
             goto done;
         }
@@ -1191,9 +1190,19 @@ static int csamAnalyseCodeStream(PVM pVM, GCPTRTYPE(uint8_t *) pInstrGC, GCPTRTY
             goto done;
 
         // For our first attempt, we'll handle only simple relative jumps and calls (immediate offset coded in instruction)
-        if ((cpu.pCurInstr->optype & OPTYPE_CONTROLFLOW) && (OP_PARM_VTYPE(cpu.pCurInstr->param1) == OP_PARM_J))
+        if (    ((cpu.pCurInstr->optype & OPTYPE_CONTROLFLOW) && (OP_PARM_VTYPE(cpu.pCurInstr->param1) == OP_PARM_J))
+            ||  (cpu.pCurInstr->opcode == OP_CALL && cpu.param1.flags == USE_DISPLACEMENT32))  /* simple indirect call (call dword ptr [address]) */
         {
-            addr = CSAMResolveBranch(&cpu, pCurInstrGC);
+            /* We need to parse 'call dword ptr [address]' type of calls to catch cpuid instructions in some recent Linux distributions (e.g. OpenSuse 10.3) */
+            if (    cpu.pCurInstr->opcode == OP_CALL 
+                &&  cpu.param1.flags == USE_DISPLACEMENT32)
+            {
+                addr = 0;
+                PGMPhysReadGCPtr(pVM, &addr, cpu.param1.disp32, sizeof(addr));
+            }
+            else
+                addr = CSAMResolveBranch(&cpu, pCurInstrGC);
+
             if (addr == 0)
             {
                 Log(("We don't support far jumps here!! (%08X)\n", cpu.param1.flags));
@@ -2251,7 +2260,29 @@ CSAMR3DECL(int) CSAMR3CheckGates(PVM pVM, uint32_t iGate, uint32_t cGates)
         return VERR_INVALID_PARAMETER;
 
     if (cGates != 1)
+    {
         pVM->csam.s.fGatesChecked = true;
+        for (unsigned i=0;i<RT_ELEMENTS(pVM->csam.s.pvCallInstruction);i++)
+        {
+            RTGCPTR pHandler = pVM->csam.s.pvCallInstruction[i];
+
+            if (pHandler)
+            {
+                CSAMP2GLOOKUPREC cacheRec = {0};            /* Cache record for PATMGCVirtToHCVirt. */
+                PCSAMPAGE pPage = NULL;
+
+                Log(("CSAMCheckGates: checking previous call instruction %VGv\n", pHandler));
+                STAM_PROFILE_START(&pVM->csam.s.StatTime, a);
+                rc = csamAnalyseCodeStream(pVM, pHandler, pHandler, true, CSAMR3AnalyseCallback, pPage, &cacheRec);
+                STAM_PROFILE_STOP(&pVM->csam.s.StatTime, a);
+                if (rc != VINF_SUCCESS)
+                {
+                    Log(("CSAMCheckGates: csamAnalyseCodeStream failed with %d\n", rc));
+                    continue;
+                }
+            }
+        }
+    }
 
     /* Determine valid upper boundary. */
     maxGates   = (cbIDT+1) / sizeof(VBOXIDTE);
@@ -2414,6 +2445,31 @@ CSAMR3DECL(int) CSAMR3CheckGates(PVM pVM, uint32_t iGate, uint32_t cGates)
     return VINF_SUCCESS;
 }
 
+/**
+ * Record previous call instruction addresses
+ *
+ * @returns VBox status code.
+ * @param   pVM         The VM to operate on.
+ * @param   GCPtrCall   Call address
+ */
+CSAMR3DECL(int) CSAMR3RecordCallAddress(PVM pVM, RTGCPTR GCPtrCall)
+{
+    for (unsigned i=0;i<RT_ELEMENTS(pVM->csam.s.pvCallInstruction);i++)
+    {
+        if (pVM->csam.s.pvCallInstruction[i] == GCPtrCall)
+            return VINF_SUCCESS;
+    }
+
+    Log(("CSAMR3RecordCallAddress %VGv\n", GCPtrCall));
+
+    pVM->csam.s.pvCallInstruction[pVM->csam.s.iCallInstruction++] = GCPtrCall;
+    if (pVM->csam.s.iCallInstruction >= RT_ELEMENTS(pVM->csam.s.pvCallInstruction))
+        pVM->csam.s.iCallInstruction = 0;
+
+    return VINF_SUCCESS;
+}
+
+
 /**
  * Query CSAM state (enabled/disabled)
  *
