@@ -1,4 +1,4 @@
-/* $Id: PATMInternal.h 4071 2007-08-07 17:07:59Z vboxsync $ */
+/* $Id: PATMInternal.h 25923 2007-11-03 09:35:04Z sandervl $ */
 /** @file
  * PATM - Internal header file.
  */
@@ -40,32 +40,32 @@
 #define MAX_PATCH_SIZE                     (1024*4)
 
 /*
- * Internal patch type flags (starts at BIT(11))
+ * Internal patch type flags (starts at RT_BIT(11))
  */
 
-#define PATMFL_CHECK_SIZE                   BIT64(11)
-#define PATMFL_FOUND_PATCHEND               BIT64(12)
-#define PATMFL_SINGLE_INSTRUCTION           BIT64(13)
-#define PATMFL_SYSENTER_XP                  BIT64(14)
-#define PATMFL_JUMP_CONFLICT                BIT64(15)
-#define PATMFL_READ_ORIGINAL_BYTES          BIT64(16) /** opcode might have already been patched */
-#define PATMFL_INT3_REPLACEMENT             BIT64(17)
-#define PATMFL_SUPPORT_CALLS                BIT64(18)
-#define PATMFL_SUPPORT_INDIRECT_CALLS       BIT64(19)
-#define PATMFL_IDTHANDLER_WITHOUT_ENTRYPOINT BIT64(20) /** internal flag to avoid duplicate entrypoints */
-#define PATMFL_INHIBIT_IRQS                 BIT64(21) /** temporary internal flag */
-#define PATMFL_GENERATE_JUMPTOGUEST         BIT64(22) /** temporary internal flag */
-#define PATMFL_RECOMPILE_NEXT               BIT64(23) /** for recompilation of the next instruction */
-#define PATMFL_CODE_MONITORED               BIT64(24) /** code pages of guest monitored for self-modifying code. */
-#define PATMFL_CALLABLE_AS_FUNCTION         BIT64(25) /** cli and pushf blocks can be used as callable functions. */
-#define PATMFL_GLOBAL_FUNCTIONS             BIT64(26) /** fake patch for global patm functions. */
-#define PATMFL_TRAMPOLINE                   BIT64(27) /** trampoline patch that clears PATM_INTERRUPTFLAG and jumps to patch destination */
-#define PATMFL_GENERATE_SETPIF              BIT64(28) /** generate set PIF for the next instruction */
-#define PATMFL_INSTR_HINT                   BIT64(29) /** Generate patch, but don't activate it. */
-#define PATMFL_PATCHED_GUEST_CODE           BIT64(30) /** Patched guest code. */
-#define PATMFL_MUST_INSTALL_PATCHJMP        BIT64(31) /** Need to patch guest code in order to activate patch. */
-#define PATMFL_INT3_REPLACEMENT_BLOCK       BIT64(32) /** int 3 replacement block */
-#define PATMFL_EXTERNAL_JUMP_INSIDE         BIT64(33) /** A trampoline patch was created that jumps to an instruction in the patch block */
+#define PATMFL_CHECK_SIZE                   RT_BIT_64(11)
+#define PATMFL_FOUND_PATCHEND               RT_BIT_64(12)
+#define PATMFL_SINGLE_INSTRUCTION           RT_BIT_64(13)
+#define PATMFL_SYSENTER_XP                  RT_BIT_64(14)
+#define PATMFL_JUMP_CONFLICT                RT_BIT_64(15)
+#define PATMFL_READ_ORIGINAL_BYTES          RT_BIT_64(16) /** opcode might have already been patched */
+#define PATMFL_INT3_REPLACEMENT             RT_BIT_64(17)
+#define PATMFL_SUPPORT_CALLS                RT_BIT_64(18)
+#define PATMFL_SUPPORT_INDIRECT_CALLS       RT_BIT_64(19)
+#define PATMFL_IDTHANDLER_WITHOUT_ENTRYPOINT RT_BIT_64(20) /** internal flag to avoid duplicate entrypoints */
+#define PATMFL_INHIBIT_IRQS                 RT_BIT_64(21) /** temporary internal flag */
+#define PATMFL_GENERATE_JUMPTOGUEST         RT_BIT_64(22) /** temporary internal flag */
+#define PATMFL_RECOMPILE_NEXT               RT_BIT_64(23) /** for recompilation of the next instruction */
+#define PATMFL_CODE_MONITORED               RT_BIT_64(24) /** code pages of guest monitored for self-modifying code. */
+#define PATMFL_CALLABLE_AS_FUNCTION         RT_BIT_64(25) /** cli and pushf blocks can be used as callable functions. */
+#define PATMFL_GLOBAL_FUNCTIONS             RT_BIT_64(26) /** fake patch for global patm functions. */
+#define PATMFL_TRAMPOLINE                   RT_BIT_64(27) /** trampoline patch that clears PATM_INTERRUPTFLAG and jumps to patch destination */
+#define PATMFL_GENERATE_SETPIF              RT_BIT_64(28) /** generate set PIF for the next instruction */
+#define PATMFL_INSTR_HINT                   RT_BIT_64(29) /** Generate patch, but don't activate it. */
+#define PATMFL_PATCHED_GUEST_CODE           RT_BIT_64(30) /** Patched guest code. */
+#define PATMFL_MUST_INSTALL_PATCHJMP        RT_BIT_64(31) /** Need to patch guest code in order to activate patch. */
+#define PATMFL_INT3_REPLACEMENT_BLOCK       RT_BIT_64(32) /** int 3 replacement block */
+#define PATMFL_EXTERNAL_JUMP_INSIDE         RT_BIT_64(33) /** A trampoline patch was created that jumps to an instruction in the patch block */
 
 #define SIZEOF_NEARJUMP8                   2 //opcode byte + 1 byte relative offset
 #define SIZEOF_NEARJUMP16                  3 //opcode byte + 2 byte relative offset
@@ -130,15 +130,15 @@ typedef struct
     AVLPVNODECORE   Core;
 
     uint32_t        uType;
-    HCPTRTYPE(uint8_t *) pRelocPos;
+    R3PTRTYPE(uint8_t *) pRelocPos;
     RTGCPTR         pSource;
     RTGCPTR         pDest;
 } RELOCREC, *PRELOCREC;
 
 typedef struct
 {
-    HCPTRTYPE(uint8_t *) pPatchLocStartHC;
-    HCPTRTYPE(uint8_t *) pPatchLocEndHC;
+    R3PTRTYPE(uint8_t *) pPatchLocStartHC;
+    R3PTRTYPE(uint8_t *) pPatchLocEndHC;
     GCPTRTYPE(uint8_t *) pGuestLoc;
     uint32_t             opsize;
 } P2GLOOKUPREC, *PP2GLOOKUPREC;
@@ -148,7 +148,7 @@ typedef struct
     /** The key is a pointer to a JUMPREC structure. */
     AVLPVNODECORE   Core;
 
-    HCPTRTYPE(uint8_t *) pJumpHC;
+    R3PTRTYPE(uint8_t *) pJumpHC;
     GCPTRTYPE(uint8_t *) pTargetGC;
     uint32_t            offDispl;
     uint32_t            opcode;
@@ -196,14 +196,14 @@ typedef struct RECGUESTTOPATCH
 typedef struct
 {
     /* Temporary tree for storing the addresses of illegal instructions. */
-    HCPTRTYPE(PAVLPVNODECORE)   IllegalInstrTree;
+    R3PTRTYPE(PAVLPVNODECORE)   IllegalInstrTree;
     uint32_t                    nrIllegalInstr;
 
     int32_t                     nrJumps;
     uint32_t                    nrRetInstr;
 
     /* Temporary tree of encountered jumps. (debug only) */
-    HCPTRTYPE(PAVLPVNODECORE)   DisasmJumpTree;
+    R3PTRTYPE(PAVLPVNODECORE)   DisasmJumpTree;
 
     int32_t                     nrCalls;
 
@@ -222,7 +222,7 @@ typedef struct _PATCHINFO
     uint32_t        uOpMode;
 
     GCPTRTYPE(uint8_t *)  pPrivInstrGC;    //GC pointer of privileged instruction
-    HCPTRTYPE(uint8_t *)  pPrivInstrHC;    //HC pointer of privileged instruction
+    R3PTRTYPE(uint8_t *)  pPrivInstrHC;    //HC pointer of privileged instruction
     uint8_t         aPrivInstr[MAX_INSTR_SIZE];
     uint32_t        cbPrivInstr;
     uint32_t        opcode;      //opcode for priv instr (OP_*)
@@ -247,19 +247,19 @@ typedef struct _PATCHINFO
     RTGCPTR                   pInstrGCHighest;
 
     /* Tree of fixup records for the patch. */
-    HCPTRTYPE(PAVLPVNODECORE) FixupTree;
+    R3PTRTYPE(PAVLPVNODECORE) FixupTree;
     int32_t         nrFixups;
 
     /* Tree of jumps inside the generated patch code. */
     int32_t         nrJumpRecs;
-    HCPTRTYPE(PAVLPVNODECORE) JumpTree;
+    R3PTRTYPE(PAVLPVNODECORE) JumpTree;
 
     /**
      * Lookup trees for determining the corresponding guest address of an
      * instruction in the patch block.
      */
-    HCPTRTYPE(PAVLU32NODECORE) Patch2GuestAddrTree;
-    HCPTRTYPE(PAVLGCPTRNODECORE) Guest2PatchAddrTree;
+    R3PTRTYPE(PAVLU32NODECORE) Patch2GuestAddrTree;
+    R3PTRTYPE(PAVLGCPTRNODECORE) Guest2PatchAddrTree;
     uint32_t                  nrPatch2GuestRecs;
 #if HC_ARCH_BITS == 64
     uint32_t        Alignment1;
@@ -269,7 +269,7 @@ typedef struct _PATCHINFO
     P2GLOOKUPREC    cacheRec;
 
     /* Temporary information during patch creation. Don't waste hypervisor memory for this. */
-    HCPTRTYPE(PPATCHINFOTEMP) pTempInfo;
+    R3PTRTYPE(PPATCHINFOTEMP) pTempInfo;
 
     /* Count the number of writes to the corresponding guest code. */
     uint32_t        cCodeWrites;
@@ -322,7 +322,7 @@ typedef struct PATMPATCHPAGE
     /** Maximum nr of pointers in the array. */
     uint32_t           cMaxPatches;
     /** Array of patch pointers for this page. */
-    HCPTRTYPE(PPATCHINFO *) aPatch;
+    R3PTRTYPE(PPATCHINFO *) aPatch;
 } PATMPATCHPAGE, *PPATMPATCHPAGE;
 
 #define PATM_PATCHREC_FROM_COREOFFSET(a)  (PPATMPATCHREC)((uintptr_t)a - RT_OFFSETOF(PATMPATCHREC, CoreOffset))
@@ -359,7 +359,7 @@ typedef struct PATM
     RTINT                   offVM;
 
     GCPTRTYPE(uint8_t *)    pPatchMemGC;
-    HCPTRTYPE(uint8_t *)    pPatchMemHC;
+    R3PTRTYPE(uint8_t *)    pPatchMemHC;
     uint32_t                cbPatchMem;
     uint32_t                offPatchMem;
     bool                    fOutOfMemory;
@@ -367,19 +367,19 @@ typedef struct PATM
     int32_t                 deltaReloc;
 
     /* GC PATM state pointers */
-    HCPTRTYPE(PPATMGCSTATE) pGCStateHC;
+    R3PTRTYPE(PPATMGCSTATE) pGCStateHC;
     GCPTRTYPE(PPATMGCSTATE) pGCStateGC;
 
     /** PATM stack page for call instruction execution. (2 parts: one for our private stack and one to store the original return address */
     GCPTRTYPE(RTGCPTR *)    pGCStackGC;
-    HCPTRTYPE(RTGCPTR *)    pGCStackHC;
+    R3PTRTYPE(RTGCPTR *)    pGCStackHC;
 
     /** GC pointer to CPUMCTX structure. */
     GCPTRTYPE(PCPUMCTX)     pCPUMCtxGC;
 
     /* GC statistics pointers */
     GCPTRTYPE(PSTAMRATIOU32) pStatsGC;
-    HCPTRTYPE(PSTAMRATIOU32) pStatsHC;
+    R3PTRTYPE(PSTAMRATIOU32) pStatsHC;
 
     /* Current free index value (uPatchRun/uPatchTrap arrays). */
     uint32_t                uCurrentPatchIdx;
@@ -398,7 +398,7 @@ typedef struct PATM
 
     /** Pointer to the patch tree for instructions replaced by 'int 3'. */
     GCPTRTYPE(PPATMTREES)   PatchLookupTreeGC;
-    HCPTRTYPE(PPATMTREES)   PatchLookupTreeHC;
+    R3PTRTYPE(PPATMTREES)   PatchLookupTreeHC;
 
     /** Global PATM lookup and call function (used by call patches). */
     RTGCPTR                 pfnHelperCallGC;
@@ -410,7 +410,7 @@ typedef struct PATM
     RTGCPTR                 pfnHelperIretGC;
 
     /** Fake patch record for global functions. */
-    HCPTRTYPE(PPATMPATCHREC) pGlobalPatchRec;
+    R3PTRTYPE(PPATMPATCHREC) pGlobalPatchRec;
 
     /** Pointer to original sysenter handler */
     RTGCPTR                 pfnSysEnterGC;
@@ -432,7 +432,7 @@ typedef struct PATM
     /* Temporary storage during load/save state */
     struct
     {
-        HCPTRTYPE(PSSMHANDLE) pSSM;
+        R3PTRTYPE(PSSMHANDLE) pSSM;
         uint32_t            cPatches;
 #if HC_ARCH_BITS == 64
         uint32_t            Alignment0; /**< Align the structure size on a 8-byte boundrary. */
@@ -523,6 +523,7 @@ DECLCALLBACK(int) patmr3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version);
 #ifdef IN_RING3
 RTGCPTR patmPatchGCPtr2GuestGCPtr(PVM pVM, PPATCHINFO pPatch, GCPTRTYPE(uint8_t *) pPatchGC);
 RTGCPTR patmGuestGCPtrToPatchGCPtr(PVM pVM, PPATCHINFO pPatch, GCPTRTYPE(uint8_t*) pInstrGC);
+RTGCPTR patmGuestGCPtrToClosestPatchGCPtr(PVM pVM, PPATCHINFO pPatch, GCPTRTYPE(uint8_t*) pInstrGC);
 #endif
 
 /* Add a patch to guest lookup record
@@ -611,7 +612,7 @@ int PATMInstallGuestSpecificPatch(PVM pVM, PDISCPUSTATE pCpu, RTGCPTR pInstrGC,
  * @returns             Host context pointer or NULL in case of an error
  *
  */
-HCPTRTYPE(uint8_t *) PATMGCVirtToHCVirt(PVM pVM, PPATCHINFO pPatch, GCPTRTYPE(uint8_t *) pGCPtr);
+R3PTRTYPE(uint8_t *) PATMGCVirtToHCVirt(PVM pVM, PPATCHINFO pPatch, GCPTRTYPE(uint8_t *) pGCPtr);
 
 
 /**
@@ -661,10 +662,10 @@ PATMDECL(const char *) patmGetInstructionString(uint32_t opcode, uint32_t fPatch
  * @param   pSrc        GC source pointer
  * @param   pDest       HC destination pointer
  * @param   size        Number of bytes to read
- * @param   dwUserdata  Callback specific user data (pCpu)
+ * @param   pvUserdata  Callback specific user data (pCpu)
  *
  */
-int32_t patmReadBytes(RTHCUINTPTR pSrc, uint8_t *pDest, uint32_t size, RTHCUINTPTR dwUserdata);
+int patmReadBytes(RTHCUINTPTR pSrc, uint8_t *pDest, unsigned size, void *pvUserdata);
 
 
 #ifndef IN_GC
@@ -680,7 +681,7 @@ typedef struct
 {
     PVM           pVM;
     PPATCHINFO    pPatchInfo;
-    HCPTRTYPE(uint8_t *) pInstrHC;
+    R3PTRTYPE(uint8_t *) pInstrHC;
     RTGCPTR       pInstrGC;
     uint32_t      fReadFlags;
 } PATMDISASM, *PPATMDISASM;
@@ -696,8 +697,8 @@ inline bool PATMR3DISInstr(PVM pVM, PPATCHINFO pPatch, DISCPUSTATE *pCpu, RTGCPT
     disinfo.pInstrGC = InstrGC;
     disinfo.fReadFlags = fReadFlags;
     (pCpu)->pfnReadBytes  = patmReadBytes;
-    (pCpu)->dwUserData[0] = (RTHCUINTPTR)&disinfo;
-    return DISInstr(pCpu, InstrGC, 0, pOpsize, pszOutput);
+    (pCpu)->apvUserData[0] = &disinfo;
+    return VBOX_SUCCESS(DISInstr(pCpu, InstrGC, 0, pOpsize, pszOutput));
 }
 #endif /* !IN_GC */
 
@@ -741,7 +742,7 @@ PPATCHINFO PATMFindActivePatchByEntrypoint(PVM pVM, RTGCPTR pInstrGC, bool fIncl
  * @note    returns failure if patching is not allowed or possible
  *
  */
-PATMR3DECL(int) PATMR3PatchBlock(PVM pVM, RTGCPTR pInstrGC, HCPTRTYPE(uint8_t *) pInstrHC,
+PATMR3DECL(int) PATMR3PatchBlock(PVM pVM, RTGCPTR pInstrGC, R3PTRTYPE(uint8_t *) pInstrHC,
                                  uint32_t uOpcode, uint32_t uOpSize, PPATMPATCHREC pPatchRec);
 
 
@@ -758,7 +759,7 @@ PATMR3DECL(int) PATMR3PatchBlock(PVM pVM, RTGCPTR pInstrGC, HCPTRTYPE(uint8_t *)
  * @note    returns failure if patching is not allowed or possible
  *
  */
-PATMR3DECL(int) PATMR3PatchInstrInt3(PVM pVM, RTGCPTR pInstrGC, HCPTRTYPE(uint8_t *) pInstrHC, DISCPUSTATE *pCpu, PPATCHINFO pPatch);
+PATMR3DECL(int) PATMR3PatchInstrInt3(PVM pVM, RTGCPTR pInstrGC, R3PTRTYPE(uint8_t *) pInstrHC, DISCPUSTATE *pCpu, PPATCHINFO pPatch);
 
 /**
  * Mark patch as dirty
