@@ -1,4 +1,4 @@
-/* $Id: PDMInternal.h 4071 2007-08-07 17:07:59Z vboxsync $ */
+/* $Id: PDMInternal.h 26583 2007-12-06 21:54:43Z bird $ */
 /** @file
  * PDM - Internal header file.
  */
@@ -23,6 +23,7 @@
 #include <VBox/param.h>
 #include <VBox/cfgm.h>
 #include <VBox/stam.h>
+#include <VBox/vusb.h>
 #include <iprt/critsect.h>
 #ifdef IN_RING3
 # include <iprt/thread.h>
@@ -78,24 +79,24 @@ typedef struct PDMDEVINSINT
 {
     /** Pointer to the next instance (HC Ptr).
      * (Head is pointed to by PDM::pDevInstances.) */
-    HCPTRTYPE(PPDMDEVINS)           pNextHC;
+    R3PTRTYPE(PPDMDEVINS)           pNextHC;
     /** Pointer to the next per device instance (HC Ptr).
      * (Head is pointed to by PDMDEV::pInstances.) */
-    HCPTRTYPE(PPDMDEVINS)           pPerDeviceNextHC;
+    R3PTRTYPE(PPDMDEVINS)           pPerDeviceNextHC;
 
     /** Pointer to device structure - HC Ptr. */
-    HCPTRTYPE(PPDMDEV)              pDevHC;
+    R3PTRTYPE(PPDMDEV)              pDevHC;
 
     /** Pointer to the VM this instance was created for - HC Ptr. */
-    HCPTRTYPE(PVM)                  pVMHC;
+    R3R0PTRTYPE(PVM)                pVMHC;
     /** Pointer to the list of logical units associated with the device. (FIFO) */
-    HCPTRTYPE(PPDMLUN)              pLunsHC;
+    R3PTRTYPE(PPDMLUN)              pLunsHC;
     /** Configuration handle to the instance node. */
-    HCPTRTYPE(PCFGMNODE)            pCfgHandle;
+    R3PTRTYPE(PCFGMNODE)            pCfgHandle;
     /** HC pointer to associated PCI device structure. */
-    HCPTRTYPE(struct PCIDevice *)   pPciDeviceHC;
+    R3R0PTRTYPE(struct PCIDevice *) pPciDeviceHC;
     /** HC pointer to associated PCI bus structure. */
-    HCPTRTYPE(PPDMPCIBUS)           pPciBusHC;
+    R3R0PTRTYPE(PPDMPCIBUS)         pPciBusHC;
 
     /** GC pointer to associated PCI device structure. */
     GCPTRTYPE(struct PCIDevice *)   pPciDeviceGC;
@@ -114,6 +115,8 @@ typedef struct PDMDEVINSINT
  */
 typedef struct PDMUSBINSINT
 {
+    /** The UUID of this instance. */
+    RTUUID                          Uuid;
     /** Pointer to the next instance.
      * (Head is pointed to by PDM::pUsbInstances.) */
     R3PTRTYPE(PPDMUSBINS)           pNext;
@@ -122,7 +125,7 @@ typedef struct PDMUSBINSINT
     R3PTRTYPE(PPDMUSBINS)           pPerDeviceNext;
 
     /** Pointer to device structure. */
-    R3PTRTYPE(PPDMUSB)              pUsb;
+    R3PTRTYPE(PPDMUSB)              pUsbDev;
 
     /** Pointer to the VM this instance was created for. */
     PVMR3                           pVM;
@@ -130,10 +133,13 @@ typedef struct PDMUSBINSINT
     R3PTRTYPE(PPDMLUN)              pLuns;
     /** The per instance device configuration. */
     R3PTRTYPE(PCFGMNODE)            pCfg;
+    /** Same as pCfg if the configuration should be deleted when detaching the device. */
+    R3PTRTYPE(PCFGMNODE)            pCfgDelete;
     /** The global device configuration. */
     R3PTRTYPE(PCFGMNODE)            pCfgGlobal;
 
-    /** Pointer to the USB hub this device is connected to. */
+    /** Pointer to the USB hub this device is attached to.
+     * This is NULL if the device isn't connected to any HUB. */
     R3PTRTYPE(PPDMUSBHUB)           pHub;
     /** The port number that we're connected to. */
     uint32_t                        iPort;
@@ -274,6 +280,8 @@ typedef struct PDMLUN
     PPDMLUN             pNext;
     /** Pointer to the top driver in the driver chain. */
     PPDMDRVINS          pTop;
+    /** Pointer to the bottom driver in the driver chain. */
+    PPDMDRVINS          pBottom;
     /** Pointer to the device instance which the LUN belongs to.
      * Either this is set or pUsbIns is set. Both is never set at the same time. */
     PPDMDEVINS          pDevIns;
@@ -292,15 +300,15 @@ typedef struct PDMLUN
 typedef struct PDMDEV
 {
     /** Pointer to the next device (HC Ptr). */
-    HCPTRTYPE(PPDMDEV)                  pNext;
+    R3PTRTYPE(PPDMDEV)                  pNext;
     /** Device name length. (search optimization) */
     RTUINT                              cchName;
     /** Registration structure. */
-    HCPTRTYPE(const struct PDMDEVREG *) pDevReg;
+    R3PTRTYPE(const struct PDMDEVREG *) pDevReg;
     /** Number of instances. */
     RTUINT                              cInstances;
     /** Pointer to chain of instances (HC Ptr). */
-    HCPTRTYPE(PPDMDEVINS)               pInstances;
+    R3PTRTYPE(PPDMDEVINS)               pInstances;
 } PDMDEV;
 
 
@@ -342,7 +350,7 @@ typedef struct PDMDRV
 typedef struct PDMPIC
 {
     /** Pointer to the PIC device instance - HC. */
-    HCPTRTYPE(PPDMDEVINS)   pDevInsR3;
+    R3PTRTYPE(PPDMDEVINS)   pDevInsR3;
     /** @copydoc PDMPICREG::pfnSetIrqHC */
     DECLR3CALLBACKMEMBER(void, pfnSetIrqR3,(PPDMDEVINS pDevIns, int iIrq, int iLevel));
     /** @copydoc PDMPICREG::pfnGetInterruptHC */
@@ -373,7 +381,7 @@ typedef struct PDMPIC
 typedef struct PDMAPIC
 {
     /** Pointer to the APIC device instance - HC Ptr. */
-    PPDMDEVINSHC                    pDevInsR3;
+    PPDMDEVINSR3                    pDevInsR3;
     /** @copydoc PDMAPICREG::pfnGetInterruptHC */
     DECLR3CALLBACKMEMBER(int,       pfnGetInterruptR3,(PPDMDEVINS pDevIns));
     /** @copydoc PDMAPICREG::pfnSetBaseHC */
@@ -431,7 +439,7 @@ typedef struct PDMAPIC
 typedef struct PDMIOAPIC
 {
     /** Pointer to the APIC device instance - HC Ptr. */
-    PPDMDEVINSHC                    pDevInsR3;
+    PPDMDEVINSR3                    pDevInsR3;
     /** @copydoc PDMIOAPICREG::pfnSetIrqHC */
     DECLR3CALLBACKMEMBER(void,      pfnSetIrqR3,(PPDMDEVINS pDevIns, int iIrq, int iLevel));
 
@@ -457,7 +465,7 @@ typedef struct PDMPCIBUS
     RTUINT          uPadding0; /**< Alignment padding.*/
 
     /** Pointer to PCI Bus device instance. */
-    PPDMDEVINSHC                    pDevInsR3;
+    PPDMDEVINSR3                    pDevInsR3;
     /** @copydoc PDMPCIBUSREG::pfnSetIrqHC */
     DECLR3CALLBACKMEMBER(void,      pfnSetIrqR3,(PPDMDEVINS pDevIns, PPCIDEVICE pPciDev, int iIrq, int iLevel));
     /** @copydoc PDMPCIBUSREG::pfnRegisterHC */
@@ -589,7 +597,7 @@ typedef struct PDMQUEUE *PPDMQUEUE;
 typedef struct PDMQUEUE
 {
     /** Pointer to the next queue in the list. */
-    HCPTRTYPE(PPDMQUEUE)    pNext;
+    R3PTRTYPE(PPDMQUEUE)    pNext;
     /** Type specific data. */
     union
     {
@@ -597,31 +605,31 @@ typedef struct PDMQUEUE
         struct
         {
             /** Pointer to consumer function. */
-            HCPTRTYPE(PFNPDMQUEUEDEV)   pfnCallback;
+            R3PTRTYPE(PFNPDMQUEUEDEV)   pfnCallback;
             /** Pointer to the device instance owning the queue. */
-            HCPTRTYPE(PPDMDEVINS)       pDevIns;
+            R3PTRTYPE(PPDMDEVINS)       pDevIns;
         } Dev;
         /** PDMQUEUETYPE_DRV */
         struct
         {
             /** Pointer to consumer function. */
-            HCPTRTYPE(PFNPDMQUEUEDRV)   pfnCallback;
+            R3PTRTYPE(PFNPDMQUEUEDRV)   pfnCallback;
             /** Pointer to the driver instance owning the queue. */
-            HCPTRTYPE(PPDMDRVINS)       pDrvIns;
+            R3PTRTYPE(PPDMDRVINS)       pDrvIns;
         } Drv;
         /** PDMQUEUETYPE_INTERNAL */
         struct
         {
             /** Pointer to consumer function. */
-            HCPTRTYPE(PFNPDMQUEUEINT)   pfnCallback;
+            R3PTRTYPE(PFNPDMQUEUEINT)   pfnCallback;
         } Int;
         /** PDMQUEUETYPE_EXTERNAL */
         struct
         {
             /** Pointer to consumer function. */
-            HCPTRTYPE(PFNPDMQUEUEEXT)   pfnCallback;
+            R3PTRTYPE(PFNPDMQUEUEEXT)   pfnCallback;
             /** Pointer to user argument. */
-            HCPTRTYPE(void *)           pvUser;
+            R3PTRTYPE(void *)           pvUser;
         } Ext;
     } u;
     /** Queue type. */
@@ -631,11 +639,11 @@ typedef struct PDMQUEUE
      * If 0, the queue will use the VM_FF_PDM_QUEUE forced action. */
     uint32_t                                cMilliesInterval;
     /** Interval timer. Only used if cMilliesInterval is non-zero. */
-    PTMTIMERHC                              pTimer;
+    PTMTIMERR3                              pTimer;
     /** Pointer to the VM. */
-    HCPTRTYPE(PVM)                          pVMHC;
+    R3R0PTRTYPE(PVM)                        pVMHC;
     /** LIFO of pending items - HC. */
-    HCPTRTYPE(PPDMQUEUEITEMCORE) volatile   pPendingHC;
+    R3R0PTRTYPE(PPDMQUEUEITEMCORE) volatile pPendingHC;
     /** Pointer to the GC VM and indicator for GC enabled queue.
      * If this is NULL, the queue cannot be used in GC.
      */
@@ -654,9 +662,9 @@ typedef struct PDMQUEUE
     struct PDMQUEUEFREEITEM
     {
         /** Pointer to the free item - HC Ptr. */
-        HCPTRTYPE(PPDMQUEUEITEMCORE) volatile pItemHC;
+        R3R0PTRTYPE(PPDMQUEUEITEMCORE) volatile pItemHC;
         /** Pointer to the free item - GC Ptr. */
-        GCPTRTYPE(PPDMQUEUEITEMCORE) volatile pItemGC;
+        GCPTRTYPE(PPDMQUEUEITEMCORE) volatile   pItemGC;
 #if HC_ARCH_BITS == 64 && GC_ARCH_BITS == 32
         uint32_t                              Alignment0;
 #endif
@@ -689,7 +697,7 @@ typedef struct PDMDEVHLPTASK
     /** The queue item core (don't touch). */
     PDMQUEUEITEMCORE        Core;
     /** Pointer to the device instance (HC Ptr). */
-    HCPTRTYPE(PPDMDEVINS)   pDevInsHC;
+    R3R0PTRTYPE(PPDMDEVINS) pDevInsHC;
     /** This operation to perform. */
     PDMDEVHLPTASKOP         enmOp;
 #if HC_ARCH_BITS == 64
@@ -725,13 +733,17 @@ typedef struct PDMUSBHUB
     /** The USB versions this hub support.
      * Note that 1.1 hubs can take on 2.0 devices. */
     uint32_t            fVersions;
-    /** The number of occupied ports. */
+    /** The number of ports on the hub. */
+    uint32_t            cPorts;
+    /** The number of available ports (0..cPorts). */
     uint32_t            cAvailablePorts;
+    /** The driver instance of the hub. */
+    PPDMDRVINS          pDrvIns;
+    /** Copy of the to the registration structure. */
+    PDMUSBHUBREG        Reg;
+
     /** Pointer to the next hub in the list. */
     struct PDMUSBHUB   *pNext;
-    /** The driver instance of the hub.. */
-    PPDMDRVINS          pDrvIns;
-
 } PDMUSBHUB;
 
 /** Pointer to a const USB HUB registration record. */
@@ -758,20 +770,20 @@ typedef struct PDM
     RTUINT                          uPadding0; /**< Alignment padding.*/
 
     /** Pointer to list of loaded modules. This is HC only! */
-    HCPTRTYPE(PPDMMOD)              pModules;
+    R3PTRTYPE(PPDMMOD)              pModules;
 
     /** List of registered devices. (FIFO) */
-    HCPTRTYPE(PPDMDEV)              pDevs;
+    R3PTRTYPE(PPDMDEV)              pDevs;
     /** List of devices instances. (FIFO) */
-    HCPTRTYPE(PPDMDEVINS)           pDevInstances;
+    R3PTRTYPE(PPDMDEVINS)           pDevInstances;
     /** List of registered USB devices. (FIFO) */
     R3PTRTYPE(PPDMUSB)              pUsbDevs;
     /** List of USB devices instances. (FIFO) */
     R3PTRTYPE(PPDMUSBINS)           pUsbInstances;
     /** List of registered drivers. (FIFO) */
-    HCPTRTYPE(PPDMDRV)              pDrvs;
+    R3PTRTYPE(PPDMDRV)              pDrvs;
     /** List of initialized critical sections. (LIFO) */
-    HCPTRTYPE(PPDMCRITSECTINT)      pCritSects;
+    R3PTRTYPE(PPDMCRITSECTINT)      pCritSects;
     /** PCI Buses. */
     PDMPCIBUS                       aPciBuses[1];
     /** The register PIC device. */
@@ -781,14 +793,14 @@ typedef struct PDM
     /** The registerd I/O APIC device. */
     PDMIOAPIC                       IoApic;
     /** The registered DMAC device. */
-    HCPTRTYPE(PPDMDMAC)             pDmac;
+    R3PTRTYPE(PPDMDMAC)             pDmac;
     /** The registered RTC device. */
-    HCPTRTYPE(PPDMRTC)              pRtc;
+    R3PTRTYPE(PPDMRTC)              pRtc;
     /** The registered USB HUBs. (FIFO) */
     R3PTRTYPE(PPDMUSBHUB)           pUsbHubs;
 
     /** Queue in which devhlp tasks are queued for R3 execution - HC Ptr. */
-    HCPTRTYPE(PPDMQUEUE)            pDevHlpQueueHC;
+    R3R0PTRTYPE(PPDMQUEUE)          pDevHlpQueueHC;
     /** Queue in which devhlp tasks are queued for R3 execution - GC Ptr. */
     GCPTRTYPE(PPDMQUEUE)            pDevHlpQueueGC;
 
@@ -799,12 +811,12 @@ typedef struct PDM
     R3PTRTYPE(PPDMCRITSECT)         apQueuedCritSectsLeaves[8];
 
     /** Linked list of timer driven PDM queues. */
-    HCPTRTYPE(struct PDMQUEUE *)    pQueuesTimer;
+    R3PTRTYPE(struct PDMQUEUE *)    pQueuesTimer;
     /** Linked list of force action driven PDM queues. */
-    HCPTRTYPE(struct PDMQUEUE *)    pQueuesForced;
+    R3PTRTYPE(struct PDMQUEUE *)    pQueuesForced;
     /** Pointer to the queue which should be manually flushed - HCPtr.
      * Only touched by EMT. */
-    HCPTRTYPE(struct PDMQUEUE *)    pQueueFlushHC;
+    R3R0PTRTYPE(struct PDMQUEUE *)  pQueueFlushHC;
     /** Pointer to the queue which should be manually flushed - GCPtr. */
     GCPTRTYPE(struct PDMQUEUE *)    pQueueFlushGC;
 #if HC_ARCH_BITS == 64
@@ -823,9 +835,9 @@ typedef struct PDM
 #if HC_ARCH_BITS == 64
     RTUINT                          padding1;
 #endif
-    HCPTRTYPE(PFNPDMDRVPOLLER)      apfnPollers[16];
-    HCPTRTYPE(PPDMDRVINS)           aDrvInsPollers[16];
-    PTMTIMERHC                      pTimerPollers;
+    R3PTRTYPE(PFNPDMDRVPOLLER)      apfnPollers[16];
+    R3PTRTYPE(PPDMDRVINS)           aDrvInsPollers[16];
+    PTMTIMERR3                      pTimerPollers;
     /** @} */
 
 #ifdef VBOX_WITH_PDM_LOCK
@@ -868,12 +880,14 @@ int         pdmR3DevFindLun(PVM pVM, const char *pszDevice, unsigned iInstance,
 
 int         pdmR3UsbLoadModules(PVM pVM);
 int         pdmR3UsbInstantiateDevices(PVM pVM);
-int         pdmR3UsbInitComplete(PVM pVM);
 PPDMUSB     pdmR3UsbLookup(PVM pVM, const char *pszName);
 int         pdmR3UsbFindLun(PVM pVM, const char *pszDevice, unsigned iInstance, unsigned iLun, PPDMLUN *ppLun);
+int         pdmR3UsbRegisterHub(PVM pVM, PPDMDRVINS pDrvIns, uint32_t fVersions, uint32_t cPorts, PCPDMUSBHUBREG pUsbHubReg, PPCPDMUSBHUBHLP ppUsbHubHlp);
+int         pdmR3UsbVMInitComplete(PVM pVM);
 
 int         pdmR3DrvInit(PVM pVM);
 int         pdmR3DrvDetach(PPDMDRVINS pDrvIns);
+void        pdmR3DrvDestroyChain(PPDMDRVINS pDrvIns);
 PPDMDRV     pdmR3DrvLookup(PVM pVM, const char *pszName);
 
 int         pdmR3LdrInit(PVM pVM);
@@ -896,6 +910,7 @@ void        pdmR3ThreadDestroyAll(PVM pVM);
 int         pdmR3ThreadResumeAll(PVM pVM);
 int         pdmR3ThreadSuspendAll(PVM pVM);
 
+
 #endif /* IN_RING3 */
 
 #ifdef VBOX_WITH_PDM_LOCK
