@@ -1,4 +1,4 @@
-/* $Id: PGMPhys.cpp 4387 2007-08-27 14:24:53Z vboxsync $ */
+/* $Id: PGMPhys.cpp 26987 2007-12-28 10:22:24Z frank $ */
 /** @file
  * PGM - Page Manager and Monitor, Physical Memory Addressing.
  */
@@ -363,25 +363,28 @@ int pgmr3PhysGrowRange(PVM pVM, RTGCPHYS GCPhys)
 
     Log(("pgmr3PhysGrowRange: allocate chunk of size 0x%X at %VGp\n", PGM_DYNAMIC_CHUNK_SIZE, GCPhys));
 
-    unsigned    cPages = PGM_DYNAMIC_CHUNK_SIZE >> PAGE_SHIFT;
-    rc = SUPPageAlloc(cPages, &pvRam);
-    if (VBOX_SUCCESS(rc))
-    {
-        VMSTATE enmVMState = VMR3GetState(pVM);
+    unsigned cPages = PGM_DYNAMIC_CHUNK_SIZE >> PAGE_SHIFT;
 
-        rc = MMR3PhysRegisterEx(pVM, pvRam, GCPhys, PGM_DYNAMIC_CHUNK_SIZE, 0, MM_PHYS_TYPE_DYNALLOC_CHUNK, "Main Memory");
-        if (    VBOX_SUCCESS(rc)
-            ||  enmVMState != VMSTATE_RUNNING)
+    for (;;)
+    {
+        rc = SUPPageAlloc(cPages, &pvRam);
+        if (VBOX_SUCCESS(rc))
         {
-            if (VBOX_FAILURE(rc))
-            {
-                AssertMsgFailed(("Out of memory while trying to allocate a guest RAM chunk at %VGp!\n", GCPhys));
-                LogRel(("PGM: Out of memory while trying to allocate a guest RAM chunk at %VGp (VMstate=%s)!\n", GCPhys, VMR3GetStateName(enmVMState)));
-            }
-            return rc;
+
+            rc = MMR3PhysRegisterEx(pVM, pvRam, GCPhys, PGM_DYNAMIC_CHUNK_SIZE, 0, MM_PHYS_TYPE_DYNALLOC_CHUNK, "Main Memory");
+            if (VBOX_SUCCESS(rc))
+                return rc;
+
+            SUPPageFree(pvRam, cPages);
         }
 
-        SUPPageFree(pvRam, cPages);
+        VMSTATE enmVMState = VMR3GetState(pVM);
+        if (enmVMState != VMSTATE_RUNNING)
+        {
+            AssertMsgFailed(("Out of memory while trying to allocate a guest RAM chunk at %VGp!\n", GCPhys));
+            LogRel(("PGM: Out of memory while trying to allocate a guest RAM chunk at %VGp (VMstate=%s)!\n", GCPhys, VMR3GetStateName(enmVMState)));
+            return rc;
+        }
 
         LogRel(("pgmr3PhysGrowRange: out of memory. pause until the user resumes execution.\n"));
 
@@ -396,9 +399,7 @@ int pgmr3PhysGrowRange(PVM pVM, RTGCPHYS GCPhys)
 
         /* Retry */
         LogRel(("pgmr3PhysGrowRange: VM execution resumed -> retry.\n"));
-        return pgmr3PhysGrowRange(pVM, GCPhys);
     }
-    return rc;
 }
 
 
