@@ -1,4 +1,4 @@
-/* $Id: TM.cpp 4071 2007-08-07 17:07:59Z vboxsync $ */
+/* $Id: TM.cpp 28144 2008-02-15 09:29:30Z frank $ */
 /** @file
  * TM - Timeout Manager.
  */
@@ -220,7 +220,7 @@ TMR3DECL(int) TMR3Init(PVM pVM)
     if (    g_pSUPGlobalInfoPage->u32Magic == SUPGLOBALINFOPAGE_MAGIC
         &&  g_pSUPGlobalInfoPage->u32UpdateIntervalNS >= 250000000 /* 0.25s */)
         return VMSetError(pVM, VERR_INTERNAL_ERROR, RT_SRC_POS,
-                          N_("The GIP update interval is too big. u32UpdateIntervalNS=%RU32 (u32UpdateHz=%RU32)\n"),
+                          N_("The GIP update interval is too big. u32UpdateIntervalNS=%RU32 (u32UpdateHz=%RU32)"),
                           g_pSUPGlobalInfoPage->u32UpdateIntervalNS, g_pSUPGlobalInfoPage->u32UpdateHz);
 
     /*
@@ -242,15 +242,15 @@ TMR3DECL(int) TMR3Init(PVM pVM)
         pVM->tm.s.fTSCVirtualized = true; /* trap rdtsc */
     else if (VBOX_FAILURE(rc))
         return VMSetError(pVM, rc, RT_SRC_POS,
-                          N_("Configuration error: Failed to querying bool value \"UseRealTSC\". (%Vrc)"), rc);
+                          N_("Configuration error: Failed to querying bool value \"UseRealTSC\""));
 
     /* source */
-    rc = CFGMR3QueryBool(pCfgHandle, "UseRealTSC", &pVM->tm.s.fTSCTicking);
+    rc = CFGMR3QueryBool(pCfgHandle, "UseRealTSC", &pVM->tm.s.fTSCUseRealTSC);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         pVM->tm.s.fTSCUseRealTSC = false; /* use virtual time */
     else if (VBOX_FAILURE(rc))
         return VMSetError(pVM, rc, RT_SRC_POS,
-                          N_("Configuration error: Failed to querying bool value \"UseRealTSC\". (%Vrc)"), rc);
+                          N_("Configuration error: Failed to querying bool value \"UseRealTSC\""));
     if (!pVM->tm.s.fTSCUseRealTSC)
         pVM->tm.s.fTSCVirtualized = true;
 
@@ -278,11 +278,11 @@ TMR3DECL(int) TMR3Init(PVM pVM)
     }
     else if (VBOX_FAILURE(rc))
         return VMSetError(pVM, rc, RT_SRC_POS,
-                          N_("Configuration error: Failed to querying uint64_t value \"TSCTicksPerSecond\". (%Vrc)"), rc);
+                          N_("Configuration error: Failed to querying uint64_t value \"TSCTicksPerSecond\""));
     else if (   pVM->tm.s.cTSCTicksPerSecond < _1M
              || pVM->tm.s.cTSCTicksPerSecond >= _4G)
         return VMSetError(pVM, VERR_INVALID_PARAMETER, RT_SRC_POS,
-                          N_("Configuration error: \"TSCTicksPerSecond\" = %RI64 is not in the range 1MHz..4GHz-1!"),
+                          N_("Configuration error: \"TSCTicksPerSecond\" = %RI64 is not in the range 1MHz..4GHz-1"),
                           pVM->tm.s.cTSCTicksPerSecond);
     else
     {
@@ -307,21 +307,21 @@ TMR3DECL(int) TMR3Init(PVM pVM)
         pVM->tm.s.u32VirtualSyncScheduleSlack           =   100000; /* 0.100ms (ASSUMES virtual time is nanoseconds) */
     else if (VBOX_FAILURE(rc))
         return VMSetError(pVM, rc, RT_SRC_POS,
-                          N_("Configuration error: Failed to querying 32-bit integer value \"ScheduleSlack\". (%Vrc)"), rc);
+                          N_("Configuration error: Failed to querying 32-bit integer value \"ScheduleSlack\""));
 
     rc = CFGMR3QueryU64(pCfgHandle, "CatchUpStopThreshold", &pVM->tm.s.u64VirtualSyncCatchUpStopThreshold);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         pVM->tm.s.u64VirtualSyncCatchUpStopThreshold    =   500000; /* 0.5ms */
     else if (VBOX_FAILURE(rc))
         return VMSetError(pVM, rc, RT_SRC_POS,
-                          N_("Configuration error: Failed to querying 64-bit integer value \"CatchUpStopThreshold\". (%Vrc)"), rc);
+                          N_("Configuration error: Failed to querying 64-bit integer value \"CatchUpStopThreshold\""));
 
     rc = CFGMR3QueryU64(pCfgHandle, "CatchUpGiveUpThreshold", &pVM->tm.s.u64VirtualSyncCatchUpGiveUpThreshold);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         pVM->tm.s.u64VirtualSyncCatchUpGiveUpThreshold  = UINT64_C(60000000000); /* 60 sec */
     else if (VBOX_FAILURE(rc))
         return VMSetError(pVM, rc, RT_SRC_POS,
-                          N_("Configuration error: Failed to querying 64-bit integer value \"CatchUpGiveUpThreshold\". (%Vrc)"), rc);
+                          N_("Configuration error: Failed to querying 64-bit integer value \"CatchUpGiveUpThreshold\""));
 
 
 #define TM_CFG_PERIOD(iPeriod, DefStart, DefPct) \
@@ -332,7 +332,7 @@ TMR3DECL(int) TMR3Init(PVM pVM)
         if (rc == VERR_CFGM_VALUE_NOT_FOUND) \
             u64 = UINT64_C(DefStart); \
         else if (VBOX_FAILURE(rc)) \
-            return VMSetError(pVM, rc, RT_SRC_POS, N_("Configuration error: Failed to querying 64-bit integer value \"CatchUpThreshold" #iPeriod "\". (%Vrc)"), rc); \
+            return VMSetError(pVM, rc, RT_SRC_POS, N_("Configuration error: Failed to querying 64-bit integer value \"CatchUpThreshold" #iPeriod "\"")); \
         if (    (iPeriod > 0 && u64 <= pVM->tm.s.aVirtualSyncCatchUpPeriods[iPeriod - 1].u64Start) \
             ||  u64 >= pVM->tm.s.u64VirtualSyncCatchUpGiveUpThreshold) \
             return VMSetError(pVM, VERR_INVALID_PARAMETER, RT_SRC_POS, N_("Configuration error: Invalid start of period #" #iPeriod ": %RU64"), u64); \
@@ -341,7 +341,7 @@ TMR3DECL(int) TMR3Init(PVM pVM)
         if (rc == VERR_CFGM_VALUE_NOT_FOUND) \
             pVM->tm.s.aVirtualSyncCatchUpPeriods[iPeriod].u32Percentage = (DefPct); \
         else if (VBOX_FAILURE(rc)) \
-            return VMSetError(pVM, rc, RT_SRC_POS, N_("Configuration error: Failed to querying 32-bit integer value \"CatchUpPrecentage" #iPeriod "\". (%Vrc)"), rc); \
+            return VMSetError(pVM, rc, RT_SRC_POS, N_("Configuration error: Failed to querying 32-bit integer value \"CatchUpPrecentage" #iPeriod "\"")); \
     } while (0)
     /* This needs more tuning. Not sure if we really need so many period and be so gentle. */
     TM_CFG_PERIOD(0,     750000,   5); /* 0.75ms at 1.05x */
@@ -365,7 +365,7 @@ TMR3DECL(int) TMR3Init(PVM pVM)
         pVM->tm.s.offUTC = 0; /* ns */
     else if (VBOX_FAILURE(rc))
         return VMSetError(pVM, rc, RT_SRC_POS,
-                          N_("Configuration error: Failed to querying 64-bit integer value \"UTCOffset\". (%Vrc)"), rc);
+                          N_("Configuration error: Failed to querying 64-bit integer value \"UTCOffset\""));
 
     /*
      * Setup the warp drive.
@@ -377,11 +377,11 @@ TMR3DECL(int) TMR3Init(PVM pVM)
         pVM->tm.s.u32VirtualWarpDrivePercentage = 100;
     else if (VBOX_FAILURE(rc))
         return VMSetError(pVM, rc, RT_SRC_POS,
-                          N_("Configuration error: Failed to querying uint32_t value \"WarpDrivePercent\". (%Vrc)"), rc);
+                          N_("Configuration error: Failed to querying uint32_t value \"WarpDrivePercent\""));
     else if (   pVM->tm.s.u32VirtualWarpDrivePercentage < 2
              || pVM->tm.s.u32VirtualWarpDrivePercentage > 20000)
         return VMSetError(pVM, VERR_INVALID_PARAMETER, RT_SRC_POS,
-                          N_("Configuration error: \"WarpDrivePercent\" = %RI32 is not in the range 2..20000!"),
+                          N_("Configuration error: \"WarpDrivePercent\" = %RI32 is not in the range 2..20000"),
                           pVM->tm.s.u32VirtualWarpDrivePercentage);
     pVM->tm.s.fVirtualWarpDrive = pVM->tm.s.u32VirtualWarpDrivePercentage != 100;
     if (pVM->tm.s.fVirtualWarpDrive)
@@ -396,7 +396,7 @@ TMR3DECL(int) TMR3Init(PVM pVM)
         u32Millies = 10;
     else if (VBOX_FAILURE(rc))
         return VMSetError(pVM, rc, RT_SRC_POS,
-                          N_("Configuration error: Failed to query uint32_t value \"TimerMillies\", rc=%Vrc.\n"), rc);
+                          N_("Configuration error: Failed to query uint32_t value \"TimerMillies\""));
     rc = RTTimerCreate(&pVM->tm.s.pTimer, u32Millies, tmR3TimerCallback, pVM);
     if (VBOX_FAILURE(rc))
     {
@@ -522,7 +522,7 @@ static bool tmR3HasFixedTSC(void)
             if (uEAX >= 0x80000007)
             {
                 ASMCpuId(0x80000007, &uEAX, &uEBX, &uECX, &uEDX);
-                if (uEDX & BIT(8) /* TscInvariant */)
+                if (uEDX & RT_BIT(8) /* TscInvariant */)
                     return true;
             }
         }
